/*
 * StreamLink JS 6.0.0-23954261
 * (c) Caplin Systems Ltd 2000-2012
 */

/**
 * A minimal (reasonably) caplin bootstrap compatible implementation of
 * caplin.namespace, caplin.extend, caplin.include (does nothing) and caplin.implement (does nothing).
 * Will not be used if bootstrap is already loaded.
 */
(function() {
	var global = this;
	
	if (global.caplin == undefined) {
		global.caplin = {};
		
		function NoOp() {};
		
		caplin.namespace = function(sPackage) {
		    var pPackageElements = sPackage.split(".");
		    var oPackageObject = global;
		    for (var n = 0, nLength = pPackageElements.length; n < nLength; n++) {
		        var sNextPackageElement = pPackageElements[n];
		        if (sNextPackageElement == "") if (n == nLength - 1) continue;
		        else throw new Error("invalid package argument: " + sPackage);
		        if (!oPackageObject[sNextPackageElement]) oPackageObject[sNextPackageElement] = {};
		        oPackageObject = oPackageObject[sNextPackageElement];
		    }
		};
		caplin.extend = function(oSubClass, oSuperClass) {
			if (oSuperClass == undefined) throw new Error("SuperClass is undefined, check the class exists.");
			function ClassDefinition() {};
			ClassDefinition.prototype = oSuperClass.prototype;
			oSubClass.prototype = new ClassDefinition();
		};
		caplin.include = NoOp;
		caplin.implement = NoOp;
	}
	
})();
caplin.namespace("caplin.utils");

caplin.utils.CommonUtils = function(){};

/**
 * Formats a date into the format <code>2012/07/09-09:00:43.755 +0100</code>, for the passed
 * date/time.
 * 
 * @param {Date} oDate the date to be formatted.
 * 
 * @type String
 * @return Returns a date stamp for the passed date/time in the standard Caplin format.
 */
caplin.utils.CommonUtils.formatDateStamp = function(oDate)
{
	var l_nTimezoneOffset = oDate.getTimezoneOffset();
	var l_nAbsoluteTimezoneOffset = Math.abs(l_nTimezoneOffset);
	var l_sOffsetSign = (l_nTimezoneOffset <= 0)?"+":"-";
	var l_sOffsetHours = caplin.utils.CommonUtils.padZeros(Math.floor(Math.abs(l_nAbsoluteTimezoneOffset) / 60), 2);
	var l_sOffsetMinutes = caplin.utils.CommonUtils.padZeros(l_nAbsoluteTimezoneOffset % 60, 2);
	return oDate.getFullYear() + "/" + caplin.utils.CommonUtils.padZeros(oDate.getMonth() +1,2) + "/" + caplin.utils.CommonUtils.padZeros(oDate.getDate(),2) + "-" + caplin.utils.CommonUtils.padZeros(oDate.getHours(), 2) + ":" + caplin.utils.CommonUtils.padZeros(oDate.getMinutes(), 2) + ":" + caplin.utils.CommonUtils.padZeros(oDate.getSeconds(), 2) + "." + caplin.utils.CommonUtils.padZeros(oDate.getMilliseconds(), 3) + " " + l_sOffsetSign + l_sOffsetHours + l_sOffsetMinutes;
};

/**
 * Determines whether the specified value needs to be padded with leading zeros or not, and if
 * so, adds the padding.
 * <br/><br/>
 * Examples:
 * <br/><br/>
 * <table>
 * <tr><th>Value</th> <th>Padded Length</th> <th>Padded Value</th></tr>
 * <tr><td>0</td>     <td>2</td>             <td>00</td>          </tr>
 * <tr><td>1</td>     <td>2</td>             <td>01</td>          </tr>
 * <tr><td>10</td>    <td>2</td>             <td>10</td>          </tr>
 * <tr><td>28</td>    <td>2</td>             <td>28</td>          </tr>
 * <tr><td>1</td>     <td>3</td>             <td>001</td>         </tr>
 * <tr><td>10</td>    <td>3</td>             <td>010</td>         </tr>
 * <tr><td>100</td>   <td>3</td>             <td>100</td>         </tr>
 * </table>
 * 
 * @param {int} l_nValue The value to be padded with leading zeros.
 * @param {String} l_nPaddedLength The number of characters the padded value must have.
 * @return The specified value, including any padding that was necessary.
 * @private
 */

caplin.utils.CommonUtils.zeroPadding = ["", "0", "00", "000"];

caplin.utils.CommonUtils.padZeros = function(l_nValue, l_nPaddedLength) 
{
	var l_nValueLength = (l_nValue + "").length;
	var l_nDifference = l_nPaddedLength - l_nValueLength;
	return caplin.utils.CommonUtils.zeroPadding[l_nDifference] + l_nValue;
};

caplin.namespace("caplin.xlang");

caplin.xlang.Utils = function(){};

caplin.xlang.Utils.dictionaryToString = function(dict)
{
	var sb = [];
	var first = true;
	for (var key in dict)
	{
		var value = dict[key];
		if (!first)
		{
			sb.push(",");
		}
		sb.push(key);
		sb.push("=");
		sb.push(value.toString());
		first = false;
	}
	
	return sb.join("");
};

if (Array.prototype.indexOf) {
	caplin.xlang.Utils.indexOf = function(arr, item) {
		return arr.indexOf(item);
	}
} else {
	caplin.xlang.Utils.indexOf = function(arr, item) {
		for (var i = 0, len = arr.length; i < len; ++i) {
			if (arr[i] == item) return i; 
		}
		return -1;
	}
}
//## seedfile.xml omitted as it is not a JavaScript source file ##//

//## LogLevel.js ##//
caplin.namespace("caplin.streamlink");

/**
 * @class  <p>Enumeration that represents the available logging levels.</p>
 *  <p>Each log message generated by StreamLink has an associated LogLevel that indicates its importance, 
 *     in the following order (highest to lowest).</p>
 *  
 *  <table>
 *   <tr>
 *   	<td>Config</td>
 *   	<td>(highest importance)</td>
 *   </tr>
 *   <tr>
 *   	<td>Critical</td>
 *   	<td></td>
 *   </tr>
 *   <tr>
 *   	<td>Error</td>
 *   	<td></td>
 *   </tr>
 *   <tr>
 *   	<td>Warning</td>
 *   	<td></td>
 *   </tr>
 *   <tr>
 *   	<td>Notify</td>
 *   	<td></td>
 *   </tr>
 *   <tr>
 *   	<td>Info</td>
 *   	<td></td>
 *   </tr>
 *   <tr>
 *   	<td>Fine</td>
 *   	<td></td>
 *   </tr>
 *   <tr>
 *   	<td>Finer</td>
 *   	<td></td>
 *   </tr>
 *   <tr>
 *   	<td>Finest</td>
 *   	<td>(lowest importance)</td>
 *   </tr>
 *   
 *  </table>
 * 
 * @name caplin.streamlink.LogLevel
 */
caplin.streamlink.LogLevel = function(name, value) {
	// Constructor body
	this._name = name;
	this._value = value;
	this._paddedName = name;
	if (this._paddedName.length < 6) {
		this._paddedName = this._paddedName + "      ";
		this._paddedName = this._paddedName.substring(0, 6);
	}
};

/**
 *  <p>Used for messages about StreamLink's configuration.</p>
 */
caplin.streamlink.LogLevel.CONFIG = new caplin.streamlink.LogLevel("CONFIG", 1);
/**
 *  <p>Used for messages indicating a serious failure.</p>
 */
caplin.streamlink.LogLevel.CRITICAL = new caplin.streamlink.LogLevel("CRITICAL", 2);
/**
 *  <p>Used for messages which indicate a problem.</p>
 */
caplin.streamlink.LogLevel.ERROR = new caplin.streamlink.LogLevel("ERROR", 3);
/**
 *  <p>Used for messages which indicate a potential problem.</p>
 */
caplin.streamlink.LogLevel.WARN = new caplin.streamlink.LogLevel("WARN", 4);
/**
 *  <p>Used for messages of more importance than INFO but may not indicate problems.</p>
 */
caplin.streamlink.LogLevel.NOTIFY = new caplin.streamlink.LogLevel("NOTIFY", 5);
/**
 *  <p>Used for information messages.</p>
 */
caplin.streamlink.LogLevel.INFO = new caplin.streamlink.LogLevel("INFO", 6);
/**
 *  <p>Used for tracing messages.</p>
 */
caplin.streamlink.LogLevel.FINE = new caplin.streamlink.LogLevel("FINE", 7);
/**
 *  <p>Used for fairly detailed tracing messages.</p>
 */
caplin.streamlink.LogLevel.FINER = new caplin.streamlink.LogLevel("FINER", 8);
/**
 *  <p>Used for the most finely detailed tracing messages.</p> 
 */
caplin.streamlink.LogLevel.FINEST = new caplin.streamlink.LogLevel("FINEST", 9);
caplin.streamlink.LogLevel.values = function() { return [caplin.streamlink.LogLevel.CONFIG, caplin.streamlink.LogLevel.CRITICAL, caplin.streamlink.LogLevel.ERROR, caplin.streamlink.LogLevel.WARN, caplin.streamlink.LogLevel.NOTIFY, caplin.streamlink.LogLevel.INFO, caplin.streamlink.LogLevel.FINE, caplin.streamlink.LogLevel.FINER, caplin.streamlink.LogLevel.FINEST];};
caplin.streamlink.LogLevel.valueOf = function(name) { return this[name]; };
caplin.streamlink.LogLevel.prototype.getValue = function() {
	return this._value;
};

caplin.streamlink.LogLevel.prototype.getPaddedName = function() {
	return this._paddedName;
};

caplin.streamlink.LogLevel.prototype.toString = function() {
	return this._name;
};



//## Receiver.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker");


caplin.streamlink.impl.bridge.worker.Receiver = function() {};

caplin.streamlink.impl.bridge.worker.Receiver.prototype.getReceiverKey = function() { throw new Error('caplin.streamlink.impl.bridge.worker.Receiver.getReceiverKey :  This is an abstract method'); };

caplin.streamlink.impl.bridge.worker.Receiver.prototype.getReceiverId = function() { throw new Error('caplin.streamlink.impl.bridge.worker.Receiver.getReceiverId :  This is an abstract method'); };

caplin.streamlink.impl.bridge.worker.Receiver.prototype.receive = function(method, args) { throw new Error('caplin.streamlink.impl.bridge.worker.Receiver.receive :  This is an abstract method'); };



//## Logger.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  <p>Allows StreamLink log messages to be written to the destination of your choice.</p>
 *  <p>You obtain an instance of <code>caplin.streamlink.Logger</code> by calling 
 *  <code>streamLinkInstance.getLogger()</code> on the StreamLink instance.</p>
 *  <p>When suitably configured (by adding <code>?debug=finer</code> to your url),
 *  StreamLink's log messages are automatically written to a dedicated
 *  window (the StreamLink Console). You would normally only enable this facility for debug purposes.
 *  You can call the Logger.log() method to write your own log messages to the StreamLink Console.</p>
 *  <p>You can use <code>caplin.streamlink.Logger</code> to write StreamLink's log messages to some
 *  other destination, such as a window that also contains log messages originating from your application.
 *  To do this, implement a {@link caplin.streamlink.LogListener} that receives the StreamLink
 *  messages and logs them to the required destination. Then obtain an instance of {@link caplin.streamlink.Logger},
 *  and call addListener() to attach your <code>Log Listener</code> to this instance. For a code example showing this, see 
 *  {@link caplin.streamlink.LogListener}.
 *  
 * 
 * @name caplin.streamlink.Logger
 */
caplin.streamlink.Logger = function() {};

/**
 *  <p>Adds a {@link caplin.streamlink.LogListener} to this Logger. The listener is
 *  called whenever a StreamLink message with the defined {@link caplin.streamlink.LogLevel} is to be logged.</p>
 *  
 * @param {caplin.streamlink.LogListener} listener The object that implements the {@link caplin.streamlink.LogListener} Interface.
 * @param {caplin.streamlink.LogLevel} level The {@link caplin.streamlink.LogLevel} of log messages for which the <code>listener</code> is to 
 *  be called. Only messages at or above this log level will invoke this <code>listener</code>.
 *  
 * @function
 */
caplin.streamlink.Logger.prototype.addListener = function(listener, level) { throw new Error('caplin.streamlink.Logger.addListener :  This is an abstract method'); };

/**
 *  <p>Sends a log message to all {@link caplin.streamlink.LogListener}'s that have
 *  been registered for this log level or above.</p>
 *  
 *  @param logLevel The logging level that will be associated with the logged message.
 * @param {String} message The message to log.
 * @param {Object} args Arguments included in the log message.
 *  <p>The <code>log()</code> method allows inline replacement of arguments. The arguments to replace these placeholders are then passed at the end.</p>
 *  <p>Example : <code>logger.log(caplin.streamlink.LogLevel.Critical, "Error {0} has occurred with code {1}.", errorString, errorCode);</code></p>
 *  
 * @function
 */
caplin.streamlink.Logger.prototype.log = function(level, message) { throw new Error('caplin.streamlink.Logger.log :  This is an abstract method'); };

/**
 *  <p>Removes a {@link caplin.streamlink.LogListener} that you previously added (see <code>addListener</code>).</p>
 *  
 * @param {caplin.streamlink.LogListener} listener The {@link caplin.streamlink.LogListener} you added.
 * @function
 */
caplin.streamlink.Logger.prototype.removeListener = function(listener) { throw new Error('caplin.streamlink.Logger.removeListener :  This is an abstract method'); };



//## LoggerImpl.js ##//
caplin.namespace("caplin.streamlink.impl.logger");

// caplin.forceload(caplin.streamlink.Logger

caplin.streamlink.impl.logger.LoggerImpl = function(runtime) {
	// Field Definitions
	this._array = [];

	// Constructor body
	this._runtime = runtime;
	this._maxLogLevelValue = 0;
};

// caplin.implement(caplin.streamlink.impl.logger.LoggerImpl, caplin.streamlink.Logger);

caplin.streamlink.impl.logger.LoggerImpl.prototype._logToListener = function(listener, info) {
	this._runtime.createTimeout(function() {
		listener.onLog(info);
	}, 0);
};

caplin.streamlink.impl.logger.LoggerImpl.prototype.addListener = function(listener, level) {
	this._array.push(new caplin.streamlink.impl.logger.ListenerAndLevel(listener, level));
	if (level.getValue() > this._maxLogLevelValue) {
		this._maxLogLevelValue = level.getValue();
	}
};

caplin.streamlink.impl.logger.LoggerImpl.prototype.log = function(level, message) {
	var args = Array.prototype.slice.call(arguments, 2);	var levelValue = level.getValue();
	if (levelValue <= this._maxLogLevelValue) {
		var info = new caplin.streamlink.impl.logger.LazyLogInfo(level, message, args);
		for (var i = 0; i < this._array.length; (i)++) {
			var listenerAndLevel = this._array[i];
			if (listenerAndLevel.getLevel().getValue() >= levelValue) {
				this._logToListener(listenerAndLevel.getListener(), info);
			}
		}
	}
};

caplin.streamlink.impl.logger.LoggerImpl.prototype.removeListener = function(listener) {
	var toRemove = -(1);
	this._maxLogLevelValue = 0;
	for (var i = 0; i < this._array.length; (i)++) {
		var listenerAndLevel = this._array[i];
		if (listenerAndLevel.getListener() == listener) {
			toRemove = i;
		}
		if (listenerAndLevel.getLevel().getValue() > this._maxLogLevelValue) {
			this._maxLogLevelValue = listenerAndLevel.getLevel().getValue();
		}
	}
	if (toRemove != -(1)) {
		this._array.splice(toRemove,1)[0];
	}
};



//## LoggerOnWorker.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.server");

// caplin.forceload(caplin.streamlink.impl.logger.LoggerImpl
// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver

caplin.streamlink.impl.bridge.worker.server.LoggerOnWorker = function(bridge, id) {
	// Super
	caplin.streamlink.impl.logger.LoggerImpl.call(this, null);
	// Constructor body
	this.bridge = bridge;
	this.id = id;
};

caplin.extend(caplin.streamlink.impl.bridge.worker.server.LoggerOnWorker, caplin.streamlink.impl.logger.LoggerImpl);

// caplin.implement(caplin.streamlink.impl.bridge.worker.server.LoggerOnWorker, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.server.LoggerOnWorker.prototype.getReceiverKey = function() {
	return null;
};

caplin.streamlink.impl.bridge.worker.server.LoggerOnWorker.prototype.getReceiverId = function() {
	return this.id;
};

caplin.streamlink.impl.bridge.worker.server.LoggerOnWorker.prototype._logToListener = function(listener, info) {
	listener.onLog(info);
};

caplin.streamlink.impl.bridge.worker.server.LoggerOnWorker.prototype.receive = function(method, args) {
	if ((method == ("addListener"))) {
		var id = args[0];
		var level = caplin.streamlink.LogLevel.valueOf(args[1]);
		var listener = new caplin.streamlink.impl.bridge.worker.server.LogListenerOnWorker(this.bridge, id);
		this.bridge.addReceiver(listener);
		this.addListener(listener, level);
	}
	else if ((method == ("removeListener"))) {
		var id = args[0];
		var level = caplin.streamlink.LogLevel.valueOf(args[1]);
		var listener = this.bridge.getReceiver(id);
		this.removeListener(listener);
	}
};



//## Message.js ##//
caplin.namespace("caplin.streamlink.impl.subscription");


caplin.streamlink.impl.subscription.Message = function(handler, packet) {
	// Constructor body
	this.handler = handler;
	this.packet = packet;
};

caplin.streamlink.impl.subscription.Message.prototype.getPacket = function() {
	return this.packet;
};

caplin.streamlink.impl.subscription.Message.prototype.getHandler = function() {
	return this.handler;
};



//## ConnectionStatusEvent.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  <p>Interface used to get the {@link caplin.streamlink.ConnectionState} and host URL of a connection.<p>
 *  <p>The <code>ConnectionStatusEvent</code> will be provided on a <code>onConnectionStatusChange</code> 
 *  callback to a {@link caplin.streamlink.ConnectionListener}.</p>
 * 
 * @name caplin.streamlink.ConnectionStatusEvent
 */
caplin.streamlink.ConnectionStatusEvent = function() {};

/**
 *  <p>Returns the {@link caplin.streamlink.ConnectionState} of a connection<p>
 *  
 *  @returns {caplin.streamlink.ConnectionState} the connection state 
 * @function
 */
caplin.streamlink.ConnectionStatusEvent.prototype.getConnectionState = function() { throw new Error('caplin.streamlink.ConnectionStatusEvent.getConnectionState :  This is an abstract method'); };

/**
 *  <p>Returns a description of the connection event<p>
 *  
 *  @returns {String} the message
 * @function
 */
caplin.streamlink.ConnectionStatusEvent.prototype.getMessage = function() { throw new Error('caplin.streamlink.ConnectionStatusEvent.getMessage :  This is an abstract method'); };



//## Serializable.js ##//
caplin.namespace("caplin.streamlink.impl");


caplin.streamlink.impl.Serializable = function() {};

caplin.streamlink.impl.Serializable.prototype._toDictionary = function() { throw new Error('caplin.streamlink.impl.Serializable._toDictionary :  This is an abstract method'); };



//## ConnectionStatusEventImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.ConnectionStatusEvent
// caplin.forceload(caplin.streamlink.impl.Serializable

caplin.streamlink.impl.event.ConnectionStatusEventImpl = function(connectionState, message) {
	// Constructor body
	this._message = message;
	this._connectionState = connectionState;
};

// caplin.implement(caplin.streamlink.impl.event.ConnectionStatusEventImpl, caplin.streamlink.ConnectionStatusEvent);
// caplin.implement(caplin.streamlink.impl.event.ConnectionStatusEventImpl, caplin.streamlink.impl.Serializable);

caplin.streamlink.impl.event.ConnectionStatusEventImpl.prototype.getConnectionState = function() {
	return this._connectionState;
};

caplin.streamlink.impl.event.ConnectionStatusEventImpl.createFromDictionary = function(serialized) {
	var connectionState = caplin.streamlink.ConnectionState.valueOf(serialized["State"]);
	return new caplin.streamlink.impl.event.ConnectionStatusEventImpl(connectionState, serialized["Message"]);
};

caplin.streamlink.impl.event.ConnectionStatusEventImpl.prototype._toDictionary = function() {
	var root = {} ;
	root["State"] = this._connectionState.toString();
	root["Message"] = this._message;
	return root;
};

caplin.streamlink.impl.event.ConnectionStatusEventImpl.prototype.getMessage = function() {
	return this._message;
};

caplin.streamlink.impl.event.ConnectionStatusEventImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("ConnectionStatusEventImpl [message=");
	builder.push(this._message);
	builder.push(", connectionState=");
	builder.push(this._connectionState);
	builder.push("]");
	return builder.join("");
};



//## SourceStatus.js ##//
caplin.namespace("caplin.streamlink");

/**
 * @class  <p>Enumeration that defines the status of a source.</p>
 *  <p>The <code>SourceStatus</code> will be provided by calling <code>getSourceStatus</code> on a {@link caplin.streamlink.SourceStatusEvent}.</p> 
 *  
 * 
 * @name caplin.streamlink.SourceStatus
 */
caplin.streamlink.SourceStatus = {
	/**
	 *  <p>A source has gone up.</p>
	 */
	DOWN: "DOWN",
	/**
	 *  <p>A source has gone down.</p>
	 */
	UP: "UP",
	values: function() {return [caplin.streamlink.SourceStatus.DOWN, caplin.streamlink.SourceStatus.UP];},
	valueOf: function(name) {return name;}
};

//## ConnectionListener.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  <p>Defines the interface for receiving information about changes in the state of the StreamLink
 *  connection, the data services available from the Liberator, and the DataSources that are normally connected 
 *  to the Liberator.</p> 
 * 
 *  <p>You should implement this interface if your application needs to take particular actions when such state 
 *  changes occur.</p>
 * 
 *  <p>Your application should not call into StreamLink when the connection status changes; StreamLink manages
 *  the connection state and will recover the connection to the Liberator without your application intervening.</p>
 *  
 *  <p>A trivial implementation would be as follows :</p>
 *  <p><code><pre>streamLink.addConnectionListener({
 *         onConnectionStatusChange : function(connectionStatusEvent)
 *         {
 *                 console.log("Connection State : " + connectionStatusEvent.getConnectionState());
 *         },
 *         onServiceStatusChange : function(serviceStatusEvent)
 *         {
 *                 console.log(serviceStatusEvent.getServiceName() + " is now " + serviceStatusEvent.getServiceStatus());
 *         },
 *         onSourceStatusChange : function(sourceStatusEvent)
 *         {
 *                 console.log(sourceStatusEvent.getSourceName() + " is now " + sourceStatusEvent.getSourceStatus());
 *         },
 *         onStatisticsChange : function(statisticsEvent)
 *         {
 *                 console.log("Latency : " + statisticsEvent.getAverageLatency());
 *         },
 *         onConnectionFail : function(reason)
 *         {
 *                 console.log("Connection Fail : " + reason);
 *         }
 * });
 * </pre></code></p>
 * 
 * @name caplin.streamlink.ConnectionListener
 */
caplin.streamlink.ConnectionListener = function() {};

/**
 *  <p>Invoked when a new connection cannot be established after trying all available connection options. 
 *  Implementing this method is optional.</p>
 *  @param message The reason why a connection cannot be established.
 * @function
 */
caplin.streamlink.ConnectionListener.prototype.onConnectionFail = function(reason) { throw new Error('caplin.streamlink.ConnectionListener.onConnectionFail :  This is an abstract method'); };

/**
 *  <p>Invoked when the state of the StreamLink connection changes. Implementing this method is optional.</p>
 * @param {caplin.streamlink.ConnectionStatusEvent} connectionStatusEvent The new status of the connection.
 * @function
 */
caplin.streamlink.ConnectionListener.prototype.onConnectionStatusChange = function(connectionStatusEvent) { throw new Error('caplin.streamlink.ConnectionListener.onConnectionStatusChange :  This is an abstract method'); };

/**
 *  <p>Invoked when the state of a DataService available from the Liberator changes. Implementing this method is optional.<p>
 * @param {caplin.streamlink.ServiceStatusEvent} serviceStatusEvent The new status of the DataService.
 * @function
 */
caplin.streamlink.ConnectionListener.prototype.onServiceStatusChange = function(serviceStatusEvent) { throw new Error('caplin.streamlink.ConnectionListener.onServiceStatusChange :  This is an abstract method'); };

/**
 *  <p>Invoked when the state of one of the Integration Adapters connected to Liberator changes. 
 *  Implementing this method is optional.</p>
 * @param {caplin.streamlink.SourceStatusEvent} sourceStatusEvent The new status of the Integration Adapter.
 * @function
 */
caplin.streamlink.ConnectionListener.prototype.onSourceStatusChange = function(sourceStatusEvent) { throw new Error('caplin.streamlink.ConnectionListener.onSourceStatusChange :  This is an abstract method'); };

/**
 *  <p>Invoked when new connection statistics are available. Implementing this method is optional.</p>
 * @param {caplin.streamlink.StatisticsEvent} statisticsEvent The new connection statistics.
 * @function
 */
caplin.streamlink.ConnectionListener.prototype.onStatisticsChange = function(statisticsEvent) { throw new Error('caplin.streamlink.ConnectionListener.onStatisticsChange :  This is an abstract method'); };



//## CommandParameters.js ##//
caplin.namespace("caplin.streamlink");

caplin.streamlink.CommandParameters = function() {};

caplin.streamlink.CommandParameters.prototype.setPersistent = function() { throw new Error('caplin.streamlink.CommandParameters.setPersistent :  This is an abstract method'); };



//## EventHandler.js ##//
caplin.namespace("caplin.streamlink.impl.subscription");


caplin.streamlink.impl.subscription.EventHandler = function() {};

caplin.streamlink.impl.subscription.EventHandler.prototype._handleEvent = function(event) { throw new Error('caplin.streamlink.impl.subscription.EventHandler._handleEvent :  This is an abstract method'); };



//## CommandResponse.js ##//
caplin.namespace("caplin.streamlink.impl.command");

// caplin.forceload(caplin.streamlink.impl.subscription.EventHandler

caplin.streamlink.impl.command.CommandResponse = function(listener, context) {
	// Field Definitions
	this.unpersist = false;

	// Constructor body
	this.listener = listener;
	this.context = context;
};

// caplin.implement(caplin.streamlink.impl.command.CommandResponse, caplin.streamlink.impl.subscription.EventHandler);

caplin.streamlink.impl.command.CommandResponse.prototype.isUnpersist = function() {
	return this.unpersist;
};

caplin.streamlink.impl.command.CommandResponse.prototype._handleEvent = function(event) {
	var self = this;
	this.context.runtime.createTimeout(function() {
		event._publishCommandResponse(self.listener);
	}, 0);
};

caplin.streamlink.impl.command.CommandResponse.prototype.disconnectOccurred = function(subject) {
	var self = this;
	var errorEvent = new caplin.streamlink.impl.event.CommandErrorEventImpl(caplin.streamlink.CommandErrorType.DISCONNECT_OCCURRED);
	this.context.runtime.createTimeout(function() {
		self.listener.onCommandError(subject, errorEvent);
	}, 0);
};

caplin.streamlink.impl.command.CommandResponse.prototype.unPersist = function() {
	this.unpersist = true;
};



//## SourceStatusEvent.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  <p>Provides information about a change in status of an Integration Adapter to which StreamLink
 *  is connected via the Liberator.</p>
 *  <p>The <code>SourceStatusEvent</code> will be provided on a <code>onSourceStatusChange</code> 
 *  callback to a {@link caplin.streamlink.ConnectionListener}.</p>
 *  
 *  <p>Within Liberator, data is considered to be available through a particular data service rather
 *  than from the underlying integration adapters. Therefore, if your application needs to know about the
 *  availability of data, it should examine the status of the data service by looking at {@link caplin.streamlink.ServiceStatusEvent}s,
 *  rather than examining {@link caplin.streamlink.SourceStatusEvent}s.</p>
 * 
 * @name caplin.streamlink.SourceStatusEvent
 */
caplin.streamlink.SourceStatusEvent = function() {};

/**
 *  <p>Returns the new {@link caplin.streamlink.SourceStatus} of an integration adapter.</p>
 *  
 *  @returns {caplin.streamlink.SourceStatus} the source status. 
 * @function
 */
caplin.streamlink.SourceStatusEvent.prototype.getSourceStatus = function() { throw new Error('caplin.streamlink.SourceStatusEvent.getSourceStatus :  This is an abstract method'); };

/**
 *  <p>Returns the name of the integration adapter that has changed state.</p>
 *  
 *  @returns {String} the source name.
 * @function
 */
caplin.streamlink.SourceStatusEvent.prototype.getSourceName = function() { throw new Error('caplin.streamlink.SourceStatusEvent.getSourceName :  This is an abstract method'); };



//## SourceStatusEventImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.SourceStatusEvent
// caplin.forceload(caplin.streamlink.impl.Serializable

caplin.streamlink.impl.event.SourceStatusEventImpl = function(sourceStatus, sourceName) {
	// Constructor body
	this._sourceStatus = sourceStatus;
	this._sourceName = sourceName;
};

// caplin.implement(caplin.streamlink.impl.event.SourceStatusEventImpl, caplin.streamlink.SourceStatusEvent);
// caplin.implement(caplin.streamlink.impl.event.SourceStatusEventImpl, caplin.streamlink.impl.Serializable);

caplin.streamlink.impl.event.SourceStatusEventImpl.prototype.getSourceStatus = function() {
	return this._sourceStatus;
};

caplin.streamlink.impl.event.SourceStatusEventImpl.createFromDictionary = function(root) {
	return new caplin.streamlink.impl.event.SourceStatusEventImpl(caplin.streamlink.SourceStatus.valueOf(root["Status"]), root["Name"]);
};

caplin.streamlink.impl.event.SourceStatusEventImpl.createFromRttp = function(rttpCode, tokens, fieldMap, encoder) {
	if (rttpCode == caplin.streamlink.impl.protocolhandler.RttpCode.SOURCE_UP) {
		return new caplin.streamlink.impl.event.SourceStatusEventImpl(caplin.streamlink.SourceStatus.UP, encoder.rttpDecode(tokens[2]));
	}
	else if (rttpCode == caplin.streamlink.impl.protocolhandler.RttpCode.SOURCE_DOWN) {
		return new caplin.streamlink.impl.event.SourceStatusEventImpl(caplin.streamlink.SourceStatus.DOWN, encoder.rttpDecode(tokens[2]));
	}
	return null;
};

caplin.streamlink.impl.event.SourceStatusEventImpl.prototype._toDictionary = function() {
	var root = {} ;
	root["Status"] = this._sourceStatus.toString();
	root["Name"] = this._sourceName;
	return root;
};

caplin.streamlink.impl.event.SourceStatusEventImpl.prototype.getSourceName = function() {
	return this._sourceName;
};

caplin.streamlink.impl.event.SourceStatusEventImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("SourceStatusEventImpl [sourceName=");
	builder.push(this._sourceName);
	builder.push(", sourceStatus=");
	builder.push(this._sourceStatus);
	builder.push("]");
	return builder.join("");
};



//## SubscriptionErrorEvent.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  <p>Provides information about a subscription error.</p>
 *  
 * 
 * @name caplin.streamlink.SubscriptionErrorEvent
 */
caplin.streamlink.SubscriptionErrorEvent = function() {};

/**
 *  <p>Returns the type of subscription error.</p>
 *  @returns {caplin.streamlink.SubscriptionErrorType} The subscription error.
 * @function
 */
caplin.streamlink.SubscriptionErrorEvent.prototype.getError = function() { throw new Error('caplin.streamlink.SubscriptionErrorEvent.getError :  This is an abstract method'); };

/**
 *  <p>Returns the subject of subscription to which the event relates.</p>
 *  @returns {String} The subject.
 * @function
 */
caplin.streamlink.SubscriptionErrorEvent.prototype.getSubject = function() { throw new Error('caplin.streamlink.SubscriptionErrorEvent.getSubject :  This is an abstract method'); };



//## SubjectErrorEventImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.SubscriptionErrorEvent
// caplin.forceload(caplin.streamlink.impl.Serializable

caplin.streamlink.impl.event.SubjectErrorEventImpl = function(subject, error) {
	// Constructor body
	this._subject = subject;
	this._error = error;
};

// caplin.implement(caplin.streamlink.impl.event.SubjectErrorEventImpl, caplin.streamlink.SubscriptionErrorEvent);
// caplin.implement(caplin.streamlink.impl.event.SubjectErrorEventImpl, caplin.streamlink.impl.Serializable);

caplin.streamlink.impl.event.SubjectErrorEventImpl.prototype.getError = function() {
	return this._error;
};

caplin.streamlink.impl.event.SubjectErrorEventImpl.rttpCodeToError = function(rttpCode) {
	switch (rttpCode) {
	case caplin.streamlink.impl.protocolhandler.RttpCode.NOT_FOUND:
	case caplin.streamlink.impl.protocolhandler.RttpCode.NOT_FOUND_DELAY:
	return caplin.streamlink.SubscriptionErrorType.NOT_FOUND;
	case caplin.streamlink.impl.protocolhandler.RttpCode.READ_DENY:
	case caplin.streamlink.impl.protocolhandler.RttpCode.READ_DENY_DELAY:
	return caplin.streamlink.SubscriptionErrorType.READ_DENIED;
	case caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY:
	case caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY_DELAY:
	return caplin.streamlink.SubscriptionErrorType.WRITE_DENIED;
	case caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY_TRADE_GROUP_LICENSE_EXCEEDED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY_TRADE_GROUP_LICENSE_EXCEEDED_DELAY:
	return caplin.streamlink.SubscriptionErrorType.WRITE_DENIED_DUE_TO_LICENSE;
	case caplin.streamlink.impl.protocolhandler.RttpCode.UNAVAILABLE:
	case caplin.streamlink.impl.protocolhandler.RttpCode.UNAVAILABLE_DELAY:
	return caplin.streamlink.SubscriptionErrorType.UNAVAILABLE;
	case caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_PARAMETERS:
	case caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_PARAMETERS_DELAY:
	return caplin.streamlink.SubscriptionErrorType.INVALID_PARAMETERS;
	case caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_REMOVED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.DELETED_DELAY:
	return caplin.streamlink.SubscriptionErrorType.DELETED;
	default :
	return caplin.streamlink.SubscriptionErrorType.OK;
	}
};

caplin.streamlink.impl.event.SubjectErrorEventImpl.createFromDictionary = function(root) {
	var subject = root["Subject"];
	var error = caplin.streamlink.SubscriptionErrorType.valueOf(root["Error"]);
	return new caplin.streamlink.impl.event.SubjectErrorEventImpl(subject, error);
};

caplin.streamlink.impl.event.SubjectErrorEventImpl.createFromRttpEvent = function(event) {
	return new caplin.streamlink.impl.event.SubjectErrorEventImpl(event.getSubject(), caplin.streamlink.impl.event.SubjectErrorEventImpl.rttpCodeToError(event._getRttpCode()));
};

caplin.streamlink.impl.event.SubjectErrorEventImpl.prototype._toDictionary = function() {
	var root = {} ;
	root["Subject"] = this._subject;
	root["Error"] = this._error.toString();
	return root;
};

caplin.streamlink.impl.event.SubjectErrorEventImpl.prototype.getSubject = function() {
	return this._subject;
};

caplin.streamlink.impl.event.SubjectErrorEventImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("SubjectErrorEvent [subject=");
	builder.push(this._subject);
	builder.push(", error=");
	builder.push(this._error);
	builder.push("]");
	return builder.join("");
};



//## KeymasterConnection.js ##//
caplin.namespace("caplin.streamlink.impl.credentials");

caplin.streamlink.impl.credentials.KeymasterConnection = function() {};

caplin.streamlink.impl.credentials.KeymasterConnection.prototype.cancel = function() { throw new Error('caplin.streamlink.impl.credentials.KeymasterConnection.cancel :  This is an abstract method'); };

caplin.streamlink.impl.credentials.KeymasterConnection.prototype.start = function() { throw new Error('caplin.streamlink.impl.credentials.KeymasterConnection.start :  This is an abstract method'); };



//## KeymasterConnectionImpl.js ##//
caplin.namespace("caplin.streamlink.js.impl.credentials");

caplin.include("caplin.streamlink.impl.credentials.KeymasterConnection", true);
caplin.include("caplin.streamlink.js.impl.connection.Channel", true);

// caplin.forceload(caplin.streamlink.impl.connection.Channel);

caplin.streamlink.js.impl.credentials.KeymasterConnectionImpl = (function()
{

	function KeymasterConnectionImpl(keymasterUrlString, listener, timeout, retries, logger)
	{
		this.keymasterUrlString = keymasterUrlString;
		this.listener = listener;
		this.logger = logger;
		this.retries = retries;
		this.totalRetries = retries;

		var self = this;
		this.requestChannel = new caplin.streamlink.js.impl.connection.Channel(keymasterUrlString, timeout, true);

		this.requestChannel.onload = function(data)
		{
			self.retries = self.totalRetries;
			self.listener.onKeymasterResponse(data);
		};

		this.requestChannel.onerror = function(status, data)
		{
			if (status == 404 || status == 500)
			{
				self.logger.log(caplin.streamlink.LogLevel.ERROR, "KeymasterConnectionImpl: Received a HTTP Status Code: {0}  Data: {1}", status,
						data);
				self.listener.onKeymasterError("KeyMaster Keep-Alive HTTP Error - HTTP Status Code: " + status + "  Data: " + data);
			}
			else
			{
				if (self.retries == 0)
				{
					self.logger.log(caplin.streamlink.LogLevel.ERROR,
							"KeymasterConnectionImpl: Failed to connect after {0} retries - HTTP Status Code: {1}  Data: {2}", self.totalRetries,
							status, data);
					self.listener.onKeymasterError("KeyMaster Keep-Alive HTTP Error - HTTP Status Code: " + status + "  Data: " + data);
				}
				else
				{
					self.logger.log(caplin.streamlink.LogLevel.ERROR, "KeymasterConnectionImpl: KeyMaster Connection Failed - Retrying on URL: {0} - {1} Retries Left",
							self.keymasterUrlString, self.retries);
					self.retries--;
					self.id = setTimeout(function(){
						self.start();
					}, 10000);
				}
			}
		};
	}
	;

	caplin.implement(KeymasterConnectionImpl, caplin.streamlink.impl.credentials.KeymasterConnection);

	var Class = KeymasterConnectionImpl.prototype;

	Class.start = function()
	{
		this.requestChannel.send(null, "application/x-www-form-urlencoded");
	};
	
	Class.cancel = function()
	{
		if (this.id != null)
		{
			clearTimeout(this.id);
		}
		this.requestChannel.abort();
	};

	return KeymasterConnectionImpl;

})();

//## RttpEvent.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.impl.Serializable

caplin.streamlink.impl.event.RttpEvent = function() {};

// caplin.implement(caplin.streamlink.impl.event.RttpEvent, caplin.streamlink.impl.Serializable);

caplin.streamlink.impl.event.RttpEvent.prototype._hasRttpObjectNumber = function() { throw new Error('caplin.streamlink.impl.event.RttpEvent._hasRttpObjectNumber :  This is an abstract method'); };

caplin.streamlink.impl.event.RttpEvent.prototype._getRttpCode = function() { throw new Error('caplin.streamlink.impl.event.RttpEvent._getRttpCode :  This is an abstract method'); };

caplin.streamlink.impl.event.RttpEvent.prototype._getRttpObjectNumber = function() { throw new Error('caplin.streamlink.impl.event.RttpEvent._getRttpObjectNumber :  This is an abstract method'); };

caplin.streamlink.impl.event.RttpEvent.prototype._getRttpSequenceNumber = function() { throw new Error('caplin.streamlink.impl.event.RttpEvent._getRttpSequenceNumber :  This is an abstract method'); };

caplin.streamlink.impl.event.RttpEvent.prototype.getSubject = function() { throw new Error('caplin.streamlink.impl.event.RttpEvent.getSubject :  This is an abstract method'); };

caplin.streamlink.impl.event.RttpEvent.prototype._publishCommandResponse = function(listener) { throw new Error('caplin.streamlink.impl.event.RttpEvent._publishCommandResponse :  This is an abstract method'); };

caplin.streamlink.impl.event.RttpEvent.prototype._publishSubscriptionResponse = function(subscription, subscriptionListener) { throw new Error('caplin.streamlink.impl.event.RttpEvent._publishSubscriptionResponse :  This is an abstract method'); };

caplin.streamlink.impl.event.RttpEvent.prototype._setSubject = function(subject) { throw new Error('caplin.streamlink.impl.event.RttpEvent._setSubject :  This is an abstract method'); };



//## RttpEventBase.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.impl.event.RttpEvent

caplin.streamlink.impl.event.RttpEventBase = function(subject, rttpCode, rttpObjectNumber, rttpSequenceNumber) {
	// Constructor body
	this._subject = subject;
	this._rttpCode = rttpCode;
	this._rttpObjectNumber = rttpObjectNumber;
	this._rttpSequenceNumber = rttpSequenceNumber;
};

// caplin.implement(caplin.streamlink.impl.event.RttpEventBase, caplin.streamlink.impl.event.RttpEvent);

caplin.streamlink.impl.event.RttpEventBase.SEQUENCE_NUMBER = "SequenceNumber";
caplin.streamlink.impl.event.RttpEventBase.OBJECT_NUMBER = "ObjectNumber";
caplin.streamlink.impl.event.RttpEventBase.CODE = "Code";
caplin.streamlink.impl.event.RttpEventBase.SUBJECT = "Subject";
caplin.streamlink.impl.event.RttpEventBase.IMAGE = "Image";

caplin.streamlink.impl.event.RttpEventBase.prototype._hasRttpObjectNumber = function() {
	return this._rttpObjectNumber != null;
};

caplin.streamlink.impl.event.RttpEventBase.prototype._getRttpCode = function() {
	return this._rttpCode;
};

caplin.streamlink.impl.event.RttpEventBase.prototype._toDictionary = function() {
	var root = {} ;
	root[caplin.streamlink.impl.event.RttpEventBase.SUBJECT] = this._subject;
	root[caplin.streamlink.impl.event.RttpEventBase.CODE] = this._rttpCode.toString();
	root[caplin.streamlink.impl.event.RttpEventBase.OBJECT_NUMBER] = this._rttpObjectNumber;
	root[caplin.streamlink.impl.event.RttpEventBase.SEQUENCE_NUMBER] = this._rttpSequenceNumber;
	return root;
};

caplin.streamlink.impl.event.RttpEventBase.prototype._getRttpObjectNumber = function() {
	return this._rttpObjectNumber;
};

caplin.streamlink.impl.event.RttpEventBase.prototype._getRttpSequenceNumber = function() {
	return this._rttpSequenceNumber;
};

caplin.streamlink.impl.event.RttpEventBase.prototype.getSubject = function() {
	return this._subject;
};

caplin.streamlink.impl.event.RttpEventBase.prototype.toString = function() { throw new Error('caplin.streamlink.impl.event.RttpEventBase.toString :  This is an abstract method'); };

caplin.streamlink.impl.event.RttpEventBase.prototype._setSubject = function(subject) {
	this._subject = subject;
};



//## ResponseUnknownEvent.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.impl.event.RttpEventBase

caplin.streamlink.impl.event.ResponseUnknownEvent = function(subject, code, objectNumber) {
	// Super
	caplin.streamlink.impl.event.RttpEventBase.call(this, subject, code, objectNumber, null);};

caplin.extend(caplin.streamlink.impl.event.ResponseUnknownEvent, caplin.streamlink.impl.event.RttpEventBase);

caplin.streamlink.impl.event.ResponseUnknownEvent.createFromRttp = function(rttpCode, tokens, fieldMap, encoder) {
	return new caplin.streamlink.impl.event.ResponseUnknownEvent(encoder.rttpDecode(tokens[1]), rttpCode, tokens[0].substring(2, 6));
};

caplin.streamlink.impl.event.ResponseUnknownEvent.prototype.toString = function() {
	var builder = [] ;
	builder.push("ResponseUnknownEvent [subject=");
	builder.push(this._subject);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.ResponseUnknownEvent.prototype._publishCommandResponse = function(listener) { };

caplin.streamlink.impl.event.ResponseUnknownEvent.prototype._publishSubscriptionResponse = function(subscription, subscriptionListener) { };



//## LogListener.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class 
 *  <p>Interface for receiving log messages from the StreamLink application.</p>
 *  <p>If you wish to capture StreamLink's log messages (for example, to write them to your own log window), 
 *     implement this interface, create an instance of it, and attach it to an instance of 
 *     {@link caplin.streamlink.Logger}.</p>
 *  <p>The following code example shows a simple anonymous implementation of LogListener 
 *     that sends all StreamLink log messages to the standard JavaScript console.</p>  
 *  <p><code><pre>streamLink.getLogger().addListener({
 *         
 *         onLog : function(logInfo)
 *         {
 *                 console.log(logInfo.getMessage());
 *         }
 *         
 * }, caplin.streamlink.LogLevel.FINEST);
 * </pre></code></p>
 * 
 * 
 * @name caplin.streamlink.LogListener
 */
caplin.streamlink.LogListener = function() {};

/**
 *  
 *  <p>Called when a StreamLink log message is to be logged.</p>
 * @param {caplin.streamlink.LogInfo} logInfo The LogInfo object, containing the log message text and log level.
 * @function
 */
caplin.streamlink.LogListener.prototype.onLog = function(logInfo) { throw new Error('caplin.streamlink.LogListener.onLog :  This is an abstract method'); };



//## ConsoleLogListener.js ##//
caplin.namespace("caplin.streamlink");

// caplin.forceload(caplin.streamlink.LogListener

caplin.streamlink.ConsoleLogListener = function() {};

// caplin.implement(caplin.streamlink.ConsoleLogListener, caplin.streamlink.LogListener);

caplin.streamlink.ConsoleLogListener.prototype.onLog = function(info) {
	console.log(info.toString());
};



//## ChatEventType.js ##//
caplin.namespace("caplin.streamlink");

/**
 * @class  <p>Enumeration that defines the type of the {@link ChatEvent}.</p>
 *  <p>The <code>ChatEventType</code> will be provided by calling <code>getUpdateType()</code> on a {@link caplin.streamlink.ChatEvent}.</p>
 * 
 * @name caplin.streamlink.ChatEventType
 */
caplin.streamlink.ChatEventType = {
	/**
	 *  <p>The event contains a chat message</p>
	 */
	MESSAGE: "MESSAGE",
	/**
	 *  <p>The event represents the event raised when a user joins a chat channel.</p>
	 */
	USER_ADDED: "USER_ADDED",
	/**
	 *  <p>The event represents the event raised when a user leaves a chat channel.</p>
	 */
	USER_REMOVED: "USER_REMOVED",
	/**
	 *  <p>The event represents the event raised when this StreamLink client joins a chat channel.</p>
	 */
	JOINED: "JOINED",
	values: function() {return [caplin.streamlink.ChatEventType.MESSAGE, caplin.streamlink.ChatEventType.USER_ADDED, caplin.streamlink.ChatEventType.USER_REMOVED, caplin.streamlink.ChatEventType.JOINED];},
	valueOf: function(name) {return name;}
};

//## DataEvent.js ##//
caplin.namespace("caplin.streamlink");

/**
 * @class  <p>Provides access to fundamental information about subscription events relating to changes in data, such as the
 *  subject and the type of event that has occurred.</p>
 *  
 *  <p>In application code, use the derived interfaces that related to specific types of data, rather than this
 *  base-level interface. For example {@link caplin.streamlink.RecordType1Event} interface is derived
 *  from {@link caplin.streamlink.DataEvent}; it provides information about changes to a subscribed
 *  subject of type record, including the fields that have changed and their new values.</p>
 *  
 *  @see caplin.streamlink.ChatEvent
 *  @see caplin.streamlink.ContainerEvent
 *  @see caplin.streamlink.NewsEvent
 *  @see caplin.streamlink.PermissionEvent
 *  @see caplin.streamlink.RecordType1Event
 *  @see caplin.streamlink.RecordType2Event
 *  @see caplin.streamlink.RecordType3Event
 *  @see caplin.streamlink.StoryEvent
 * 
 * @name caplin.streamlink.DataEvent
 */
caplin.streamlink.DataEvent = function() {};

/**
 *  <p>Returns whether this was an image event or an update event.</p>
 *  
 *  <p>If this method returns true, all data for the subject is contained within this event; 
 *  you should therefore clear the application's caches of any current data for the subject of this event before refreshing them with the new data.</p>
 * 
 *  @returns {boolean} true if this event contains all data for the subject (an "image"), 
 *  false if the event contains only some of the data (an "update").
 * @function
 */
caplin.streamlink.DataEvent.prototype.isImage = function() { throw new Error('caplin.streamlink.DataEvent.isImage :  This is an abstract method'); };

/**
 *  <p>Returns the subject to which the event relates.</p>
 * 
 *  @returns {String} the subject
 * @function
 */
caplin.streamlink.DataEvent.prototype.getSubject = function() { throw new Error('caplin.streamlink.DataEvent.getSubject :  This is an abstract method'); };



//## ChatEvent.js ##//
caplin.namespace("caplin.streamlink");

// caplin.forceload(caplin.streamlink.DataEvent

/**
 * @class  <p>Provides information about updates to a chat subject.</p>
 *  <p>The <code>ChatEvent</code> will be provided on a <code>onChatUpdate</code> callback to a {@link caplin.streamlink.SubscriptionListener}.</p>
 *  
 *  @extends caplin.streamlink.DataEvent
 * 
 * @name caplin.streamlink.ChatEvent
 */
caplin.streamlink.ChatEvent = function() {};

// caplin.implement(caplin.streamlink.ChatEvent, caplin.streamlink.DataEvent);

/**
 *  <p>Returns the type of chat event that this event represents.</p>
 *  @returns {caplin.streamlink.ChatEventType} the event type.
 * @function
 */
caplin.streamlink.ChatEvent.prototype.getUpdateType = function() { throw new Error('caplin.streamlink.ChatEvent.getUpdateType :  This is an abstract method'); };

/**
 *  <p>Returns the chat message.</p>
 *  
 *  <p>This field will return null for all {@link caplin.streamlink.ChatEventType} events apart from
 *  {@link caplin.streamlink.ChatEventType.MESSAGE}.</p>
 *  @returns {String} The chat message.
 * @function
 */
caplin.streamlink.ChatEvent.prototype.getMessage = function() { throw new Error('caplin.streamlink.ChatEvent.getMessage :  This is an abstract method'); };

/**
 *  <p>Returns the time that the chat event was raised.</p>
 *  
 *  <p>The time stamp is in the format HH:MM:SS.</p>
 *  @returns {String} The timestamp.
 * @function
 */
caplin.streamlink.ChatEvent.prototype.getTime = function() { throw new Error('caplin.streamlink.ChatEvent.getTime :  This is an abstract method'); };

/**
 *  <p>Returns the username of the user that generated this chat event.</p>
 *  @returns {String} The username.
 * @function
 */
caplin.streamlink.ChatEvent.prototype.getUser = function() { throw new Error('caplin.streamlink.ChatEvent.getUser :  This is an abstract method'); };



//## ChatEventImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.ChatEvent
// caplin.forceload(caplin.streamlink.impl.Serializable
// caplin.forceload(caplin.streamlink.impl.event.RttpEventBase
// caplin.forceload(caplin.streamlink.DataEvent

caplin.streamlink.impl.event.ChatEventImpl = function(subject, image, type, fields, rttpCode, objectNumber, sequenceNumber) {
	// Super
	caplin.streamlink.impl.event.RttpEventBase.call(this, subject, rttpCode, objectNumber, sequenceNumber);
	// Constructor body
	this._image = image;
	this._fields = fields;
	this._type = type;
};

caplin.extend(caplin.streamlink.impl.event.ChatEventImpl, caplin.streamlink.impl.event.RttpEventBase);

// caplin.implement(caplin.streamlink.impl.event.ChatEventImpl, caplin.streamlink.ChatEvent);
// caplin.implement(caplin.streamlink.impl.event.ChatEventImpl, caplin.streamlink.DataEvent);
// caplin.implement(caplin.streamlink.impl.event.ChatEventImpl, caplin.streamlink.impl.Serializable);

caplin.streamlink.impl.event.ChatEventImpl.FIELDS = "Fields";
caplin.streamlink.impl.event.ChatEventImpl.IMAGE = "Image";
caplin.streamlink.impl.event.ChatEventImpl.TYPE = "Type";
caplin.streamlink.impl.event.ChatEventImpl.USER_FIELD = "user";
caplin.streamlink.impl.event.ChatEventImpl.TIME_FIELD = "time";
caplin.streamlink.impl.event.ChatEventImpl.MSG_FIELD = "msg";
caplin.streamlink.impl.event.ChatEventImpl.STATUS_FIELD = "status";

caplin.streamlink.impl.event.ChatEventImpl.prototype.isImage = function() {
	return this._image;
};

caplin.streamlink.impl.event.ChatEventImpl.prototype.getUpdateType = function() {
	return this._type;
};

caplin.streamlink.impl.event.ChatEventImpl.createFromDictionary = function(root) {
	return new caplin.streamlink.impl.event.ChatEventImpl(root[caplin.streamlink.impl.event.RttpEventBase.SUBJECT], root[caplin.streamlink.impl.event.ChatEventImpl.IMAGE], caplin.streamlink.ChatEventType.valueOf(root[caplin.streamlink.impl.event.ChatEventImpl.TYPE]), root[caplin.streamlink.impl.event.ChatEventImpl.FIELDS], caplin.streamlink.impl.utils.RttpUtil.getRttpCode(root[caplin.streamlink.impl.event.RttpEventBase.CODE]), root[caplin.streamlink.impl.event.RttpEventBase.OBJECT_NUMBER], root[caplin.streamlink.impl.event.RttpEventBase.SEQUENCE_NUMBER]);
};

caplin.streamlink.impl.event.ChatEventImpl.createFromRttp = function(rttpCode, tokens, fieldMap, encoder) {
	switch (rttpCode) {
	case caplin.streamlink.impl.protocolhandler.RttpCode.CHAT_RESP:
	return new caplin.streamlink.impl.event.ChatEventImpl(encoder.rttpDecode(tokens[1]), true, caplin.streamlink.ChatEventType.JOINED, caplin.streamlink.impl.event.ParseUtils.getFields(tokens, fieldMap, encoder, 2), rttpCode, tokens[0].substring(2, 6), null);
	case caplin.streamlink.impl.protocolhandler.RttpCode.CHAT_UPD:
	var type = caplin.streamlink.ChatEventType.MESSAGE;
	var fields = caplin.streamlink.impl.event.ParseUtils.getFields(tokens, fieldMap, encoder, 1);
	var status = fields[caplin.streamlink.impl.event.ChatEventImpl.STATUS_FIELD];
	if (status != null) {
		if ((status == ("1"))) {
			type = caplin.streamlink.ChatEventType.USER_ADDED;
		}
		else if ((status == ("2"))) {
			type = caplin.streamlink.ChatEventType.USER_REMOVED;
		}
		else if ((status == ("3"))) {
			type = caplin.streamlink.ChatEventType.JOINED;
		}
	}
	return new caplin.streamlink.impl.event.ChatEventImpl(null, false, type, fields, rttpCode, tokens[0].substring(4, 8), tokens[0].substring(2, 4));
	}
	return null;
};

caplin.streamlink.impl.event.ChatEventImpl.prototype._toDictionary = function() {
	var root = caplin.streamlink.impl.event.RttpEventBase.prototype._toDictionary.call(this);
	root[caplin.streamlink.impl.event.ChatEventImpl.IMAGE] = this._image;
	root[caplin.streamlink.impl.event.ChatEventImpl.FIELDS] = this._fields;
	root[caplin.streamlink.impl.event.ChatEventImpl.TYPE] = this._type.toString();
	return root;
};

caplin.streamlink.impl.event.ChatEventImpl.prototype.getMessage = function() {
	return this._fields[caplin.streamlink.impl.event.ChatEventImpl.MSG_FIELD];
};

caplin.streamlink.impl.event.ChatEventImpl.prototype.getTime = function() {
	return this._fields[caplin.streamlink.impl.event.ChatEventImpl.TIME_FIELD];
};

caplin.streamlink.impl.event.ChatEventImpl.prototype.getUser = function() {
	return this._fields[caplin.streamlink.impl.event.ChatEventImpl.USER_FIELD];
};

caplin.streamlink.impl.event.ChatEventImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("ChatDataEventImpl [subject=");
	builder.push(this._subject);
	builder.push(", fields=");
	builder.push(caplin.xlang.Utils.dictionaryToString(this._fields));
	builder.push(", image=");
	builder.push(this._image);
	builder.push(", type=");
	builder.push(this._type);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.ChatEventImpl.prototype._publishCommandResponse = function(listener) { };

caplin.streamlink.impl.event.ChatEventImpl.prototype._publishSubscriptionResponse = function(subscription, subscriptionListener) {
	(subscriptionListener.onChatUpdate && subscriptionListener.onChatUpdate(subscription, this));
};



//## ActionFailResponseEvent.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.impl.event.RttpEventBase

caplin.streamlink.impl.event.ActionFailResponseEvent = function(subject, code) {
	// Super
	caplin.streamlink.impl.event.RttpEventBase.call(this, subject, code, null, null);};

caplin.extend(caplin.streamlink.impl.event.ActionFailResponseEvent, caplin.streamlink.impl.event.RttpEventBase);

caplin.streamlink.impl.event.ActionFailResponseEvent.createFromRttp = function(rttpCode, tokens, fieldMap, encoder) {
	return new caplin.streamlink.impl.event.ActionFailResponseEvent(encoder.rttpDecode(tokens[1]), rttpCode);
};

caplin.streamlink.impl.event.ActionFailResponseEvent.prototype.toString = function() {
	var builder = [] ;
	builder.push("ActionFailResponseEvent [subject=");
	builder.push(this._subject);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.ActionFailResponseEvent.prototype._publishCommandResponse = function(listener) {
	listener.onCommandError(this._subject, caplin.streamlink.impl.event.CommandErrorEventImpl._createFromRttpCode(this._rttpCode));
};

caplin.streamlink.impl.event.ActionFailResponseEvent.prototype._publishSubscriptionResponse = function(subscription, subscriptionListener) {
	subscriptionListener.onSubscriptionError(subscription, caplin.streamlink.impl.event.SubjectErrorEventImpl.createFromRttpEvent(this));
};



//## Packet.js ##//
caplin.namespace("caplin.streamlink.impl.packet");


caplin.streamlink.impl.packet.Packet = function() {};

caplin.streamlink.impl.packet.Packet.prototype.getType = function() { throw new Error('caplin.streamlink.impl.packet.Packet.getType :  This is an abstract method'); };

caplin.streamlink.impl.packet.Packet.prototype.getSubject = function() { throw new Error('caplin.streamlink.impl.packet.Packet.getSubject :  This is an abstract method'); };

caplin.streamlink.impl.packet.Packet.prototype.toRttpString = function(encoder) { throw new Error('caplin.streamlink.impl.packet.Packet.toRttpString :  This is an abstract method'); };



//## PacketCreate.js ##//
caplin.namespace("caplin.streamlink.impl.packet");

// caplin.forceload(caplin.streamlink.impl.packet.Packet

caplin.streamlink.impl.packet.PacketCreate = function(subject, subjectType) {
	// Constructor body
	this.subject = subject;
	this.subjectType = subjectType;
};

// caplin.implement(caplin.streamlink.impl.packet.PacketCreate, caplin.streamlink.impl.packet.Packet);

caplin.streamlink.impl.packet.PacketCreate.prototype.getType = function() {
	return caplin.streamlink.impl.packet.PacketType.CREATE;
};

caplin.streamlink.impl.packet.PacketCreate.prototype.getSubject = function() {
	return this.subject;
};

caplin.streamlink.impl.packet.PacketCreate.prototype.toRttpString = function(encoder) {
	var result = [] ;
	result.push(caplin.streamlink.impl.packet.PacketType.CREATE.getCode());
	result.push(" ");
	result.push(encoder.rttpEncode(this.subject));
	result.push(";");
	result.push(caplin.streamlink.impl.event.ParseUtils.getNumberFromSubjectType(this.subjectType));
	return result.join("");
};

caplin.streamlink.impl.packet.PacketCreate.prototype.toString = function() {
	var builder = [] ;
	builder.push("Create [subject=");
	builder.push(this.subject);
	builder.push(", subjectType=");
	builder.push(this.subjectType);
	builder.push("]");
	return builder.join("");
};



//## DirectoryChangeElement.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  <p>Provides information about a change to a directory element.</p>
 * 
 * 
 * @name caplin.streamlink.DirectoryChangeElement
 */
caplin.streamlink.DirectoryChangeElement = function() {};

/**
 *  <p>Returns whether the element has been added or removed from the directory.</p>
 *  
 *  @returns {boolean} Whether the element has been added or removed.
 * @function
 */
caplin.streamlink.DirectoryChangeElement.prototype.isAdded = function() { throw new Error('caplin.streamlink.DirectoryChangeElement.isAdded :  This is an abstract method'); };

/**
 *  <p>Returns the type of {@link caplin.streamlink.SubjectType} that the element is.</p>
 *  
 *  @returns {caplin.streamlink.SubjectType} The {@link caplin.streamlink.SubjectType}
 * @function
 */
caplin.streamlink.DirectoryChangeElement.prototype.getType = function() { throw new Error('caplin.streamlink.DirectoryChangeElement.getType :  This is an abstract method'); };

/**
 *  <p>Returns the subject of the requested directory element.</p>
 *  
 *  @returns {String} The subject
 * @function
 */
caplin.streamlink.DirectoryChangeElement.prototype.getSubject = function() { throw new Error('caplin.streamlink.DirectoryChangeElement.getSubject :  This is an abstract method'); };



//## Timeout.js ##//
caplin.namespace("caplin.streamlink.impl.protocolhandler");


caplin.streamlink.impl.protocolhandler.Timeout = function() {};



//## StoryEvent.js ##//
caplin.namespace("caplin.streamlink");

// caplin.forceload(caplin.streamlink.DataEvent

/**
 * @class  <p>Provides information about a change to story data in a Story.
 * 
 *  Stories may be sent from the Liberator in one or more {@link caplin.streamlink.StoryEvent}s.
 *  The start of the story is indicated by the <code>isImage()</code> method returning 
 *  true.</p>
 *  
 *  @extends caplin.streamlink.DataEvent
 * 
 * @name caplin.streamlink.StoryEvent
 */
caplin.streamlink.StoryEvent = function() {};

// caplin.implement(caplin.streamlink.StoryEvent, caplin.streamlink.DataEvent);

/**
 *  <p>Returns the story text relating to the event.</p>
 *  
 *  @returns {String} The story text chunk for this event.
 * @function
 */
caplin.streamlink.StoryEvent.prototype.getText = function() { throw new Error('caplin.streamlink.StoryEvent.getText :  This is an abstract method'); };



//## StoryEventImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.impl.Serializable
// caplin.forceload(caplin.streamlink.impl.event.RttpEventBase
// caplin.forceload(caplin.streamlink.StoryEvent

caplin.streamlink.impl.event.StoryEventImpl = function(subject, image, text, rttpCode, objectNumber, sequenceNumber) {
	// Super
	caplin.streamlink.impl.event.RttpEventBase.call(this, subject, rttpCode, objectNumber, sequenceNumber);
	// Constructor body
	this._text = text;
	this._image = image;
};

caplin.extend(caplin.streamlink.impl.event.StoryEventImpl, caplin.streamlink.impl.event.RttpEventBase);

// caplin.implement(caplin.streamlink.impl.event.StoryEventImpl, caplin.streamlink.StoryEvent);
// caplin.implement(caplin.streamlink.impl.event.StoryEventImpl, caplin.streamlink.impl.Serializable);

caplin.streamlink.impl.event.StoryEventImpl.prototype._addMoreText = function(encodedText, encoder) {
	var text = encoder.rttpDecode(encodedText);
	if ((text == ("."))) {
		return false;
	}
	var ret = true;
	if ((text.substring(text.length - 1) == ("."))) {
		ret = false;
		text = text.substring(0, text.length - 1);
	}
	this._text += text;
	if (ret == true) {
		this._text += "\n";
	}
	return ret;
};

caplin.streamlink.impl.event.StoryEventImpl.prototype.isImage = function() {
	return this._image;
};

caplin.streamlink.impl.event.StoryEventImpl.createFromDictionary = function(root) {
	var subject = root[caplin.streamlink.impl.event.RttpEventBase.SUBJECT];
	var image = root[caplin.streamlink.impl.event.RttpEventBase.IMAGE];
	var text = root["Text"];
	var rttpCode = caplin.streamlink.impl.utils.RttpUtil.getRttpCode(root[caplin.streamlink.impl.event.RttpEventBase.CODE]);
	var objectNumber = root[caplin.streamlink.impl.event.RttpEventBase.OBJECT_NUMBER];
	var sequenceNumber = root[caplin.streamlink.impl.event.RttpEventBase.SEQUENCE_NUMBER];
	return new caplin.streamlink.impl.event.StoryEventImpl(subject, image, text, rttpCode, objectNumber, sequenceNumber);
};

caplin.streamlink.impl.event.StoryEventImpl.createFromRttp = function(rttpCode, tokens, fieldMap, encoder) {
	if (rttpCode == caplin.streamlink.impl.protocolhandler.RttpCode.STORY_RESP) {
		var subjectName = tokens[1];
		if (tokens[0].length > 6 && (tokens[0].substring(6, 7) == ("-"))) {
			subjectName = tokens[0].substring(7);
		}
		return new caplin.streamlink.impl.event.StoryEventImpl(encoder.rttpDecode(subjectName), true, "", rttpCode, tokens[0].substring(2, 6), null);
	}
	else if (rttpCode == caplin.streamlink.impl.protocolhandler.RttpCode.STORY_IMG) {
		return new caplin.streamlink.impl.event.StoryEventImpl(null, true, "", rttpCode, tokens[0].substring(4, 8), tokens[0].substring(2, 4));
	}
	else if (rttpCode == caplin.streamlink.impl.protocolhandler.RttpCode.STORY_UPD) {
		return new caplin.streamlink.impl.event.StoryEventImpl(null, false, "", rttpCode, tokens[0].substring(4, 8), tokens[0].substring(2, 4));
	}
	return null;
};

caplin.streamlink.impl.event.StoryEventImpl.prototype._toDictionary = function() {
	var root = caplin.streamlink.impl.event.RttpEventBase.prototype._toDictionary.call(this);
	root[caplin.streamlink.impl.event.RttpEventBase.IMAGE] = this._image;
	root["Text"] = this._text;
	return root;
};

caplin.streamlink.impl.event.StoryEventImpl.prototype.getText = function() {
	return this._text;
};

caplin.streamlink.impl.event.StoryEventImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("StoryDataEventImpl [subject=");
	builder.push(this._subject);
	builder.push(", text=");
	builder.push(this._text);
	builder.push(", image=");
	builder.push(this._image);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.StoryEventImpl.prototype._publishCommandResponse = function(listener) { };

caplin.streamlink.impl.event.StoryEventImpl.prototype._publishSubscriptionResponse = function(subscription, subscriptionListener) {
	(subscriptionListener.onStoryUpdate && subscriptionListener.onStoryUpdate(subscription, this));
};



//## CommandSubscription.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  
 *  <p>Stored information on the subscription and lets you unpersist the command.</p>
 * 
 * @name caplin.streamlink.CommandSubscription
 */
caplin.streamlink.CommandSubscription = function() {};

/**
 *  <p>Returns the {@link caplin.streamlink.CommandListener} that is informed of success or
 *  failure of this command.</p>
 *  
 *  @returns {caplin.streamlink.CommandListener} The command listener.
 * @function
 */
caplin.streamlink.CommandSubscription.prototype.getCommandListener = function() { throw new Error('caplin.streamlink.CommandSubscription.getCommandListener :  This is an abstract method'); };

/**
 *  <p>Returns the subject the command was executed on.</p>
 *  @returns {String} The subject of the command.
 * @function
 */
caplin.streamlink.CommandSubscription.prototype.getSubject = function() { throw new Error('caplin.streamlink.CommandSubscription.getSubject :  This is an abstract method'); };

/**
 *  
 *  <p>For commands that have been set as persistent (using the <code>setPersistent</code> method on 
 *  {@link caplin.streamlink.CommandParameters}) this method will turn off that persistence. 
 *  Once persistence is turned off the command will not be resent to the server when a reconnection occurs.
 *  
 * @function
 */
caplin.streamlink.CommandSubscription.prototype.unPersist = function() { throw new Error('caplin.streamlink.CommandSubscription.unPersist :  This is an abstract method'); };



//## PacketRequest.js ##//
caplin.namespace("caplin.streamlink.impl.packet");

// caplin.forceload(caplin.streamlink.impl.packet.Packet

caplin.streamlink.impl.packet.PacketRequest = function(subject, params) {
	// Constructor body
	this.subject = subject;
	this.params = params;
};

// caplin.implement(caplin.streamlink.impl.packet.PacketRequest, caplin.streamlink.impl.packet.Packet);

caplin.streamlink.impl.packet.PacketRequest.prototype.getType = function() {
	return caplin.streamlink.impl.packet.PacketType.REQUEST;
};

caplin.streamlink.impl.packet.PacketRequest.prototype.getRttpParams = function(encoder) {
	return (this.params == null) ? "" : this.params.toRttpString(encoder);
};

caplin.streamlink.impl.packet.PacketRequest.prototype.getSubject = function() {
	return this.subject;
};

caplin.streamlink.impl.packet.PacketRequest.prototype.toRttpString = function(encoder) {
	return caplin.streamlink.impl.packet.PacketType.REQUEST.getCode() + " " + encoder.rttpEncode(this.subject) + this.getRttpParams(encoder);
};

caplin.streamlink.impl.packet.PacketRequest.prototype.toString = function() {
	return "Request [subject=" + this.subject + ", params=" + this.params + "]";
};



//## Version.js ##//
caplin.namespace("caplin.streamlink.impl");

caplin.streamlink.impl.Version = function() {};

caplin.streamlink.impl.Version.getVersion = function() {
	return "6.0.0-23954261";
};



//## ProtocolListener.js ##//
caplin.namespace("caplin.streamlink.impl.protocolhandler");


caplin.streamlink.impl.protocolhandler.ProtocolListener = function() {};

caplin.streamlink.impl.protocolhandler.ProtocolListener.prototype.onDisconnect = function() { throw new Error('caplin.streamlink.impl.protocolhandler.ProtocolListener.onDisconnect :  This is an abstract method'); };

caplin.streamlink.impl.protocolhandler.ProtocolListener.prototype.onLogin = function() { throw new Error('caplin.streamlink.impl.protocolhandler.ProtocolListener.onLogin :  This is an abstract method'); };

caplin.streamlink.impl.protocolhandler.ProtocolListener.prototype.onReconnect = function() { throw new Error('caplin.streamlink.impl.protocolhandler.ProtocolListener.onReconnect :  This is an abstract method'); };

caplin.streamlink.impl.protocolhandler.ProtocolListener.prototype.onResponse = function(event) { throw new Error('caplin.streamlink.impl.protocolhandler.ProtocolListener.onResponse :  This is an abstract method'); };

caplin.streamlink.impl.protocolhandler.ProtocolListener.prototype.onTerminalUpdate = function(event) { throw new Error('caplin.streamlink.impl.protocolhandler.ProtocolListener.onTerminalUpdate :  This is an abstract method'); };

caplin.streamlink.impl.protocolhandler.ProtocolListener.prototype.onUpdate = function(event) { throw new Error('caplin.streamlink.impl.protocolhandler.ProtocolListener.onUpdate :  This is an abstract method'); };



//## SubscriptionParameters.js ##//
caplin.namespace("caplin.streamlink");

/**
 * @class  
 *   
 * @private
 *   
 *  <p>The base interface for all Subscription parameter types.
 * 
 *  Use the create methods in the StreamLink instance to obtain concrete implementations of this interface that
 *  represent particular types of subscription parameters.</p>
 *  
 * 
 * @name caplin.streamlink.SubscriptionParameters
 */
caplin.streamlink.SubscriptionParameters = function() {};



//## RecordSubscriptionParameters.js ##//
caplin.namespace("caplin.streamlink");

// caplin.forceload(caplin.streamlink.SubscriptionParameters

/**
 * @class  
 * @private
 *  
 *  <p>Defines the parameters used for Record subscriptions.</p>
 *  
 *  @extends caplin.streamlink.SubscriptionParameters
 * 
 * @name caplin.streamlink.RecordSubscriptionParameters
 */
caplin.streamlink.RecordSubscriptionParameters = function() {};

// caplin.implement(caplin.streamlink.RecordSubscriptionParameters, caplin.streamlink.SubscriptionParameters);

/**
 *  <p>Sets the fields that will be requested when a subscription is made using these parameters.</p>
 *  
 *  <p>If no fields are defined, then your {@link caplin.streamlink.SubscriptionListener} will 
 *  receive all fields that are available for the subject.</p>
 *  
 * @param {List} fields The fields to be requested.
 *  @returns {caplin.streamlink.RecordSubscriptionParameters} A {@link caplin.streamlink.RecordSubscriptionParameters} object to permit a fluent style.
 * @function
 */
caplin.streamlink.RecordSubscriptionParameters.prototype.setFields = function(fields) { throw new Error('caplin.streamlink.RecordSubscriptionParameters.setFields :  This is an abstract method'); };

/**
 *  <p>Sets the filter associated with the subscription.</p>
 *  
 *  <p>For more details on the format of the filter string, please see the <b>StreamLink Overview</b> document.</p>
 *  
 * @param {String} filter The filter string.
 * @param {boolean} isImage Whether the filter string should be applied only to the initial response (true) or on every update (false).
 *  @returns {caplin.streamlink.RecordSubscriptionParameters} A {@link caplin.streamlink.RecordSubscriptionParameters} object to permit a fluent style.
 * @function
 */
caplin.streamlink.RecordSubscriptionParameters.prototype.setFilter = function(filter, isImage) { throw new Error('caplin.streamlink.RecordSubscriptionParameters.setFilter :  This is an abstract method'); };



//## PacketGlobalThrottle.js ##//
caplin.namespace("caplin.streamlink.impl.packet");

// caplin.forceload(caplin.streamlink.impl.packet.Packet

caplin.streamlink.impl.packet.PacketGlobalThrottle = function(command) {
	// Constructor body
	this.subject = "";
	this.command = command;
};

// caplin.implement(caplin.streamlink.impl.packet.PacketGlobalThrottle, caplin.streamlink.impl.packet.Packet);

caplin.streamlink.impl.packet.PacketGlobalThrottle.prototype.getType = function() {
	return caplin.streamlink.impl.packet.PacketType.GLOBAL_THROTTLE;
};

caplin.streamlink.impl.packet.PacketGlobalThrottle.prototype.getSubject = function() {
	return this.subject;
};

caplin.streamlink.impl.packet.PacketGlobalThrottle.prototype.toRttpString = function(encoder) {
	return caplin.streamlink.impl.packet.PacketType.GLOBAL_THROTTLE.getCode() + " " + this.command.toString().toLowerCase();
};

caplin.streamlink.impl.packet.PacketGlobalThrottle.prototype.toString = function() {
	var builder = [] ;
	builder.push("GlobalThrottle [subject=");
	builder.push(this.subject);
	builder.push(", command=");
	builder.push(this.command);
	builder.push("]");
	return builder.join("");
};



//## SubscriptionStatusType.js ##//
caplin.namespace("caplin.streamlink");

/**
 * @class  <p>Enumeration that defines the status of a subject.</p>
 *  <p>The <code>SubscriptionStatusType</code> will be provided by calling <code>getStatus()</code> on a {@link caplin.streamlink.SubscriptionStatusEvent}.</p>
 * 
 * @name caplin.streamlink.SubscriptionStatusType
 */
caplin.streamlink.SubscriptionStatusType = {
	/**
	 *  <p>The data associated with the subject is correct and up-to-date.</p>
	 */
	STATUS_OK: "STATUS_OK",
	/**
	 *  <p>One of the sources of data for the subject is not available. This may affect the subject's data.</p>
	 */
	STATUS_LIMITED: "STATUS_LIMITED",
	/**
	 *  <p>The subject's data may not be correct and up-to-date.</p>
	 */
	STATUS_STALE: "STATUS_STALE",
	values: function() {return [caplin.streamlink.SubscriptionStatusType.STATUS_OK, caplin.streamlink.SubscriptionStatusType.STATUS_LIMITED, caplin.streamlink.SubscriptionStatusType.STATUS_STALE];},
	valueOf: function(name) {return name;}
};

//## FactoryOnClient.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.client");

// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver

caplin.streamlink.impl.bridge.worker.client.FactoryOnClient = function(bridge) {
	// Constructor body
	this._bridge = bridge;
	this._id = "Factory";
	bridge.addReceiver(this);
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.client.FactoryOnClient, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.client.FactoryOnClient.prototype.createStreamLink = function(configuration) {
	var config = configuration;
	var logger = new caplin.streamlink.impl.bridge.worker.client.LoggerOnClient(this._bridge, this._bridge.getNextId());
	this._bridge.addReceiver(logger);
	var streamLink = new caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient(this._bridge, this._bridge.getNextId(), logger);
	this._bridge.addReceiver(streamLink);
	var keymasterResponseListenerId = null;
	var keymasterResponseListener = configuration._getKeymasterKeepAliveListener();
	if (keymasterResponseListener != null) {
		var keymasterResponseListenerOnClient = new caplin.streamlink.impl.bridge.worker.client.KeymasterResponseListenerOnClient(this._bridge.getNextId(), keymasterResponseListener);
		this._bridge.addReceiver(keymasterResponseListenerOnClient);
		keymasterResponseListenerId = keymasterResponseListenerOnClient.getReceiverId();
	}
	var args = [streamLink.getReceiverId(), logger.getReceiverId(), config._toDictionary(), keymasterResponseListenerId];
	this._bridge.send(this._id, "createStreamLink", args);
	return streamLink;
};

caplin.streamlink.impl.bridge.worker.client.FactoryOnClient.prototype.getReceiverKey = function() {
	return null;
};

caplin.streamlink.impl.bridge.worker.client.FactoryOnClient.prototype.getReceiverId = function() {
	return this._id;
};

caplin.streamlink.impl.bridge.worker.client.FactoryOnClient.prototype.receive = function(method, args) { };



//## ConnectionFactory.js ##//
caplin.namespace("caplin.streamlink");


caplin.streamlink.ConnectionFactory = function() {};

caplin.streamlink.ConnectionFactory.prototype.isSupported = function(url) { throw new Error('caplin.streamlink.ConnectionFactory.isSupported :  This is an abstract method'); };

caplin.streamlink.ConnectionFactory.prototype.getConnection = function(url) { throw new Error('caplin.streamlink.ConnectionFactory.getConnection :  This is an abstract method'); };

caplin.streamlink.ConnectionFactory.prototype.getKeymasterConnection = function(url, keymasterResponseListener, keymasterTimeout, keymasterRetries) { throw new Error('caplin.streamlink.ConnectionFactory.getKeymasterConnection :  This is an abstract method'); };

caplin.streamlink.ConnectionFactory.prototype.setContext = function(context) { throw new Error('caplin.streamlink.ConnectionFactory.setContext :  This is an abstract method'); };



//## ListenerAndLevel.js ##//
caplin.namespace("caplin.streamlink.impl.logger");


caplin.streamlink.impl.logger.ListenerAndLevel = function(listener, level) {
	// Constructor body
	this.listener = listener;
	this.level = level;
};

caplin.streamlink.impl.logger.ListenerAndLevel.prototype.getLevel = function() {
	return this.level;
};

caplin.streamlink.impl.logger.ListenerAndLevel.prototype.getListener = function() {
	return this.listener;
};

caplin.streamlink.impl.logger.ListenerAndLevel.prototype.toString = function() {
	var builder = [] ;
	builder.push("ListenerAndLevel [listener=");
	builder.push(this.listener);
	builder.push(", level=");
	builder.push(this.level);
	builder.push("]");
	return builder.join("");
};



//## SubscriptionParametersInternal.js ##//
caplin.namespace("caplin.streamlink.impl");


caplin.streamlink.impl.SubscriptionParametersInternal = function() {};

caplin.streamlink.impl.SubscriptionParametersInternal.prototype.toRttpString = function(encoder) { throw new Error('caplin.streamlink.impl.SubscriptionParametersInternal.toRttpString :  This is an abstract method'); };



//## FreeFormSubscriptionParameters.js ##//
caplin.namespace("caplin.streamlink");

// caplin.forceload(caplin.streamlink.SubscriptionParameters

/**
 * @class   
 * @private
 * 
 * 
 * @name caplin.streamlink.FreeFormSubscriptionParameters
 */
caplin.streamlink.FreeFormSubscriptionParameters = function() {};

// caplin.implement(caplin.streamlink.FreeFormSubscriptionParameters, caplin.streamlink.SubscriptionParameters);

caplin.streamlink.FreeFormSubscriptionParameters.prototype.setParameters = function(parameters) { throw new Error('caplin.streamlink.FreeFormSubscriptionParameters.setParameters :  This is an abstract method'); };



//## FreeFormSubscriptionParametersImpl.js ##//
caplin.namespace("caplin.streamlink.impl.subscription");

// caplin.forceload(caplin.streamlink.impl.SubscriptionParametersInternal
// caplin.forceload(caplin.streamlink.FreeFormSubscriptionParameters

caplin.streamlink.impl.subscription.FreeFormSubscriptionParametersImpl = function() {
	// Field Definitions
	this._parameters = "";
};

// caplin.implement(caplin.streamlink.impl.subscription.FreeFormSubscriptionParametersImpl, caplin.streamlink.FreeFormSubscriptionParameters);
// caplin.implement(caplin.streamlink.impl.subscription.FreeFormSubscriptionParametersImpl, caplin.streamlink.impl.SubscriptionParametersInternal);

caplin.streamlink.impl.subscription.FreeFormSubscriptionParametersImpl.prototype.setParameters = function(parameters) {
	this._parameters = parameters;
	return this;
};

caplin.streamlink.impl.subscription.FreeFormSubscriptionParametersImpl.prototype.toRttpString = function(encoder) {
	if (this._parameters == null || (this._parameters == (""))) {
		return "";
	}
	return ";" + this._parameters;
};



//## StreamLinkConfiguration.js ##//
caplin.namespace("caplin.streamlink");


caplin.streamlink.StreamLinkConfiguration = function() {};

caplin.streamlink.StreamLinkConfiguration.prototype.addLiberatorUrlGroup = function() { throw new Error('caplin.streamlink.StreamLinkConfiguration.addLiberatorUrlGroup :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setApplicationId = function(applicationId) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setApplicationId :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setClockOffsetInterval = function(milliseconds) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setClockOffsetInterval :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setClockOffsetWindow = function(milliseconds) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setClockOffsetWindow :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setForeverFrameReconnectCount = function(count) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setForeverFrameReconnectCount :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setHttpTimeout = function(timeout) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setHttpTimeout :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setKeymasterAttempts = function(timeout) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setKeymasterAttempts :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setKeymasterKeepAliveInterval = function(timeout) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setKeymasterKeepAliveInterval :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setKeymasterKeepAliveListener = function(keymasterPollListener) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setKeymasterKeepAliveListener :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setKeymasterKeepAliveUrl = function(url) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setKeymasterKeepAliveUrl :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setKeymasterUrl = function(url) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setKeymasterUrl :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setLatencyInterval = function(milliseconds) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setLatencyInterval :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setLatencyWindow = function(milliseconds) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setLatencyWindow :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setLogWindowLogLevel = function(level) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setLogWindowLogLevel :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setLogWindowVisible = function(show) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setLogWindowVisible :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setPassword = function(password) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setPassword :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setPollInterval = function(milliseconds) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setPollInterval :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setRequestTimeout = function(milliseconds) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setRequestTimeout :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setRunOnWorker = function() { throw new Error('caplin.streamlink.StreamLinkConfiguration.setRunOnWorker :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setUsername = function(username) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setUsername :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setWorkerUrl = function(workerUrl) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setWorkerUrl :  This is an abstract method'); };

caplin.streamlink.StreamLinkConfiguration.prototype.setXhrStreamingReconnectCount = function(count) { throw new Error('caplin.streamlink.StreamLinkConfiguration.setXhrStreamingReconnectCount :  This is an abstract method'); };



//## StreamLinkConfigurationImpl.js ##//
caplin.namespace("caplin.streamlink.impl");

// caplin.forceload(caplin.streamlink.impl.Serializable
// caplin.forceload(caplin.streamlink.StreamLinkConfiguration

caplin.streamlink.impl.StreamLinkConfigurationImpl = function(attributes) {
	// Constructor body
	this._attributes = attributes;
};

// caplin.implement(caplin.streamlink.impl.StreamLinkConfigurationImpl, caplin.streamlink.StreamLinkConfiguration);
// caplin.implement(caplin.streamlink.impl.StreamLinkConfigurationImpl, caplin.streamlink.impl.Serializable);

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._isRunOnWorker = function() {
	return this._attributes["run_on_worker"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.isBasicAuthentication = function() {
	return this._getUsername() != null && this._getPassword() != null;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.isKeyMasterAuthentication = function() {
	return this._getKeymasterUrl() != null && this._getKeymasterKeepAliveUrl() != null;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getCredentialsProvider = function(context, connectionFactory) {
	if (this.isBasicAuthentication()) {
		context.logger.log(caplin.streamlink.LogLevel.INFO, "Using username {0} and password {1}", this._getUsername(), this._getPassword());
		return new caplin.streamlink.impl.credentials.PasswordCredentialsProvider(this._getUsername(), this._getPassword());
	}
	else {
		var keymasterUrl = this._getKeymasterUrl();
		context.logger.log(caplin.streamlink.LogLevel.INFO, "Using username keymaster {0}", keymasterUrl);
		return new caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl(connectionFactory, context);
	}
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getKeymasterKeepAliveListener = function() {
	return this._keymasterPollListener;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getLogWindowLogLevel = function() {
	return this._attributes["log_window_log_level"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setLogWindowLogLevel = function(level) {
	this._attributes["log_window_log_level"] = level;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setLogWindowVisible = function(show) {
	this._attributes["log_window_visible"] = show;
	return this;
};

/**
 *  
 *  ======== Set Methods ========
 *  
 * @function
 */
caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.addLiberatorUrlGroup = function() {
	var urls = Array.prototype.slice.call(arguments, 0);	var groups = this._attributes["liberator_urls"];
	var copy = [];
	Array.prototype.push.apply(copy, urls);
	groups.push(copy);
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.create = function() {
	var attributes = {} ;
	attributes["run_on_worker"] = false;
	attributes["worker_url"] = "streamlink-main.js";
	attributes["http_timeout"] = 30000;
	attributes["keymaster_url"] = null;
	attributes["keymaster_attempts"] = 3;
	attributes["keymaster_keep_alive_url"] = null;
	attributes["keymaster_keep_alive_interval"] = 60000;
	attributes["poll_interval"] = 1000;
	attributes["forever_frame_reconnect_count"] = 10000;
	attributes["xhr_streaming_reconnect_count"] = 10000;
	attributes["request_timeout"] = 10000;
	attributes["clock_offset_interval"] = 10000;
	attributes["clock_offset_window"] = 120000;
	attributes["latency_interval"] = 10000;
	attributes["latency_window"] = 120000;
	attributes["liberator_urls"] = [];
	attributes["log_window_log_level"] = caplin.streamlink.LogLevel.FINE;
	return new caplin.streamlink.impl.StreamLinkConfigurationImpl(attributes);
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.createFromDictionary = function(attributes) {
	return new caplin.streamlink.impl.StreamLinkConfigurationImpl(attributes);
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setApplicationId = function(applicationId) {
	this._attributes["application_id"] = applicationId;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setClockOffsetInterval = function(value) {
	this._attributes["clock_offset_interval"] = value;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setClockOffsetWindow = function(value) {
	this._attributes["clock_offset_window"] = value;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setForeverFrameReconnectCount = function(count) {
	this._attributes["forever_frame_reconnect_count"] = count;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setHttpTimeout = function(timeout) {
	this._attributes["http_timeout"] = timeout;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setKeymasterAttempts = function(timeout) {
	this._attributes["keymaster_attempts"] = timeout;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setKeymasterKeepAliveInterval = function(timeout) {
	this._attributes["keymaster_keep_alive_interval"] = timeout;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setKeymasterKeepAliveListener = function(keymasterPollListener) {
	this._keymasterPollListener = keymasterPollListener;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setKeymasterKeepAliveUrl = function(url) {
	this._attributes["keymaster_keep_alive_url"] = url;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setKeymasterUrl = function(url) {
	this._attributes["keymaster_url"] = url;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setLatencyInterval = function(value) {
	this._attributes["latency_interval"] = value;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setLatencyWindow = function(value) {
	this._attributes["latency_window"] = value;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setPassword = function(password) {
	this._attributes["password"] = password;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setPollInterval = function(interval) {
	this._attributes["poll_interval"] = interval;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setRequestTimeout = function(value) {
	this._attributes["request_timeout"] = value;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setRunOnWorker = function() {
	this._attributes["run_on_worker"] = true;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setUsername = function(username) {
	this._attributes["username"] = username;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setWorkerUrl = function(workerUrl) {
	this._attributes["worker_url"] = workerUrl;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.setXhrStreamingReconnectCount = function(count) {
	this._attributes["xhr_streaming_reconnect_count"] = count;
	return this;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getLiberatorUrls = function() {
	return this._attributes["liberator_urls"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._toDictionary = function() {
	return this._attributes;
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getClockOffsetInterval = function() {
	return this._attributes["clock_offset_interval"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getClockOffsetWindow = function() {
	return this._attributes["clock_offset_window"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getClockOffsetsInWindow = function() {
	return this._getClockOffsetWindow() / this._getClockOffsetInterval();
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getForeverFrameReconnectCount = function() {
	return this._attributes["forever_frame_reconnect_count"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getHttpTimeout = function() {
	return this._attributes["http_timeout"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getKeymasterAttempts = function() {
	return this._attributes["keymaster_attempts"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getKeymasterKeepAliveInterval = function() {
	return this._attributes["keymaster_keep_alive_interval"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getLatencyInterval = function() {
	return this._attributes["latency_interval"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getLatencyWindow = function() {
	return this._attributes["latency_window"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getLatencysInWindow = function() {
	return this._getLatencyWindow() / this._getLatencyInterval();
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getPollInterval = function() {
	return this._attributes["poll_interval"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getRequestTimeout = function() {
	return this._attributes["request_timeout"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getXhrStreamingReconnectCount = function() {
	return this._attributes["xhr_streaming_reconnect_count"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getLogWindowVisible = function() {
	return this._attributes["log_window_visible"];
};

/**
 *  
 *  ======== Get Methods ========
 *  
 * @function
 */
caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getApplicationId = function() {
	return this._attributes["application_id"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getKeymasterKeepAliveUrl = function() {
	return this._attributes["keymaster_keep_alive_url"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getKeymasterUrl = function() {
	return this._attributes["keymaster_url"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getPassword = function() {
	return this._attributes["password"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getUsername = function() {
	return this._attributes["username"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._getWorkerUrl = function() {
	return this._attributes["worker_url"];
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._checkCredentialsConfig = function() {
	var username = this._getUsername();
	var password = this._getPassword();
	var keymasterUrl = this._getKeymasterUrl();
	var keymasterKeepAliveUrl = this._getKeymasterKeepAliveUrl();
	if (!(this.isKeyMasterAuthentication()) && !(this.isBasicAuthentication())) {
		throw new Error("You must define a keymaster url and keymaster keep-alive url OR a username and password.");
	}
	else if ((keymasterUrl != null || keymasterKeepAliveUrl != null) && (username != null || password != null)) {
		throw new Error("You must define either a keymaster url and keymaster keep-alive url OR a username and password, not both.");
	}
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._checkLatencyConfig = function() {
	if (this._getClockOffsetInterval() > this._getClockOffsetWindow()) {
		throw new Error("The clock_offset_window must be greater than or equal to the clock_offset_interval.");
	}
	if (this._getLatencyInterval() > this._getLatencyWindow()) {
		throw new Error("The latency_window must be greater than or equal to the latency_window.");
	}
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._log = function(logger) {
	logger.log(caplin.streamlink.LogLevel.CONFIG, "Configuration:");
	for (var key in this._attributes) {
		logger.log(caplin.streamlink.LogLevel.CONFIG, "{0} = {1}", key, this._attributes[key]);
	}
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype._validate = function() {
	this.assertIntegerPositive("request_timeout");
	this.assertIntegerPositive("latency_interval");
	this.assertIntegerPositive("latency_window");
	this.assertIntegerPositive("clock_offset_window");
	this.assertIntegerPositive("latency_interval");
	this.assertIntegerPositive("clock_offset_interval");
	this.assertIntegerPositive("http_timeout");
	this.assertIntegerPositive("keymaster_attempts");
	this.assertIntegerPositive("keymaster_keep_alive_interval");
	this.assertIntegerPositive("http_timeout");
	this.assertIntegerPositive("poll_interval");
	this.assertIntegerPositive("forever_frame_reconnect_count");
	this.assertIntegerPositive("xhr_streaming_reconnect_count");
	this._checkCredentialsConfig();
	this._checkLatencyConfig();
	if (this._getRequestTimeout() <= 0) {
		throw new Error("The request timeout must be positive.");
	}
};

caplin.streamlink.impl.StreamLinkConfigurationImpl.prototype.assertIntegerPositive = function(key) {
	if (this._attributes[key] <= 0) {
		throw new Error("The " + key.replace(new RegExp("_", 'g'), " ") + " must be positive.");
	}
};



//## Encoder.js ##//
caplin.namespace("caplin.streamlink.impl.protocolhandler");

caplin.streamlink.impl.protocolhandler.Encoder = function() {};

caplin.streamlink.impl.protocolhandler.Encoder.prototype.rttpDecode = function(rttp) { throw new Error('caplin.streamlink.impl.protocolhandler.Encoder.rttpDecode :  This is an abstract method'); };

caplin.streamlink.impl.protocolhandler.Encoder.prototype.rttpEncode = function(rttp) { throw new Error('caplin.streamlink.impl.protocolhandler.Encoder.rttpEncode :  This is an abstract method'); };



//## CommandListener.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  <p>Provides information on the result of a command.</p>
 *  
 *  <p>A trivial implementation would be as follows :</p>
 *  <p><code><pre>streamLink.publishToSubject("/SUBJECT", fieldData, {
 *         onCommandError : function(subject, commandErrorEvent)
 *         {
 *                 console.log("Publish Error : " + commandErrorEvent.getError());
 *         },
 *         onCommandOk : function(subject)
 *         {
 *                 console.log("Publish to " + subject + " Succesful");
 *         }
 * }, null);
 * </pre></code></p>
 * 
 * @name caplin.streamlink.CommandListener
 */
caplin.streamlink.CommandListener = function() {};

/**
 *  <p>The command failed or is in an unknown state.</p>
 * @param {String} subject The subject of the command.
 * @param {caplin.streamlink.CommandErrorEvent} commandErrorEvent Provides error information for commands.
 * @function
 */
caplin.streamlink.CommandListener.prototype.onCommandError = function(subject, commandErrorEvent) { throw new Error('caplin.streamlink.CommandListener.onCommandError :  This is an abstract method'); };

/**
 *  <p>The command was successful. Implementing this method is optional.</p>
 * @param {String} subject The subject of the command.
 * @function
 */
caplin.streamlink.CommandListener.prototype.onCommandOk = function(subject) { throw new Error('caplin.streamlink.CommandListener.onCommandOk :  This is an abstract method'); };



//## CommandErrorEvent.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  <p>Provides error information for commands.</p>
 *  <p>The <code>CommandErrorEvent</code> will be provided on a <code>onCommandError</code> callback to a {@link caplin.streamlink.CommandListener}.</p>
 * 
 * 
 * @name caplin.streamlink.CommandErrorEvent
 */
caplin.streamlink.CommandErrorEvent = function() {};

/**
 *  <p>Returns the {@link caplin.streamlink.CommandErrorType} of the event.</p>
 *  
 *  @returns {caplin.streamlink.CommandErrorType} The type of error.
 * @function
 */
caplin.streamlink.CommandErrorEvent.prototype.getError = function() { throw new Error('caplin.streamlink.CommandErrorEvent.getError :  This is an abstract method'); };



//## ParseUtils.js ##//
caplin.namespace("caplin.streamlink.impl.event");


caplin.streamlink.impl.event.ParseUtils = function() {};

caplin.streamlink.impl.event.ParseUtils.getSubjectType = function(type) {
	if ((type == ("200"))) {
		return caplin.streamlink.SubjectType.UNKNOWN;
	}
	else if ((type == ("220"))) {
		return caplin.streamlink.SubjectType.DIRECTORY;
	}
	else if ((type == ("221"))) {
		return caplin.streamlink.SubjectType.PAGE;
	}
	else if ((type == ("222"))) {
		return caplin.streamlink.SubjectType.RECORD;
	}
	else if ((type == ("223"))) {
		return caplin.streamlink.SubjectType.NEWS_HEADLINE;
	}
	else if ((type == ("224"))) {
		return caplin.streamlink.SubjectType.NEWS_STORY;
	}
	else if ((type == ("227"))) {
		return caplin.streamlink.SubjectType.CHAT;
	}
	else if ((type == ("228"))) {
		return caplin.streamlink.SubjectType.CONTAINER;
	}
	else if ((type == ("230"))) {
		return caplin.streamlink.SubjectType.PERMISSION;
	}
	else {
		return caplin.streamlink.SubjectType.UNKNOWN;
	}
};

/**
 *  Standard field helper methods
 * @function
 */
caplin.streamlink.impl.event.ParseUtils.getFields = function(tokens, fieldMap, encoder, fieldsStartToken) {
	var fields = {} ;
	for (var i = fieldsStartToken; i < tokens.length; (i)++) {
		var pair = tokens[i].split("=");
		var fieldName = fieldMap[pair[0]];
		if (fieldName == null) {
			fieldName = encoder.rttpDecode(pair[0]);
		}
		fields[fieldName] = (pair.length == 2) ? encoder.rttpDecode(pair[1]) : "";
	}
	return fields;
};

caplin.streamlink.impl.event.ParseUtils.getNumberFromSubjectType = function(type) {
	switch (type) {
	case caplin.streamlink.SubjectType.DIRECTORY:
	return 220;
	case caplin.streamlink.SubjectType.PAGE:
	return 221;
	case caplin.streamlink.SubjectType.RECORD:
	return 222;
	case caplin.streamlink.SubjectType.NEWS_HEADLINE:
	return 223;
	case caplin.streamlink.SubjectType.NEWS_STORY:
	return 224;
	case caplin.streamlink.SubjectType.CHAT:
	return 227;
	case caplin.streamlink.SubjectType.CONTAINER:
	return 228;
	case caplin.streamlink.SubjectType.PERMISSION:
	return 230;
	default :
	return 200;
	}
};

/**
 *  Get the name of the first field
 * @function
 */
caplin.streamlink.impl.event.ParseUtils.getNameOfFirstField = function(tokens, fieldMap) {
	if (tokens.length > 1) {
		var keyPair = tokens[1].split("=");
		var fieldName = fieldMap[keyPair[0]];
		if (fieldName == null) {
			fieldName = keyPair[0];
		}
		return fieldName;
	}
	return null;
};

/**
 *  Permission helper methods
 * @function
 */
caplin.streamlink.impl.event.ParseUtils.getPermissionsKeyFromTokens = function(tokens, encoder, keyStartIndex) {
	var tokenKey = null;
	if (keyStartIndex > 0 && tokens.length > keyStartIndex && (tokens[keyStartIndex].indexOf("=")>=0)) {
		var keyPair = tokens[keyStartIndex].split("=");
		tokenKey = encoder.rttpDecode(keyPair[1]);
	}
	return tokenKey;
};



//## CredentialsProvider.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  
 *  
 * @private
 *  
 *  <p>This interface defines how to provide the user credentials that StreamLink uses to log into a Liberator
 *  server.</p>
 *  
 *  <p>If your StreamLink application does not use one of the credential provider classes supplied with
 *  StreamLink, you must define your own class that implements this interface, along with an implementation of the
 *  {@link caplin.streamlink.Credentials} interface (which is passed to the {@link caplin.streamlink.CredentialsReceiver}
 *  instance provided).</p>
 * 
 * @name caplin.streamlink.CredentialsProvider
 */
caplin.streamlink.CredentialsProvider = function() {};

/**
 *  <p>Attempts to cancel a request to get credentials on the CredentialsProvider.</p>
 * @function
 */
caplin.streamlink.CredentialsProvider.prototype.cancel = function() { throw new Error('caplin.streamlink.CredentialsProvider.cancel :  This is an abstract method'); };

/**
 *  <p>Get the credentials used to login to the Liberator.</p>
 *  
 *  <p>This method will be called by the StreamLink library to trigger an attempt to retrieve credentials.</p>
 *  
 *  <p>This method should not block. If retrieving the credentials involves communicating with an external
 *  service then asynchronous methods should be used and control returned to the StreamLink library as quickly
 *  as possible.</p>
 *  
 * @param {caplin.streamlink.CredentialsReceiver} listener Interface to call back on to provide either the credentials or an error indication.
 * @function
 */
caplin.streamlink.CredentialsProvider.prototype.onCredentialsRequired = function(listener) { throw new Error('caplin.streamlink.CredentialsProvider.onCredentialsRequired :  This is an abstract method'); };



//## KeymasterCredentialsProvider.js ##//
caplin.namespace("caplin.streamlink.impl.credentials");

// caplin.forceload(caplin.streamlink.CredentialsProvider

caplin.streamlink.impl.credentials.KeymasterCredentialsProvider = function() {};

// caplin.implement(caplin.streamlink.impl.credentials.KeymasterCredentialsProvider, caplin.streamlink.CredentialsProvider);

caplin.streamlink.impl.credentials.KeymasterCredentialsProvider.prototype.cancelPolling = function() { throw new Error('caplin.streamlink.impl.credentials.KeymasterCredentialsProvider.cancelPolling :  This is an abstract method'); };

caplin.streamlink.impl.credentials.KeymasterCredentialsProvider.prototype.startKeymasterPollingInterval = function() { throw new Error('caplin.streamlink.impl.credentials.KeymasterCredentialsProvider.startKeymasterPollingInterval :  This is an abstract method'); };



//## SubscriptionErrorType.js ##//
caplin.namespace("caplin.streamlink");

/**
 * @class  <p>Enumeration which represents the errors that can occur for a subject.</p>
 *  <p>The <code>SubscriptionErrorType</code> will be provided on a <code>onSubscriptionError</code> callback to a {@link caplin.streamlink.SubscriptionListener}.</p>
 * 
 * @name caplin.streamlink.SubscriptionErrorType
 */
caplin.streamlink.SubscriptionErrorType = {
	/**
	 *   <p>No error.</p>
	 */
	OK: "OK",
	/**
	 *  <p>The subject could not be found.</p>
	 */
	NOT_FOUND: "NOT_FOUND",
	/**
	 *  <p>The subject was not available.</p>
	 */
	UNAVAILABLE: "UNAVAILABLE",
	/**
	 *  <p>The subject has been deleted.</p>
	 */
	DELETED: "DELETED",
	/**
	 *  <p>The client does not have permission to read the subject.</p>
	 */
	READ_DENIED: "READ_DENIED",
	/**
	 *  <p>The client does not have permission to write to the subject.</p>
	 */
	WRITE_DENIED: "WRITE_DENIED",
	/**
	 *  <p>The client cannot contribute due to the Liberator's licensing limits being exceeded.</p>
	 */
	WRITE_DENIED_DUE_TO_LICENSE: "WRITE_DENIED_DUE_TO_LICENSE",
	/**
	 *  <p>Supplied parameters are invalid for this subject type.</p>
	 */
	INVALID_PARAMETERS: "INVALID_PARAMETERS",
	values: function() {return [caplin.streamlink.SubscriptionErrorType.OK, caplin.streamlink.SubscriptionErrorType.NOT_FOUND, caplin.streamlink.SubscriptionErrorType.UNAVAILABLE, caplin.streamlink.SubscriptionErrorType.DELETED, caplin.streamlink.SubscriptionErrorType.READ_DENIED, caplin.streamlink.SubscriptionErrorType.WRITE_DENIED, caplin.streamlink.SubscriptionErrorType.WRITE_DENIED_DUE_TO_LICENSE, caplin.streamlink.SubscriptionErrorType.INVALID_PARAMETERS];},
	valueOf: function(name) {return name;}
};

//## StatisticsCalculator.js ##//
caplin.namespace("caplin.streamlink.impl.protocolhandler");


caplin.streamlink.impl.protocolhandler.StatisticsCalculator = function(clockOffsetsInWindow, latencysInWindow) {
	// Field Definitions
	this.clockOffset = 0;
	this.averageLatency = 0;
	this.clockOffsets = [];
	this.roundTripTimes = [];
	this.latencies = [];

	// Constructor body
	this.clockOffsetsInWindow = clockOffsetsInWindow;
	this.latencysInWindow = latencysInWindow;
};

caplin.streamlink.impl.protocolhandler.StatisticsCalculator.prototype.newStatisticsEvent = function() {
	return new caplin.streamlink.impl.event.StatisticsEventImpl(this.getLastLatency(), this.averageLatency, this.clockOffset);
};

caplin.streamlink.impl.protocolhandler.StatisticsCalculator.prototype.getSortedLatencies = function() {
	var result = [];
	Array.prototype.push.apply(result, this.latencies);
	result.sort(function(left, right) {
		if (left < right) {
			return -(1);
		}
		if (left > right) {
			return 1;
		}
		return 0;
	});
	return result;
};

caplin.streamlink.impl.protocolhandler.StatisticsCalculator.prototype.calculateAverageLatency = function() {
	var sortedLatencies = this.getSortedLatencies();
	var quartileLength = Math.floor(sortedLatencies.length / 4);
	var interquartileSortedLatencies = sortedLatencies.slice(quartileLength, -(quartileLength));
	if ((interquartileSortedLatencies.length==0)) {
		return this.getLastLatency();
	}
	var latencySum = 0;
	for (var i = 0; i < interquartileSortedLatencies.length; ++(i)) {
		latencySum += interquartileSortedLatencies[i];
	}
	return Math.round(latencySum / interquartileSortedLatencies.length);
};

caplin.streamlink.impl.protocolhandler.StatisticsCalculator.prototype.calculateClockOffset = function() {
	var indexOfShortestRoundTripTime = 0;
	for (var i = 0; i < this.roundTripTimes.length; ++(i)) {
		if (this.roundTripTimes[i] < this.roundTripTimes[indexOfShortestRoundTripTime]) {
			indexOfShortestRoundTripTime = i;
		}
	}
	return this.clockOffsets[indexOfShortestRoundTripTime];
};

caplin.streamlink.impl.protocolhandler.StatisticsCalculator.prototype.getAverageLatency = function() {
	return this.averageLatency;
};

caplin.streamlink.impl.protocolhandler.StatisticsCalculator.prototype.getClockOffset = function() {
	return this.clockOffset;
};

caplin.streamlink.impl.protocolhandler.StatisticsCalculator.prototype.getLastLatency = function() {
	if ((this.latencies.length==0)) {
		return 0;
	}
	return this.latencies[this.latencies.length - 1];
};

caplin.streamlink.impl.protocolhandler.StatisticsCalculator.prototype.clear = function() {
	(this.clockOffsets.length = 0);
	(this.roundTripTimes.length = 0);
	(this.latencies.length = 0);
	this.clockOffset = 0;
	this.averageLatency = 0;
};

caplin.streamlink.impl.protocolhandler.StatisticsCalculator.prototype.updateLatencyStatistics = function(timeSentFromServer, timeReceivedOnClient) {
	if ((this.clockOffsets.length==0)) {
		return;
	}
	this.latencies.push(Math.max(0, timeReceivedOnClient - timeSentFromServer + this.clockOffset));
	if (this.latencies.length > this.latencysInWindow) {
		this.latencies.shift();
	}
	this.averageLatency = this.calculateAverageLatency();
};

caplin.streamlink.impl.protocolhandler.StatisticsCalculator.prototype.updateSyncStatistics = function(timeSentFromClient, timeSentFromServer, timeBetweenClientToServer, timeReceivedOnClient) {
	this.clockOffsets.push(Math.floor((timeBetweenClientToServer - timeReceivedOnClient + timeSentFromServer) / 2.0));
	this.roundTripTimes.push(timeSentFromClient - timeReceivedOnClient);
	if (this.clockOffsets.length > this.clockOffsetsInWindow) {
		this.clockOffsets.shift();
		this.roundTripTimes.shift();
	}
	this.clockOffset = this.calculateClockOffset();
};



//## Credentials.js ##//
caplin.namespace("caplin.streamlink");

/**
 * @class  
 *  
 * @private
 *  
 *  <p>Encapsulates authentication information used to log in to a Liberator server.</p>
 *  
 *  <p>If your StreamLink application does not use one of the {@link caplin.streamlink.CredentialsProvider}
 *  implementations supplied with StreamLink, you must define a class that implements the 
 *  {@link caplin.streamlink.CredentialsProvider} interface.</p>
 * 
 * @name caplin.streamlink.Credentials
 */
caplin.streamlink.Credentials = function() {};

/**
 *  <p>Returns the password.</p>
 *  @returns {String} The password.
 * @function
 */
caplin.streamlink.Credentials.prototype.getPassword = function() { throw new Error('caplin.streamlink.Credentials.getPassword :  This is an abstract method'); };

/**
 *  <p>Returns the username.</p>
 *  @returns {String} The username.
 * @function
 */
caplin.streamlink.Credentials.prototype.getUsername = function() { throw new Error('caplin.streamlink.Credentials.getUsername :  This is an abstract method'); };



//## CredentialsImpl.js ##//
caplin.namespace("caplin.streamlink.impl.credentials");

// caplin.forceload(caplin.streamlink.Credentials

caplin.streamlink.impl.credentials.CredentialsImpl = function(username, password) {
	// Constructor body
	this._username = username;
	this._password = password;
};

// caplin.implement(caplin.streamlink.impl.credentials.CredentialsImpl, caplin.streamlink.Credentials);

caplin.streamlink.impl.credentials.CredentialsImpl.prototype.getPassword = function() {
	return this._password;
};

caplin.streamlink.impl.credentials.CredentialsImpl.prototype.getUsername = function() {
	return this._username;
};

caplin.streamlink.impl.credentials.CredentialsImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("Credentials [username=");
	builder.push(this._username);
	builder.push(", password=");
	builder.push(this._password);
	builder.push("]");
	return builder.join("");
};



//## CommandErrorType.js ##//
caplin.namespace("caplin.streamlink");

/**
 * @class  <p>Enumeration that defines errors that can occur for a subject.</p>
 *  <p>The <code>CommandErrorType</code> will be provided by calling <code>getError()</code> on a {@link caplin.streamlink.CommandErrorEvent}.</p>
 * 
 * @name caplin.streamlink.CommandErrorType
 */
caplin.streamlink.CommandErrorType = {
	/**
	 *  <p>The subject has been deleted.</p>
	 */
	DELETED: "DELETED",
	/**
	 *  <p>May have been sent to server, but disconnect occurred.</p>
	 */
	DISCONNECT_OCCURRED: "DISCONNECT_OCCURRED",
	/**
	 *  <p>Supplied parameters are invalid for this subject type.</p>
	 */
	INVALID_PARAMETERS: "INVALID_PARAMETERS",
	/**
	 *  <p>The subject could not be found.</p>
	 */
	NOT_FOUND: "NOT_FOUND",
	/**
	 *  <p>The client does not have permission to read the subject.</p>
	 */
	READ_DENIED: "READ_DENIED",
	/**
	 *  <p>Throttle failed.</p>
	 */
	THROTTLE_FAILED: "THROTTLE_FAILED",
	/**
	 *  <p>The subject was not available.</p>
	 */
	UNAVAILABLE: "UNAVAILABLE",
	/**
	 *  <p>The client does not have permission to write to the subject.</p>
	 */
	WRITE_DENIED: "WRITE_DENIED",
	/**
	 *  <p>The Liberator has detected a breach of license parameters and has rejected the attempted trade.</p>
	 */
	WRITE_DENIED_TRADE_GROUP_LICENSE_EXCEEDED: "WRITE_DENIED_TRADE_GROUP_LICENSE_EXCEEDED",
	values: function() {return [caplin.streamlink.CommandErrorType.DELETED, caplin.streamlink.CommandErrorType.DISCONNECT_OCCURRED, caplin.streamlink.CommandErrorType.INVALID_PARAMETERS, caplin.streamlink.CommandErrorType.NOT_FOUND, caplin.streamlink.CommandErrorType.READ_DENIED, caplin.streamlink.CommandErrorType.THROTTLE_FAILED, caplin.streamlink.CommandErrorType.UNAVAILABLE, caplin.streamlink.CommandErrorType.WRITE_DENIED, caplin.streamlink.CommandErrorType.WRITE_DENIED_TRADE_GROUP_LICENSE_EXCEEDED];},
	valueOf: function(name) {return name;}
};

//## LogInfo.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  <p>Stores log information including message and level of a log entry.</p> 
 * 
 * @name caplin.streamlink.LogInfo
 */
caplin.streamlink.LogInfo = function() {};

/**
 *  <p>Returns the log level for this log entry.</p>
 *  @returns {caplin.streamlink.LogLevel} The log level.
 * @function
 */
caplin.streamlink.LogInfo.prototype.getLevel = function() { throw new Error('caplin.streamlink.LogInfo.getLevel :  This is an abstract method'); };

/**
 *  <p>Returns the message text for this log entry.</p>
 *  @returns {String} The message text.
 * @function
 */
caplin.streamlink.LogInfo.prototype.getMessage = function() { throw new Error('caplin.streamlink.LogInfo.getMessage :  This is an abstract method'); };

/**
 *  <p>Returns the date for this log entry.</p>
 *  @returns {Date} The date for this log level.
 * @function
 */
caplin.streamlink.LogInfo.prototype.getDate = function() { throw new Error('caplin.streamlink.LogInfo.getDate :  This is an abstract method'); };



//## Timer.js ##//
caplin.namespace("caplin.streamlink.impl");

caplin.streamlink.impl.Timer = function() {};

caplin.streamlink.impl.Timer.prototype.cancel = function() { throw new Error('caplin.streamlink.impl.Timer.cancel :  This is an abstract method'); };



//## ConnectionState.js ##//
caplin.namespace("caplin.streamlink");

/**
 * @class  <p>Enumeration that defines the state of a StreamLink connection.</p>
 *  <p>The <code>ConnectionState</code> will be provided by calling <code>getConnectionState</code> on a {@link caplin.streamlink.ConnectionStatusEvent}.</p>
 * 
 * @name caplin.streamlink.ConnectionState
 */
caplin.streamlink.ConnectionState = {
	/**
	 *  <p>Disconnected from the Liberator.</p>
	 */
	DISCONNECTED: "DISCONNECTED",
	/**
	 *  <p>A connection attempt is in progress.</p> 
	 */
	CONNECTING: "CONNECTING",
	/**
	 *  <p>A connection attempt has failed.</p>
	 */
	FAILED: "FAILED",
	/**
	 *  <p>A network connection has been established to the Liberator.</p> 
	 */
	CONNECTED: "CONNECTED",
	/**
	 *  <p>The client is now logged into the server.</p>
	 */
	LOGGEDIN: "LOGGEDIN",
	/**
	 *  <p>The client could not login to the Liberator.</p>
	 */
	LOGINFAILED: "LOGINFAILED",
	/**
	 *  <p>The client has been ejected from the Liberator.</p> 
	 */
	EJECTED: "EJECTED",
	/**
	 *  <p>Connection to the Liberator has been lost. The API will attempt to reconnect.</p> 
	 */
	LOST: "LOST",
	/**
	 *  <p>Connection to the Liberator has been re-established.</p>
	 */
	RECONNECTED: "RECONNECTED",
	/**
	 *  <p>Connection to the Liberator is not possible.</p>
	 */
	UNREACHABLE: "UNREACHABLE",
	values: function() {return [caplin.streamlink.ConnectionState.DISCONNECTED, caplin.streamlink.ConnectionState.CONNECTING, caplin.streamlink.ConnectionState.FAILED, caplin.streamlink.ConnectionState.CONNECTED, caplin.streamlink.ConnectionState.LOGGEDIN, caplin.streamlink.ConnectionState.LOGINFAILED, caplin.streamlink.ConnectionState.EJECTED, caplin.streamlink.ConnectionState.LOST, caplin.streamlink.ConnectionState.RECONNECTED, caplin.streamlink.ConnectionState.UNREACHABLE];},
	valueOf: function(name) {return name;}
};

//## ServiceStatusEvent.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  <p>Provides information about the change in status of a data service to which StreamLink is connected
 *  via the Liberator.</p>
 *  <p>The <code>ServiceStatusEvent</code> will be provided on a <code>onServiceStatusChange</code> 
 *  callback to a {@link caplin.streamlink.ConnectionListener}.</p>
 * 
 * @name caplin.streamlink.ServiceStatusEvent
 */
caplin.streamlink.ServiceStatusEvent = function() {};

/**
 *  <p>Returns the new {@link caplin.streamlink.ServiceStatus} of a service.</p>
 *  
 *  @returns {caplin.streamlink.ServiceStatus} the service status 
 * @function
 */
caplin.streamlink.ServiceStatusEvent.prototype.getServiceStatus = function() { throw new Error('caplin.streamlink.ServiceStatusEvent.getServiceStatus :  This is an abstract method'); };

/**
 *  <p>Returns the name of the service that has changed state.</p>
 *  
 *  @returns {String} the service name
 * @function
 */
caplin.streamlink.ServiceStatusEvent.prototype.getServiceName = function() { throw new Error('caplin.streamlink.ServiceStatusEvent.getServiceName :  This is an abstract method'); };



//## PacketAuto.js ##//
caplin.namespace("caplin.streamlink.impl.packet");

// caplin.forceload(caplin.streamlink.impl.packet.Packet

caplin.streamlink.impl.packet.PacketAuto = function(subject) {
	// Constructor body
	this.subject = subject;
};

// caplin.implement(caplin.streamlink.impl.packet.PacketAuto, caplin.streamlink.impl.packet.Packet);

caplin.streamlink.impl.packet.PacketAuto.prototype.getType = function() {
	return caplin.streamlink.impl.packet.PacketType.AUTO;
};

caplin.streamlink.impl.packet.PacketAuto.prototype.getSubject = function() {
	return this.subject;
};

caplin.streamlink.impl.packet.PacketAuto.prototype.toRttpString = function(encoder) {
	return null;
};



//## ContainerElement.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  <p>This interface represents a subject that is within a container.</p>
 *  <p>The <code>ContainerElement</code> will be provided on an <code>insert</code>, <code>remove</code> or <code>move</code> 
 *   callback to a {@link caplin.streamlink.ContainerModel}.</p>
 * 
 * @name caplin.streamlink.ContainerElement
 */
caplin.streamlink.ContainerElement = function() {};

/**
 *  <p>Returns the SubjectType of the element.</p>
 *  @returns {caplin.streamlink.SubjectType} The SubjectType.
 * @function
 */
caplin.streamlink.ContainerElement.prototype.getType = function() { throw new Error('caplin.streamlink.ContainerElement.getType :  This is an abstract method'); };

/**
 *  <p>Returns the subject of the element.</p>
 *  @returns {String} The subject.
 * @function
 */
caplin.streamlink.ContainerElement.prototype.getSubject = function() { throw new Error('caplin.streamlink.ContainerElement.getSubject :  This is an abstract method'); };



//## ContainerElementImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.ContainerElement
// caplin.forceload(caplin.streamlink.impl.Serializable

caplin.streamlink.impl.event.ContainerElementImpl = function(objectNumber, subject, type, position) {
	// Constructor body
	this._objectNumber = objectNumber;
	this._type = type;
	this._subject = subject;
	this._position = position;
};

// caplin.implement(caplin.streamlink.impl.event.ContainerElementImpl, caplin.streamlink.ContainerElement);
// caplin.implement(caplin.streamlink.impl.event.ContainerElementImpl, caplin.streamlink.impl.Serializable);

caplin.streamlink.impl.event.ContainerElementImpl.prototype.getType = function() {
	return this._type;
};

caplin.streamlink.impl.event.ContainerElementImpl.createFromDictionary = function(root) {
	var objectNumber = root["objectNumber"];
	var subject = root["subject"];
	var type = caplin.streamlink.SubjectType.valueOf(root["type"]);
	var position = root["position"];
	return new caplin.streamlink.impl.event.ContainerElementImpl(objectNumber, subject, type, position);
};

caplin.streamlink.impl.event.ContainerElementImpl.prototype._toDictionary = function() {
	var root = {} ;
	root["objectNumber"] = this._objectNumber;
	root["subject"] = this._subject;
	root["type"] = this._type.toString();
	root["position"] = this._position;
	return root;
};

caplin.streamlink.impl.event.ContainerElementImpl.prototype._getPosition = function() {
	return this._position;
};

caplin.streamlink.impl.event.ContainerElementImpl.prototype._getObjectNumber = function() {
	return this._objectNumber;
};

caplin.streamlink.impl.event.ContainerElementImpl.prototype.getSubject = function() {
	return this._subject;
};

caplin.streamlink.impl.event.ContainerElementImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("ContainerElement [subject=");
	builder.push(this._subject);
	builder.push(", type=");
	builder.push(this._type);
	builder.push(", position=");
	builder.push(this._position);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.ContainerElementImpl.prototype._setPosition = function(position) {
	this._position = position;
};



//## Connection.js ##//
caplin.namespace("caplin.streamlink.impl.connection");


caplin.streamlink.impl.connection.Connection = function() {};

caplin.streamlink.impl.connection.Connection.prototype.isReadyToSend = function() { throw new Error('caplin.streamlink.impl.connection.Connection.isReadyToSend :  This is an abstract method'); };

caplin.streamlink.impl.connection.Connection.prototype.getURL = function() { throw new Error('caplin.streamlink.impl.connection.Connection.getURL :  This is an abstract method'); };

caplin.streamlink.impl.connection.Connection.prototype.connect = function() { throw new Error('caplin.streamlink.impl.connection.Connection.connect :  This is an abstract method'); };

caplin.streamlink.impl.connection.Connection.prototype.disconnect = function() { throw new Error('caplin.streamlink.impl.connection.Connection.disconnect :  This is an abstract method'); };

caplin.streamlink.impl.connection.Connection.prototype.send = function(message) { throw new Error('caplin.streamlink.impl.connection.Connection.send :  This is an abstract method'); };

caplin.streamlink.impl.connection.Connection.prototype.setRttpConnectionListener = function(listener) { throw new Error('caplin.streamlink.impl.connection.Connection.setRttpConnectionListener :  This is an abstract method'); };

caplin.streamlink.impl.connection.Connection.prototype.setSessionId = function(sessionId) { throw new Error('caplin.streamlink.impl.connection.Connection.setSessionId :  This is an abstract method'); };



//## BaseConnection.js ##//
caplin.namespace("caplin.streamlink.impl.connection");

// caplin.forceload(caplin.streamlink.impl.connection.Connection

caplin.streamlink.impl.connection.BaseConnection = function(context, url, ssl) {
	// Constructor body
	this.context = context;
	this.logger = context.logger;
	this.url = url;
	this.ssl = ssl;
	this.host = this.getHost(url);
	this.port = this.getPort(url);
	this.messageCount = 0;
	this.connectionState = caplin.streamlink.impl.connection.RttpConnectionState.DISCONNECTED;
	this.readyToSend = false;
};

// caplin.implement(caplin.streamlink.impl.connection.BaseConnection, caplin.streamlink.impl.connection.Connection);

caplin.streamlink.impl.connection.BaseConnection.prototype.isReadyToSend = function() {
	return this.connectionState == caplin.streamlink.impl.connection.RttpConnectionState.CONNECTED && this.readyToSend;
};

caplin.streamlink.impl.connection.BaseConnection.prototype.getDefaultPort = function() {
	if (this.ssl) {
		return 443;
	}
	else {
		return 80;
	}
};

caplin.streamlink.impl.connection.BaseConnection.prototype.getPort = function(urlString) {
	var tempUrlString = urlString.substring(urlString.indexOf("//") + 2);
	if ((tempUrlString.indexOf(":")>=0)) {
		var portString = tempUrlString.substring(tempUrlString.indexOf(':') + 1);
		if ((portString.indexOf("/")>=0)) {
			portString = portString.substring(0, portString.indexOf('/'));
		}
		if ((portString == (""))) {
			return this.getDefaultPort();
		}
		else {
			try {
				this.context.runtime.parseInt(portString);
			}
			catch (e) {
				this.logger.log(caplin.streamlink.LogLevel.ERROR, "Cannot parse port in url: {0} - Using default port {1} ", urlString, this.getDefaultPort());
			}
		}
	}
	return this.getDefaultPort();
};

caplin.streamlink.impl.connection.BaseConnection.prototype.getHost = function(urlString) {
	var hostString = urlString.substring(urlString.indexOf("//") + 2);
	var end = hostString.indexOf(':');
	if (end == -(1)) {
		end = hostString.indexOf('/');
	}
	if (end != -(1)) {
		hostString = hostString.substring(0, end);
	}
	return hostString;
};

caplin.streamlink.impl.connection.BaseConnection.prototype.getURL = function() {
	return this.url;
};

caplin.streamlink.impl.connection.BaseConnection.prototype.changeState = function(newState) {
	if (this.connectionState != newState) {
		this.context.logger.log(caplin.streamlink.LogLevel.FINEST, "Rttp Connection changing state to {0}", newState);
		this.connectionState = newState;
		this.fireStateChange();
	}
};

caplin.streamlink.impl.connection.BaseConnection.prototype.fireData = function(data) {
	if (this.connectionState == caplin.streamlink.impl.connection.RttpConnectionState.CONNECTED && this.listener != null) {
		this.listener.onData(data);
		(this.messageCount)++;
	}
};

caplin.streamlink.impl.connection.BaseConnection.prototype.fireReadyToSend = function() {
	if (this.connectionState == caplin.streamlink.impl.connection.RttpConnectionState.CONNECTED && this.listener != null) {
		this.listener.onReady();
	}
};

caplin.streamlink.impl.connection.BaseConnection.prototype.fireReconnectRequest = function() {
	if (this.connectionState == caplin.streamlink.impl.connection.RttpConnectionState.CONNECTED && this.listener != null) {
		this.context.logger.log(caplin.streamlink.LogLevel.FINE, "Requesting reconnection.");
		this.listener.onRequestReconnect();
	}
};

caplin.streamlink.impl.connection.BaseConnection.prototype.fireStateChange = function() {
	if (this.listener != null) {
		this.listener.onStateChange(this.connectionState);
	}
};

caplin.streamlink.impl.connection.BaseConnection.prototype.logDataIn = function(message) {
	this.context.logger.log(caplin.streamlink.LogLevel.FINER, "< {0}", message);
};

caplin.streamlink.impl.connection.BaseConnection.prototype.logDataOut = function(message) {
	this.context.logger.log(caplin.streamlink.LogLevel.FINER, "> {0}", message);
};

caplin.streamlink.impl.connection.BaseConnection.prototype.setRttpConnectionListener = function(listener) {
	this.listener = listener;
};

caplin.streamlink.impl.connection.BaseConnection.prototype.setSessionId = function(sessionid) { };



//## TestConnection.js ##//
caplin.namespace("caplin.streamlink.impl.connection");

// caplin.forceload(caplin.streamlink.impl.connection.BaseConnection

/**
 * @class  Test connection used for testing
 *  
 *  To make streamlink use a test connection use a url with prefix dummy://
 *  Then get hold of the connection created like this:
 *  
 *  	StreamLinkImpl streamLinkImpl = (StreamLinkImpl)streamLink;
 * 		TestConnection testConnection = streamLinkImpl.getTestConnection();
 * 
 *  you can then add a listener to the testConnection to get callbacks for connect, send, disconnect if needed
 *  and test for messages sent using was??? methods 
 *  and receive simulated liberator messages using receive()
 *  
 *  e.g.
 *  	testConnection.receive("1b LOGIN+OK HttpRequestLineLength=960,HttpBodyLength=65536,MergedCommands=1");
 * 		System.out.println(testConnection); // prints all messages sent
 * 		
 * 		Assert.assertTrue(testConnection.wasLastSent("sessionId REQUEST /"));
 * 		testConnection.receive("380001 /");
 * 
 *  
 * @private
 * 
 * 
 * @name caplin.streamlink.impl.connection.TestConnection
 */
caplin.streamlink.impl.connection.TestConnection = function(context, url, ssl) {
	// Super
	caplin.streamlink.impl.connection.BaseConnection.call(this, context, url, ssl);
	// Constructor body
	this.context.logger.log(caplin.streamlink.LogLevel.INFO, "TestConnection: created: {0}:{1} - SSL: {2}", [this.host, this.port, ssl]);
	this.sentMessages = [];
};

caplin.extend(caplin.streamlink.impl.connection.TestConnection, caplin.streamlink.impl.connection.BaseConnection);

caplin.streamlink.impl.connection.TestConnection.prototype.wasLastSent = function(message) {
	if (this.sentMessages.length == 0) {
		return false;
	}
	return (this.sentMessages[this.sentMessages.length - 1] == (message));
};

caplin.streamlink.impl.connection.TestConnection.prototype.wasSent = function(message) {
	for (var i = 0; i < this.sentMessages.length; (i)++) {
		if ((this.sentMessages[i] == (message))) {
			return true;
		}
	}
	return false;
};

caplin.streamlink.impl.connection.TestConnection.prototype.getListener = function() {
	return this.listener;
};

caplin.streamlink.impl.connection.TestConnection.prototype.getSentMessages = function() {
	return this.sentMessages;
};

caplin.streamlink.impl.connection.TestConnection.prototype.getDefaultPort = function() {
	return 0;
};

caplin.streamlink.impl.connection.TestConnection.prototype.toString = function() {
	var builder = [] ;
	builder.push("TestConnection [sentMessages=");
	builder.push(this.sentMessages);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.connection.TestConnection.prototype.clearSentMessages = function() {
	(this.sentMessages.length = 0);
};

caplin.streamlink.impl.connection.TestConnection.prototype.connect = function() {
	this.context.logger.log(caplin.streamlink.LogLevel.INFO, "TestConnection: connect", null);
	this.connectionState = caplin.streamlink.impl.connection.RttpConnectionState.CONNECTING;
	this.fireStateChange();
	this.connectionState = caplin.streamlink.impl.connection.RttpConnectionState.CONNECTED;
	this.fireStateChange();
	this.readyToSend = true;
	this.fireReadyToSend();
	if (this.listener != null) {
		this.listener.onConnect(this.context);
	}
	else {
		this.receive("01 sessionId host=testhost version=2.1 server=rttpd/5.0.7 time=1335507948 timezone=0000");
	}
};

caplin.streamlink.impl.connection.TestConnection.prototype.disconnect = function() {
	this.context.logger.log(caplin.streamlink.LogLevel.INFO, "TestConnection: disconnect");
	this.connectionState = caplin.streamlink.impl.connection.RttpConnectionState.DISCONNECTED;
	this.fireStateChange();
	if (this.listener != null) {
		this.listener.onDisconnect(this.context);
	}
};

caplin.streamlink.impl.connection.TestConnection.prototype.receive = function(message) {
	this.logDataIn(message);
	this.fireData(message + "\n");
};

caplin.streamlink.impl.connection.TestConnection.prototype.send = function(message) {
	this.logDataOut(message);
	this.sentMessages.push(message);
	this.fireReadyToSend();
	if (this.listener != null) {
		this.listener.onSend(this.context, message);
	}
};

caplin.streamlink.impl.connection.TestConnection.prototype.setListener = function(listener) {
	this.listener = listener;
};



//## ConnectionProvider.js ##//
caplin.namespace("caplin.streamlink.impl.connection");


caplin.streamlink.impl.connection.ConnectionProvider = function() {};

caplin.streamlink.impl.connection.ConnectionProvider.prototype.getNextConnection = function(previouslyConnected) { throw new Error('caplin.streamlink.impl.connection.ConnectionProvider.getNextConnection :  This is an abstract method'); };

caplin.streamlink.impl.connection.ConnectionProvider.prototype.setConnectionFactory = function(connectionFactory) { throw new Error('caplin.streamlink.impl.connection.ConnectionProvider.setConnectionFactory :  This is an abstract method'); };



//## ConnectionProviderImpl.js ##//
caplin.namespace("caplin.streamlink.impl.connection");

// caplin.forceload(caplin.streamlink.impl.connection.ConnectionProvider

caplin.streamlink.impl.connection.ConnectionProviderImpl = function(logger, urls) {
	// Field Definitions
	this.retryGroup = -(1);
	this.retryIndex = -(1);

	// Constructor body
	this.logger = logger;
	if ((urls.length==0)) {
		throw new Error("You must provide liberator urls.");
	}
	for (var i = 0; i < urls.length; ++(i)) {
		if ((urls[i].length==0)) {
			throw new Error("You may not configure an empty group of liberator urls.");
		}
	}
	this.urls = urls;
	this.currentIndex = -(1);
	this.randomiseGroupIndex();
};

// caplin.implement(caplin.streamlink.impl.connection.ConnectionProviderImpl, caplin.streamlink.impl.connection.ConnectionProvider);

caplin.streamlink.impl.connection.ConnectionProviderImpl.prototype.getNextConnection = function(previouslyConnected) {
	if (previouslyConnected) {
		this.retryGroup = this.currentGroup;
		this.retryIndex = this.currentIndex;
	}
	else {
		this.increment();
		if (this.retryGroup == this.currentGroup && this.retryIndex == this.currentIndex) {
			this.retryGroup = -(1);
			this.retryIndex = -(1);
			return null;
		}
		if (this.retryGroup == -(1)) {
			this.retryGroup = this.currentGroup;
			this.retryIndex = this.currentIndex;
		}
	}
	var nextURL = this.urls[this.currentGroup][this.currentIndex];
	this.logger.log(caplin.streamlink.LogLevel.INFO, "Trying next connection: {0}", nextURL);
	return this.connectionFactory.getConnection(nextURL);
};

caplin.streamlink.impl.connection.ConnectionProviderImpl.prototype.expandUrls = function(urls) {
	for (var i = urls.length - 1; i >= 0; --(i)) {
		var group = urls[i];
		for (var j = group.length - 1; j >= 0; --(j)) {
			var url = group[j];
			if (caplin.streamlink.impl.utils.Utils.startsWith(url, "rttp:")) {
				var withoutProtocal = url.substring("rttp:".length);
				group.splice(j, 1, "ws:" + withoutProtocal, "http:" + withoutProtocal, "poll:" + withoutProtocal);
			}
			else if (caplin.streamlink.impl.utils.Utils.startsWith(url, "rttps:")) {
				var withoutProtocal = url.substring("rttps:".length);
				group.splice(j, 1, "wss:" + withoutProtocal, "https:" + withoutProtocal, "polls:" + withoutProtocal);
			}
		}
	}
};

caplin.streamlink.impl.connection.ConnectionProviderImpl.prototype.filterGroupsAndUrls = function(connectionFactory, urls) {
	for (var i = urls.length - 1; i >= 0; --(i)) {
		var group = urls[i];
		for (var j = group.length - 1; j >= 0; --(j)) {
			var url = group[j];
			if (connectionFactory.isSupported(url) == false) {
				group.splice(j,1)[0];
			}
		}
		if ((group.length==0)) {
			urls.splice(i,1)[0];
		}
	}
};

caplin.streamlink.impl.connection.ConnectionProviderImpl.prototype.increment = function() {
	if (++(this.currentIndex) >= this.urls[this.currentGroup].length) {
		this.currentIndex = 0;
		this.currentGroup = (this.currentGroup + 1) % this.urls.length;
	}
};

caplin.streamlink.impl.connection.ConnectionProviderImpl.prototype.randomiseGroupIndex = function() {
	this.currentGroup = Math.floor(Math.random() * this.urls.length);
};

caplin.streamlink.impl.connection.ConnectionProviderImpl.prototype.setConnectionFactory = function(newConnectionFactory) {
	this.connectionFactory = newConnectionFactory;
	this.expandUrls(this.urls);
	this.filterGroupsAndUrls(this.connectionFactory, this.urls);
	if ((this.urls.length==0)) {
		throw new Error("No supported connections configured.");
	}
};

caplin.streamlink.impl.connection.ConnectionProviderImpl.prototype.setGroupIndex = function(currentGroup) {
	this.currentGroup = currentGroup;
};



//## WebSocketConnection.js ##//
caplin.namespace("caplin.streamlink.js.impl.connection");

caplin.include("caplin.streamlink.impl.connection.BaseConnection", true);

caplin.streamlink.js.impl.connection.WebSocketConnection = function(state, url, ssl)
{
	caplin.streamlink.impl.connection.BaseConnection.call(this, state, url, ssl);

	this.logger.log(caplin.streamlink.LogLevel.INFO, "Using Connection Type: {0}", "WebSocketConnection");
};

caplin.extend(caplin.streamlink.js.impl.connection.WebSocketConnection, caplin.streamlink.impl.connection.BaseConnection);

caplin.streamlink.js.impl.connection.WebSocketConnection.prototype.connect = function()
{
	var global = new Function("return this;")();
	var WebSocketConstructor = global.WebSocket || global.MozWebSocket;
	this.websocket = new WebSocketConstructor(this.url + "/RTTP-TYPE8");

	var self = this;

	this.websocket.onopen = function()
	{
		self.changeState(caplin.streamlink.impl.connection.RttpConnectionState.CONNECTED);
	};

	this.websocket.onclose = function()
	{
		self.changeState(caplin.streamlink.impl.connection.RttpConnectionState.DISCONNECTED);
	};

	this.websocket.onmessage = function(evt)
	{
		self.logDataIn(evt.data);
		self.fireData(evt.data);
	};

	this.websocket.onerror = function(evt)
	{
		self.logger.log(caplin.streamlink.LogLevel.ERROR, "WebSocketConnection: {0}", evt.data);
	};

	this.changeState(caplin.streamlink.impl.connection.RttpConnectionState.CONNECTING);
};

caplin.streamlink.js.impl.connection.WebSocketConnection.prototype.disconnect = function()
{
	this.websocket.close();
	this.websocket = null;
	this.changeState(caplin.streamlink.impl.connection.RttpConnectionState.DISCONNECTED);
};

caplin.streamlink.js.impl.connection.WebSocketConnection.prototype.send = function(message)
{
	this.logDataOut(message);
	this.websocket.send(message);
};

caplin.streamlink.js.impl.connection.WebSocketConnection.prototype.isReadyToSend = function()
{
	return Boolean(this.websocket);
};

caplin.streamlink.js.impl.connection.WebSocketConnection.prototype.getDefaultPort = function()
{
	return 80;
};


//## Capabilities.js ##//
caplin.namespace("caplin.streamlink.impl.protocolhandler");


caplin.streamlink.impl.protocolhandler.Capabilities = function() {
	// Constructor body
	this.reset();
};

caplin.streamlink.impl.protocolhandler.Capabilities.prototype.getCapability = function(name) {
	return this.capabilitiesMap[name];
};

caplin.streamlink.impl.protocolhandler.Capabilities.prototype.toString = function() {
	return "Capabilities [capabilitiesMap=" + this.capabilitiesMap + "]";
};

caplin.streamlink.impl.protocolhandler.Capabilities.prototype.addCapability = function(name, value) {
	this.capabilitiesMap[name] = value;
};

caplin.streamlink.impl.protocolhandler.Capabilities.prototype.load = function(runtime, tokens) {
	this.reset();
	if (tokens.length >= 3) {
		var pairs = tokens[2].split(",");
		for (var i = 0; i < pairs.length; (i)++) {
			var pair = pairs[i];
			var nameValue = pair.split("=");
			var name = nameValue[0];
			var value = nameValue[1];
			if ((name == ("HttpRequestLineLength"))) {
				this.addCapability(name, runtime.parseInt(value));
			}
			else if ((name == ("HttpBodyLength"))) {
				this.addCapability(name, runtime.parseInt(value));
			}
			else if ((name == ("MergedCommands"))) {
				this.addCapability(name, ((value == ("1"))) ? true : false);
			}
			else {
				this.addCapability(name, value);
			}
		}
	}
};

caplin.streamlink.impl.protocolhandler.Capabilities.prototype.reset = function() {
	this.capabilitiesMap = {} ;
	this.addCapability("HttpBodyLength", 65536);
	this.addCapability("HttpRequestLineLength", 128);
	this.addCapability("MergedCommands", false);
};



//## TestConnectionListener.js ##//
caplin.namespace("caplin.streamlink.impl.connection");


caplin.streamlink.impl.connection.TestConnectionListener = function() {};

caplin.streamlink.impl.connection.TestConnectionListener.prototype.onConnect = function(context) { throw new Error('caplin.streamlink.impl.connection.TestConnectionListener.onConnect :  This is an abstract method'); };

caplin.streamlink.impl.connection.TestConnectionListener.prototype.onDisconnect = function(context) { throw new Error('caplin.streamlink.impl.connection.TestConnectionListener.onDisconnect :  This is an abstract method'); };

caplin.streamlink.impl.connection.TestConnectionListener.prototype.onSend = function(context, message) { throw new Error('caplin.streamlink.impl.connection.TestConnectionListener.onSend :  This is an abstract method'); };



//## KeymasterResponseListener.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  <p>Defines the interface that should be implemented by the application to receive KeyMaster keep alive Responses.</p>
 *  
 *  <p>The KeymasterResponseListener needs to be added to the {@link caplin.streamlink.StreamLinkFactory} create method in order to receive callbacks.</p>
 *  
 *  <p>Typically an application would notify the user when <code>onKeymasterError</code> is called and ask them to sign in again.</p>
 *  
 *  <p>A trivial implementation would be as follows :</p>
 *  <p><code><pre>caplin.streamlink.StreamLinkFactory.create({
 *         liberator_urls: "rttp://liberator1:8080",
 *         keymaster_url: "keymaster",
 *         keymaster_keep_alive_url: "keymasterpoll",
 *         keymaster_keep_alive_listener: {
 *                 onKeymasterError: function(reason)
 *                 {
 *                         console.log("KeyMaster Failed due to : " + reason);
 *                 },
 *                 onKeymasterResponse : function(response)
 *                 {
 *                         console.log("KeyMaster Success. Response Message : " + response);
 *                 }
 *         });
 * </pre></code></p>
 * 
 * @name caplin.streamlink.KeymasterResponseListener
 */
caplin.streamlink.KeymasterResponseListener = function() {};

/**
 *  
 *  <p>Returns an unsuccessful KeyMaster keep alive Response.</p>
 *  
 * @param {String} reason The error text.
 * @function
 */
caplin.streamlink.KeymasterResponseListener.prototype.onKeymasterError = function(reason) { throw new Error('caplin.streamlink.KeymasterResponseListener.onKeymasterError :  This is an abstract method'); };

/**
 *  
 *  <p>Returns a successful KeyMaster keep alive Response. Implementing this method is optional.</p>
 *  
 * @param {String} response The response text.
 * @function
 */
caplin.streamlink.KeymasterResponseListener.prototype.onKeymasterResponse = function(response) { throw new Error('caplin.streamlink.KeymasterResponseListener.onKeymasterResponse :  This is an abstract method'); };



//## DirectoryEvent.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  <p>Provides information about a change to a directory.</p>
 *  
 * 
 * @name caplin.streamlink.DirectoryEvent
 */
caplin.streamlink.DirectoryEvent = function() {};

/**
 *  <p>Returns the subject of the requested directory.</p>
 *  
 *  @returns {String} The subject.
 * @function
 */
caplin.streamlink.DirectoryEvent.prototype.getSubject = function() { throw new Error('caplin.streamlink.DirectoryEvent.getSubject :  This is an abstract method'); };

/**
 *  <p>Returns a list of changes to the Directory. Each {@link caplin.streamlink.DirectoryChangeElement} in the list
 *  represents an element which has been added or deleted from the directory.</p>
 *  
 *  @returns {List} An Array of element changes.
 * @function
 */
caplin.streamlink.DirectoryEvent.prototype.getChangedElements = function() { throw new Error('caplin.streamlink.DirectoryEvent.getChangedElements :  This is an abstract method'); };



//## ContainerEvent.js ##//
caplin.namespace("caplin.streamlink");

// caplin.forceload(caplin.streamlink.DataEvent

/**
 * @class  <p>Represents an update to a container subject.</p>
 *  <p>The <code>ContainerEvent</code> will be provided on a <code>onContainerUpdate</code> callback to a {@link caplin.streamlink.SubscriptionListener}.</p>
 * 
 *  @extends caplin.streamlink.DataEvent
 * 
 * @name caplin.streamlink.ContainerEvent
 */
caplin.streamlink.ContainerEvent = function() {};

// caplin.implement(caplin.streamlink.ContainerEvent, caplin.streamlink.DataEvent);

/**
 *  <p>Gets the size of the container.</p>
 *  
 *  <p>The value returned here is the true size of the container, which may be larger than the size of the container
 *  window that you have subscribed to.</p>
 *  
 *  <p>The value returned here is useful for sizing the scroll bars of a data view showing the elements
 *  within a container.</p>
 *  
 *  @returns {int} The size of the container.
 *  
 *  
 * @function
 */
caplin.streamlink.ContainerEvent.prototype.getSize = function() { throw new Error('caplin.streamlink.ContainerEvent.getSize :  This is an abstract method'); };

/**
 *  <p>Gets the index of the end of the container window.</p>
 *  @returns {int} The end of the window, or -1 if no window has been specified.
 * @function
 */
caplin.streamlink.ContainerEvent.prototype.getWindowEnd = function() { throw new Error('caplin.streamlink.ContainerEvent.getWindowEnd :  This is an abstract method'); };

/**
 *  <p>Gets the index of the start of the window.</p>
 *  
 *  @returns {int} The start of the window, or -1 if no window has been specified.
 * @function
 */
caplin.streamlink.ContainerEvent.prototype.getWindowStart = function() { throw new Error('caplin.streamlink.ContainerEvent.getWindowStart :  This is an abstract method'); };

/**
 *  <p>Update your implementation of {@link caplin.streamlink.ContainerModel} with the contents of this ContainerEvent.
 *  
 * @param {caplin.streamlink.ContainerModel} containerModel Your implementation of {@link caplin.streamlink.ContainerModel} that should
 *  be updated with the contents of this ContainerEvent.
 * @function
 */
caplin.streamlink.ContainerEvent.prototype.updateModel = function(containerModel) { throw new Error('caplin.streamlink.ContainerEvent.updateModel :  This is an abstract method'); };



//## RttpUtil.js ##//
caplin.namespace("caplin.streamlink.impl.utils");


caplin.streamlink.impl.utils.RttpUtil = function() {};

;

caplin.streamlink.impl.utils.RttpUtil.getRttpCode = function(code) {
	if (caplin.streamlink.impl.utils.RttpUtil.map == null) {
		caplin.streamlink.impl.utils.RttpUtil.map = {} ;
		var codes = caplin.streamlink.impl.protocolhandler.RttpCode.values();
		for (var i = 0; i < codes.length; (i)++) {
			caplin.streamlink.impl.utils.RttpUtil.map[codes[i].getName()] = codes[i];
		}
	}
	return caplin.streamlink.impl.utils.RttpUtil.map[code];
};



//## Subscription.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  <p>Represents a subscription to a subject on the Liberator.</p>
 * 
 * @name caplin.streamlink.Subscription
 */
caplin.streamlink.Subscription = function() {};

/**
 *  <p>Returns the {@link caplin.streamlink.SubscriptionListener} that is informed of all
 *  subscription events relating to this subscription.</p>
 *  @returns {caplin.streamlink.SubscriptionListener} The subscription listener.
 * @function
 */
caplin.streamlink.Subscription.prototype.getSubscriptionListener = function() { throw new Error('caplin.streamlink.Subscription.getSubscriptionListener :  This is an abstract method'); };

/**
 *  <p>Returns the subject name to which the subscription relates (for example, /FX/EURUSD).</p>
 *  @returns {String} The subject of this subscription.
 * @function
 */
caplin.streamlink.Subscription.prototype.getSubject = function() { throw new Error('caplin.streamlink.Subscription.getSubject :  This is an abstract method'); };

/**
 *  <p>Sets the container window for this subscription.</p>
 *  
 *  <p>If the subject type of this subscription is not a container then this method will throw
 *  an exception.</p>
 *  
 *  <p>Note: A container element is referred to by its index within the container. 
 *  The first element of a container has index 0 (zero).</p>
 *  
 * @param {int} start The index of the container element that is at the start of the container window.
 * @param {int} size The number of elements in the container window.
 * @function
 */
caplin.streamlink.Subscription.prototype.setWindow = function(start, size) { throw new Error('caplin.streamlink.Subscription.setWindow :  This is an abstract method'); };

/**
 *  <p>Unsubscribes to the subject on the Liberator.</p>
 * @function
 */
caplin.streamlink.Subscription.prototype.unsubscribe = function() { throw new Error('caplin.streamlink.Subscription.unsubscribe :  This is an abstract method'); };



//## SubscriptionImpl.js ##//
caplin.namespace("caplin.streamlink.impl.subscription");

// caplin.forceload(caplin.streamlink.impl.subscription.EventHandler
// caplin.forceload(caplin.streamlink.Subscription

caplin.streamlink.impl.subscription.SubscriptionImpl = function(subscriptionManager, context, subject, listener, parameters) {
	// Field Definitions
	this._subscribed = true;

	// Constructor body
	this._subscriptionManager = subscriptionManager;
	this._context = context;
	this._subject = subject;
	this._parameters = parameters;
	this._listener = listener;
};

// caplin.implement(caplin.streamlink.impl.subscription.SubscriptionImpl, caplin.streamlink.Subscription);
// caplin.implement(caplin.streamlink.impl.subscription.SubscriptionImpl, caplin.streamlink.impl.subscription.EventHandler);

caplin.streamlink.impl.subscription.SubscriptionImpl.prototype.getSubscriptionListener = function() {
	return this._listener;
};

caplin.streamlink.impl.subscription.SubscriptionImpl.prototype._getParameters = function() {
	return this._parameters;
};

caplin.streamlink.impl.subscription.SubscriptionImpl.prototype._getRttpObjectNumber = function() {
	return this._rttpObjectNumber;
};

caplin.streamlink.impl.subscription.SubscriptionImpl.prototype.getSubject = function() {
	return this._subject;
};

caplin.streamlink.impl.subscription.SubscriptionImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("SubscriptionImpl [subject=");
	builder.push(this._subject);
	builder.push(", listener=");
	builder.push(this._listener);
	builder.push(", parameters=");
	builder.push(this._parameters);
	builder.push(", rttpObjectNumber=");
	builder.push(this._rttpObjectNumber);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.subscription.SubscriptionImpl.prototype._clear = function() {
	this._rttpObjectNumber = null;
	this._containerModel = null;
};

caplin.streamlink.impl.subscription.SubscriptionImpl.prototype._handleEvent = function(event) {
	if (!(this._subscribed)) {
		return;
	}
	if (this._containerModel != null) {
		if (!((event instanceof caplin.streamlink.impl.event.ContainerEventInternal)) && this._parameters != null && (this._parameters instanceof caplin.streamlink.impl.subscription.ContainerSubscriptionParametersImpl)) {
			var containerParameters = this._parameters;
			if (!(containerParameters._inWindow(this._containerModel.findSubject(event.getSubject())))) {
				return;
			}
		}
	}
	var self = this;
	this._context.runtime.createTimeout(function() {
		event._publishSubscriptionResponse(self, self._listener);
	}, 0);
	if ((event instanceof caplin.streamlink.impl.event.ContainerEventInternal)) {
		if (this._containerModel == null) {
			this._containerModel = new caplin.streamlink.impl.subscription.container.ContainerModelInternal();
		}
		var containerEventInternal = event;
		this._updateAutoSubscriptions(containerEventInternal);
		var actions = this._containerModel.update(containerEventInternal);
		var containerEvent = new caplin.streamlink.impl.event.ContainerEventImpl(this._subject, this._containerModel.getSize(), this._containerModel.getWindowStart(), this._containerModel.getWindowEnd(), actions);
		this._context.logger.log(caplin.streamlink.LogLevel.FINE, "{0}", containerEvent);
		this._context.runtime.createTimeout(function() {
			(self._listener.onContainerUpdate && self._listener.onContainerUpdate(self, containerEvent));
		}, 0);
	}
};

caplin.streamlink.impl.subscription.SubscriptionImpl.prototype._removeAllAutoSubscriptions = function() {
	var elements = this._containerModel.getElements();
	for (var i = 0; i < elements.length; (i)++) {
		var element = elements[i];
		this._subscriptionManager.removeAutoSubscription(this, element._getObjectNumber());
	}
};

caplin.streamlink.impl.subscription.SubscriptionImpl.prototype._setRttpObjectNumber = function(rttpObjectNumber) {
	this._rttpObjectNumber = rttpObjectNumber;
};

caplin.streamlink.impl.subscription.SubscriptionImpl.prototype._updateAutoSubscriptions = function(containerEventInternal) {
	if (containerEventInternal.isImage()) {
		this._removeAllAutoSubscriptions();
	}
	var items = containerEventInternal.getItems();
	for (var i = 0; i < items.length; (i)++) {
		var item = items[i];
		if (item.isAdded()) {
			this._subscriptionManager.addAutoSubscription(this, item.getObjectNumber(), item.getSubject());
		}
		else {
			this._subscriptionManager.removeAutoSubscription(this, item.getObjectNumber());
		}
	}
};

caplin.streamlink.impl.subscription.SubscriptionImpl.prototype.setWindow = function(start, size) {
	if (!((this._parameters instanceof caplin.streamlink.impl.subscription.ContainerSubscriptionParametersImpl))) {
		throw new Error("Cannot change window when ContainerSubscriptionParameters were not used in the orginal subscription");
	}
	var containerParameters = this._parameters;
	var newContainerParameters = new caplin.streamlink.impl.subscription.ContainerSubscriptionParametersImpl(containerParameters._getCtrid());
	newContainerParameters._copy(containerParameters);
	newContainerParameters.setWindow(start, size);
	this._subscriptionManager.send(this, new caplin.streamlink.impl.packet.PacketRequest(this._subject, newContainerParameters));
	this._parameters = newContainerParameters;
};

caplin.streamlink.impl.subscription.SubscriptionImpl.prototype.unsubscribe = function() {
	if (this._subscribed) {
		this._subscriptionManager.send(this, new caplin.streamlink.impl.packet.PacketDiscard(this._subject));
		if (this._containerModel != null) {
			this._removeAllAutoSubscriptions();
		}
	}
	this._subscribed = false;
};



//## SendQueue.js ##//
caplin.namespace("caplin.streamlink.impl.protocolhandler");


caplin.streamlink.impl.protocolhandler.SendQueue = function() {
	// Field Definitions
	this.sendQueue = [];
	this.systemSendQueue = [];
};

caplin.streamlink.impl.protocolhandler.SendQueue.prototype.isEmpty = function() {
	return (this.sendQueue.length==0) && (this.systemSendQueue.length==0);
};

caplin.streamlink.impl.protocolhandler.SendQueue.prototype.peekNextMessage = function() {
	if (!((this.systemSendQueue.length==0))) {
		return this.systemSendQueue[0];
	}
	if (!((this.sendQueue.length==0))) {
		return this.sendQueue[0];
	}
	return null;
};

caplin.streamlink.impl.protocolhandler.SendQueue.prototype.popNextMessage = function() {
	if (!((this.systemSendQueue.length==0))) {
		return this.systemSendQueue.shift();
	}
	if (!((this.sendQueue.length==0))) {
		return this.sendQueue.shift();
	}
	return null;
};

caplin.streamlink.impl.protocolhandler.SendQueue.prototype.clearSystemMessages = function() {
	(this.systemSendQueue.length = 0);
};

caplin.streamlink.impl.protocolhandler.SendQueue.prototype.pushNextMessage = function(packet) {
	if (packet.getType().isSystem()) {
		this.systemSendQueue.push(packet);
	}
	else {
		this.sendQueue.push(packet);
	}
};



//## Timer.js ##//
caplin.namespace("caplin.streamlink.js.impl");

caplin.streamlink.js.impl.TimerImpl = function(id, isInterval) 
{
	this.id = id;
	this.isInterval = isInterval;
};

caplin.streamlink.js.impl.TimerImpl.prototype.cancel = function()
{
	if (this.id != -1)
	{
		if (this.isInterval)
		{
			clearInterval(this.id);
		}
		else
		{
			clearTimeout(this.id);
		}
		this.id = -1;
	}
};


//## PacketLogin.js ##//
caplin.namespace("caplin.streamlink.impl.packet");

// caplin.forceload(caplin.streamlink.impl.packet.Packet

caplin.streamlink.impl.packet.PacketLogin = function(reconnectSession, tech_appId, user_appId, username, password, latencyInterval) {
	// Constructor body
	this.reconnectSession = reconnectSession;
	this.username = username;
	this.password = password;
	this.tech_appId = tech_appId;
	this.user_appId = user_appId;
	this.latencyInterval = latencyInterval;
};

// caplin.implement(caplin.streamlink.impl.packet.PacketLogin, caplin.streamlink.impl.packet.Packet);

caplin.streamlink.impl.packet.PacketLogin.prototype.getType = function() {
	return caplin.streamlink.impl.packet.PacketType.LOGIN;
};

caplin.streamlink.impl.packet.PacketLogin.prototype.getSubject = function() {
	return null;
};

caplin.streamlink.impl.packet.PacketLogin.prototype.toRttpString = function(encoder) {
	var machineId = "";
	var result = [] ;
	result.push(caplin.streamlink.impl.packet.PacketType.LOGIN.getCode());
	result.push(" ");
	result.push(this.reconnectSession);
	result.push(" ");
	result.push(encoder.rttpEncode(this.tech_appId));
	if (this.user_appId != null) {
		result.push(":");
		result.push(encoder.rttpEncode(this.user_appId));
	}
	result.push("/");
	result.push(machineId);
	result.push(" RTTP/2.1 ");
	result.push(encoder.rttpEncode(this.username));
	result.push(" ");
	result.push(encoder.rttpEncode(this.password));
	result.push(" HttpRequestLineLength=,HttpBodyLength=,MergedCommands=");
	result.push(",HeartbeatInterval=" + this.latencyInterval);
	return result.join("");
};

caplin.streamlink.impl.packet.PacketLogin.prototype.toString = function() {
	var builder = [] ;
	builder.push("Login [username=");
	builder.push(this.username);
	builder.push(", password=");
	builder.push(this.password);
	builder.push(", reconnectSession=");
	builder.push(this.reconnectSession);
	builder.push("]");
	return builder.join("");
};



//## NewsEvent.js ##//
caplin.namespace("caplin.streamlink");

// caplin.forceload(caplin.streamlink.DataEvent

/**
 * @class  <p>Provides information about a news headline update.</p>
 *  
 *  @extends caplin.streamlink.DataEvent
 * 
 * @name caplin.streamlink.NewsEvent
 */
caplin.streamlink.NewsEvent = function() {};

// caplin.implement(caplin.streamlink.NewsEvent, caplin.streamlink.DataEvent);

/**
 *  <p>Returns the date/time associated with this headline update.</p>
 *  
 *  <p>The format of the date/time string is dependent on the implementation
 *  of the DataSoure. Generally it will be in a displayable format.</p>
 *  
 *  @returns {String} The date/time stamp.
 * @function
 */
caplin.streamlink.NewsEvent.prototype.getDateTime = function() { throw new Error('caplin.streamlink.NewsEvent.getDateTime :  This is an abstract method'); };

/**
 *  <p>Returns the headline for this update.</p>
 *  @returns {String} The headline.
 * @function
 */
caplin.streamlink.NewsEvent.prototype.getHeadline = function() { throw new Error('caplin.streamlink.NewsEvent.getHeadline :  This is an abstract method'); };

/**
 *  <p>Returns the story code associated with this update.</p>
 *  
 *  <p>The value of the code can be used to construct a news story
 *  subject to obtain the full text of the news item.</p>
 *  
 *  @returns {String} The story code.
 * @function
 */
caplin.streamlink.NewsEvent.prototype.getStoryCode = function() { throw new Error('caplin.streamlink.NewsEvent.getStoryCode :  This is an abstract method'); };

/**
 *  <p>Returns all field/value pairs that were updated in this update.</p>
 *  
 *  <p>This data structure allows access to any additional information associated
 *  with the news headline update which aren't available directly.</p>
 *  
 *  @returns {Object} A collection of field/value pairs.
 * @function
 */
caplin.streamlink.NewsEvent.prototype.getFields = function() { throw new Error('caplin.streamlink.NewsEvent.getFields :  This is an abstract method'); };



//## NewsEventImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.NewsEvent
// caplin.forceload(caplin.streamlink.impl.Serializable
// caplin.forceload(caplin.streamlink.impl.event.RttpEventBase
// caplin.forceload(caplin.streamlink.DataEvent

caplin.streamlink.impl.event.NewsEventImpl = function(subject, image, fields, rttpCode, objectNumber, sequenceNumber) {
	// Super
	caplin.streamlink.impl.event.RttpEventBase.call(this, subject, rttpCode, objectNumber, sequenceNumber);
	// Constructor body
	this._image = image;
	this._fields = fields;
};

caplin.extend(caplin.streamlink.impl.event.NewsEventImpl, caplin.streamlink.impl.event.RttpEventBase);

// caplin.implement(caplin.streamlink.impl.event.NewsEventImpl, caplin.streamlink.DataEvent);
// caplin.implement(caplin.streamlink.impl.event.NewsEventImpl, caplin.streamlink.NewsEvent);
// caplin.implement(caplin.streamlink.impl.event.NewsEventImpl, caplin.streamlink.impl.Serializable);

caplin.streamlink.impl.event.NewsEventImpl.prototype.isImage = function() {
	return this._image;
};

caplin.streamlink.impl.event.NewsEventImpl.createFromDictionary = function(root) {
	return new caplin.streamlink.impl.event.NewsEventImpl(root[caplin.streamlink.impl.event.RttpEventBase.SUBJECT], root["Image"], root["Fields"], caplin.streamlink.impl.utils.RttpUtil.getRttpCode(root[caplin.streamlink.impl.event.RttpEventBase.CODE]), root[caplin.streamlink.impl.event.RttpEventBase.OBJECT_NUMBER], root[caplin.streamlink.impl.event.RttpEventBase.SEQUENCE_NUMBER]);
};

caplin.streamlink.impl.event.NewsEventImpl.createFromRttp = function(rttpCode, tokens, fieldMap, encoder) {
	switch (rttpCode) {
	case caplin.streamlink.impl.protocolhandler.RttpCode.NEWS_RESP:
	return new caplin.streamlink.impl.event.NewsEventImpl(encoder.rttpDecode(tokens[1]), true, caplin.streamlink.impl.event.ParseUtils.getFields(tokens, fieldMap, encoder, 2), rttpCode, tokens[0].substring(2, 6), null);
	case caplin.streamlink.impl.protocolhandler.RttpCode.NEWS_UPD:
	case caplin.streamlink.impl.protocolhandler.RttpCode.NEWS_IMG:
	return new caplin.streamlink.impl.event.NewsEventImpl(null, false, caplin.streamlink.impl.event.ParseUtils.getFields(tokens, fieldMap, encoder, 1), rttpCode, tokens[0].substring(4, 8), tokens[0].substring(2, 4));
	case caplin.streamlink.impl.protocolhandler.RttpCode.NEWS_CLR:
	return new caplin.streamlink.impl.event.NewsEventImpl(null, true, caplin.streamlink.impl.event.ParseUtils.getFields(tokens, fieldMap, encoder, 1), rttpCode, tokens[0].substring(4, 8), tokens[0].substring(2, 4));
	}
	return null;
};

caplin.streamlink.impl.event.NewsEventImpl.prototype._toDictionary = function() {
	var root = caplin.streamlink.impl.event.RttpEventBase.prototype._toDictionary.call(this);
	root["Image"] = this._image;
	root["Fields"] = this._fields;
	return root;
};

caplin.streamlink.impl.event.NewsEventImpl.prototype.getDateTime = function() {
	return this._fields["date"];
};

caplin.streamlink.impl.event.NewsEventImpl.prototype.getHeadline = function() {
	return this._fields["headline"];
};

caplin.streamlink.impl.event.NewsEventImpl.prototype.getStoryCode = function() {
	return this._fields["code"];
};

caplin.streamlink.impl.event.NewsEventImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("NewsDataEventImpl [subject=");
	builder.push(this._subject);
	builder.push(", fields=");
	builder.push(caplin.xlang.Utils.dictionaryToString(this._fields));
	builder.push(", image=");
	builder.push(this._image);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.NewsEventImpl.prototype.getFields = function() {
	return this._fields;
};

caplin.streamlink.impl.event.NewsEventImpl.prototype._publishCommandResponse = function(listener) { };

caplin.streamlink.impl.event.NewsEventImpl.prototype._publishSubscriptionResponse = function(subscription, subscriptionListener) {
	(subscriptionListener.onNewsUpdate && subscriptionListener.onNewsUpdate(subscription, this));
};



//## RecordType2Event.js ##//
caplin.namespace("caplin.streamlink");

// caplin.forceload(caplin.streamlink.DataEvent

/**
 * @class  <p>Provides information about a change to type2 data in a record.</p>
 *  <p>The <code>RecordType2Event</code> will be provided on a <code>onRecordType2Update</code> callback
 *  to a {@link caplin.streamlink.SubscriptionListener}.</p>
 *  
 *  <p>Type 2 data is a 2-dimensional array, where the type 2 index acts as a row key and
 *  the field-value pairs are column keys and values. This type of record is often used
 *  to represent a level 2 order book, for example:</p>
 *  
 *  <table>
 *  <tr><th>Stock (type2Index)</th>        <th>BidSize</th>        <th>Bid</th>    <th>Ask</th>    <th>AskSize</th></tr>
 *   <tr><td>ABN1</td>      <td>100</td>    <td>10.22</td>  <td>10.28</td>  <td>55</td></tr>
 *   <tr><td>ZBN2</td>      <td>200</td>    <td>10.20</td>  <td>20.33</td>  <td>250</td></tr>
 *   <tr><td>ARC</td>       <td>1</td>      <td>10.16</td>  <td>10.41</td>  <td>1</td></tr>
 *  </table>
 *  
 *  <p>When the type 2 data is updated, the new values are sent as a set of {@link caplin.streamlink.RecordType2Event}s,
 *  one event for each row of the record that has changed. Each such event identifies the row that has changed.</p>
 *  
 *  
 *  @extends caplin.streamlink.DataEvent 
 * 
 * @name caplin.streamlink.RecordType2Event
 */
caplin.streamlink.RecordType2Event = function() {};

// caplin.implement(caplin.streamlink.RecordType2Event, caplin.streamlink.DataEvent);

/**
 *  <p>Returns whether all rows should be deleted.</p>
 *  
 *  @returns {boolean} true if all rows should be deleted, otherwise false.
 * @function
 */
caplin.streamlink.RecordType2Event.prototype.deleteAllIndices = function() { throw new Error('caplin.streamlink.RecordType2Event.deleteAllIndices :  This is an abstract method'); };

/**
 *  <p>Returns whether the row should be deleted.</p>
 *  
 *  @returns {boolean} true if the row should be deleted, otherwise false.
 * @function
 */
caplin.streamlink.RecordType2Event.prototype.deleteIndex = function() { throw new Error('caplin.streamlink.RecordType2Event.deleteIndex :  This is an abstract method'); };

/**
 *  <p>Gets the type 2 index field relating to the event.</p>
 *  
 *  <p>A {@link caplin.streamlink.RecordType2Event} contains information about one
 *  row of the type 2 data. The row is identified by the value of the type 2 index fieldname
 *  returned by this method.</p>
 *  
 *  @returns {String} the index field.
 * @function
 */
caplin.streamlink.RecordType2Event.prototype.getType2IndexField = function() { throw new Error('caplin.streamlink.RecordType2Event.getType2IndexField :  This is an abstract method'); };

/**
 *  <p>Returns the record fields relating to the event, as a collection of key-value pairs.</p>
 * 
 *  <p>When a call to isImage() returns true, the subscription data event contains an image of the data
 *  for the subscribed item. All the fields comprising the record are present in the collection. The application
 *  must discard any values previously received and replace them with the values contained within the
 *  collection.</p>
 *  
 *  <p>When a call to isImage() returns false, the subscription data event contains new data for the subscribed
 *  record. The values in the collection returned are an update to any values previously received and any
 *  application caches should be updated appropriately.</p>
 *  
 *  @returns {Object} the fields
 * @function
 */
caplin.streamlink.RecordType2Event.prototype.getFields = function() { throw new Error('caplin.streamlink.RecordType2Event.getFields :  This is an abstract method'); };



//## RecordType2EventImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.impl.Serializable
// caplin.forceload(caplin.streamlink.impl.event.RttpEventBase
// caplin.forceload(caplin.streamlink.DataEvent
// caplin.forceload(caplin.streamlink.RecordType2Event

caplin.streamlink.impl.event.RecordType2EventImpl = function(subject, image, fields, indexFieldName, deleteIndex, deleteAll, rttpCode, objectNumber, sequenceNumber) {
	// Super
	caplin.streamlink.impl.event.RttpEventBase.call(this, subject, rttpCode, objectNumber, sequenceNumber);
	// Constructor body
	this._image = image;
	this._fields = fields;
	this._indexFieldName = indexFieldName;
	this._deleteIndex = deleteIndex;
	this._allIndexDelete = deleteAll;
};

caplin.extend(caplin.streamlink.impl.event.RecordType2EventImpl, caplin.streamlink.impl.event.RttpEventBase);

// caplin.implement(caplin.streamlink.impl.event.RecordType2EventImpl, caplin.streamlink.DataEvent);
// caplin.implement(caplin.streamlink.impl.event.RecordType2EventImpl, caplin.streamlink.RecordType2Event);
// caplin.implement(caplin.streamlink.impl.event.RecordType2EventImpl, caplin.streamlink.impl.Serializable);

caplin.streamlink.impl.event.RecordType2EventImpl.prototype.deleteAllIndices = function() {
	return this._allIndexDelete;
};

caplin.streamlink.impl.event.RecordType2EventImpl.prototype.deleteIndex = function() {
	return this._deleteIndex;
};

caplin.streamlink.impl.event.RecordType2EventImpl.prototype.isImage = function() {
	return this._image;
};

caplin.streamlink.impl.event.RecordType2EventImpl.createFromDictionary = function(root) {
	var subject = root[caplin.streamlink.impl.event.RttpEventBase.SUBJECT];
	var image = root["Image"];
	var fields = root["Fields"];
	var indexField = root["IndexField"];
	var deleteIndex = root["DeleteIndex"];
	var deleteAll = root["DeleteAll"];
	var rttpCode = caplin.streamlink.impl.utils.RttpUtil.getRttpCode(root[caplin.streamlink.impl.event.RttpEventBase.CODE]);
	var objectNumber = root[caplin.streamlink.impl.event.RttpEventBase.OBJECT_NUMBER];
	var sequenceNumber = root[caplin.streamlink.impl.event.RttpEventBase.SEQUENCE_NUMBER];
	return new caplin.streamlink.impl.event.RecordType2EventImpl(subject, image, fields, indexField, deleteIndex, deleteAll, rttpCode, objectNumber, sequenceNumber);
};

caplin.streamlink.impl.event.RecordType2EventImpl.createFromRttp = function(rttpCode, tokens, fieldMap, encoder) {
	var objectNumber = tokens[0].substring(4, 8);
	var sequenceNumber = tokens[0].substring(2, 4);
	switch (rttpCode) {
	case caplin.streamlink.impl.protocolhandler.RttpCode.REC2_UPD:
	case caplin.streamlink.impl.protocolhandler.RttpCode.REC2_IMG:
	{
		var fields = caplin.streamlink.impl.event.ParseUtils.getFields(tokens, fieldMap, encoder, 1);
		var indexField = caplin.streamlink.impl.event.ParseUtils.getNameOfFirstField(tokens, fieldMap);
		return new caplin.streamlink.impl.event.RecordType2EventImpl(null, rttpCode == caplin.streamlink.impl.protocolhandler.RttpCode.REC2_IMG, fields, indexField, false, false, rttpCode, objectNumber, sequenceNumber);
	}
	case caplin.streamlink.impl.protocolhandler.RttpCode.REC2_DEL:
	{
		var fields = caplin.streamlink.impl.event.ParseUtils.getFields(tokens, fieldMap, encoder, 1);
		var indexField = caplin.streamlink.impl.event.ParseUtils.getNameOfFirstField(tokens, fieldMap);
		return new caplin.streamlink.impl.event.RecordType2EventImpl(null, true, fields, indexField, true, false, rttpCode, objectNumber, sequenceNumber);
	}
	case caplin.streamlink.impl.protocolhandler.RttpCode.REC2_CLR:
	return new caplin.streamlink.impl.event.RecordType2EventImpl(null, false, {} , null, false, true, rttpCode, objectNumber, sequenceNumber);
	}
	return null;
};

caplin.streamlink.impl.event.RecordType2EventImpl.prototype._toDictionary = function() {
	var root = caplin.streamlink.impl.event.RttpEventBase.prototype._toDictionary.call(this);
	root["Image"] = this._image;
	root["Fields"] = this._fields;
	root["IndexField"] = this._indexFieldName;
	root["DeleteIndex"] = this._deleteIndex;
	root["DeleteAll"] = this._allIndexDelete;
	return root;
};

caplin.streamlink.impl.event.RecordType2EventImpl.prototype.getType2IndexField = function() {
	return this._indexFieldName;
};

caplin.streamlink.impl.event.RecordType2EventImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("RecordType2DataEvent [subject=");
	builder.push(this._subject);
	builder.push(", fields=[");
	builder.push(caplin.xlang.Utils.dictionaryToString(this._fields));
	builder.push("]");
	builder.push(", indexField=");
	builder.push(this._indexFieldName);
	builder.push(", delete=");
	builder.push(this._deleteIndex);
	builder.push(", deleteall=");
	builder.push(this._allIndexDelete);
	builder.push(", image=");
	builder.push(this._image);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.RecordType2EventImpl.prototype.getFields = function() {
	return this._fields;
};

caplin.streamlink.impl.event.RecordType2EventImpl.prototype._publishCommandResponse = function(listener) { };

caplin.streamlink.impl.event.RecordType2EventImpl.prototype._publishSubscriptionResponse = function(subscription, subscriptionListener) {
	(subscriptionListener.onRecordType2Update && subscriptionListener.onRecordType2Update(subscription, this));
};



//## CommandListenerOnWorker.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.server");

// caplin.forceload(caplin.streamlink.CommandListener
// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver

caplin.streamlink.impl.bridge.worker.server.CommandListenerOnWorker = function(bridge, id) {
	// Constructor body
	this.bridge = bridge;
	this.id = id;
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.server.CommandListenerOnWorker, caplin.streamlink.CommandListener);
// caplin.implement(caplin.streamlink.impl.bridge.worker.server.CommandListenerOnWorker, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.server.CommandListenerOnWorker.prototype.getReceiverKey = function() {
	return null;
};

caplin.streamlink.impl.bridge.worker.server.CommandListenerOnWorker.prototype.getReceiverId = function() {
	return this.id;
};

caplin.streamlink.impl.bridge.worker.server.CommandListenerOnWorker.prototype.onCommandError = function(subject, commandErrorEvent) {
	var args = [subject, commandErrorEvent._toDictionary()];
	this.bridge.send(this.id, "onCommandError", args);
};

caplin.streamlink.impl.bridge.worker.server.CommandListenerOnWorker.prototype.onCommandOk = function(subject) {
	var args = [subject];
	this.bridge.send(this.id, "onCommandOk", args);
};

caplin.streamlink.impl.bridge.worker.server.CommandListenerOnWorker.prototype.receive = function(method, args) { };



//## CredentialsReceiver.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  
 *  
 * @private
 *  
 *  <p>Interface that your custom implementation of {@link caplin.streamlink.CredentialsProvider} must call to
 *  provide credential information to StreamLink.</p>
 *  
 *  <p>Your implementation of {@link caplin.streamlink.CredentialsProvider} will be called when a set
 *  of {@link caplin.streamlink.Credentials} is required. Your implementation will be passed an instance
 *  of this interface which you should use to callback with either a set of credentials or an error message.</p>
 * 
 * @name caplin.streamlink.CredentialsReceiver
 */
caplin.streamlink.CredentialsReceiver = function() {};

/**
 *  <p>Call this method to indicate an error retrieving a set of credentials.</p>
 *  
 * @param {String} reason A textual error message that will be visible within the StreamLink logs.
 * @function
 */
caplin.streamlink.CredentialsReceiver.prototype.credentialsUnavailable = function(reason) { throw new Error('caplin.streamlink.CredentialsReceiver.credentialsUnavailable :  This is an abstract method'); };

/**
 *  <p>Call this method to provide a set of {@link caplin.streamlink.Credentials} to StreamLink.</p>
 *  
 * @param {caplin.streamlink.Credentials} credentials The credentials that should be used to login to the Liberator server.
 * @function
 */
caplin.streamlink.CredentialsReceiver.prototype.provideCredentials = function(credentials) { throw new Error('caplin.streamlink.CredentialsReceiver.provideCredentials :  This is an abstract method'); };



//## RttpConnectionListener.js ##//
caplin.namespace("caplin.streamlink.impl.connection");


/**
 * @class  NOTE: this class is called RttpConnectionListener (rather than ConnectionListener)
 *  because its the low level listener for a Connection rather than the client level
 *  (protocol level) listener, called ConnectionListener
 *  
 *  
 * @private
 * 
 * @name caplin.streamlink.impl.connection.RttpConnectionListener
 */
caplin.streamlink.impl.connection.RttpConnectionListener = function() {};

caplin.streamlink.impl.connection.RttpConnectionListener.prototype.onData = function(data) { throw new Error('caplin.streamlink.impl.connection.RttpConnectionListener.onData :  This is an abstract method'); };

caplin.streamlink.impl.connection.RttpConnectionListener.prototype.onReady = function() { throw new Error('caplin.streamlink.impl.connection.RttpConnectionListener.onReady :  This is an abstract method'); };

caplin.streamlink.impl.connection.RttpConnectionListener.prototype.onRequestReconnect = function() { throw new Error('caplin.streamlink.impl.connection.RttpConnectionListener.onRequestReconnect :  This is an abstract method'); };

caplin.streamlink.impl.connection.RttpConnectionListener.prototype.onStateChange = function(rttpConnectionState) { throw new Error('caplin.streamlink.impl.connection.RttpConnectionListener.onStateChange :  This is an abstract method'); };



//## ProtocolHandler.js ##//
caplin.namespace("caplin.streamlink.impl.protocolhandler");


caplin.streamlink.impl.protocolhandler.ProtocolHandler = function() {};

caplin.streamlink.impl.protocolhandler.ProtocolHandler.prototype.getConnectionState = function() { throw new Error('caplin.streamlink.impl.protocolhandler.ProtocolHandler.getConnectionState :  This is an abstract method'); };

caplin.streamlink.impl.protocolhandler.ProtocolHandler.prototype.addConnectionListener = function(listener) { throw new Error('caplin.streamlink.impl.protocolhandler.ProtocolHandler.addConnectionListener :  This is an abstract method'); };

caplin.streamlink.impl.protocolhandler.ProtocolHandler.prototype.connect = function() { throw new Error('caplin.streamlink.impl.protocolhandler.ProtocolHandler.connect :  This is an abstract method'); };

caplin.streamlink.impl.protocolhandler.ProtocolHandler.prototype.disconnect = function() { throw new Error('caplin.streamlink.impl.protocolhandler.ProtocolHandler.disconnect :  This is an abstract method'); };

caplin.streamlink.impl.protocolhandler.ProtocolHandler.prototype.fireOnStatisticsChange = function(event) { throw new Error('caplin.streamlink.impl.protocolhandler.ProtocolHandler.fireOnStatisticsChange :  This is an abstract method'); };

caplin.streamlink.impl.protocolhandler.ProtocolHandler.prototype.reconnect = function() { throw new Error('caplin.streamlink.impl.protocolhandler.ProtocolHandler.reconnect :  This is an abstract method'); };

caplin.streamlink.impl.protocolhandler.ProtocolHandler.prototype.removeConnectionListener = function(listener) { throw new Error('caplin.streamlink.impl.protocolhandler.ProtocolHandler.removeConnectionListener :  This is an abstract method'); };

caplin.streamlink.impl.protocolhandler.ProtocolHandler.prototype.sendPacket = function(packet) { throw new Error('caplin.streamlink.impl.protocolhandler.ProtocolHandler.sendPacket :  This is an abstract method'); };

caplin.streamlink.impl.protocolhandler.ProtocolHandler.prototype.setConnectionMonitor = function(connectionMonitor) { throw new Error('caplin.streamlink.impl.protocolhandler.ProtocolHandler.setConnectionMonitor :  This is an abstract method'); };

caplin.streamlink.impl.protocolhandler.ProtocolHandler.prototype.setProtocolListener = function(listener) { throw new Error('caplin.streamlink.impl.protocolhandler.ProtocolHandler.setProtocolListener :  This is an abstract method'); };



//## ProtocolHandlerImpl.js ##//
caplin.namespace("caplin.streamlink.impl.protocolhandler");

// caplin.forceload(caplin.streamlink.CredentialsReceiver
// caplin.forceload(caplin.streamlink.impl.connection.RttpConnectionListener
// caplin.forceload(caplin.streamlink.impl.protocolhandler.ProtocolHandler

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl = function(context, connectionProvider, credentialsProvider) {
	// Field Definitions
	this.fieldMap = {} ;
	this.queuedRawData = "";
	this.connectionState = caplin.streamlink.ConnectionState.DISCONNECTED;
	this.userWantsToBeConnected = false;
	this.sendQueue = new caplin.streamlink.impl.protocolhandler.SendQueue();
	this.connectionListeners = [];
	this.lastSequenceNumber = "";
	this.previouslyConnected = false;
	this.eventInProgress = null;

	// Constructor body
	this.context = context;
	this.connectionProvider = connectionProvider;
	this.credentialsProvider = credentialsProvider;
	if ((credentialsProvider instanceof caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl)) {
		this.addConnectionListener(credentialsProvider);
	}
	this.responseQueue = new caplin.streamlink.impl.protocolhandler.ResponseQueue(this, context.runtime, context.logger, context.configuration._getRequestTimeout());
	this.capabilities = new caplin.streamlink.impl.protocolhandler.Capabilities();
};

// caplin.implement(caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl, caplin.streamlink.CredentialsReceiver);
// caplin.implement(caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl, caplin.streamlink.impl.connection.RttpConnectionListener);
// caplin.implement(caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl, caplin.streamlink.impl.protocolhandler.ProtocolHandler);

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.LOG_PREFIX = "ProtocolHandler: ";

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.getConnectionState = function() {
	return this.connectionState;
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.createLoginPacket = function(credentials) {
	var sessionReconnect = this.lastSessionId != null && this.responseQueue.isEmpty();
	var reconnectToSession;
	if (sessionReconnect) {
		reconnectToSession = this.lastSessionId + "/" + this.lastSequenceNumber;
	}
	else {
		reconnectToSession = "0";
		this.fieldMap = {} ;
	}
	var technologyApplicationId = this.context.technologyApplicationId;
	var applicationId = this.context.configuration._getApplicationId();
	var username = credentials.getUsername();
	var password = credentials.getPassword();
	var latencyInterval = this.context.configuration._getLatencyInterval();
	var loginPacket = new caplin.streamlink.impl.packet.PacketLogin(reconnectToSession, technologyApplicationId, applicationId, username, password, latencyInterval);
	this.lastSessionId = null;
	this.lastSequenceNumber = "";
	return loginPacket;
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype._disconnectInternal = function() {
	if (this.connection != null) {
		try {
			if (!(this.userWantsToBeConnected)) {
				this.lastSessionId = null;
				if (this.sessionId != null) {
					this.sendPacket(new caplin.streamlink.impl.packet.PacketLogout());
				}
			}
			this.connection.disconnect();
		}
		catch (errorDisconnecting) {
			this.context.logger.log(caplin.streamlink.LogLevel.WARN, "Error disconnecting: {1}", errorDisconnecting);
			this.onStateChange(caplin.streamlink.impl.connection.RttpConnectionState.DISCONNECTED);
		}
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.addConnectionListener = function(listener) {
	this.connectionListeners.push(listener);
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.cancelReconnectTimer = function() {
	if (this.reconnectTimer != null) {
		this.reconnectTimer.cancel();
		this.reconnectTimer = null;
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.changeConnectionState = function(state, message) {
	this.context.logger.log(caplin.streamlink.LogLevel.INFO, "Connection state changed to: {0} {1}", state, message);
	this.connectionState = state;
	var event = new caplin.streamlink.impl.event.ConnectionStatusEventImpl(state, message);
	for (var i = 0; i < this.connectionListeners.length; (i)++) {
		this.onConnectionStatusChange(this.connectionListeners[i], event);
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.connect = function() {
	if (this.userWantsToBeConnected == false) {
		this.userWantsToBeConnected = true;
		this.nextConnection();
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.credentialsUnavailable = function(message) {
	if (!(this.expectingCredentials)) {
		this.context.logger.log(caplin.streamlink.LogLevel.WARN, "Received credentials unavailable event when not expecting - ignoring: {0}", message);
	}
	else {
		this.expectingCredentials = false;
		this.context.logger.log(caplin.streamlink.LogLevel.WARN, "Unable to get credentials: {0}", message);
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.disconnect = function() {
	this.userWantsToBeConnected = false;
	this._disconnectInternal();
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.fireOnStatisticsChange = function(event) {
	for (var i = 0; i < this.connectionListeners.length; (i)++) {
		var listener = this.connectionListeners[i];
		this.onFireOnStatisticsChange(listener, event);
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.nextConnection = function() {
	var wasConnected = this.previouslyConnected;
	this.previouslyConnected = false;
	this.connection = this.connectionProvider.getNextConnection(wasConnected);
	if (this.connection == null) {
		this.context.logger.log(caplin.streamlink.LogLevel.INFO, "No more connections to try.");
		this.disconnect();
		this.sendConnectionFail("Unable to connect/reconnect to any configured Liberator.");
		return;
	}
	var self = this;
	var callReconnect = function() {
		self.context.logger.log(caplin.streamlink.LogLevel.FINE, "Connection not established within timeout period (" + self.context.configuration._getHttpTimeout() + "ms)");
		self.reconnect();
	}
	this.reconnectTimer = this.context.runtime.createTimeout(callReconnect, this.context.configuration._getHttpTimeout());
	this.connection.setRttpConnectionListener(this);
	this.connection.connect();
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.onConnectionStatusChange = function(listener, event) {
	this.context.runtime.createTimeout(function() {
		(listener.onConnectionStatusChange && listener.onConnectionStatusChange(event));
	}, 0);
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.onData = function(newData) {
	try {
		this.responseQueue.updateReceived();
		this.queuedRawData += newData;
		var indexLastNewLine = this.queuedRawData.lastIndexOf('\n') + 1;
		if (indexLastNewLine == 0) {
			return;
		}
		var lines = this.queuedRawData.substring(0, indexLastNewLine).split("\n");
		this.queuedRawData = this.queuedRawData.substring(indexLastNewLine);
		this.processCompleteData(lines);
	}
	catch (parsingError) {
		this.context.logger.log(caplin.streamlink.LogLevel.WARN, "Error parsing: {0}", parsingError.toString());
		this.reconnect();
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.onFireOnStatisticsChange = function(listener, event) {
	this.context.runtime.createTimeout(function() {
		(listener.onStatisticsChange && listener.onStatisticsChange(event));
	}, 0);
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.onProcessServiceStatus = function(listener, event) {
	this.context.runtime.createTimeout(function() {
		(listener.onServiceStatusChange && listener.onServiceStatusChange(event));
	}, 0);
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.onProcessSourceStatus = function(listener, event) {
	this.context.runtime.createTimeout(function() {
		(listener.onSourceStatusChange && listener.onSourceStatusChange(event));
	}, 0);
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.onReady = function() {
	this.sendNextMessage();
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.onRequestReconnect = function() {
	if (this.responseQueue.isEmpty()) {
		this.reconnect();
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.onResponse = function(rttpEvent) {
	this.responseQueue.checkResponseReceived(rttpEvent);
	this.protocolListener.onResponse(rttpEvent);
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.onStateChange = function(newState) {
	switch (newState) {
	case caplin.streamlink.impl.connection.RttpConnectionState.CONNECTING:
	{
		this.changeConnectionState(caplin.streamlink.ConnectionState.CONNECTING, this.connection.getURL());
		break;
	}
	case caplin.streamlink.impl.connection.RttpConnectionState.CONNECTED:
	{
		this.cancelReconnectTimer();
		break;
	}
	case caplin.streamlink.impl.connection.RttpConnectionState.DISCONNECTED:
	{
		this.cancelReconnectTimer();
		this.expectingCredentials = false;
		this.credentialsProvider.cancel();
		this.sendQueue.clearSystemMessages();
		this.queuedRawData = "";
		this.responseQueue.clearSystemMessages();
		this.changeConnectionState(caplin.streamlink.ConnectionState.DISCONNECTED, this.connection.getURL());
		this.context.logger.log(caplin.streamlink.LogLevel.FINE, "Disconnected from: {0}", this.connection.getURL());
		this.protocolListener.onDisconnect();
		this.connection.setRttpConnectionListener(null);
		if (this.userWantsToBeConnected) {
			this.nextConnection();
		}
		break;
	}
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.onTerminalUpdate = function(rttpEvent) {
	this.lastSequenceNumber = rttpEvent._getRttpSequenceNumber();
	this.protocolListener.onTerminalUpdate(rttpEvent);
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.onUpdate = function(rttpEvent) {
	this.lastSequenceNumber = rttpEvent._getRttpSequenceNumber();
	this.protocolListener.onUpdate(rttpEvent);
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.parseRttpEvent = function(rttpCode, message) {
	var tokens = message.split(" ");
	switch (rttpCode) {
	case caplin.streamlink.impl.protocolhandler.RttpCode.REC1_IMG:
	case caplin.streamlink.impl.protocolhandler.RttpCode.REC1_UPD:
	case caplin.streamlink.impl.protocolhandler.RttpCode.REC1_CLR:
	{
		var timeReceived = this.context.runtime.currentTimeMillis();
		var clockOffset = this.connectionMonitor.getClockOffset();
		this.onUpdate(caplin.streamlink.impl.event.RecordType1EventImpl.createFromRttp(rttpCode, tokens, this.fieldMap, clockOffset, timeReceived, this.context));
	}
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.RECORD_RESP:
	{
		var timeReceived = this.context.runtime.currentTimeMillis();
		var clockOffset = this.connectionMonitor.getClockOffset();
		this.onResponse(caplin.streamlink.impl.event.RecordType1EventImpl.createFromRttp(rttpCode, tokens, this.fieldMap, clockOffset, timeReceived, this.context));
	}
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.CONNECT_OK:
	this.sessionId = tokens[1];
	this.context.logger.log(caplin.streamlink.LogLevel.FINEST, "Requesting credentials from credentials provider.");
	this.expectingCredentials = true;
	this.credentialsProvider.onCredentialsRequired(this);
	this.connection.setSessionId(this.sessionId);
	this.changeConnectionState(caplin.streamlink.ConnectionState.CONNECTED, this.connection.getURL());
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.LOGIN_OK:
	this.capabilities.load(this.context.runtime, tokens);
	this.previouslyConnected = true;
	this.eventInProgress = null;
	this.responseQueue.clear();
	this.lastSessionId = this.sessionId;
	this.changeConnectionState(caplin.streamlink.ConnectionState.LOGGEDIN, this.connection.getURL());
	this.protocolListener.onLogin();
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.RECON_OK:
	this.previouslyConnected = true;
	this.responseQueue.responseReceived();
	this.lastSessionId = this.sessionId;
	this.context.logger.log(caplin.streamlink.LogLevel.FINE, "Session reconnection");
	this.changeConnectionState(caplin.streamlink.ConnectionState.RECONNECTED, this.connection.getURL());
	this.protocolListener.onReconnect();
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_USER:
	case caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_PASS:
	case caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_IP:
	case caplin.streamlink.impl.protocolhandler.RttpCode.ACCT_EXPIRED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.ALREADY_LOGGED_IN:
	case caplin.streamlink.impl.protocolhandler.RttpCode.LICENSE_SITE:
	case caplin.streamlink.impl.protocolhandler.RttpCode.LICENSE_USER:
	case caplin.streamlink.impl.protocolhandler.RttpCode.GOTO_URL:
	case caplin.streamlink.impl.protocolhandler.RttpCode.AUTH_ERROR_STR:
	this.context.logger.log(caplin.streamlink.LogLevel.FINE, "Login Failure: " + rttpCode.getDescription());
	this.changeConnectionState(caplin.streamlink.ConnectionState.LOGINFAILED, rttpCode.getDescription());
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.SERVICE_OK:
	case caplin.streamlink.impl.protocolhandler.RttpCode.SERVICE_LIMITED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.SERVICE_DOWN:
	this.processServiceStatus(rttpCode, tokens);
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.SOURCE_UP:
	case caplin.streamlink.impl.protocolhandler.RttpCode.SOURCE_DOWN:
	this.processSourceStatus(rttpCode, tokens);
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.FIELD_MAP:
	this.processFieldMap(tokens);
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.RESP_UNKNOWN:
	this.onResponse(caplin.streamlink.impl.event.ResponseUnknownEvent.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.NOT_FOUND:
	case caplin.streamlink.impl.protocolhandler.RttpCode.READ_DENY:
	case caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY:
	case caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY_TRADE_GROUP_LICENSE_EXCEEDED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.UNAVAILABLE:
	case caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_PARAMETERS:
	case caplin.streamlink.impl.protocolhandler.RttpCode.CONTRIB_FAILED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_FAILED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_UP_FAILED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_DOWN_FAILED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_MIN_FAILED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_MAX_FAILED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_STOP_FAILED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_START_FAILED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_DEF_FAILED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_NL_FAILED:
	this.onResponse(caplin.streamlink.impl.event.ActionFailResponseEvent.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_OK:
	case caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_STALE:
	case caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_LIMITED:
	this.onUpdate(caplin.streamlink.impl.event.SubjectStatusEventImpl.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.DIRECTORY_RESP:
	this.onResponse(caplin.streamlink.impl.event.DirectoryEventImpl.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.DIR_UPD:
	this.onUpdate(caplin.streamlink.impl.event.DirectoryEventImpl.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.NEWS_RESP:
	this.onResponse(caplin.streamlink.impl.event.NewsEventImpl.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.NEWS_CLR:
	case caplin.streamlink.impl.protocolhandler.RttpCode.NEWS_IMG:
	case caplin.streamlink.impl.protocolhandler.RttpCode.NEWS_UPD:
	this.onUpdate(caplin.streamlink.impl.event.NewsEventImpl.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.STORY_IMG:
	case caplin.streamlink.impl.protocolhandler.RttpCode.STORY_RESP:
	case caplin.streamlink.impl.protocolhandler.RttpCode.STORY_UPD:
	this.eventInProgress = caplin.streamlink.impl.event.StoryEventImpl.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder);
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.CHAT_RESP:
	this.onResponse(caplin.streamlink.impl.event.ChatEventImpl.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.CHAT_UPD:
	this.onUpdate(caplin.streamlink.impl.event.ChatEventImpl.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.PERM_RESP:
	this.onResponse(caplin.streamlink.impl.event.PermissionEventImpl.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.PERM_IMG:
	case caplin.streamlink.impl.protocolhandler.RttpCode.PERM_UPD:
	case caplin.streamlink.impl.protocolhandler.RttpCode.PERM_DEL:
	case caplin.streamlink.impl.protocolhandler.RttpCode.PERM_CLR:
	this.onUpdate(caplin.streamlink.impl.event.PermissionEventImpl.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.REC2_CLR:
	case caplin.streamlink.impl.protocolhandler.RttpCode.REC2_DEL:
	case caplin.streamlink.impl.protocolhandler.RttpCode.REC2_IMG:
	case caplin.streamlink.impl.protocolhandler.RttpCode.REC2_UPD:
	this.onUpdate(caplin.streamlink.impl.event.RecordType2EventImpl.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.REC3_CLR:
	case caplin.streamlink.impl.protocolhandler.RttpCode.REC3_IMG:
	case caplin.streamlink.impl.protocolhandler.RttpCode.REC3_UPD:
	this.onUpdate(caplin.streamlink.impl.event.RecordType3EventImpl.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.CREATE_OK:
	case caplin.streamlink.impl.protocolhandler.RttpCode.DELETE_OK:
	case caplin.streamlink.impl.protocolhandler.RttpCode.CONTRIB_OK:
	case caplin.streamlink.impl.protocolhandler.RttpCode.DISCARD_OK:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_OK:
	this.onResponse(caplin.streamlink.impl.event.OkEvent.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.CONT_RESP:
	this.onResponse(caplin.streamlink.impl.event.ContainerEventInternal.createFromRttp(this.context, rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.CONT_UPD:
	case caplin.streamlink.impl.protocolhandler.RttpCode.CONT_IMG:
	this.onUpdate(caplin.streamlink.impl.event.ContainerEventInternal.createFromRttp(this.context, rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.CONTRIB_WAIT:
	this.onResponse(caplin.streamlink.impl.event.ContribEvent.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.CONTRIB_OK_DELAY:
	this.onUpdate(caplin.streamlink.impl.event.ContribEvent.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.NOT_FOUND_DELAY:
	case caplin.streamlink.impl.protocolhandler.RttpCode.READ_DENY_DELAY:
	case caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY_DELAY:
	case caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY_TRADE_GROUP_LICENSE_EXCEEDED_DELAY:
	case caplin.streamlink.impl.protocolhandler.RttpCode.UNAVAILABLE_DELAY:
	case caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_PARAMETERS_DELAY:
	case caplin.streamlink.impl.protocolhandler.RttpCode.DELETED_DELAY:
	case caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_REMOVED:
	this.onTerminalUpdate(caplin.streamlink.impl.event.ActionFailEvent.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder));
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.SYNC_OK:
	this.responseQueue.responseReceived();
	this.connectionMonitor.onSyncOk(tokens);
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.PAGE_RESP:
	case caplin.streamlink.impl.protocolhandler.RttpCode.AUTODIR_RESP:
	case caplin.streamlink.impl.protocolhandler.RttpCode.NOOP_OK:
	case caplin.streamlink.impl.protocolhandler.RttpCode.LOGOUT_OK:
	this.responseQueue.responseReceived();
	break;
	case caplin.streamlink.impl.protocolhandler.RttpCode.TIMED_HEARTBEAT:
	this.connectionMonitor.onTimedHeartbeat(tokens);
	break;
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.processCompleteData = function(lines) {
	for (var i = 0; i < lines.length; (i)++) {
		var line = lines[i];
		if (line.length > 0) {
			this.processMessage(line);
		}
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.processFieldMap = function(tokens) {
	for (var i = 1; i < tokens.length; (i)++) {
		var nameValue = tokens[i].split("=");
		this.fieldMap[nameValue[0]] = this.context.encoder.rttpDecode(nameValue[1]);
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.processMessage = function(message) {
	this.context.logger.log(caplin.streamlink.LogLevel.FINEST, caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.LOG_PREFIX + "in " + message);
	if (this.eventInProgress != null) {
		switch (this.eventInProgress._getRttpCode()) {
		case caplin.streamlink.impl.protocolhandler.RttpCode.STORY_IMG:
		case caplin.streamlink.impl.protocolhandler.RttpCode.STORY_UPD:
		case caplin.streamlink.impl.protocolhandler.RttpCode.STORY_RESP:
		if (this.eventInProgress._addMoreText(message, this.context.encoder) == false) {
			if (this.eventInProgress._getRttpCode() == caplin.streamlink.impl.protocolhandler.RttpCode.STORY_RESP) {
				this.onResponse(this.eventInProgress);
			}
			else {
				this.onUpdate(this.eventInProgress);
			}
			this.eventInProgress = null;
		}
		}
		return;
	}
	var rttpCode = caplin.streamlink.impl.utils.RttpUtil.getRttpCode(message.substring(0, 2));
	if (rttpCode == null) {
		throw new Error(caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.LOG_PREFIX + "Rttp code: " + message.substring(0, 2) + " not recognized");
	}
	this.parseRttpEvent(rttpCode, message);
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.processServiceStatus = function(rttpCode, tokens) {
	var event = caplin.streamlink.impl.event.ServiceStatusEventImpl.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder);
	for (var i = 0; i < this.connectionListeners.length; (i)++) {
		var listener = this.connectionListeners[i];
		this.onProcessServiceStatus(listener, event);
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.processSourceStatus = function(rttpCode, tokens) {
	var event = caplin.streamlink.impl.event.SourceStatusEventImpl.createFromRttp(rttpCode, tokens, this.fieldMap, this.context.encoder);
	for (var i = 0; i < this.connectionListeners.length; (i)++) {
		var listener = this.connectionListeners[i];
		this.onProcessSourceStatus(listener, event);
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.provideCredentials = function(credentials) {
	if (!(this.expectingCredentials)) {
		this.context.logger.log(caplin.streamlink.LogLevel.WARN, "Received credentials when not expecting - ignoring: {0}", credentials);
	}
	else {
		this.expectingCredentials = false;
		this.context.logger.log(caplin.streamlink.LogLevel.FINE, "Received credentials: {0}", credentials);
		var packetLogin = this.createLoginPacket(credentials);
		this.sendPacket(packetLogin);
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.reconnect = function() {
	this.userWantsToBeConnected = true;
	this._disconnectInternal();
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.removeConnectionListener = function(listener) {
	var index = caplin.xlang.Utils.indexOf(this.connectionListeners, listener);
	if (index >= 0) {
		this.connectionListeners.splice(index,1)[0];
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.sendConnectionFail = function(message) {
	for (var i = 0; i < this.connectionListeners.length; (i)++) {
		this.sendConnectionFailedOnTimer(this.connectionListeners[i], message);
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.sendConnectionFailedOnTimer = function(listener, message) {
	this.context.runtime.createTimeout(function() {
		(listener.onConnectionFail && listener.onConnectionFail(message));
	}, 0);
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.sendNextMessage = function() {
	var mergeCommands = this.capabilities.getCapability("MergedCommands");
	var bodyLength = this.capabilities.getCapability("HttpBodyLength");
	if (!(this.sendQueue.isEmpty()) && this.connection != null && this.connection.isReadyToSend()) {
		var packet = this.sendQueue.popNextMessage();
		var msg = this.context.encoder.rttpEncode(this.sessionId + " " + packet.toRttpString(this.context.encoder));
		if (packet.getType().isResponseRequired()) {
			this.responseQueue.add(packet);
		}
		if (mergeCommands && packet.getType().isBatchable()) {
			packet = this.sendQueue.peekNextMessage();
			while (packet != null && packet.getType().isBatchable()) {
				var msgPart = this.context.encoder.rttpEncode(this.sessionId + " " + packet.toRttpString(this.context.encoder));
				if (msg.length + msgPart.length + 1 >= bodyLength) {
					break;
				}
				msg += "\n" + msgPart;
				this.sendQueue.popNextMessage();
				if (packet.getType().isResponseRequired()) {
					this.responseQueue.add(packet);
				}
				packet = this.sendQueue.peekNextMessage();
			}
		}
		this.context.logger.log(caplin.streamlink.LogLevel.FINEST, "{0}out {1} ", caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.LOG_PREFIX, msg);
		this.connection.send(msg);
	}
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.sendPacket = function(packet) {
	if (!(packet.getType().isSystem())) {
		this.context.logger.log(caplin.streamlink.LogLevel.FINE, "Send {0}", packet);
	}
	this.sendQueue.pushNextMessage(packet);
	this.sendNextMessage();
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.setConnectionMonitor = function(connectionMonitor) {
	this.connectionMonitor = connectionMonitor;
};

caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl.prototype.setProtocolListener = function(listener) {
	this.protocolListener = listener;
};



//## StreamLinkContext.js ##//
caplin.namespace("caplin.streamlink.impl");


caplin.streamlink.impl.StreamLinkContext = function(runtime, logger, technologyApplicationId, configuration, encoder) {
	// Constructor body
	this.runtime = runtime;
	this.logger = logger;
	this.configuration = configuration;
	this.encoder = encoder;
	this.technologyApplicationId = technologyApplicationId;
};

caplin.streamlink.impl.StreamLinkContext.prototype.getTestConnection = function() {
	return this.testConnection;
};

caplin.streamlink.impl.StreamLinkContext.prototype.setTestConnection = function(testConnection) {
	this.testConnection = testConnection;
};



//## CommandErrorEventImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.CommandErrorEvent

caplin.streamlink.impl.event.CommandErrorEventImpl = function(commandErrorType) {
	// Constructor body
	this._commandErrorType = commandErrorType;
};

// caplin.implement(caplin.streamlink.impl.event.CommandErrorEventImpl, caplin.streamlink.CommandErrorEvent);

caplin.streamlink.impl.event.CommandErrorEventImpl.prototype.getError = function() {
	return this._commandErrorType;
};

caplin.streamlink.impl.event.CommandErrorEventImpl._createFromDictionary = function(data) {
	var commandErrorType = caplin.streamlink.CommandErrorType.valueOf(data["CommandError"]);
	return new caplin.streamlink.impl.event.CommandErrorEventImpl(commandErrorType);
};

caplin.streamlink.impl.event.CommandErrorEventImpl._createFromRttpCode = function(rttpCode) {
	switch (rttpCode) {
	case caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY:
	case caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY_DELAY:
	return new caplin.streamlink.impl.event.CommandErrorEventImpl(caplin.streamlink.CommandErrorType.WRITE_DENIED);
	case caplin.streamlink.impl.protocolhandler.RttpCode.READ_DENY:
	case caplin.streamlink.impl.protocolhandler.RttpCode.READ_DENY_DELAY:
	return new caplin.streamlink.impl.event.CommandErrorEventImpl(caplin.streamlink.CommandErrorType.READ_DENIED);
	case caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY_TRADE_GROUP_LICENSE_EXCEEDED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY_TRADE_GROUP_LICENSE_EXCEEDED_DELAY:
	return new caplin.streamlink.impl.event.CommandErrorEventImpl(caplin.streamlink.CommandErrorType.WRITE_DENIED_TRADE_GROUP_LICENSE_EXCEEDED);
	case caplin.streamlink.impl.protocolhandler.RttpCode.NOT_FOUND:
	case caplin.streamlink.impl.protocolhandler.RttpCode.NOT_FOUND_DELAY:
	return new caplin.streamlink.impl.event.CommandErrorEventImpl(caplin.streamlink.CommandErrorType.NOT_FOUND);
	case caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_PARAMETERS:
	case caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_PARAMETERS_DELAY:
	return new caplin.streamlink.impl.event.CommandErrorEventImpl(caplin.streamlink.CommandErrorType.INVALID_PARAMETERS);
	case caplin.streamlink.impl.protocolhandler.RttpCode.UNAVAILABLE:
	case caplin.streamlink.impl.protocolhandler.RttpCode.UNAVAILABLE_DELAY:
	return new caplin.streamlink.impl.event.CommandErrorEventImpl(caplin.streamlink.CommandErrorType.UNAVAILABLE);
	case caplin.streamlink.impl.protocolhandler.RttpCode.DELETED_DELAY:
	return new caplin.streamlink.impl.event.CommandErrorEventImpl(caplin.streamlink.CommandErrorType.DELETED);
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_FAILED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_UP_FAILED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_DOWN_FAILED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_MIN_FAILED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_MAX_FAILED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_STOP_FAILED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_START_FAILED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_DEF_FAILED:
	case caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_NL_FAILED:
	return new caplin.streamlink.impl.event.CommandErrorEventImpl(caplin.streamlink.CommandErrorType.THROTTLE_FAILED);
	default :
	return null;
	}
};

caplin.streamlink.impl.event.CommandErrorEventImpl.prototype._toDictionary = function() {
	var root = {} ;
	root["CommandError"] = this._commandErrorType.toString();
	return root;
};



//## ContainerEventInternal.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.impl.event.RttpEventBase

caplin.streamlink.impl.event.ContainerEventInternal = function(subject, image, rttpCode, rttpObjectNumber, rttpSequenceNumber, size, ctrstart, ctrend, ctrid, elements, order) {
	// Super
	caplin.streamlink.impl.event.RttpEventBase.call(this, subject, rttpCode, rttpObjectNumber, rttpSequenceNumber);
	// Constructor body
	this.size = size;
	this.ctrstart = ctrstart;
	this.ctrend = ctrend;
	this.ctrid = ctrid;
	this.elements = elements;
	this.order = order;
	this._image = image;
};

caplin.extend(caplin.streamlink.impl.event.ContainerEventInternal, caplin.streamlink.impl.event.RttpEventBase);

caplin.streamlink.impl.event.ContainerEventInternal.prototype.isImage = function() {
	return this._image;
};

caplin.streamlink.impl.event.ContainerEventInternal.createFromRttp = function(context, rttpCode, tokens, fieldMap, encoder) {
	switch (rttpCode) {
	case caplin.streamlink.impl.protocolhandler.RttpCode.CONT_RESP:
	return caplin.streamlink.impl.event.ContainerEventInternal.parseContainerElements(context, encoder.rttpDecode(tokens[1]), tokens, rttpCode, tokens[0].substring(2, 6), null, true);
	case caplin.streamlink.impl.protocolhandler.RttpCode.CONT_UPD:
	return caplin.streamlink.impl.event.ContainerEventInternal.parseContainerElements(context, null, tokens, rttpCode, tokens[0].substring(4, 8), tokens[0].substring(2, 4), false);
	case caplin.streamlink.impl.protocolhandler.RttpCode.CONT_IMG:
	return caplin.streamlink.impl.event.ContainerEventInternal.parseContainerElements(context, null, tokens, rttpCode, tokens[0].substring(4, 8), tokens[0].substring(2, 4), true);
	}
	return null;
};

caplin.streamlink.impl.event.ContainerEventInternal.parseContainerElements = function(context, subject, tokens, rttpCode, objectNumber, sequenceNumber, image) {
	var elements = [];
	var order = [];
	var size = 0;
	var ctrstart = 0;
	var ctrend = 0;
	var ctrid = null;
	for (var i = 1; i < tokens.length; (i)++) {
		var parts = tokens[i].split(";");
		if (parts.length == 3) {
			caplin.streamlink.impl.event.ContainerEventInternal.parseContainerElement(elements, parts, i, context.encoder);
		}
		else {
			var parts2 = tokens[i].split("=");
			if (parts2.length == 2) {
				var key = parts2[0];
				var value = parts2[1];
				if ((key == ("size"))) {
					size = context.runtime.parseInt(value);
				}
				else if ((key == ("ctrstart"))) {
					ctrstart = context.runtime.parseInt(value);
				}
				else if ((key == ("ctrend"))) {
					ctrend = context.runtime.parseInt(value);
				}
				else if ((key == ("ctrid"))) {
					ctrid = value;
				}
				else if ((key == ("order"))) {
					caplin.streamlink.impl.event.ContainerEventInternal.parseOrder(context, order, value);
				}
			}
		}
	}
	return new caplin.streamlink.impl.event.ContainerEventInternal(subject, image, rttpCode, objectNumber, sequenceNumber, size, ctrstart, ctrend, ctrid, elements, order);
};

caplin.streamlink.impl.event.ContainerEventInternal.prototype.getItems = function() {
	return this.elements;
};

caplin.streamlink.impl.event.ContainerEventInternal.prototype.getOrder = function() {
	return this.order;
};

caplin.streamlink.impl.event.ContainerEventInternal.prototype.getSize = function() {
	return this.size;
};

caplin.streamlink.impl.event.ContainerEventInternal.prototype.getWindowEnd = function() {
	return this.ctrend;
};

caplin.streamlink.impl.event.ContainerEventInternal.prototype.getWindowStart = function() {
	return this.ctrstart;
};

caplin.streamlink.impl.event.ContainerEventInternal.prototype.getId = function() {
	return this.ctrid;
};

caplin.streamlink.impl.event.ContainerEventInternal.prototype.toString = function() {
	var builder = [] ;
	builder.push("ContainerEvent [");
	builder.push("subject=");
	builder.push(this._subject);
	builder.push(", size=");
	builder.push(this.size);
	builder.push(", elements=[");
	builder.push(this.elements.toString());
	builder.push("]");
	builder.push(", order=[");
	builder.push(this.order.toString());
	builder.push("]");
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.ContainerEventInternal.prototype._publishCommandResponse = function(listener) { };

caplin.streamlink.impl.event.ContainerEventInternal.prototype._publishSubscriptionResponse = function(subscription, subscriptionListener) { };

caplin.streamlink.impl.event.ContainerEventInternal.parseContainerElement = function(elements, parts, postion, encoder) {
	var subject = encoder.rttpDecode(parts[0]);
	var type = caplin.streamlink.impl.event.ParseUtils.getSubjectType(parts[1]);
	var objectNumber = parts[2];
	var added = !((parts[1] == ("0")));
	elements.push(new caplin.streamlink.impl.event.ContainerItem(subject, type, objectNumber, added));
};

caplin.streamlink.impl.event.ContainerEventInternal.parseOrder = function(context, order, value) {
	var tokens = value.split(",");
	for (var i = 0; i < tokens.length; (i)++) {
		var token = tokens[i];
		var objectNumber = token.substring(0, 4);
		var posStr = token.substring(5);
		var pos = context.runtime.parseInt(posStr);
		var orderElement = new caplin.streamlink.impl.event.ContainerOrderItem(objectNumber, pos);
		order.push(orderElement);
	}
};



//## StreamLink.js ##//

/**
 * @class  <p>The main StreamLink Class.</p>
 * <p>To create an instance of this class, use the {@link caplin.streamlink.StreamLinkFactory}.</p>
 * 
 * @name caplin.streamlink.StreamLink
 */
caplin.streamlink.StreamLink = function(streamLinkCore, streamLinkId, logger, configuration) 
{
	this._streamLinkCore = streamLinkCore;
	this._streamLinkId = streamLinkId;
	this._logger = logger;
	
	// add a window log listener if  ?debug=<loggingLevel> defined on query string
	var debugWindowLevelStr = caplin.streamlink.js.impl.Utils.getParameter("debug");
	var logWindowVisible = configuration?configuration._getLogWindowVisible():null;
	
	if (logWindowVisible == true || (logWindowVisible == null && debugWindowLevelStr))
	{
		var logLevel = null;
		if (logWindowVisible == true)
		{
			logLevel = configuration._getLogWindowLogLevel()
		}
		else
		{
			logLevel = caplin.streamlink.LogLevel.valueOf(debugWindowLevelStr.toUpperCase());
		}
		
		if (logLevel)
		{
			this._addWindowLogListener(logLevel);
		}
	}
};

caplin.streamlink.StreamLink.prototype._createLogWindowHTML = function(id, logLevel) 
{
	return "<head>\n" +
	"<title>Log Window: " + logLevel + "</title>\n" +
	"<style type=\"text/css\">\n" +
	"body {font-family:Verdana;background-color:Black;color:#00ff00;}\n" +
	"pre {margin:0px;}\n" +
	"</style>\n" +
	"</head>\n" +
	"<body leftmargin=\"1\" topmargin=\"1\" rightmargin=\"1\" bottommargin=\"1\">\n" +
	"</body>\n" +
	"</html>";
};

caplin.streamlink.StreamLink.prototype._addWindowLogListener = function(logLevel) 
{
	var windowParameters = "height=400,width=600,status=no,toolbar=no,menubar=no,location=no,resizable=yes,scrollbars=yes";
	this._debugWindowOpen = false;
	
	var sHTML = this._createLogWindowHTML(this._streamLinkId, logLevel);
	this._debugWindow = window.open("", "", windowParameters); // second arg must be "" for IE
	
	if (this._debugWindow != null)
	{
		this._debugWindow.document.writeln(sHTML);
		this._debugWindow.document.close();
	
		var self = this;
		this._debugWindow.onunload = function()
		{
			 self._logger.removeListener(self._debugListener, logLevel);
		};
			
		this._debugListener =  
		{
			onLog: function(logInfo)
			{
				self._logMessage(logInfo.toString());
			}
		}; 
		
		this._logger.addListener(this._debugListener, logLevel );
	}
};

caplin.streamlink.StreamLink.prototype._logMessage = function(message)
{
	var doc = null;
	try
	{
		doc = this._debugWindow.document;
	}
	catch(e)
	{
		return; // if document cannot be obtained (permission error after closing window in IE8) then return
	}
		
	// checks if should scroll
	var trackEnd = ((doc.body.scrollTop + doc.body.clientHeight) == doc.body.scrollHeight);
	var pre = doc.createElement("pre");
	var txt = doc.createTextNode(message);
	pre.appendChild(txt);
	doc.body.appendChild(pre);
	
	// scroll to bottom
	if(trackEnd)
	{
		doc.body.scrollTop = doc.body.scrollHeight;
	}
}

/**
 * <p>Instructs this <code>StreamLink</code> instance to attempt to connect to the Liberator.</p>
 */
caplin.streamlink.StreamLink.prototype.connect = function() 
{
	this._streamLinkCore.connect();
};

/**
 * <p>Instructs this <code>StreamLink</code> instance to disconnect from the Liberator.</p>
 */
caplin.streamlink.StreamLink.prototype.disconnect = function() 
{
	this._streamLinkCore.disconnect();
};

/**
 * <p>Adds a {@link caplin.streamlink.ConnectionListener} to this StreamLink instance. 
 * This connectionListener is called whenever a connection event occurs.</p>
 * 
 * @param {caplin.streamlink.ConnectionListener} connectionListener The <code>ConnectionListener</code> to be added.
 */
caplin.streamlink.StreamLink.prototype.addConnectionListener = function(connectionListener) 
{
	this._streamLinkCore.addConnectionListener(connectionListener);
};

/**
 * <p>Removes a previously added {@link caplin.streamlink.ConnectionListener} from this StreamLink instance. 
 * After removal, no further callbacks are received by this ConnectionListener.</p>
 * 
 * @param {caplin.streamlink.ConnectionListener} connectionListener The ConnectionListener to be removed.
 */
caplin.streamlink.StreamLink.prototype.removeConnectionListener = function(connectionListener) 
{
	this._streamLinkCore.removeConnectionListener(connectionListener);
};

/**
 *  <p>Subscribes to a subject to receive updates.</p>
 *  @param {String} subject The subject to subscribe to.
 *  @param {caplin.streamlink.SubscriptionListener} subscriptionListener A listener that is called to provide information about the result of the subscription request, the subscribed data, and subsequent updates to that data.
 *  @param {Map<String>} subscriptionParameters Optional JSON configuration attributes:
 *  
 *  <table>
 *  <tr>
 *  	<th>Attribute name</th>
 *  	<th>Type</th>
 *  	<th>Default</th>
 *  	<th>Optional</th>
 *      <th>Description</th>
 *  </tr>
 *  <tr>
 *  	<td>fields</td>
 *  	<td>[string]</td>
 *  	<td>All fields are subscribed to.</td>
 *  	<td>Yes</td>
 *  	<td>An array of the the fields to subscribe to.
 *          <p>For example: <code>["field_a","field_b","field_d"]</code></p>
 *      </td>
 *  <tr>
 *  	<td>filter</td>
 *  	<td>{value: string, image: boolean}</td>
 *  	<td>No filter is applied.</td> 
 *  	<td>Yes</td>
 *  	<td>A filter that restricts the number of field updates that will be received.
 *          <p>The <code>value</code> string specifies the filter criteria. For example: <code>(dLast >50) & (dVolumeAcc > 1000)/code>, 
 *          means that the Liberator sends updates only when the value of the field <code>dLast</code> is greater than 50 
 *          and the <code>dVolumeAcc</code> field is greater than 1000.</p>
 *          <p>When the <code>image</code> boolean is true, the filter not applied until after the data within a particular update 
 *          has been applied to the image of the data item. When <code>image</code> is false, the update filter is applied just to 
 *          the data within a particular update of the data item.</p>
 *          For more information about filters, see the 'Filtering data' section of the document <b>StreamLink Overview</b>.</p>
 *      </td>
 *  </tr>
 *  <tr>
 *  	<td>window</td>
 *  	<td>{start: int, size: int}</td>
 *  	<td>Updates are returned for the whole container (subject to any filter applied).</td> 
 *  	<td>Yes</td>
 *  	<td>When the subscription is to a container, this attribute defines a window that begins at the element defined by <code>start</code>, 
 *         and is <code>size</code> elements long. Liberator only returns updates for the elements that are in the window. 
 *         The first element in a container starts at position 0 (zero).</td>
 *  </tr>
 *  </table>
 *  
 *  @returns {caplin.streamlink.Subscription} The subscription data. You can subsequently unsubscribe from the subject by calling this object's <code>unsubcribe()</code> method.
 *  @function
 */
caplin.streamlink.StreamLink.prototype.subscribe = function(subject, subscriptionListener, subscriptionParameters) 
{
	return this._streamLinkCore.subscribe(subject, subscriptionListener, this._toSubscriptionParameters(subscriptionParameters));
};

caplin.streamlink.StreamLink.prototype._toSubscriptionParameters = function(parameters) 
{
	if (!parameters)
	{
		return this._streamLinkCore.createRecordSubscriptionParameters();
	}
	
	var utils = caplin.streamlink.js.impl.Utils;
	var typeSchema = {
		window: function (window, attribute) {
			if (!utils.isNumber(window.start) || !utils.isNumber(window.size))
			{
				throw Error('window.start and window.size must of type Number.');
			}
		},
		fields: caplin.streamlink.js.impl.Utils.assertArrayOfStrings,
		filter: function (filter, attribute) {
			if (typeof filter.value !== 'string')
			{
				throw new Error('filter.value must be of type String.');
			}
			if ('image' in filter)
			{
				utils.assertBoolean(filter.image, 'image');
			}
		},
		freeform: caplin.streamlink.js.impl.Utils.assertString
	};
	utils.validateTypeSchema(typeSchema, parameters);
	
	var result;
	if (parameters.freeform)
	{
		result = this._streamLinkCore.createFreeFormSubscriptionParameters();
		result.setParameters(parameters.freeform);
		return result;
	}
	
	if (parameters.window)
	{
		result = this._streamLinkCore.createContainerSubscriptionParameters();
		result.setWindow(parameters.window.start, parameters.window.size);
	}
	else
	{
		result = this._streamLinkCore.createRecordSubscriptionParameters();
	}
	
	if (parameters.fields)
	{
		result.setFields(parameters.fields);
	}	
	if (parameters.filter)
	{
		result.setFilter(parameters.filter.value, parameters.filter.image === true);
	}
	return result;
};

/**
 * <p>Creates a subject in the Liberator.</p>
 * 
 * @param {String} subject The name of the subject to create.
 * @param {caplin.streamlink.SubjectType} subjectType The type of subject to create.
 * @param {caplin.streamlink.CommandListener} commandListener A listener that is called to provide information about the result of the "create subject" command.
 * @param {Map<String>} commandParameters Optional JSON configuration attributes:
 *  
 *  <table>
 *  <tr>
 *  	<th>Attribute name</th>
 *  	<th>Type</th>
 *  	<th>Default</th>
 *  	<th>Optional</th>
 *      <th>Description</th>
 *  </tr>
 *  <tr>
 *  	<td>persistent</td>
 *  	<td>boolean</td>
 *  	<td>false</td>
 *  	<td>Yes</td>
 *  	<td><p>When true, the throttle setting for this command persists across a failure of the connection to Liberator 
 *             and subsequent reconnection.</p>
 *          <p>When false, the throttle setting only applies to the current Liberator session 
 *             and does not persist following a connection failure.</p>
 *          <p>For a fuller explanation, see the 'Persistence' section of the <b>StreamLink Overview</b> document.</p>
 *      </td>
 *  </tr>
 *  </table>
 * 
 * @returns {caplin.streamlink.CommandSubscription} Information about the 'create subject' command. If the <code>commandParameters</code> made the command persistent, you can subsequently cancel this by calling the <code>CommandSubscription</code>'s <code>unPersist()</code> method.
 */
caplin.streamlink.StreamLink.prototype.createSubject = function(subject, subjectType, commandListener, commandParameters) 
{
	return this._streamLinkCore.createSubject(subject, subjectType, commandListener, this._toCommandParameters(commandParameters));
};

/**
 * <p>Deletes a subject in the Liberator.</p>
 * 
 * @param {String} subject The name of the subject to delete.
 * @param {caplin.streamlink.CommandListener} commandListener A listener that is called to provide information about the result of the 'delete subject' command.
 */
caplin.streamlink.StreamLink.prototype.deleteSubject = function(subject, commandListener) 
{
	this._streamLinkCore.deleteSubject(subject, commandListener);
};

/**
 *  <p>Publishes data to a subject.</p>
 *  @param {String} subject The subject to publish to.
 *  @param {Map<String>} fieldData The data, as key-value pairs, to be published to the subject.
 *  @param {caplin.streamlink.CommandListener} commandListener A listener that receives notification of the publish command's success or failure.
 *  @param {Map<String>} commandParameters Optional JSON configuration attributes:
 *  
 *  <table>
 *  <tr>
 *  	<th>Attribute name</th>
 *  	<th>Type</th>
 *  	<th>Default</th>
 *  	<th>Optional</th>
 *      <th>Description</th>
 *  </tr>
 *  <tr>
 *  	<td>persistent</td>
 *  	<td>boolean</td>
 *  	<td>false</td>
 *  	<td>Yes</td>
 *  	<td><p>When true, if the connection to the Liberator fails after publication, 
 *             the data is resent when StreamLink reconnects.</p>
 *          <p>When false, if the connection to the Liberator fails after publication, 
 *             the data is not resent when StreamLink reconnects.</p>
 *      </td>
 *  </tr>
 *  </table>
 *  
 *  @returns {caplin.streamlink.CommandSubscription} Information about the 'publish' command. If the <code>commandParameters</code> made the command persistent, you can subsequently cancel this by calling the <code>CommandSubscription</code>'s <code>unPersist()</code> method.
 *  @function
 */
caplin.streamlink.StreamLink.prototype.publishToSubject = function(subject, fieldData, commandListener, commandParameters) 
{
	return this._streamLinkCore.publishToSubject(subject, fieldData, commandListener, this._toCommandParameters(commandParameters));
};

/**
 * <p>Throttles the specified subject, reducing the number of updates that are sent every second.</p>
 * <p>For a fuller explanation of throttling, refer to the 'Throttling' section of the document <b>StreamLink Overview.</b></p>
 * 
 * @param {String} subject The subject for which received data updates are to be throttled.
 * @param {caplin.streamlink.ThrottleCommand} throttleCommand The Throttle command to apply.
 * @param {caplin.streamlink.CommandListener} commandListener A listener that is called to provide information about the result of the throttle command.
 * @param {Map<String>} commandParameters Optional JSON configuration attributes:
 *  
 *  <table>
 *  <tr>
 *  	<th>Attribute name</th>
 *  	<th>Type</th>
 *  	<th>Default</th>
 *  	<th>Optional</th>
 *      <th>Description</th>
 *  </tr>
 *  <tr>
 *  	<td>persistent</td>
 *  	<td>boolean</td>
 *  	<td>false</td>
 *  	<td>Yes</td>
 *  	<td><p>When true, the throttle setting for this command persists across a failure of the connection to Liberator 
 *             and subsequent reconnection.</p>
 *          <p>When false, the throttle setting only applies to the current Liberator session 
 *             and does not persist following a connection failure.</p>
 *          <p>For a fuller explanation, see the 'Persistence' section of the <b>StreamLink Overview</b> document.</p>
 *      </td>
 *  </tr>
 *  </table>
 * 
 * @return {caplin.streamlink.CommandSubscription} Information about the 'throttle' command. If the <code>commandParameters</code> made the command persistent, you can subsequently cancel this by calling the <code>CommandSubscription</code>'s <code>unPersist()</code> method.
 */
caplin.streamlink.StreamLink.prototype.throttleSubject = function(subject, throttleCommand, commandListener, commandParameters) 
{
	return this._streamLinkCore.throttleSubject(subject, throttleCommand, commandListener, this._toCommandParameters(commandParameters));
};

/**
 * <p>Throttles all the currently subscribed subjects, reducing the number of updates that are sent every second.</p>
 * <p>For a fuller explanation of throttling, refer to the 'Throttling' section of the document <b>StreamLink Overview.</b></p>
 * 
 * @param {caplin.streamlink.ThrottleCommand} throttleCommand The throttle command to apply.
 * @param {caplin.streamlink.CommandListener} commandListener A listener that is called to provide information about the result of the throttle command.
 * @param {Map<String>} commandParameters Optional JSON configuration attributes; for details, see the <code>commandParameters</code> parameter of <code>throttleSubject().</code>
 *  
 * @return {caplin.streamlink.CommandSubscription} Information about the 'throttle' command. If the <code>commandParameters</code> made the command persistent, you can subsequently cancel this by calling the <code>CommandSubscription</code>'s <code>unPersist()</code> method.
 */
caplin.streamlink.StreamLink.prototype.throttleEverything = function(throttleCommand, commandListener, commandParameters) 
{
	return this._streamLinkCore.throttleEverything(throttleCommand, commandListener, this._toCommandParameters(commandParameters));
};

caplin.streamlink.StreamLink.prototype._toCommandParameters = function(parameters) 
{
	if (!parameters)
	{
		return this._streamLinkCore.createCommandParameters();
	}
	var utils = caplin.streamlink.js.impl.Utils;
	var typeSchema = {
		persistent: utils.assertBoolean
	};
	utils.validateTypeSchema(typeSchema, parameters);
	
	var result = this._streamLinkCore.createCommandParameters();
	if (parameters.persistent)
	{
		result.setPersistent();
	}
	return result;
};

caplin.streamlink.StreamLink.prototype.setReachability = function(isReachable) 
{
	this._streamLinkCore.setReachability(isReachable === true);
};


/**
 * <p>Returns the StreamLink logger object. 
 * You can receive StreamLink log messages by adding your own {@link caplin.streamlink.LogListener} to this object.</p>
 *
 * @return {caplin.streamlink.Logger} The <code>Logger</code> object of this StreamLink instance.
 */
caplin.streamlink.StreamLink.prototype.getLogger = function() 
{
	return this._streamLinkCore.getLogger();
};

/**
 * <p>Returns the version number and build identifier for this version of the StreamLink library.</p> 
 * 
 * @return String the version number and build identifier for this version of the StreamLink library.
 */
caplin.streamlink.StreamLink.prototype.getVersion = function()
{
	return caplin.streamlink.impl.Version.getVersion();
};



//## StreamLinkCore.js ##//
caplin.namespace("caplin.streamlink.impl");


caplin.streamlink.impl.StreamLinkCore = function() {};

caplin.streamlink.impl.StreamLinkCore.prototype.createCommandParameters = function() { throw new Error('caplin.streamlink.impl.StreamLinkCore.createCommandParameters :  This is an abstract method'); };

caplin.streamlink.impl.StreamLinkCore.prototype.createSubject = function(subject, type, listener, parameters) { throw new Error('caplin.streamlink.impl.StreamLinkCore.createSubject :  This is an abstract method'); };

caplin.streamlink.impl.StreamLinkCore.prototype.publishToSubject = function(subject, fieldData, listener, parameters) { throw new Error('caplin.streamlink.impl.StreamLinkCore.publishToSubject :  This is an abstract method'); };

caplin.streamlink.impl.StreamLinkCore.prototype.throttleEverything = function(command, listener, parameters) { throw new Error('caplin.streamlink.impl.StreamLinkCore.throttleEverything :  This is an abstract method'); };

caplin.streamlink.impl.StreamLinkCore.prototype.throttleSubject = function(subject, command, listener, parameters) { throw new Error('caplin.streamlink.impl.StreamLinkCore.throttleSubject :  This is an abstract method'); };

caplin.streamlink.impl.StreamLinkCore.prototype.createContainerSubscriptionParameters = function() { throw new Error('caplin.streamlink.impl.StreamLinkCore.createContainerSubscriptionParameters :  This is an abstract method'); };

caplin.streamlink.impl.StreamLinkCore.prototype.createFreeFormSubscriptionParameters = function() { throw new Error('caplin.streamlink.impl.StreamLinkCore.createFreeFormSubscriptionParameters :  This is an abstract method'); };

caplin.streamlink.impl.StreamLinkCore.prototype.getLogger = function() { throw new Error('caplin.streamlink.impl.StreamLinkCore.getLogger :  This is an abstract method'); };

caplin.streamlink.impl.StreamLinkCore.prototype.createRecordSubscriptionParameters = function() { throw new Error('caplin.streamlink.impl.StreamLinkCore.createRecordSubscriptionParameters :  This is an abstract method'); };

caplin.streamlink.impl.StreamLinkCore.prototype.subscribe = function(subject, listener, parameters) { throw new Error('caplin.streamlink.impl.StreamLinkCore.subscribe :  This is an abstract method'); };

caplin.streamlink.impl.StreamLinkCore.prototype.getVersion = function() { throw new Error('caplin.streamlink.impl.StreamLinkCore.getVersion :  This is an abstract method'); };

caplin.streamlink.impl.StreamLinkCore.prototype.addConnectionListener = function(listener) { throw new Error('caplin.streamlink.impl.StreamLinkCore.addConnectionListener :  This is an abstract method'); };

caplin.streamlink.impl.StreamLinkCore.prototype.connect = function() { throw new Error('caplin.streamlink.impl.StreamLinkCore.connect :  This is an abstract method'); };

caplin.streamlink.impl.StreamLinkCore.prototype.deleteSubject = function(subject, listener) { throw new Error('caplin.streamlink.impl.StreamLinkCore.deleteSubject :  This is an abstract method'); };

caplin.streamlink.impl.StreamLinkCore.prototype.disconnect = function() { throw new Error('caplin.streamlink.impl.StreamLinkCore.disconnect :  This is an abstract method'); };

caplin.streamlink.impl.StreamLinkCore.prototype.removeConnectionListener = function(listener) { throw new Error('caplin.streamlink.impl.StreamLinkCore.removeConnectionListener :  This is an abstract method'); };



//## RttpConnectionState.js ##//
caplin.namespace("caplin.streamlink.impl.connection");

caplin.streamlink.impl.connection.RttpConnectionState = {
		DISCONNECTED: "DISCONNECTED",
		CONNECTING: "CONNECTING",
		CONNECTED: "CONNECTED",
	values: function() {return [caplin.streamlink.impl.connection.RttpConnectionState.DISCONNECTED, caplin.streamlink.impl.connection.RttpConnectionState.CONNECTING, caplin.streamlink.impl.connection.RttpConnectionState.CONNECTED];},
	valueOf: function(name) {return name;}
};

//## PollingConnection.js ##//
caplin.namespace("caplin.streamlink.js.impl.connection");

caplin.include("caplin.streamlink.impl.connection.BaseConnection", true);
caplin.include("caplin.streamlink.impl.connection.RttpConnectionState", true);
caplin.include("caplin.streamlink.js.impl.connection.Channel");

//caplin.forceload(caplin.streamlink.impl.connection.BaseConnection);
//caplin.forceload(caplin.streamlink.impl.connection.RttpConnectionState);

caplin.streamlink.js.impl.connection.PollingConnection = (function() {
	
	var ConnectionState = caplin.streamlink.impl.connection.RttpConnectionState;
	var Super = caplin.streamlink.impl.connection.BaseConnection;
	
	function PollingConnection(context, url, ssl) {
		Super.call(this, context, url, ssl);

		this.logger.log(caplin.streamlink.LogLevel.INFO, "Using Connection Type: {0}", "PollingConnection");

		this.adjustedURL = url;
		this.sessionId = null;
		this.timer = null;
		
		if (url.indexOf("poll:") == 0)
		{
			this.adjustedURL = "http:" + url.substring(5);
		}
		else if (url.indexOf("polls:") == 0)
		{
			this.adjustedURL = "https:" + url.substring(6);
		}
		
		this.channel = new caplin.streamlink.js.impl.connection.Channel(this.adjustedURL+"/RTTP-TYPE3", context.configuration._getHttpTimeout());

		var self = this;
		this.channel.onload = function(data) {
			if (self.connectionState == ConnectionState.CONNECTING) {
				self.changeState(ConnectionState.CONNECTED);
			}

			if (self.connectionState == ConnectionState.CONNECTED) {
				self.logDataIn(data);
				self.fireData(data);

				self.readyToSend = true;
				self.fireReadyToSend();
			}
		};
		
		this.channel.onerror = function(status, data) {
			self.logger.log(caplin.streamlink.LogLevel.ERROR, "Request failed, code {0} : {1}", status, data);
			self.disconnect();
		};
	}
	
	caplin.extend(PollingConnection, Super);
	var Class = PollingConnection.prototype; 
	
	Class.connect = function() {
		this.changeState(ConnectionState.CONNECTING);
		this.readyToSend = false;
		this.channel.send();
	};
	
	Class.setSessionId = function(sessionId){
		this.sessionId = sessionId;
		this._resetTimer(); // start timer when first connected and sessionId available
	};

	Class.send = function(message) {
		this._sendInternal(message, true);
	};
	
	Class._sendInternal = function(message, log) {
		this.readyToSend = false;
		if (log)
			this.logDataOut(message);
		this.channel.send(message, "application/x-www-form-urlencoded");
		this._resetTimer();
	};
	
	Class._resetTimer = function()
	{
		if (this.timer) {
			clearInterval(this.timer);
			this.timer = null;
		}
		
		if (this.connectionState == ConnectionState.CONNECTED && this.sessionId != null) {
			var self = this;
			this.timer = setInterval(function() {
				if (self.isReadyToSend()) {
					self._sendInternal(self.sessionId, false)
				} else {
					self.logger.log(caplin.streamlink.LogLevel.WARN, "Attempt to send message when not ready.");
					self._resetTimer();
				}
			} , this.context.configuration._getPollInterval());
		}
	};	
	
	Class.disconnect = function() {
		if (this.timer) {
			clearInterval(this.timer);
			this.timer = null;
		}
		this.changeState(ConnectionState.DISCONNECTED);
	};
	
	return PollingConnection;
})();


//## LogListenerOnClient.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.client");

// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver

caplin.streamlink.impl.bridge.worker.client.LogListenerOnClient = function(id, listener) {
	// Constructor body
	this._id = id;
	this._listener = listener;
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.client.LogListenerOnClient, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.client.LogListenerOnClient.prototype.getReceiverKey = function() {
	return this._listener;
};

caplin.streamlink.impl.bridge.worker.client.LogListenerOnClient.prototype.getReceiverId = function() {
	return this._id;
};

caplin.streamlink.impl.bridge.worker.client.LogListenerOnClient.prototype.receive = function(method, args) {
	if ((method == ("onLog"))) {
		this._listener.onLog(new caplin.streamlink.impl.logger.LazyLogInfo(caplin.streamlink.LogLevel.valueOf(args[0]), "W: " + args[1], null));
	}
};



//## CommandListenerOnClient.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.client");

// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver

caplin.streamlink.impl.bridge.worker.client.CommandListenerOnClient = function(id, listener) {
	// Constructor body
	this._id = id;
	this._listener = listener;
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.client.CommandListenerOnClient, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.client.CommandListenerOnClient.prototype.getReceiverKey = function() {
	return this._listener;
};

caplin.streamlink.impl.bridge.worker.client.CommandListenerOnClient.prototype.getReceiverId = function() {
	return this._id;
};

caplin.streamlink.impl.bridge.worker.client.CommandListenerOnClient.prototype.receive = function(method, args) {
	if ((method == ("onCommandOk"))) {
		var subject = args[0];
		(this._listener.onCommandOk && this._listener.onCommandOk(subject));
	}
	else if ((method == ("onCommandError"))) {
		var subject = args[0];
		var commandErrorEvent = caplin.streamlink.impl.event.CommandErrorEventImpl._createFromDictionary(args[1]);
		this._listener.onCommandError(subject, commandErrorEvent);
	}
};



//## LoggerOnClient.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.client");

// caplin.forceload(caplin.streamlink.Logger
// caplin.forceload(caplin.streamlink.impl.logger.LoggerImpl
// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver

caplin.streamlink.impl.bridge.worker.client.LoggerOnClient = function(bridge, id) {
	// Super
	caplin.streamlink.impl.logger.LoggerImpl.call(this, null);
	// Constructor body
	this._bridge = bridge;
	this._id = id;
};

caplin.extend(caplin.streamlink.impl.bridge.worker.client.LoggerOnClient, caplin.streamlink.impl.logger.LoggerImpl);

// caplin.implement(caplin.streamlink.impl.bridge.worker.client.LoggerOnClient, caplin.streamlink.Logger);
// caplin.implement(caplin.streamlink.impl.bridge.worker.client.LoggerOnClient, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.client.LoggerOnClient.prototype.getReceiverKey = function() {
	return null;
};

caplin.streamlink.impl.bridge.worker.client.LoggerOnClient.prototype.getReceiverId = function() {
	return this._id;
};

caplin.streamlink.impl.bridge.worker.client.LoggerOnClient.prototype._logToListener = function(listener, info) {
	listener.onLog(info);
};

caplin.streamlink.impl.bridge.worker.client.LoggerOnClient.prototype.addListener = function(listener, level) {
	caplin.streamlink.impl.logger.LoggerImpl.prototype.addListener.call(this, listener, level);
	var wrappedListener = new caplin.streamlink.impl.bridge.worker.client.LogListenerOnClient(this._bridge.getNextId(), listener);
	this._bridge.addReceiver(wrappedListener);
	var args = [wrappedListener.getReceiverId(), level.toString()];
	this._bridge.send(this._id, "addListener", args);
};

caplin.streamlink.impl.bridge.worker.client.LoggerOnClient.prototype.log = function(level, message) {
	var args = Array.prototype.slice.call(arguments, 2);	caplin.streamlink.impl.logger.LoggerImpl.prototype.log.call(this, level, message, args);
};

caplin.streamlink.impl.bridge.worker.client.LoggerOnClient.prototype.receive = function(method, args) { };

caplin.streamlink.impl.bridge.worker.client.LoggerOnClient.prototype.removeListener = function(listener) {
	caplin.streamlink.impl.logger.LoggerImpl.prototype.removeListener.call(this, listener);
	var receiver = this._bridge.findReceiver(listener);
	if (receiver != null) {
		var args = [receiver.getReceiverId()];
		this._bridge.send(this._id, "removeListener", args);
	}
};



//## Action.js ##//
caplin.namespace("caplin.streamlink.impl.subscription.container");

// caplin.forceload(caplin.streamlink.impl.Serializable

caplin.streamlink.impl.subscription.container.Action = function() {};

// caplin.implement(caplin.streamlink.impl.subscription.container.Action, caplin.streamlink.impl.Serializable);

caplin.streamlink.impl.subscription.container.Action.prototype.applyToList = function(list) { throw new Error('caplin.streamlink.impl.subscription.container.Action.applyToList :  This is an abstract method'); };

caplin.streamlink.impl.subscription.container.Action.prototype.applyToModel = function(model) { throw new Error('caplin.streamlink.impl.subscription.container.Action.applyToModel :  This is an abstract method'); };



//## InsertAction.js ##//
caplin.namespace("caplin.streamlink.impl.subscription.container");

// caplin.forceload(caplin.streamlink.impl.subscription.container.Action

caplin.streamlink.impl.subscription.container.InsertAction = function(obj, at) {
	// Constructor body
	this._obj = obj;
	this._at = at;
};

// caplin.implement(caplin.streamlink.impl.subscription.container.InsertAction, caplin.streamlink.impl.subscription.container.Action);

caplin.streamlink.impl.subscription.container.InsertAction.createFromDictionary = function(root) {
	var at = root["At"];
	var dict = root["Obj"];
	var obj = caplin.streamlink.impl.event.ContainerElementImpl.createFromDictionary(dict);
	return new caplin.streamlink.impl.subscription.container.InsertAction(obj, at);
};

caplin.streamlink.impl.subscription.container.InsertAction.prototype._toDictionary = function() {
	var root = {} ;
	root["actionType"] = "Insert";
	root["At"] = this._at;
	root["Obj"] = this._obj._toDictionary();
	return root;
};

caplin.streamlink.impl.subscription.container.InsertAction.prototype.toString = function() {
	return "Insert " + this._obj.getSubject() + " at " + this._at;
};

caplin.streamlink.impl.subscription.container.InsertAction.prototype.applyToList = function(list) {
	list.splice(this._at, 0, this._obj);
};

caplin.streamlink.impl.subscription.container.InsertAction.prototype.applyToModel = function(model) {
	model.insert(this._at, this._obj);
};



//## ConnectionListenerOnWorker.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.server");

// caplin.forceload(caplin.streamlink.ConnectionListener
// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver

caplin.streamlink.impl.bridge.worker.server.ConnectionListenerOnWorker = function(bridge, id) {
	// Constructor body
	this.bridge = bridge;
	this.id = id;
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.server.ConnectionListenerOnWorker, caplin.streamlink.ConnectionListener);
// caplin.implement(caplin.streamlink.impl.bridge.worker.server.ConnectionListenerOnWorker, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.server.ConnectionListenerOnWorker.prototype.getReceiverKey = function() {
	return null;
};

caplin.streamlink.impl.bridge.worker.server.ConnectionListenerOnWorker.prototype.getReceiverId = function() {
	return this.id;
};

caplin.streamlink.impl.bridge.worker.server.ConnectionListenerOnWorker.prototype.onConnectionFail = function(reason) {
	var args = [reason];
	this.bridge.send(this.id, "onSourceStatusChange", args);
};

caplin.streamlink.impl.bridge.worker.server.ConnectionListenerOnWorker.prototype.onConnectionStatusChange = function(event) {
	var args = [event._toDictionary()];
	this.bridge.send(this.id, "onConnectionStatusChange", args);
};

caplin.streamlink.impl.bridge.worker.server.ConnectionListenerOnWorker.prototype.onServiceStatusChange = function(event) {
	var args = [event._toDictionary()];
	this.bridge.send(this.id, "onServiceStatusChange", args);
};

caplin.streamlink.impl.bridge.worker.server.ConnectionListenerOnWorker.prototype.onSourceStatusChange = function(event) {
	var args = [event._toDictionary()];
	this.bridge.send(this.id, "onSourceStatusChange", args);
};

caplin.streamlink.impl.bridge.worker.server.ConnectionListenerOnWorker.prototype.onStatisticsChange = function(event) {
	var args = [event._toDictionary()];
	this.bridge.send(this.id, "onStatisticsChange", args);
};

caplin.streamlink.impl.bridge.worker.server.ConnectionListenerOnWorker.prototype.receive = function(method, args) { };



//## PasswordCredentialsProvider.js ##//
caplin.namespace("caplin.streamlink.impl.credentials");

// caplin.forceload(caplin.streamlink.CredentialsProvider

caplin.streamlink.impl.credentials.PasswordCredentialsProvider = function(username, password) {
	// Constructor body
	this.username = username;
	this.password = password;
};

// caplin.implement(caplin.streamlink.impl.credentials.PasswordCredentialsProvider, caplin.streamlink.CredentialsProvider);

caplin.streamlink.impl.credentials.PasswordCredentialsProvider.prototype.cancel = function() { };

caplin.streamlink.impl.credentials.PasswordCredentialsProvider.prototype.onCredentialsRequired = function(listener) {
	listener.provideCredentials(new caplin.streamlink.impl.credentials.CredentialsImpl(this.username, this.password));
};



//## Utils.js ##//
caplin.namespace("caplin.streamlink.js.impl");

caplin.streamlink.js.impl.Utils = function()
{
};

caplin.streamlink.js.impl.Utils.getParameter = function(name)
{
	if (typeof window == 'undefined') return null;
	var regEx = new RegExp("[?&]" + name + "=([^&]+)");
	var match = window.location.search.match(regEx);
	return match == null ? null : match[1];
};

caplin.streamlink.js.impl.Utils.isNumber = function(number)
{
	return typeof number === 'number' && isFinite(number);
};

caplin.streamlink.js.impl.Utils.isInteger = function(number)
{
	return caplin.streamlink.js.impl.Utils.isNumber(number) && number % 1 === 0;
};

caplin.streamlink.js.impl.Utils.isArray = function(array)
{
	return Object.prototype.toString.call(array) === '[object Array]';
};

caplin.streamlink.js.impl.Utils.isString = function(text)
{
	return typeof text === 'string';
};

caplin.streamlink.js.impl.Utils.isStringArray = function(array)
{
	var self = caplin.streamlink.js.impl.Utils;
	if (!self.isArray(array))
	{
		return false;
	}
	for (var i = 0; i !== array.length; ++i)
	{
		if (!self.isString(array[i]))
		{
			return false;
		}
	}
	return true;
};

caplin.streamlink.js.impl.Utils.assertString = function (value, attribute)
{
	if (!caplin.streamlink.js.impl.Utils.isString(value))
	{
		throw new Error(attribute + ' must be of type String.');
	}
};

caplin.streamlink.js.impl.Utils.assertInteger = function (value, attribute)
{
	if (!caplin.streamlink.js.impl.Utils.isInteger(value))
	{
		throw new Error(attribute + ' must be of type Integer.');
	}
};

caplin.streamlink.js.impl.Utils.assertBoolean = function (value, attribute)
{
	if (value !== true && value !== false)
	{
		throw new Error(attribute + ' must be of type Boolean.');
	}
};

caplin.streamlink.js.impl.Utils.assertArrayOfStrings = function (value, attribute)
{
	var self = caplin.streamlink.js.impl.Utils;
	if (!caplin.streamlink.js.impl.Utils.isArray(value))
	{
		throw new Error(attribute + ' must be of type [String].');
	}
	for ( var i = 0; i !== value.length; ++i)
	{
		if (typeof value[i] !== 'string')
		{
			throw new Error(attribute + ' must be of type [String].');
		}
	}
};

caplin.streamlink.js.impl.Utils.assertBoolean = function (value, attribute)
{
	if (value !== true && value !== false)
	{
		throw new Error(attribute + ' must be of type Boolean.');
	}
};

caplin.streamlink.js.impl.Utils.assertLogLevel = function (value, attribute)
{
	if (!(value instanceof caplin.streamlink.LogLevel))
	{
		throw new Error(attribute + ' must be of type caplin.streamlink.LogLevel.');
	}
};

caplin.streamlink.js.impl.Utils.isFunction = function (object)
{
	return Object.prototype.toString.call(object) === '[object Function]';
};

caplin.streamlink.js.impl.Utils.assertKeymasterResponseListener = function (value, attribute)
{
	var self = caplin.streamlink.js.impl.Utils;
	if (!value || !self.isFunction(value.onKeymasterError))
	{
		throw new Error(attribute + ' must be of type caplin.streamlink.KeymasterResponseListener.');
	}
};

caplin.streamlink.js.impl.Utils.validateTypeSchema = function(typeSchema, json)
{
	for (var attribute in json)
	{
		if (!json.hasOwnProperty(attribute))
		{
			continue;
		}
		if (!typeSchema[attribute])
		{
			var attributes = [];
			for (var key in typeSchema)
			{
				attributes.push(key);
			}
			throw new Error('Invalid attribute: <' + attribute + '> valid attributes are: ' + attributes.join(", "));
		}
		typeSchema[attribute](json[attribute], attribute);
	}
};


//## RecordType1Event.js ##//
caplin.namespace("caplin.streamlink");

// caplin.forceload(caplin.streamlink.DataEvent

/**
 * @class  <p>Provides information about a change to type 1 data (fieldname and data pairs) in a record.</p>
 *  <p>The <code>RecordType1Event</code> is provided in an <code>onRecordUpdate</code> 
 *  callback to a {@link caplin.streamlink.SubscriptionListener}.</p>
 *  
 *  @extends caplin.streamlink.DataEvent
 * 
 * @name caplin.streamlink.RecordType1Event
 */
caplin.streamlink.RecordType1Event = function() {};

// caplin.implement(caplin.streamlink.RecordType1Event, caplin.streamlink.DataEvent);

/**
 *  <p>Returns the record fields relating to the event, as a collection of key-value pairs.</p>
 * 
 *  <p>When a call to isImage() returns true, the subscription data event contains an image of the data
 *  for the subscribed item. All the fields comprising the record are present in the collection. The application
 *  must discard any values previously received and replace them with the values contained within the
 *  collection.</p>
 *  
 *  <p>When a call to isImage() returns false, the subscription data event contains new data for the subscribed
 *  record. The values in the collection returned are an update to any values previously received and any
 *  application caches should be updated appropriately.</p>
 *  
 *  @returns {Object} the fields
 * @function
 */
caplin.streamlink.RecordType1Event.prototype.getFields = function() { throw new Error('caplin.streamlink.RecordType1Event.getFields :  This is an abstract method'); };

/**
 *  <p>Gets the latency of the record in milliseconds. StreamLink calculates the latency using the clock offset 
 *  and the timestamp supplied in the <c>timeStampField</c> field of the record.</p>
 *  <p>The latency cannot be calculated if <c>timeStampField</c> is not present in the record or if the clock 
 *  offset has not yet been established. For further information, see <a href="./Latency.html">Clock Offset and 
 *  Latency Measurement</a>.</p>
 * @param {String} timeStampField The name of the field in the record containing the timestamp that StreamLink uses to calculate the latency of the record.
 *  @returns {long} The latency of the record in milliseconds, or -1 if the latency cannot be calculated.
 * @function
 */
caplin.streamlink.RecordType1Event.prototype.getLatency = function(timeStampField) { throw new Error('caplin.streamlink.RecordType1Event.getLatency :  This is an abstract method'); };

/**
 *  <p>Returns the time received in milliseconds since 1 January 1970 00:00:00 UTC.</p>
 *  @returns {long} Returns the time received in milliseconds since 1 January 1970 00:00:00 UTC.
 * @function
 */
caplin.streamlink.RecordType1Event.prototype.getTimeReceived = function() { throw new Error('caplin.streamlink.RecordType1Event.getTimeReceived :  This is an abstract method'); };



//## RecordType1EventImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.RecordType1Event
// caplin.forceload(caplin.streamlink.impl.Serializable
// caplin.forceload(caplin.streamlink.DataEvent
// caplin.forceload(caplin.streamlink.impl.event.RttpEventBase

caplin.streamlink.impl.event.RecordType1EventImpl = function(runtime, subject, image, fields, rttpCode, objectNumber, sequenceNumber, clockOffset, timeReceived) {
	// Super
	caplin.streamlink.impl.event.RttpEventBase.call(this, subject, rttpCode, objectNumber, sequenceNumber);
	// Constructor body
	this._image = image;
	this._fields = fields;
	this._clockOffset = clockOffset;
	this._runtime = runtime;
	this._timeReceived = timeReceived;
};

caplin.extend(caplin.streamlink.impl.event.RecordType1EventImpl, caplin.streamlink.impl.event.RttpEventBase);

// caplin.implement(caplin.streamlink.impl.event.RecordType1EventImpl, caplin.streamlink.DataEvent);
// caplin.implement(caplin.streamlink.impl.event.RecordType1EventImpl, caplin.streamlink.RecordType1Event);
// caplin.implement(caplin.streamlink.impl.event.RecordType1EventImpl, caplin.streamlink.impl.Serializable);

caplin.streamlink.impl.event.RecordType1EventImpl.prototype.isImage = function() {
	return this._image;
};

caplin.streamlink.impl.event.RecordType1EventImpl.createFromDictionary = function(runtime, root) {
	var subject = root[caplin.streamlink.impl.event.RttpEventBase.SUBJECT];
	var image = root["Image"];
	var fields = root["Fields"];
	var code = caplin.streamlink.impl.utils.RttpUtil.getRttpCode(root[caplin.streamlink.impl.event.RttpEventBase.CODE]);
	var objectNumber = root[caplin.streamlink.impl.event.RttpEventBase.OBJECT_NUMBER];
	var sequenceNumber = root[caplin.streamlink.impl.event.RttpEventBase.SEQUENCE_NUMBER];
	var clockOffset = root["ClockOffset"];
	var timeReceived = root["TimeReceived"];
	return new caplin.streamlink.impl.event.RecordType1EventImpl(runtime, subject, image, fields, code, objectNumber, sequenceNumber, clockOffset, timeReceived);
};

caplin.streamlink.impl.event.RecordType1EventImpl.createFromRttp = function(rttpCode, tokens, fieldMap, clockOffset, timeReceived, context) {
	switch (rttpCode) {
	case caplin.streamlink.impl.protocolhandler.RttpCode.REC1_UPD:
	{
		var fields = caplin.streamlink.impl.event.ParseUtils.getFields(tokens, fieldMap, context.encoder, 1);
		var objectNumber = tokens[0].substring(4, 8);
		var sequenceNumber = tokens[0].substring(2, 4);
		return new caplin.streamlink.impl.event.RecordType1EventImpl(context.runtime, null, false, fields, rttpCode, objectNumber, sequenceNumber, clockOffset, timeReceived);
	}
	case caplin.streamlink.impl.protocolhandler.RttpCode.RECORD_RESP:
	{
		var subject = context.encoder.rttpDecode(tokens[1]);
		var fields = caplin.streamlink.impl.event.ParseUtils.getFields(tokens, fieldMap, context.encoder, 2);
		var objectNumber = tokens[0].substring(2, 6);
		return new caplin.streamlink.impl.event.RecordType1EventImpl(context.runtime, subject, true, fields, rttpCode, objectNumber, null, clockOffset, timeReceived);
	}
	case caplin.streamlink.impl.protocolhandler.RttpCode.REC1_IMG:
	{
		var fields = caplin.streamlink.impl.event.ParseUtils.getFields(tokens, fieldMap, context.encoder, 1);
		var objectNumber = tokens[0].substring(4, 8);
		var sequenceNumber = tokens[0].substring(2, 4);
		return new caplin.streamlink.impl.event.RecordType1EventImpl(context.runtime, null, true, fields, rttpCode, objectNumber, sequenceNumber, clockOffset, timeReceived);
	}
	case caplin.streamlink.impl.protocolhandler.RttpCode.REC1_CLR:
	{
		var fields = {} ;
		var objectNumber = tokens[0].substring(4, 8);
		var sequenceNumber = tokens[0].substring(2, 4);
		return new caplin.streamlink.impl.event.RecordType1EventImpl(context.runtime, null, true, fields, rttpCode, objectNumber, sequenceNumber, clockOffset, timeReceived);
	}
	default :
	return null;
	}
};

caplin.streamlink.impl.event.RecordType1EventImpl.prototype._toDictionary = function() {
	var root = {} ;
	root["Subject"] = this._subject;
	root["Code"] = this._rttpCode.toString();
	root["ObjectNumber"] = this._rttpObjectNumber;
	root["SequenceNumber"] = this._rttpSequenceNumber;
	root["Image"] = this._image;
	root["Fields"] = this._fields;
	root["ClockOffset"] = this._clockOffset;
	root["TimeReceived"] = this._timeReceived;
	return root;
};

caplin.streamlink.impl.event.RecordType1EventImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("RecordType1DataEvent [subject=");
	builder.push(this._subject);
	builder.push(", fields=[");
	builder.push(caplin.xlang.Utils.dictionaryToString(this._fields));
	builder.push("]");
	builder.push(", image=");
	builder.push(this._image);
	builder.push(", timeReceived=");
	builder.push(this._timeReceived);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.RecordType1EventImpl.prototype.getFields = function() {
	return this._fields;
};

caplin.streamlink.impl.event.RecordType1EventImpl.prototype.getLatency = function(timeStampField) {
	try {
		return Math.max(0, this._timeReceived - this._runtime.parseLong(this._fields[timeStampField]) + this._clockOffset);
	}
	catch (ignore) {
		return -(1);
	}
};

caplin.streamlink.impl.event.RecordType1EventImpl.prototype.getTimeReceived = function() {
	return this._timeReceived;
};

caplin.streamlink.impl.event.RecordType1EventImpl.prototype._publishCommandResponse = function(listener) { };

caplin.streamlink.impl.event.RecordType1EventImpl.prototype._publishSubscriptionResponse = function(subscription, subscriptionListener) {
	(subscriptionListener.onRecordUpdate && subscriptionListener.onRecordUpdate(subscription, this));
};



//## LogListenerOnWorker.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.server");

// caplin.forceload(caplin.streamlink.LogListener
// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver

caplin.streamlink.impl.bridge.worker.server.LogListenerOnWorker = function(bridge, id) {
	// Constructor body
	this.bridge = bridge;
	this.id = id;
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.server.LogListenerOnWorker, caplin.streamlink.LogListener);
// caplin.implement(caplin.streamlink.impl.bridge.worker.server.LogListenerOnWorker, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.server.LogListenerOnWorker.prototype.getReceiverKey = function() {
	return null;
};

caplin.streamlink.impl.bridge.worker.server.LogListenerOnWorker.prototype.getReceiverId = function() {
	return this.id;
};

caplin.streamlink.impl.bridge.worker.server.LogListenerOnWorker.prototype.onLog = function(logInfo) {
	var args = [logInfo.getLevel().toString(), logInfo.getMessage()];
	this.bridge.send(this.id, "onLog", args);
};

caplin.streamlink.impl.bridge.worker.server.LogListenerOnWorker.prototype.receive = function(method, args) { };



//## PacketSync.js ##//
caplin.namespace("caplin.streamlink.impl.packet");

// caplin.forceload(caplin.streamlink.impl.packet.Packet

caplin.streamlink.impl.packet.PacketSync = function(connectionMonitor, latency, clockOffset) {
	// Constructor body
	this.connectionMonitor = connectionMonitor;
	this.latency = latency;
	this.clockOffset = clockOffset;
};

// caplin.implement(caplin.streamlink.impl.packet.PacketSync, caplin.streamlink.impl.packet.Packet);

caplin.streamlink.impl.packet.PacketSync.prototype.getType = function() {
	return caplin.streamlink.impl.packet.PacketType.SYNC;
};

caplin.streamlink.impl.packet.PacketSync.prototype.getSubject = function() {
	return null;
};

caplin.streamlink.impl.packet.PacketSync.prototype.toRttpString = function(encoder) {
	var now = this.connectionMonitor.updateTimeSentFromClient();
	var result = [] ;
	result.push(caplin.streamlink.impl.packet.PacketType.SYNC.getCode());
	result.push(" ");
	result.push(now);
	result.push(" ");
	result.push(this.clockOffset);
	result.push(" ");
	result.push(this.latency);
	return result.join("");
};

caplin.streamlink.impl.packet.PacketSync.prototype.toString = function() {
	var builder = [] ;
	builder.push("Sync [clockOffset=");
	builder.push(this.clockOffset);
	builder.push(", latency=");
	builder.push(this.latency);
	builder.push("]");
	return builder.join("");
};



//## ArraySet.js ##//
caplin.namespace("caplin.streamlink.impl.utils");


caplin.streamlink.impl.utils.ArraySet = function() {
	// Field Definitions
	this.data = [];
};

caplin.streamlink.impl.utils.ArraySet.prototype.get = function(index) {
	return this.data[index];
};

caplin.streamlink.impl.utils.ArraySet.prototype.contains = function(item) {
	return (caplin.xlang.Utils.indexOf(this.data, item)>=0);
};

caplin.streamlink.impl.utils.ArraySet.prototype.isEmpty = function() {
	return (this.data.length==0);
};

caplin.streamlink.impl.utils.ArraySet.prototype.remove = function(item) {
	for (var i = 0; i < this.data.length; (i)++) {
		if ((item == (this.data[i]))) {
			this.data.splice(i,1)[0];
			return true;
		}
	}
	return false;
};

caplin.streamlink.impl.utils.ArraySet.create = function() {
	var data = Array.prototype.slice.call(arguments, 0);	var result = new caplin.streamlink.impl.utils.ArraySet();
	for (var i = 0; i < data.length; (i)++) {
		result.add(data[i]);
	}
	return result;
};

caplin.streamlink.impl.utils.ArraySet.prototype.size = function() {
	return this.data.length;
};

caplin.streamlink.impl.utils.ArraySet.prototype.add = function(item) {
	if (!((caplin.xlang.Utils.indexOf(this.data, item)>=0))) {
		this.data.push(item);
	}
};

caplin.streamlink.impl.utils.ArraySet.prototype.clear = function() {
	(this.data.length = 0);
};



//## ServiceStatusEventImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.impl.Serializable
// caplin.forceload(caplin.streamlink.ServiceStatusEvent

caplin.streamlink.impl.event.ServiceStatusEventImpl = function(serviceStatus, serviceName) {
	// Constructor body
	this._serviceStatus = serviceStatus;
	this._serviceName = serviceName;
};

// caplin.implement(caplin.streamlink.impl.event.ServiceStatusEventImpl, caplin.streamlink.ServiceStatusEvent);
// caplin.implement(caplin.streamlink.impl.event.ServiceStatusEventImpl, caplin.streamlink.impl.Serializable);

caplin.streamlink.impl.event.ServiceStatusEventImpl.prototype.getServiceStatus = function() {
	return this._serviceStatus;
};

caplin.streamlink.impl.event.ServiceStatusEventImpl.createFromDictionary = function(root) {
	return new caplin.streamlink.impl.event.ServiceStatusEventImpl(caplin.streamlink.ServiceStatus.valueOf(root["Status"]), root["Name"]);
};

caplin.streamlink.impl.event.ServiceStatusEventImpl.createFromRttp = function(rttpCode, tokens, fieldMap, encoder) {
	switch (rttpCode) {
	case caplin.streamlink.impl.protocolhandler.RttpCode.SERVICE_OK:
	return new caplin.streamlink.impl.event.ServiceStatusEventImpl(caplin.streamlink.ServiceStatus.OK, encoder.rttpDecode(tokens[1]));
	case caplin.streamlink.impl.protocolhandler.RttpCode.SERVICE_LIMITED:
	return new caplin.streamlink.impl.event.ServiceStatusEventImpl(caplin.streamlink.ServiceStatus.LIMITED, encoder.rttpDecode(tokens[1]));
	case caplin.streamlink.impl.protocolhandler.RttpCode.SERVICE_DOWN:
	return new caplin.streamlink.impl.event.ServiceStatusEventImpl(caplin.streamlink.ServiceStatus.DOWN, encoder.rttpDecode(tokens[1]));
	}
	return null;
};

caplin.streamlink.impl.event.ServiceStatusEventImpl.prototype._toDictionary = function() {
	var root = {} ;
	root["Status"] = this._serviceStatus.toString();
	root["Name"] = this._serviceName;
	return root;
};

caplin.streamlink.impl.event.ServiceStatusEventImpl.prototype.getServiceName = function() {
	return this._serviceName;
};

caplin.streamlink.impl.event.ServiceStatusEventImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("ServiceStatusEventImpl [serviceName=");
	builder.push(this._serviceName);
	builder.push(", serviceStatus=");
	builder.push(this._serviceStatus);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.ServiceStatusEventImpl.prototype.parse = function(rttpCode, tokens, fieldMap, encoder) { };



//## RuntimeImpl.js ##//
caplin.namespace("caplin.streamlink.js.impl");

caplin.streamlink.js.impl.RuntimeImpl = function() 
{
};

caplin.streamlink.js.impl.RuntimeImpl.prototype.createTimeout = function(runnable, milliseconds)
{
	var id = setTimeout(runnable, milliseconds);
	return new caplin.streamlink.js.impl.TimerImpl(id, false);
};

caplin.streamlink.js.impl.RuntimeImpl.prototype.createInterval = function(runnable, milliseconds)
{
	var id = setInterval(runnable, milliseconds);
	return new caplin.streamlink.js.impl.TimerImpl(id, true);
};

caplin.streamlink.js.impl.RuntimeImpl.prototype.currentTimeMillis = Date.now || function()
{
	return new Date().getTime();
};

caplin.streamlink.js.impl.RuntimeImpl.prototype.parseLong = function(number)
{
	var result = parseInt(number, 10);
	if (isNaN(result))
	{
		throw new Error("cannot parse: " + number);
	}
	return result;
};

caplin.streamlink.js.impl.RuntimeImpl.prototype.parseInt = caplin.streamlink.js.impl.RuntimeImpl.prototype.parseLong;




//## CommandParametersImpl.js ##//
caplin.namespace("caplin.streamlink.impl.command");

// caplin.forceload(caplin.streamlink.CommandParameters

caplin.streamlink.impl.command.CommandParametersImpl = function() {
	// Field Definitions
	this._persistent = false;
};

// caplin.implement(caplin.streamlink.impl.command.CommandParametersImpl, caplin.streamlink.CommandParameters);

caplin.streamlink.impl.command.CommandParametersImpl.prototype._isPersistent = function() {
	return this._persistent;
};

caplin.streamlink.impl.command.CommandParametersImpl._createFromDictionary = function(data) {
	var result = new caplin.streamlink.impl.command.CommandParametersImpl();
	if (data["Persistent"]) {
		result.setPersistent();
	}
	return result;
};

caplin.streamlink.impl.command.CommandParametersImpl.prototype.setPersistent = function() {
	this._persistent = true;
	return this;
};

caplin.streamlink.impl.command.CommandParametersImpl.prototype._toDictionary = function() {
	var result = {} ;
	result["Persistent"] = this._persistent;
	return result;
};



//## SubscriptionListener.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  <p>Interface for receiving updates to subscribed data from the Liberator.</p> 
 *  <p>Your application should implement this interface.</p> 
 * 
 *  <p>A trivial implementation of <code>SubscriptionListener</code> would be: </p>
 * 
 *  <p><code><pre>// every possible callback.
 * //
 * // This trivial implementation just logs the update event
 * // for each type of data that can be received.
 * //
 * 
 * streamLink.subscribe("/SUBJECT", {
 *         onSubscriptionStatus : function(subscription, event)
 *         {
 *                 console.log(event);
 *         },
 *         onSubscriptionError : function(subscription, event)
 *         {
 *                 console.log(event);
 *         },
 *         onStoryUpdate : function(subscription, event)
 *         {
 *                 console.log(event);
 *         },
 *         onRecordUpdate : function(subscription, event)
 *         {
 *                 console.log(event);
 *         },
 *         onRecordType2Update : function(subscription, event)
 *         {
 *                 console.log(event);
 *         },
 *         onRecordType3Update : function(subscription, event)
 *         {
 *                 console.log(event);
 *         },
 *         onPermissionUpdate : function(subscription, event)
 *         {
 *                 console.log(event);
 *         },
 *         onNewsUpdate : function(subscription, event)
 *         {
 *                 console.log(event);
 *         },
 *         onDirectoryUpdate : function(subscription, event)
 *         {
 *                 console.log(event);
 *         },
 *         onContainerUpdate : function(subscription, event)
 *         {
 *                 console.log(event);
 *         },
 *         onChatUpdate : function(subscription, event)
 *         {
 *                 console.log(event);
 *         }
 *         
 * }, null);
 * </pre></code></p>
 * 
 * @name caplin.streamlink.SubscriptionListener
 */
caplin.streamlink.SubscriptionListener = function() {};

/**
 *  <p>Called when a chat update is received. Implementing this method is optional.</p>
 * @param {caplin.streamlink.Subscription} subscription The subscription for which the update occurred.
 * @param {caplin.streamlink.ChatEvent} event The chat update.
 * @function
 */
caplin.streamlink.SubscriptionListener.prototype.onChatUpdate = function(subscription, event) { throw new Error('caplin.streamlink.SubscriptionListener.onChatUpdate :  This is an abstract method'); };

/**
 *  <p>Called when a container update is received. Implementing this method is optional.</p>
 * @param {caplin.streamlink.Subscription} subscription The subscription for which the update occurred.
 * @param {caplin.streamlink.ContainerEvent} event The container update.
 * @function
 */
caplin.streamlink.SubscriptionListener.prototype.onContainerUpdate = function(subscription, event) { throw new Error('caplin.streamlink.SubscriptionListener.onContainerUpdate :  This is an abstract method'); };

/**
 *  <p>Called when a directory update is received. Implementing this method is optional.</p>
 * @param {caplin.streamlink.Subscription} subscription The subscription for which the update occurred.
 * @param {caplin.streamlink.DirectoryEvent} event The directory update.
 * @function
 */
caplin.streamlink.SubscriptionListener.prototype.onDirectoryUpdate = function(subscription, event) { throw new Error('caplin.streamlink.SubscriptionListener.onDirectoryUpdate :  This is an abstract method'); };

/**
 *  <p>Called when a news headline update is received. Implementing this method is optional.</p>
 * @param {caplin.streamlink.Subscription} subscription The subscription for which the update occurred.
 * @param {caplin.streamlink.NewsEvent} event The news headline update.
 * @function
 */
caplin.streamlink.SubscriptionListener.prototype.onNewsUpdate = function(subscription, event) { throw new Error('caplin.streamlink.SubscriptionListener.onNewsUpdate :  This is an abstract method'); };

/**
 *  <p>Called when a permission update is received. Implementing this method is optional.</p>
 * @param {caplin.streamlink.Subscription} subscription The subscription for which the update occurred.
 * @param {caplin.streamlink.PermissionEvent} event The permission update.
 * @function
 */
caplin.streamlink.SubscriptionListener.prototype.onPermissionUpdate = function(subscription, event) { throw new Error('caplin.streamlink.SubscriptionListener.onPermissionUpdate :  This is an abstract method'); };

/**
 *  <p>Called when an update to type 2 data in a record is received. Implementing this method is optional.</p>
 * @param {caplin.streamlink.Subscription} subscription The subscription for which the update occurred.
 * @param {caplin.streamlink.RecordType2Event} event The record update.
 * @function
 */
caplin.streamlink.SubscriptionListener.prototype.onRecordType2Update = function(subscription, event) { throw new Error('caplin.streamlink.SubscriptionListener.onRecordType2Update :  This is an abstract method'); };

/**
 *  <p>Called when an update to type 3 data in a record is received. Implementing this method is optional.</p>
 * @param {caplin.streamlink.Subscription} subscription The subscription for which the update occurred.
 * @param {caplin.streamlink.RecordType3Event} event The record update.
 * @function
 */
caplin.streamlink.SubscriptionListener.prototype.onRecordType3Update = function(subscription, event) { throw new Error('caplin.streamlink.SubscriptionListener.onRecordType3Update :  This is an abstract method'); };

/**
 *  <p>Called when an update to type 1 data in a record is received. Implementing this method is optional.</p>
 * @param {caplin.streamlink.Subscription} subscription The subscription for which the update occurred.
 * @param {caplin.streamlink.RecordType1Event} event The record update.
 * @function
 */
caplin.streamlink.SubscriptionListener.prototype.onRecordUpdate = function(subscription, event) { throw new Error('caplin.streamlink.SubscriptionListener.onRecordUpdate :  This is an abstract method'); };

/**
 *  <p>Called when a news story update is received. Implementing this method is optional.</p>
 * @param {caplin.streamlink.Subscription} subscription The subscription for which the update occurred.
 * @param {caplin.streamlink.StoryEvent} event The news story update.
 * @function
 */
caplin.streamlink.SubscriptionListener.prototype.onStoryUpdate = function(subscription, event) { throw new Error('caplin.streamlink.SubscriptionListener.onStoryUpdate :  This is an abstract method'); };

/**
 *  <p>Called when there is an error in a subscription.</p>
 * @param {caplin.streamlink.Subscription} subscription The subscription for which the update occurred.
 * @param {caplin.streamlink.SubscriptionErrorEvent} event The subscription error event.
 * @function
 */
caplin.streamlink.SubscriptionListener.prototype.onSubscriptionError = function(subscription, event) { throw new Error('caplin.streamlink.SubscriptionListener.onSubscriptionError :  This is an abstract method'); };

/**
 *  <p>Called when there is a change in the status of a subscription.</p>
 * @param {caplin.streamlink.Subscription} subscription The subscription for which the update occurred.
 * @param {caplin.streamlink.SubscriptionStatusEvent} event The subscription status event.
 * @function
 */
caplin.streamlink.SubscriptionListener.prototype.onSubscriptionStatus = function(subscription, event) { throw new Error('caplin.streamlink.SubscriptionListener.onSubscriptionStatus :  This is an abstract method'); };



//## SubscriptionListenerOnWorker.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.server");

// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver
// caplin.forceload(caplin.streamlink.SubscriptionListener

caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker = function(bridge, subscriptionId, id) {
	// Constructor body
	this.bridge = bridge;
	this.id = id;
	this.subscriptionId = subscriptionId;
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker, caplin.streamlink.SubscriptionListener);
// caplin.implement(caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker.prototype.getReceiverKey = function() {
	return null;
};

caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker.prototype.getReceiverId = function() {
	return this.id;
};

caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker.prototype.onChatUpdate = function(subscription, event) {
	var args = [this.subscriptionId, event._toDictionary()];
	this.bridge.send(this.id, "onStoryUpdate", args);
};

caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker.prototype.onContainerUpdate = function(subscription, event) {
	var args = [this.subscriptionId, event._toDictionary()];
	this.bridge.send(this.id, "onContainerUpdate", args);
};

caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker.prototype.onDirectoryUpdate = function(subscription, event) {
	var args = [this.subscriptionId, event._toDictionary()];
	this.bridge.send(this.id, "onDirectoryUpdate", args);
};

caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker.prototype.onNewsUpdate = function(subscription, event) {
	var args = [this.subscriptionId, event._toDictionary()];
	this.bridge.send(this.id, "onNewsUpdate", args);
};

caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker.prototype.onPermissionUpdate = function(subscription, event) {
	var args = [this.subscriptionId, event._toDictionary()];
	this.bridge.send(this.id, "onPermissionUpdate", args);
};

caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker.prototype.onRecordType2Update = function(subscription, event) {
	var args = [this.subscriptionId, event._toDictionary()];
	this.bridge.send(this.id, "onRecordType2Update", args);
};

caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker.prototype.onRecordType3Update = function(subscription, event) {
	var args = [this.subscriptionId, event._toDictionary()];
	this.bridge.send(this.id, "onRecordType3Update", args);
};

caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker.prototype.onRecordUpdate = function(subscription, event) {
	var args = [this.subscriptionId, event._toDictionary()];
	this.bridge.send(this.id, "onRecordUpdate", args);
};

caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker.prototype.onStoryUpdate = function(subscription, event) {
	var args = [this.subscriptionId, event._toDictionary()];
	this.bridge.send(this.id, "onStoryUpdate", args);
};

caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker.prototype.onSubscriptionError = function(subscription, event) {
	var args = [this.subscriptionId, event._toDictionary()];
	this.bridge.send(this.id, "onSubscriptionError", args);
};

caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker.prototype.onSubscriptionStatus = function(subscription, event) {
	var args = [this.subscriptionId, event._toDictionary()];
	this.bridge.send(this.id, "onSubscriptionStatus", args);
};

caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker.prototype.receive = function(method, args) { };



//## KeymasterResponseListenerOnWorker.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.server");

// caplin.forceload(caplin.streamlink.KeymasterResponseListener
// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver

caplin.streamlink.impl.bridge.worker.server.KeymasterResponseListenerOnWorker = function(bridge, keymasterResponseListenerId) {
	// Constructor body
	this.bridge = bridge;
	this.keymasterResponseListenerId = keymasterResponseListenerId;
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.server.KeymasterResponseListenerOnWorker, caplin.streamlink.KeymasterResponseListener);
// caplin.implement(caplin.streamlink.impl.bridge.worker.server.KeymasterResponseListenerOnWorker, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.server.KeymasterResponseListenerOnWorker.prototype.getReceiverKey = function() {
	return null;
};

caplin.streamlink.impl.bridge.worker.server.KeymasterResponseListenerOnWorker.prototype.getReceiverId = function() {
	return this.keymasterResponseListenerId;
};

caplin.streamlink.impl.bridge.worker.server.KeymasterResponseListenerOnWorker.prototype.onKeymasterError = function(reason) {
	this.bridge.send(this.keymasterResponseListenerId, "onKeymasterError", [reason]);
};

caplin.streamlink.impl.bridge.worker.server.KeymasterResponseListenerOnWorker.prototype.onKeymasterResponse = function(response) {
	this.bridge.send(this.keymasterResponseListenerId, "onKeymasterResponse", [response]);
};

caplin.streamlink.impl.bridge.worker.server.KeymasterResponseListenerOnWorker.prototype.receive = function(method, args) { };



//## SubscriptionManager.js ##//
caplin.namespace("caplin.streamlink.impl.subscription");

// caplin.forceload(caplin.streamlink.impl.protocolhandler.ProtocolListener

caplin.streamlink.impl.subscription.SubscriptionManager = function(protocolHandler, currentSubscriptions, logger) {
	// Field Definitions
	this.persistBeforeResubscribing = [];
	this.persistAfterResubscribing = [];
	this.responseQueue = [];
	this.requestQueue = [];
	this.pendingUnsubscribes = caplin.streamlink.impl.utils.ArraySet.create();

	// Constructor body
	this.protocolHandler = protocolHandler;
	this.logger = logger;
	this.subscriptions = currentSubscriptions;
	this.loggedIn = false;
};

// caplin.implement(caplin.streamlink.impl.subscription.SubscriptionManager, caplin.streamlink.impl.protocolhandler.ProtocolListener);

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.getNextType = function() {
	return this.requestQueue[0].getPacket().getType();
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.indexOf = function(queue, commandResponse) {
	for (var i = 0; i < queue.length; (i)++) {
		if (commandResponse == queue[i].getHandler()) {
			return i;
		}
	}
	return -(1);
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.addAutoSubscription = function(eventHandler, objectNumber, subject) {
	var message = new caplin.streamlink.impl.subscription.Message(eventHandler, new caplin.streamlink.impl.packet.PacketAuto(subject));
	this.subscriptions.set(objectNumber, message);
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.addObjectNumber = function(objectNumber, message) {
	this.subscriptions.set(objectNumber, message);
	var handler = message.getHandler();
	if ((handler instanceof caplin.streamlink.impl.subscription.SubscriptionImpl)) {
		handler._setRttpObjectNumber(objectNumber);
	}
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.addPendingMessages = function(too, from, addCommands) {
	for (var i = 0; i < from.length; (i)++) {
		var message = from[i];
		var handler = message.getHandler();
		var isCommandResponse = (handler instanceof caplin.streamlink.impl.command.CommandResponse);
		if (isCommandResponse) {
			var commandResponse = handler;
			if (!(commandResponse.isUnpersist()) && this.indexOf(too, handler) == -(1)) {
				if (addCommands) {
					too.push(from[i]);
				}
				else {
					commandResponse.disconnectOccurred(message.getPacket().getSubject());
				}
			}
		}
		else if (!(isCommandResponse)) {
			too.push(from[i]);
		}
	}
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.clearDown = function(newPendingQueue) {
	(this.responseQueue.length = 0);
	this.pendingUnsubscribes.clear();
	this.subscriptions.clear();
	this.requestQueue = newPendingQueue;
	this.clearDownSubscriptions();
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.clearDownSubscriptions = function() {
	for (var i = 0; i < this.requestQueue.length; (i)++) {
		var eventHandler = this.requestQueue[i].getHandler();
		if ((eventHandler instanceof caplin.streamlink.impl.subscription.SubscriptionImpl)) {
			var subscription = eventHandler;
			subscription._clear();
		}
	}
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.logEvent = function(event) {
	if (!((event instanceof caplin.streamlink.impl.event.ContainerEventInternal))) {
		this.logger.log(caplin.streamlink.LogLevel.FINE, "{0}", event);
	}
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.onDisconnect = function() {
	if (this.loggedIn) {
		this.loggedIn = false;
		this.updateSubscriptionStatus(caplin.streamlink.SubscriptionStatusType.STATUS_STALE, caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_STALE);
	}
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.onLogin = function() {
	this.loggedIn = true;
	var newPendingQueue = [];
	this.addPendingMessages(newPendingQueue, this.persistBeforeResubscribing, true);
	this.addPendingMessages(newPendingQueue, this.subscriptions.getAllNonAutoMessages(), false);
	this.addPendingMessages(newPendingQueue, this.responseQueue, false);
	this.addPendingMessages(newPendingQueue, this.requestQueue, true);
	this.addPendingMessages(newPendingQueue, this.persistAfterResubscribing, true);
	this.clearDown(newPendingQueue);
	this.sendQueued();
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.onReconnect = function() {
	this.loggedIn = true;
	this.updateSubscriptionStatus(caplin.streamlink.SubscriptionStatusType.STATUS_OK, caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_OK);
	this.sendQueued();
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.onResponse = function(event) {
	this.logEvent(event);
	var message = this.responseQueue.shift();
	if (message == null) {
		this.logger.log(caplin.streamlink.LogLevel.ERROR, "Received response without a request - Event: " + event);
		return;
	}
	var handler = message.getHandler();
	var packet = message.getPacket();
	if (event._hasRttpObjectNumber()) {
		this.addObjectNumber(event._getRttpObjectNumber(), message);
	}
	if (packet.getType() == caplin.streamlink.impl.packet.PacketType.REQUEST) {
		event._setSubject(packet.getSubject());
		if (this.pendingUnsubscribes.remove(handler)) {
			this.sendQueued();
		}
	}
	handler._handleEvent(event);
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.onTerminalUpdate = function(event) {
	this.logEvent(event);
	var messages = this.subscriptions.getMessages(event._getRttpObjectNumber());
	if (messages == null) {
		return;
	}
	var packet = messages[0].getPacket();
	event._setSubject(packet.getSubject());
	this.sendToHandlers(messages, event);
	this.subscriptions.removeAll(event._getRttpObjectNumber());
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.onUpdate = function(event) {
	this.logEvent(event);
	var messages = this.subscriptions.getMessages(event._getRttpObjectNumber());
	if (messages == null) {
		return;
	}
	var packet = messages[0].getPacket();
	event._setSubject(packet.getSubject());
	this.sendToHandlers(messages, event);
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.processDiscard = function() {
	var subscription = this.requestQueue[0].getHandler();
	var objectNumber = subscription._getRttpObjectNumber();
	if (objectNumber == null) {
		this.pendingUnsubscribes.add(subscription);
	}
	else {
		this.subscriptions.remove(objectNumber, subscription);
		if (this.subscriptions.containsMessages(objectNumber)) {
			this.requestQueue.shift();
		}
		else {
			this.sendNext();
		}
	}
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.removeAutoSubscription = function(eventHandler, objectNumber) {
	this.subscriptions.remove(objectNumber, eventHandler);
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.removeFromQueue = function(persist, commandResponse) {
	var index = this.indexOf(persist, commandResponse);
	if (index != -(1)) {
		persist.splice(index,1)[0];
	}
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.send = function(handler, packet) {
	this.requestQueue.push(new caplin.streamlink.impl.subscription.Message(handler, packet));
	this.sendQueued();
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.sendNext = function() {
	var message = this.requestQueue.shift();
	if (message.getPacket().getType().isResponseRequired()) {
		this.responseQueue.push(message);
	}
	this.protocolHandler.sendPacket(message.getPacket());
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.sendPersist = function(commandResponse, packet, persist) {
	var message = new caplin.streamlink.impl.subscription.Message(commandResponse, packet);
	persist.push(message);
	this.requestQueue.push(message);
	this.sendQueued();
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.sendPersistAfterCommand = function(commandResponse, packet) {
	this.sendPersist(commandResponse, packet, this.persistAfterResubscribing);
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.sendPersistBeforeCommand = function(commandResponse, packet) {
	this.sendPersist(commandResponse, packet, this.persistBeforeResubscribing);
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.sendQueued = function() {
	var state = this.protocolHandler.getConnectionState();
	if (state != caplin.streamlink.ConnectionState.LOGGEDIN && state != caplin.streamlink.ConnectionState.RECONNECTED) {
		return;
	}
	while ((this.requestQueue.length==0) == false && this.pendingUnsubscribes.isEmpty()) {
		if (this.getNextType() == caplin.streamlink.impl.packet.PacketType.DISCARD) {
			this.processDiscard();
		}
		else {
			this.sendNext();
		}
	}
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.sendToHandlers = function(messages, event) {
	for (var i = 0; i < messages.length; (i)++) {
		messages[i].getHandler()._handleEvent(event);
	}
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.unPersist = function(commandResponse) {
	this.removeFromQueue(this.persistBeforeResubscribing, commandResponse);
	this.removeFromQueue(this.persistAfterResubscribing, commandResponse);
};

caplin.streamlink.impl.subscription.SubscriptionManager.prototype.updateSubscriptionStatus = function(type, code) {
	var requests = this.subscriptions.getAllMessages();
	for (var i = 0; i < requests.length; (i)++) {
		var message = requests[i];
		var statusEvent = new caplin.streamlink.impl.event.SubjectStatusEventImpl(message.getPacket().getSubject(), type, code, null, null);
		this.logEvent(statusEvent);
		message.getHandler()._handleEvent(statusEvent);
	}
};



//## RttpCode.js ##//
caplin.namespace("caplin.streamlink.impl.protocolhandler");

caplin.streamlink.impl.protocolhandler.RttpCode = function(name, description) {
	// Constructor body
	this.name = name;
	this.description = description;
};

caplin.streamlink.impl.protocolhandler.RttpCode.REQUEST_ACK = new caplin.streamlink.impl.protocolhandler.RttpCode("00", "Request Ack");
caplin.streamlink.impl.protocolhandler.RttpCode.CONNECT_OK = new caplin.streamlink.impl.protocolhandler.RttpCode("01", "Connection greeting message");
caplin.streamlink.impl.protocolhandler.RttpCode.FIELD_MAP = new caplin.streamlink.impl.protocolhandler.RttpCode("0o", "Field name mapping");
caplin.streamlink.impl.protocolhandler.RttpCode.SESSION_EJECTED = new caplin.streamlink.impl.protocolhandler.RttpCode("1Q", "Server has ejected your session");
caplin.streamlink.impl.protocolhandler.RttpCode.LOGIN_OK = new caplin.streamlink.impl.protocolhandler.RttpCode("1b", "Login ok, capabilities are a comma separated list of key value pairs of the form <name>=<value>");
caplin.streamlink.impl.protocolhandler.RttpCode.RECON_OK = new caplin.streamlink.impl.protocolhandler.RttpCode("1l", "Recon ok, reconnected to previous session");
caplin.streamlink.impl.protocolhandler.RttpCode.RESP_UNKNOWN = new caplin.streamlink.impl.protocolhandler.RttpCode("38", "Blank Response (object being requested from an active datasrc)");
caplin.streamlink.impl.protocolhandler.RttpCode.DIRECTORY_RESP = new caplin.streamlink.impl.protocolhandler.RttpCode("3S", "Directory object");
caplin.streamlink.impl.protocolhandler.RttpCode.PAGE_RESP = new caplin.streamlink.impl.protocolhandler.RttpCode("3T", "Page object");
caplin.streamlink.impl.protocolhandler.RttpCode.RECORD_RESP = new caplin.streamlink.impl.protocolhandler.RttpCode("3U", "Record object (type 1 fields only)");
caplin.streamlink.impl.protocolhandler.RttpCode.NEWS_RESP = new caplin.streamlink.impl.protocolhandler.RttpCode("3V", "News headline object");
caplin.streamlink.impl.protocolhandler.RttpCode.STORY_RESP = new caplin.streamlink.impl.protocolhandler.RttpCode("3W", "News story object");
caplin.streamlink.impl.protocolhandler.RttpCode.CHAT_RESP = new caplin.streamlink.impl.protocolhandler.RttpCode("3Z", "Chat object");
caplin.streamlink.impl.protocolhandler.RttpCode.CONT_RESP = new caplin.streamlink.impl.protocolhandler.RttpCode("3a", "Container object");
caplin.streamlink.impl.protocolhandler.RttpCode.AUTODIR_RESP = new caplin.streamlink.impl.protocolhandler.RttpCode("3b", "Auto Directory Object");
caplin.streamlink.impl.protocolhandler.RttpCode.PERM_RESP = new caplin.streamlink.impl.protocolhandler.RttpCode("3c", "Permissions Object");
caplin.streamlink.impl.protocolhandler.RttpCode.CONTRIB_OK = new caplin.streamlink.impl.protocolhandler.RttpCode("4j", "Contrib command ok");
caplin.streamlink.impl.protocolhandler.RttpCode.DISCARD_OK = new caplin.streamlink.impl.protocolhandler.RttpCode("4k", "Discard command ok");
caplin.streamlink.impl.protocolhandler.RttpCode.CREATE_OK = new caplin.streamlink.impl.protocolhandler.RttpCode("4l", "Create command ok");
caplin.streamlink.impl.protocolhandler.RttpCode.DELETE_OK = new caplin.streamlink.impl.protocolhandler.RttpCode("4m", "Delete command ok");
caplin.streamlink.impl.protocolhandler.RttpCode.NOOP_OK = new caplin.streamlink.impl.protocolhandler.RttpCode("4n", "Noop command ok");
caplin.streamlink.impl.protocolhandler.RttpCode.LOGOUT_OK = new caplin.streamlink.impl.protocolhandler.RttpCode("4o", "Logout command ok");
caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_OK = new caplin.streamlink.impl.protocolhandler.RttpCode("4p", "Throttle command ok");
caplin.streamlink.impl.protocolhandler.RttpCode.SYNC_OK = new caplin.streamlink.impl.protocolhandler.RttpCode("4r", "Clock sync ok");
caplin.streamlink.impl.protocolhandler.RttpCode.CONTRIB_WAIT = new caplin.streamlink.impl.protocolhandler.RttpCode("4t", "Contrib wait");
caplin.streamlink.impl.protocolhandler.RttpCode.CONTRIB_OK_DELAY = new caplin.streamlink.impl.protocolhandler.RttpCode("5V", "Contrib command ok, delayed");
caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_OK = new caplin.streamlink.impl.protocolhandler.RttpCode("6V", "Object Status OK");
caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_STALE = new caplin.streamlink.impl.protocolhandler.RttpCode("6W", "Object Status Stale, some required sources are down");
caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_LIMITED = new caplin.streamlink.impl.protocolhandler.RttpCode("6X", "Object Status Limited, some optional sources are down");
caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_REMOVED = new caplin.streamlink.impl.protocolhandler.RttpCode("6Y", "Object Status Removed, subscription removed");
caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_INFO = new caplin.streamlink.impl.protocolhandler.RttpCode("6Z", "Object Status Info, message only");
caplin.streamlink.impl.protocolhandler.RttpCode.DIR_UPD = new caplin.streamlink.impl.protocolhandler.RttpCode("6a", "Directory update");
caplin.streamlink.impl.protocolhandler.RttpCode.PAGE_UPD = new caplin.streamlink.impl.protocolhandler.RttpCode("6b", "Page update");
caplin.streamlink.impl.protocolhandler.RttpCode.REC1_UPD = new caplin.streamlink.impl.protocolhandler.RttpCode("6c", "Record type 1 update");
caplin.streamlink.impl.protocolhandler.RttpCode.NEWS_UPD = new caplin.streamlink.impl.protocolhandler.RttpCode("6d", "News headline");
caplin.streamlink.impl.protocolhandler.RttpCode.STORY_UPD = new caplin.streamlink.impl.protocolhandler.RttpCode("6e", "News story update");
caplin.streamlink.impl.protocolhandler.RttpCode.REC2_UPD = new caplin.streamlink.impl.protocolhandler.RttpCode("6f", "Record type 2 update");
caplin.streamlink.impl.protocolhandler.RttpCode.REC3_UPD = new caplin.streamlink.impl.protocolhandler.RttpCode("6g", "Record type 3 update");
caplin.streamlink.impl.protocolhandler.RttpCode.CHAT_UPD = new caplin.streamlink.impl.protocolhandler.RttpCode("6h", "Chat update");
caplin.streamlink.impl.protocolhandler.RttpCode.CONT_UPD = new caplin.streamlink.impl.protocolhandler.RttpCode("6i", "Container update");
caplin.streamlink.impl.protocolhandler.RttpCode.AUTODIR_UPD = new caplin.streamlink.impl.protocolhandler.RttpCode("6j", "Autodir Update");
caplin.streamlink.impl.protocolhandler.RttpCode.PERM_UPD = new caplin.streamlink.impl.protocolhandler.RttpCode("6k", "Permissions Update");
caplin.streamlink.impl.protocolhandler.RttpCode.REC1_IMG = new caplin.streamlink.impl.protocolhandler.RttpCode("7O", "Record type 1 cache/image data");
caplin.streamlink.impl.protocolhandler.RttpCode.NEWS_IMG = new caplin.streamlink.impl.protocolhandler.RttpCode("7P", "News headline from cache");
caplin.streamlink.impl.protocolhandler.RttpCode.STORY_IMG = new caplin.streamlink.impl.protocolhandler.RttpCode("7Q", "News story image");
caplin.streamlink.impl.protocolhandler.RttpCode.REC2_IMG = new caplin.streamlink.impl.protocolhandler.RttpCode("7R", "Record type 2 cache/image data");
caplin.streamlink.impl.protocolhandler.RttpCode.REC3_IMG = new caplin.streamlink.impl.protocolhandler.RttpCode("7S", "Record type 3 cache/image data");
caplin.streamlink.impl.protocolhandler.RttpCode.CONT_IMG = new caplin.streamlink.impl.protocolhandler.RttpCode("7U", "Container cache/image data");
caplin.streamlink.impl.protocolhandler.RttpCode.AUTODIR_IMG = new caplin.streamlink.impl.protocolhandler.RttpCode("7V", "Autodir cache/image data");
caplin.streamlink.impl.protocolhandler.RttpCode.PERM_IMG = new caplin.streamlink.impl.protocolhandler.RttpCode("72", "(new) Permissions cache/image data");
caplin.streamlink.impl.protocolhandler.RttpCode.REC1_CLR = new caplin.streamlink.impl.protocolhandler.RttpCode("7Y", "Record type 1 clear cache");
caplin.streamlink.impl.protocolhandler.RttpCode.NEWS_CLR = new caplin.streamlink.impl.protocolhandler.RttpCode("7F", "(new) News headline clear cache");
caplin.streamlink.impl.protocolhandler.RttpCode.REC2_CLR = new caplin.streamlink.impl.protocolhandler.RttpCode("7b", "Record type 2 clear cache");
caplin.streamlink.impl.protocolhandler.RttpCode.REC3_CLR = new caplin.streamlink.impl.protocolhandler.RttpCode("7c", "Record type 3 clear cache");
caplin.streamlink.impl.protocolhandler.RttpCode.PERM_CLR = new caplin.streamlink.impl.protocolhandler.RttpCode("7M", "(new) Permission Object clear cache");
caplin.streamlink.impl.protocolhandler.RttpCode.REC1_DEL = new caplin.streamlink.impl.protocolhandler.RttpCode("7i", "Record type 1 field delete (Unused)");
caplin.streamlink.impl.protocolhandler.RttpCode.REC2_DEL = new caplin.streamlink.impl.protocolhandler.RttpCode("7l", "Record type 2 entry delete");
caplin.streamlink.impl.protocolhandler.RttpCode.REC3_DEL = new caplin.streamlink.impl.protocolhandler.RttpCode("7m", "Record type 3 entry delete (Unused)");
caplin.streamlink.impl.protocolhandler.RttpCode.PERM_DEL = new caplin.streamlink.impl.protocolhandler.RttpCode("7g", "(new) Permission Object delete entry");
caplin.streamlink.impl.protocolhandler.RttpCode.MISC_HEARTBEAT = new caplin.streamlink.impl.protocolhandler.RttpCode("7r", "Heartbeat message");
caplin.streamlink.impl.protocolhandler.RttpCode.INIT_HEARTBEAT = new caplin.streamlink.impl.protocolhandler.RttpCode("7s", "Heartbeat initialisation");
caplin.streamlink.impl.protocolhandler.RttpCode.TIMED_HEARTBEAT = new caplin.streamlink.impl.protocolhandler.RttpCode("7t", "Timed Heartbeat");
caplin.streamlink.impl.protocolhandler.RttpCode.SOURCE_UP = new caplin.streamlink.impl.protocolhandler.RttpCode("7_", "DataSource up message");
caplin.streamlink.impl.protocolhandler.RttpCode.SOURCE_DOWN = new caplin.streamlink.impl.protocolhandler.RttpCode("80", "DataSource down message");
caplin.streamlink.impl.protocolhandler.RttpCode.SOURCE_WARN = new caplin.streamlink.impl.protocolhandler.RttpCode("81", "DataSource status/warning message");
caplin.streamlink.impl.protocolhandler.RttpCode.SERVICE_OK = new caplin.streamlink.impl.protocolhandler.RttpCode("83", "Data Service ok message");
caplin.streamlink.impl.protocolhandler.RttpCode.SERVICE_DOWN = new caplin.streamlink.impl.protocolhandler.RttpCode("84", "Data Service down message");
caplin.streamlink.impl.protocolhandler.RttpCode.SERVICE_LIMITED = new caplin.streamlink.impl.protocolhandler.RttpCode("85", "Data Service limited message");
caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_MSG = new caplin.streamlink.impl.protocolhandler.RttpCode("88", "Liberator status message");
caplin.streamlink.impl.protocolhandler.RttpCode.WARNING_MSG = new caplin.streamlink.impl.protocolhandler.RttpCode("8I", "Liberator warning message");
caplin.streamlink.impl.protocolhandler.RttpCode.ERROR_MSG = new caplin.streamlink.impl.protocolhandler.RttpCode("8S", "Liberator error message");
caplin.streamlink.impl.protocolhandler.RttpCode.NOT_FOUND_DELAY = new caplin.streamlink.impl.protocolhandler.RttpCode("8c", "Active object not found");
caplin.streamlink.impl.protocolhandler.RttpCode.READ_DENY_DELAY = new caplin.streamlink.impl.protocolhandler.RttpCode("8m", "Active object read access denied");
caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY_DELAY = new caplin.streamlink.impl.protocolhandler.RttpCode("8w", "Active object write access denied");
caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY_TRADE_GROUP_LICENSE_EXCEEDED_DELAY = new caplin.streamlink.impl.protocolhandler.RttpCode("90", "Attempt to perform a trade by contributing when trade group license exceeded.");
caplin.streamlink.impl.protocolhandler.RttpCode.UNAVAILABLE_DELAY = new caplin.streamlink.impl.protocolhandler.RttpCode("94", "Active object unavailable");
caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_PARAMETERS_DELAY = new caplin.streamlink.impl.protocolhandler.RttpCode("95", "Active object invalid parameters");
caplin.streamlink.impl.protocolhandler.RttpCode.DELETED_DELAY = new caplin.streamlink.impl.protocolhandler.RttpCode("9E", "Object Deleted");
caplin.streamlink.impl.protocolhandler.RttpCode.NOT_FOUND = new caplin.streamlink.impl.protocolhandler.RttpCode("9O", "Object not found");
caplin.streamlink.impl.protocolhandler.RttpCode.READ_DENY = new caplin.streamlink.impl.protocolhandler.RttpCode("9Y", "Object read access denied");
caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY = new caplin.streamlink.impl.protocolhandler.RttpCode("9i", "Object write access denied");
caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY_TRADE_GROUP_LICENSE_EXCEEDED = new caplin.streamlink.impl.protocolhandler.RttpCode("9o", "Attempt to perform a trade by contributing when trade group license exceeded.");
caplin.streamlink.impl.protocolhandler.RttpCode.UNAVAILABLE = new caplin.streamlink.impl.protocolhandler.RttpCode("9s", "Object unavailable");
caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_PARAMETERS = new caplin.streamlink.impl.protocolhandler.RttpCode("9t", "Object invalid parameters");
caplin.streamlink.impl.protocolhandler.RttpCode.CONTRIB_FAILED = new caplin.streamlink.impl.protocolhandler.RttpCode("AA", "Contrib failed (unused)");
caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_FAILED = new caplin.streamlink.impl.protocolhandler.RttpCode("AK", "Throttle failed");
caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_UP_FAILED = new caplin.streamlink.impl.protocolhandler.RttpCode("AL", "Throttle Up failed");
caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_DOWN_FAILED = new caplin.streamlink.impl.protocolhandler.RttpCode("AM", "Throttle Down failed");
caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_MIN_FAILED = new caplin.streamlink.impl.protocolhandler.RttpCode("AN", "Throttle Min failed");
caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_MAX_FAILED = new caplin.streamlink.impl.protocolhandler.RttpCode("AO", "Throttle Max failed");
caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_STOP_FAILED = new caplin.streamlink.impl.protocolhandler.RttpCode("AP", "Throttle Stop failed");
caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_START_FAILED = new caplin.streamlink.impl.protocolhandler.RttpCode("AQ", "Throttle Start failed");
caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_DEF_FAILED = new caplin.streamlink.impl.protocolhandler.RttpCode("AR", "Throttle Default failed");
caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_NL_FAILED = new caplin.streamlink.impl.protocolhandler.RttpCode("AS", "Throttle No Listener");
caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_USER = new caplin.streamlink.impl.protocolhandler.RttpCode("Az", "Invalid Username");
caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_PASS = new caplin.streamlink.impl.protocolhandler.RttpCode("A-", "Invalid Password");
caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_IP = new caplin.streamlink.impl.protocolhandler.RttpCode("A_", "Invalid IP Address");
caplin.streamlink.impl.protocolhandler.RttpCode.ACCT_EXPIRED = new caplin.streamlink.impl.protocolhandler.RttpCode("B0", "Account Expired");
caplin.streamlink.impl.protocolhandler.RttpCode.ALREADY_LOGGED_IN = new caplin.streamlink.impl.protocolhandler.RttpCode("B1", "Already Logged In");
caplin.streamlink.impl.protocolhandler.RttpCode.LICENSE_SITE = new caplin.streamlink.impl.protocolhandler.RttpCode("B7", "Site License Exceeded");
caplin.streamlink.impl.protocolhandler.RttpCode.LICENSE_USER = new caplin.streamlink.impl.protocolhandler.RttpCode("B8", "User License Exceeded");
caplin.streamlink.impl.protocolhandler.RttpCode.GOTO_URL = new caplin.streamlink.impl.protocolhandler.RttpCode("BH", "Login Failed, Goto this URL (unused)");
caplin.streamlink.impl.protocolhandler.RttpCode.AUTH_ERROR_STR = new caplin.streamlink.impl.protocolhandler.RttpCode("BR", "Auth Module Error");
caplin.streamlink.impl.protocolhandler.RttpCode.NOT_LOGGED_IN = new caplin.streamlink.impl.protocolhandler.RttpCode("CW", "Not logged in");
caplin.streamlink.impl.protocolhandler.RttpCode.REQUEST_ERROR = new caplin.streamlink.impl.protocolhandler.RttpCode("Ch", "Request error (invalid login format)");
caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_REQUEST = new caplin.streamlink.impl.protocolhandler.RttpCode("Cg", "Invalid request");
caplin.streamlink.impl.protocolhandler.RttpCode.SERVER_ERROR = new caplin.streamlink.impl.protocolhandler.RttpCode("Cq", "Server error (unused)");
caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_SESSION_ID = new caplin.streamlink.impl.protocolhandler.RttpCode("D0", "Invalid session ID");
caplin.streamlink.impl.protocolhandler.RttpCode.MULTI_RESPONSE = new caplin.streamlink.impl.protocolhandler.RttpCode("E4", "Container code, other messages follow");
caplin.streamlink.impl.protocolhandler.RttpCode.values = function() { return [caplin.streamlink.impl.protocolhandler.RttpCode.REQUEST_ACK, caplin.streamlink.impl.protocolhandler.RttpCode.CONNECT_OK, caplin.streamlink.impl.protocolhandler.RttpCode.FIELD_MAP, caplin.streamlink.impl.protocolhandler.RttpCode.SESSION_EJECTED, caplin.streamlink.impl.protocolhandler.RttpCode.LOGIN_OK, caplin.streamlink.impl.protocolhandler.RttpCode.RECON_OK, caplin.streamlink.impl.protocolhandler.RttpCode.RESP_UNKNOWN, caplin.streamlink.impl.protocolhandler.RttpCode.DIRECTORY_RESP, caplin.streamlink.impl.protocolhandler.RttpCode.PAGE_RESP, caplin.streamlink.impl.protocolhandler.RttpCode.RECORD_RESP, caplin.streamlink.impl.protocolhandler.RttpCode.NEWS_RESP, caplin.streamlink.impl.protocolhandler.RttpCode.STORY_RESP, caplin.streamlink.impl.protocolhandler.RttpCode.CHAT_RESP, caplin.streamlink.impl.protocolhandler.RttpCode.CONT_RESP, caplin.streamlink.impl.protocolhandler.RttpCode.AUTODIR_RESP, caplin.streamlink.impl.protocolhandler.RttpCode.PERM_RESP, caplin.streamlink.impl.protocolhandler.RttpCode.CONTRIB_OK, caplin.streamlink.impl.protocolhandler.RttpCode.DISCARD_OK, caplin.streamlink.impl.protocolhandler.RttpCode.CREATE_OK, caplin.streamlink.impl.protocolhandler.RttpCode.DELETE_OK, caplin.streamlink.impl.protocolhandler.RttpCode.NOOP_OK, caplin.streamlink.impl.protocolhandler.RttpCode.LOGOUT_OK, caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_OK, caplin.streamlink.impl.protocolhandler.RttpCode.SYNC_OK, caplin.streamlink.impl.protocolhandler.RttpCode.CONTRIB_WAIT, caplin.streamlink.impl.protocolhandler.RttpCode.CONTRIB_OK_DELAY, caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_OK, caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_STALE, caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_LIMITED, caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_REMOVED, caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_INFO, caplin.streamlink.impl.protocolhandler.RttpCode.DIR_UPD, caplin.streamlink.impl.protocolhandler.RttpCode.PAGE_UPD, caplin.streamlink.impl.protocolhandler.RttpCode.REC1_UPD, caplin.streamlink.impl.protocolhandler.RttpCode.NEWS_UPD, caplin.streamlink.impl.protocolhandler.RttpCode.STORY_UPD, caplin.streamlink.impl.protocolhandler.RttpCode.REC2_UPD, caplin.streamlink.impl.protocolhandler.RttpCode.REC3_UPD, caplin.streamlink.impl.protocolhandler.RttpCode.CHAT_UPD, caplin.streamlink.impl.protocolhandler.RttpCode.CONT_UPD, caplin.streamlink.impl.protocolhandler.RttpCode.AUTODIR_UPD, caplin.streamlink.impl.protocolhandler.RttpCode.PERM_UPD, caplin.streamlink.impl.protocolhandler.RttpCode.REC1_IMG, caplin.streamlink.impl.protocolhandler.RttpCode.NEWS_IMG, caplin.streamlink.impl.protocolhandler.RttpCode.STORY_IMG, caplin.streamlink.impl.protocolhandler.RttpCode.REC2_IMG, caplin.streamlink.impl.protocolhandler.RttpCode.REC3_IMG, caplin.streamlink.impl.protocolhandler.RttpCode.CONT_IMG, caplin.streamlink.impl.protocolhandler.RttpCode.AUTODIR_IMG, caplin.streamlink.impl.protocolhandler.RttpCode.PERM_IMG, caplin.streamlink.impl.protocolhandler.RttpCode.REC1_CLR, caplin.streamlink.impl.protocolhandler.RttpCode.NEWS_CLR, caplin.streamlink.impl.protocolhandler.RttpCode.REC2_CLR, caplin.streamlink.impl.protocolhandler.RttpCode.REC3_CLR, caplin.streamlink.impl.protocolhandler.RttpCode.PERM_CLR, caplin.streamlink.impl.protocolhandler.RttpCode.REC1_DEL, caplin.streamlink.impl.protocolhandler.RttpCode.REC2_DEL, caplin.streamlink.impl.protocolhandler.RttpCode.REC3_DEL, caplin.streamlink.impl.protocolhandler.RttpCode.PERM_DEL, caplin.streamlink.impl.protocolhandler.RttpCode.MISC_HEARTBEAT, caplin.streamlink.impl.protocolhandler.RttpCode.INIT_HEARTBEAT, caplin.streamlink.impl.protocolhandler.RttpCode.TIMED_HEARTBEAT, caplin.streamlink.impl.protocolhandler.RttpCode.SOURCE_UP, caplin.streamlink.impl.protocolhandler.RttpCode.SOURCE_DOWN, caplin.streamlink.impl.protocolhandler.RttpCode.SOURCE_WARN, caplin.streamlink.impl.protocolhandler.RttpCode.SERVICE_OK, caplin.streamlink.impl.protocolhandler.RttpCode.SERVICE_DOWN, caplin.streamlink.impl.protocolhandler.RttpCode.SERVICE_LIMITED, caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_MSG, caplin.streamlink.impl.protocolhandler.RttpCode.WARNING_MSG, caplin.streamlink.impl.protocolhandler.RttpCode.ERROR_MSG, caplin.streamlink.impl.protocolhandler.RttpCode.NOT_FOUND_DELAY, caplin.streamlink.impl.protocolhandler.RttpCode.READ_DENY_DELAY, caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY_DELAY, caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY_TRADE_GROUP_LICENSE_EXCEEDED_DELAY, caplin.streamlink.impl.protocolhandler.RttpCode.UNAVAILABLE_DELAY, caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_PARAMETERS_DELAY, caplin.streamlink.impl.protocolhandler.RttpCode.DELETED_DELAY, caplin.streamlink.impl.protocolhandler.RttpCode.NOT_FOUND, caplin.streamlink.impl.protocolhandler.RttpCode.READ_DENY, caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY, caplin.streamlink.impl.protocolhandler.RttpCode.WRITE_DENY_TRADE_GROUP_LICENSE_EXCEEDED, caplin.streamlink.impl.protocolhandler.RttpCode.UNAVAILABLE, caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_PARAMETERS, caplin.streamlink.impl.protocolhandler.RttpCode.CONTRIB_FAILED, caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_FAILED, caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_UP_FAILED, caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_DOWN_FAILED, caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_MIN_FAILED, caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_MAX_FAILED, caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_STOP_FAILED, caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_START_FAILED, caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_DEF_FAILED, caplin.streamlink.impl.protocolhandler.RttpCode.THROTTLE_NL_FAILED, caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_USER, caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_PASS, caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_IP, caplin.streamlink.impl.protocolhandler.RttpCode.ACCT_EXPIRED, caplin.streamlink.impl.protocolhandler.RttpCode.ALREADY_LOGGED_IN, caplin.streamlink.impl.protocolhandler.RttpCode.LICENSE_SITE, caplin.streamlink.impl.protocolhandler.RttpCode.LICENSE_USER, caplin.streamlink.impl.protocolhandler.RttpCode.GOTO_URL, caplin.streamlink.impl.protocolhandler.RttpCode.AUTH_ERROR_STR, caplin.streamlink.impl.protocolhandler.RttpCode.NOT_LOGGED_IN, caplin.streamlink.impl.protocolhandler.RttpCode.REQUEST_ERROR, caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_REQUEST, caplin.streamlink.impl.protocolhandler.RttpCode.SERVER_ERROR, caplin.streamlink.impl.protocolhandler.RttpCode.INVALID_SESSION_ID, caplin.streamlink.impl.protocolhandler.RttpCode.MULTI_RESPONSE];};
caplin.streamlink.impl.protocolhandler.RttpCode.valueOf = function(name) { return this[name]; };
caplin.streamlink.impl.protocolhandler.RttpCode.prototype.getDescription = function() {
	return this.description;
};

caplin.streamlink.impl.protocolhandler.RttpCode.prototype.getName = function() {
	return this.name;
};

caplin.streamlink.impl.protocolhandler.RttpCode.prototype.toString = function() {
	return this.name;
};



//## SubscriptionListenerOnClient.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.client");

// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver

caplin.streamlink.impl.bridge.worker.client.SubscriptionListenerOnClient = function(bridge, id, listener) {
	// Constructor body
	this._bridge = bridge;
	this._id = id;
	this._listener = listener;
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.client.SubscriptionListenerOnClient, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.client.SubscriptionListenerOnClient.prototype.getReceiverKey = function() {
	return this._listener;
};

caplin.streamlink.impl.bridge.worker.client.SubscriptionListenerOnClient.prototype.getReceiverId = function() {
	return this._id;
};

caplin.streamlink.impl.bridge.worker.client.SubscriptionListenerOnClient.prototype.receive = function(method, args) {
	if ((method == ("onRecordUpdate"))) {
		var subscription = this._bridge.getReceiver(args[0]);
		(this._listener.onRecordUpdate && this._listener.onRecordUpdate(subscription, caplin.streamlink.impl.event.RecordType1EventImpl.createFromDictionary(this._bridge.getRuntime(), args[1])));
	}
	else if ((method == ("onRecordType2Update"))) {
		var subscription = this._bridge.getReceiver(args[0]);
		(this._listener.onRecordType2Update && this._listener.onRecordType2Update(subscription, caplin.streamlink.impl.event.RecordType2EventImpl.createFromDictionary(args[1])));
	}
	else if ((method == ("onRecordType3Update"))) {
		var subscription = this._bridge.getReceiver(args[0]);
		(this._listener.onRecordType3Update && this._listener.onRecordType3Update(subscription, caplin.streamlink.impl.event.RecordType3EventImpl.createFromDictionary(args[1])));
	}
	else if ((method == ("onNewsUpdate"))) {
		var subscription = this._bridge.getReceiver(args[0]);
		(this._listener.onNewsUpdate && this._listener.onNewsUpdate(subscription, caplin.streamlink.impl.event.NewsEventImpl.createFromDictionary(args[1])));
	}
	else if ((method == ("onStoryUpdate"))) {
		var subscription = this._bridge.getReceiver(args[0]);
		(this._listener.onStoryUpdate && this._listener.onStoryUpdate(subscription, caplin.streamlink.impl.event.StoryEventImpl.createFromDictionary(args[1])));
	}
	else if ((method == ("onChatUpdate"))) {
		var subscription = this._bridge.getReceiver(args[0]);
		(this._listener.onChatUpdate && this._listener.onChatUpdate(subscription, caplin.streamlink.impl.event.ChatEventImpl.createFromDictionary(args[1])));
	}
	else if ((method == ("onDirectoryUpdate"))) {
		var subscription = this._bridge.getReceiver(args[0]);
		(this._listener.onDirectoryUpdate && this._listener.onDirectoryUpdate(subscription, caplin.streamlink.impl.event.DirectoryEventImpl.createFromDictionary(args[1])));
	}
	else if ((method == ("onSubscriptionError"))) {
		var subscription = this._bridge.getReceiver(args[0]);
		this._listener.onSubscriptionError(subscription, caplin.streamlink.impl.event.SubjectErrorEventImpl.createFromDictionary(args[1]));
	}
	else if ((method == ("onSubscriptionStatus"))) {
		var subscription = this._bridge.getReceiver(args[0]);
		this._listener.onSubscriptionStatus(subscription, caplin.streamlink.impl.event.SubjectStatusEventImpl.createFromDictionary(args[1]));
	}
	else if ((method == ("onPermissionUpdate"))) {
		var subscription = this._bridge.getReceiver(args[0]);
		(this._listener.onPermissionUpdate && this._listener.onPermissionUpdate(subscription, caplin.streamlink.impl.event.PermissionEventImpl.createFromDictionary(args[1])));
	}
	else if ((method == ("onContainerUpdate"))) {
		var subscription = this._bridge.getReceiver(args[0]);
		(this._listener.onContainerUpdate && this._listener.onContainerUpdate(subscription, caplin.streamlink.impl.event.ContainerEventImpl.createFromDictionary(args[1])));
	}
};



//## PermissionEvent.js ##//
caplin.namespace("caplin.streamlink");

// caplin.forceload(caplin.streamlink.DataEvent

/**
 * @class  <p>Provides information about a change to permission data.</p>
 *  
 *  @extends caplin.streamlink.DataEvent
 * 
 * @name caplin.streamlink.PermissionEvent
 */
caplin.streamlink.PermissionEvent = function() {};

// caplin.implement(caplin.streamlink.PermissionEvent, caplin.streamlink.DataEvent);

/**
 *  <p>Returns the boolean that indicates whether all permission keys for this subject have been deleted.</p>
 *  
 *  @returns {boolean} If true, all permission data for this subject should be removed from your cache.
 * @function
 */
caplin.streamlink.PermissionEvent.prototype.deleteAllKeys = function() { throw new Error('caplin.streamlink.PermissionEvent.deleteAllKeys :  This is an abstract method'); };

/**
 *  <p>Returns the boolean that indicates whether the permission key within this update has been deleted.</p>
 *  
 *  @returns {boolean} Whether the specified key should be deleted from your cache.
 * @function
 */
caplin.streamlink.PermissionEvent.prototype.deleteKey = function() { throw new Error('caplin.streamlink.PermissionEvent.deleteKey :  This is an abstract method'); };

/**
 *  <p>Returns the permission key relating to this event.</p>
 *  
 *  @returns {String} The value of the permission key.
 * @function
 */
caplin.streamlink.PermissionEvent.prototype.getKey = function() { throw new Error('caplin.streamlink.PermissionEvent.getKey :  This is an abstract method'); };

/**
 *  <p>Returns the collection of fields relating to this PermissionEvent.</p>
 *  
 *  <p>For example, if this event is not an image, the fields are the ones that have been updated.</p>
 *  
 *  @returns {Object} A collection of name-value pairs.
 * @function
 */
caplin.streamlink.PermissionEvent.prototype.getFields = function() { throw new Error('caplin.streamlink.PermissionEvent.getFields :  This is an abstract method'); };



//## PermissionEventImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.PermissionEvent
// caplin.forceload(caplin.streamlink.impl.Serializable
// caplin.forceload(caplin.streamlink.impl.event.RttpEventBase
// caplin.forceload(caplin.streamlink.DataEvent

caplin.streamlink.impl.event.PermissionEventImpl = function(subject, image, fields, key, deleteIndex, deleteAll, rttpCode, objectNumber, sequenceNumber) {
	// Super
	caplin.streamlink.impl.event.RttpEventBase.call(this, subject, rttpCode, objectNumber, sequenceNumber);
	// Field Definitions
	this._key = null;
	this._image = false;
	this._deleteAllKeys = false;
	this._deleteKey = false;

	// Constructor body
	this._image = image;
	this._fields = fields;
	this._key = key;
	this._deleteKey = deleteIndex;
	this._deleteAllKeys = deleteAll;
};

caplin.extend(caplin.streamlink.impl.event.PermissionEventImpl, caplin.streamlink.impl.event.RttpEventBase);

// caplin.implement(caplin.streamlink.impl.event.PermissionEventImpl, caplin.streamlink.DataEvent);
// caplin.implement(caplin.streamlink.impl.event.PermissionEventImpl, caplin.streamlink.PermissionEvent);
// caplin.implement(caplin.streamlink.impl.event.PermissionEventImpl, caplin.streamlink.impl.Serializable);

caplin.streamlink.impl.event.PermissionEventImpl.prototype.deleteAllKeys = function() {
	return this._deleteAllKeys;
};

caplin.streamlink.impl.event.PermissionEventImpl.prototype.deleteKey = function() {
	return this._deleteKey;
};

caplin.streamlink.impl.event.PermissionEventImpl.prototype.isImage = function() {
	return this._image;
};

caplin.streamlink.impl.event.PermissionEventImpl.createFromDictionary = function(root) {
	return new caplin.streamlink.impl.event.PermissionEventImpl(root[caplin.streamlink.impl.event.RttpEventBase.SUBJECT], root["Image"], root["Fields"], root["Key"], root["DeleteKey"], root["DeleteAll"], caplin.streamlink.impl.utils.RttpUtil.getRttpCode(root[caplin.streamlink.impl.event.RttpEventBase.CODE]), root[caplin.streamlink.impl.event.RttpEventBase.OBJECT_NUMBER], root[caplin.streamlink.impl.event.RttpEventBase.SEQUENCE_NUMBER]);
};

caplin.streamlink.impl.event.PermissionEventImpl.createFromRttp = function(rttpCode, tokens, fieldMap, encoder) {
	switch (rttpCode) {
	case caplin.streamlink.impl.protocolhandler.RttpCode.PERM_RESP:
	{
		var fields = caplin.streamlink.impl.event.ParseUtils.getFields(tokens, fieldMap, encoder, 3);
		var key = caplin.streamlink.impl.event.ParseUtils.getPermissionsKeyFromTokens(tokens, encoder, 2);
		return new caplin.streamlink.impl.event.PermissionEventImpl(encoder.rttpDecode(tokens[1]), true, fields, key, false, false, rttpCode, tokens[0].substring(2, 6), null);
	}
	case caplin.streamlink.impl.protocolhandler.RttpCode.PERM_UPD:
	{
		var fields = caplin.streamlink.impl.event.ParseUtils.getFields(tokens, fieldMap, encoder, 2);
		var key = caplin.streamlink.impl.event.ParseUtils.getPermissionsKeyFromTokens(tokens, encoder, 1);
		return new caplin.streamlink.impl.event.PermissionEventImpl(null, false, fields, key, false, false, rttpCode, tokens[0].substring(4, 8), tokens[0].substring(2, 4));
	}
	case caplin.streamlink.impl.protocolhandler.RttpCode.PERM_IMG:
	{
		var fields = caplin.streamlink.impl.event.ParseUtils.getFields(tokens, fieldMap, encoder, 2);
		var key = caplin.streamlink.impl.event.ParseUtils.getPermissionsKeyFromTokens(tokens, encoder, 1);
		return new caplin.streamlink.impl.event.PermissionEventImpl(null, true, fields, key, false, false, rttpCode, tokens[0].substring(4, 8), tokens[0].substring(2, 4));
	}
	case caplin.streamlink.impl.protocolhandler.RttpCode.PERM_CLR:
	{
		var fields = {} ;
		return new caplin.streamlink.impl.event.PermissionEventImpl(null, false, fields, null, false, true, rttpCode, tokens[0].substring(4, 8), tokens[0].substring(2, 4));
	}
	case caplin.streamlink.impl.protocolhandler.RttpCode.PERM_DEL:
	{
		var fields = {} ;
		var key = caplin.streamlink.impl.event.ParseUtils.getPermissionsKeyFromTokens(tokens, encoder, 1);
		return new caplin.streamlink.impl.event.PermissionEventImpl(null, true, fields, key, true, false, rttpCode, tokens[0].substring(4, 8), tokens[0].substring(2, 4));
	}
	}
	return null;
};

caplin.streamlink.impl.event.PermissionEventImpl.prototype._toDictionary = function() {
	var root = caplin.streamlink.impl.event.RttpEventBase.prototype._toDictionary.call(this);
	root["Image"] = this._image;
	root["Fields"] = this._fields;
	root["Key"] = this._key;
	root["DeleteAll"] = this._deleteAllKeys;
	root["DeleteKey"] = this._deleteKey;
	return root;
};

caplin.streamlink.impl.event.PermissionEventImpl.prototype.getKey = function() {
	return this._key;
};

caplin.streamlink.impl.event.PermissionEventImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("PermissionDataEventImpl [subject=");
	builder.push(this._subject);
	builder.push(", fields=");
	builder.push(caplin.xlang.Utils.dictionaryToString(this._fields));
	builder.push(", key=");
	builder.push(this._key);
	builder.push(", image=");
	builder.push(this._image);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.PermissionEventImpl.prototype.getFields = function() {
	return this._fields;
};

caplin.streamlink.impl.event.PermissionEventImpl.prototype._publishCommandResponse = function(listener) { };

caplin.streamlink.impl.event.PermissionEventImpl.prototype._publishSubscriptionResponse = function(subscription, subscriptionListener) {
	if (this._key != null || this._deleteAllKeys) {
		(subscriptionListener.onPermissionUpdate && subscriptionListener.onPermissionUpdate(subscription, this));
	}
};



//## Type2Connection.js ##//
caplin.namespace("caplin.streamlink.js.impl.connection");

caplin.include("caplin.streamlink.impl.connection.BaseConnection", true);
caplin.include("caplin.streamlink.js.impl.connection.Channel", true);

//caplin.forceload(caplin.streamlink.impl.connection.Channel);

caplin.streamlink.js.impl.connection.Type2Connection = (function()
{

	function Type2Connection(context, url, ssl)
	{
		caplin.streamlink.impl.connection.BaseConnection.call(this, context, url, ssl);
		var self = this;
		var timeout = context.configuration._getHttpTimeout();
		
		this.requestChannel = new caplin.streamlink.js.impl.connection.Channel(url + "/RTTP-TYPE2", timeout);
		this.backChannel = new caplin.streamlink.js.impl.connection.Channel(url + "/RTTP-TYPE2?X-RTTP-Type2-Pad-Length=4096", timeout);
		
		this.logger.log(caplin.streamlink.LogLevel.INFO, "Using Connection Type: {0}", "Type2Connection");
		this.logger.log(caplin.streamlink.LogLevel.INFO, "Using Channels: {0}", this.backChannel.className);

		this.backChannel.onprogress = function(data)
		{
			if (self.connectionState == caplin.streamlink.impl.connection.RttpConnectionState.CONNECTING)
			{
				self.changeState(caplin.streamlink.impl.connection.RttpConnectionState.CONNECTED);
			}

			if (data.length > 0)
			{
				self.logDataIn(data);
				self.fireData(data);
			}
			
			if (self.messageCount > self.context.configuration._getXhrStreamingReconnectCount())
			{
				self.fireReconnectRequest();
			}
		};

		this.backChannel.onload = function(data)
		{
			self.changeState(caplin.streamlink.impl.connection.RttpConnectionState.DISCONNECTED);
		};

		this.requestChannel.onload = function(data)
		{
			if (self.connectionState == caplin.streamlink.impl.connection.RttpConnectionState.CONNECTED)
			{
				self.readyToSend = true;
				self.fireReadyToSend();
			}
		};
		
		/**
		 * On Error
		 */
		this.requestChannel.onerror = function(status, data)
		{
			self.logger.log(caplin.streamlink.LogLevel.ERROR, "Type2Connection: {0}  Status: {1}  Data: {2}" , "Error in the request channel.", status, data);
			self.changeState(caplin.streamlink.impl.connection.RttpConnectionState.DISCONNECTED);
		};
		
		this.backChannel.onerror = function(status, data)
		{
			self.logger.log(caplin.streamlink.LogLevel.ERROR, "Type2Connection: {0}  Status: {1}  Data: {2}" , "Error in the back channel.", status, data);
			self.changeState(caplin.streamlink.impl.connection.RttpConnectionState.DISCONNECTED);
		};

		this.readyToSend = true;
	}
	;

	caplin.extend(Type2Connection, caplin.streamlink.impl.connection.BaseConnection);

	var Class = Type2Connection.prototype;
	
	Class.connect = function(message)
	{
		this.changeState(caplin.streamlink.impl.connection.RttpConnectionState.CONNECTING);
		this.backChannel.send(null, "text/plain");
	};

	Class.send = function(message)
	{
		if (this.isReadyToSend())
		{
			this.logDataOut(message);
			this.requestChannel.send(message, "application/x-www-form-urlencoded");
			this.readyToSend = false;
		}
	};

	Class.disconnect = function()
	{
		this.changeState(caplin.streamlink.impl.connection.RttpConnectionState.DISCONNECTED);
		this.backChannel.abort();
		this.requestChannel.abort();
	};

	Class.isReadyToSend = function()
	{
		return this.readyToSend;
	};

	Class.getDefaultPort = function()
	{
		return 80;
	};

	return Type2Connection;

})();


//## ForeverFrame.js ##//
caplin.namespace("caplin.streamlink.js.impl.connection");

caplin.include("caplin.streamlink.impl.connection.BaseConnection", true);
caplin.include("caplin.streamlink.impl.connection.RttpConnectionState", true);
caplin.include("caplin.streamlink.js.impl.connection.Channel");

//caplin.forceload(caplin.streamlink.impl.connection.BaseConnection);
//caplin.forceload(caplin.streamlink.impl.connection.RttpConnectionState);


caplin.streamlink.js.impl.connection.ForeverFrame = (function() {
	
	var ConnectionState = caplin.streamlink.impl.connection.RttpConnectionState;
	var Super = caplin.streamlink.impl.connection.BaseConnection;
	
	function ForeverFrame(context, url, ssl) {
		Super.call(this, context, url, ssl);
		
		this.logger.log(caplin.streamlink.LogLevel.INFO, "Using Connection Type: {0}", "ForeverFrame");
		
		this.iframe = null;
		this.requestChannel = new caplin.streamlink.js.impl.connection.Channel(url+"/RTTP-TYPE5", context.configuration._getHttpTimeout());
		this.requestChannel.onload = function(data) {
			self.onRequestChannelData(data);
		};
		this.requestChannel.onerror = function(status, data) {
			self.logger.log(caplin.streamlink.LogLevel.ERROR, "Request failed, code {0} : {1}", status, data);
			self.changeState(ConnectionState.DISCONNECTED);
		};

		this.readyToSend = true;

		var self = this;
		function receiveMessage(evt) {
			evt = evt || event;
			var data = evt.data;
			if (self.iframe === null || evt.origin !== self.url || data.indexOf(self.uniqueId) !== 0)
			{
				return;
			}
			data = data.slice(self.uniqueIdLength);
			if (self.connectionState === ConnectionState.CONNECTING)
			{
				self.changeState(ConnectionState.CONNECTED);
			}
			
			if (data === "disconnected")
			{
				self.disconnect();
				return;
			}
			
			self.logDataIn(data);
			self.fireData(data);
			
			if (self.messageCount > self.context.configuration._getForeverFrameReconnectCount())
			{
				self.fireReconnectRequest();
			}
			
		}
		
		this.fnReceiveMessage = receiveMessage;
		
		if (window.addEventListener) {
			window.addEventListener("message", receiveMessage, false);
		} else {
			window.attachEvent('onmessage', receiveMessage);
		}
	}
	
	caplin.extend(ForeverFrame, Super);
	var Class = ForeverFrame.prototype; 
	
	ForeverFrame.uniqueId = 0;
	
	Class.connect = function() {
		this.changeState(ConnectionState.CONNECTING);
		var self = this;

		this._cleanUp();

		this.uniqueId = (caplin.streamlink.js.impl.connection.ForeverFrame.uniqueId++) + "";
		this.uniqueIdLength = this.uniqueId.length + 1;

		var iframe = document.createElement('iframe');

		if (window.ActiveXObject) {
			iframe.setAttribute("src", this.url + "/sljs/htmlfile.html?uniqueId=" + this.uniqueId);
		} else {
			iframe.setAttribute("src", this.url + "/RTTP-TYPE5?X-RTTP-Type5-Client=sljs?uniqueId=" + this.uniqueId);
			iframe.onload = function() {
				self.disconnect();
			};
		}
		iframe.style.display = "none";
		
		if(document.body)
		{
			document.body.appendChild(iframe);
		}
		else
		{
			document.head.appendChild(iframe);
		}
		
		this.iframe = iframe;
	};
	
	Class.send = function(message) {
		if (this.isReadyToSend()) {
			this.readyToSend = false;
			this.logDataOut(message);
			this.requestChannel.send(message, "application/x-www-form-urlencoded");
		} else {
			this.logger.log(caplin.streamlink.LogLevel.ERROR, "Instructed to send message when not ready: {0}", message);
			this._cleanUp();
		}
	};
	
	Class.disconnect = function() {
		this._cleanUp();
		this.changeState(ConnectionState.DISCONNECTED);
	};
	
	Class.onRequestChannelData = function(data) {
		if (this.connectionState == ConnectionState.CONNECTED) {
			this.readyToSend = true;
			this.fireReadyToSend();
		}
	};

	Class._cleanUp = function() {
		if (this.iframe != null) {
			this.iframe.onload = null;
			try
			{
				this.iframe.document.execCommand('Stop');
			}
			catch (ignore)
			{
			}
			this.iframe.parentNode.removeChild(this.iframe);
			this.iframe = null;
			
			if (window.removeEventListener) {
				window.removeEventListener("message", this.fnReceiveMessage, false);
			} else {
				window.detachEvent('onmessage', this.fnReceiveMessage);
			}
			
		}
	}
	
	return ForeverFrame;
})();


//## ServiceStatus.js ##//
caplin.namespace("caplin.streamlink");

/**
 * @class  <p>Enumeration that defines the status of a Service.</p>
 *  <p>The <code>SourceStatus</code> will be provided by calling <code>getSourceStatus</code> on a {@link caplin.streamlink.SourceStatusEvent}.</p> 
 *  
 *  <p>The three states that are possible for a Liberator data service are:</p>
 *  <ul>
 *  <li><b>OK</b> - All of the data sources that make up the service are up.</li>
 *  <li><b>Down</b> - One or more <i>required</i> data sources for the service are down.</li>
 *  <li><b>Limited</b> - One or more <i>non-required</i> data sources for the service are down.</li>
 *  </ul></p>
 *  
 *  @see caplin.streamlink.ServiceStatusEvent
 *  
 * 
 * @name caplin.streamlink.ServiceStatus
 */
caplin.streamlink.ServiceStatus = {
	/**
	 *  <p>Indicates that a service is down, which means that at least one of the required sources
	 *  defined for the service are down.</p>
	 */
	DOWN: "DOWN",
	/**
	 *  <p>Indicates that a service is limited, which means that at least one of the non required
	 *  sources defined for the service are down (but all of the required sources are up).</p>
	 */
	LIMITED: "LIMITED",
	/**
	 *  <p>Indicates that the service is OK, which means that all the required and non required
	 *  sources defined for the service are up.</p>
	 */
	OK: "OK",
	values: function() {return [caplin.streamlink.ServiceStatus.DOWN, caplin.streamlink.ServiceStatus.LIMITED, caplin.streamlink.ServiceStatus.OK];},
	valueOf: function(name) {return name;}
};

//## Bridge.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker");


caplin.streamlink.impl.bridge.worker.Bridge = function() {};

caplin.streamlink.impl.bridge.worker.Bridge.prototype.getRuntime = function() { throw new Error('caplin.streamlink.impl.bridge.worker.Bridge.getRuntime :  This is an abstract method'); };

caplin.streamlink.impl.bridge.worker.Bridge.prototype.findReceiver = function(obj) { throw new Error('caplin.streamlink.impl.bridge.worker.Bridge.findReceiver :  This is an abstract method'); };

caplin.streamlink.impl.bridge.worker.Bridge.prototype.getReceiver = function(id) { throw new Error('caplin.streamlink.impl.bridge.worker.Bridge.getReceiver :  This is an abstract method'); };

caplin.streamlink.impl.bridge.worker.Bridge.prototype.getNextId = function() { throw new Error('caplin.streamlink.impl.bridge.worker.Bridge.getNextId :  This is an abstract method'); };

caplin.streamlink.impl.bridge.worker.Bridge.prototype.addReceiver = function(receiver) { throw new Error('caplin.streamlink.impl.bridge.worker.Bridge.addReceiver :  This is an abstract method'); };

caplin.streamlink.impl.bridge.worker.Bridge.prototype.log = function(message) { throw new Error('caplin.streamlink.impl.bridge.worker.Bridge.log :  This is an abstract method'); };

caplin.streamlink.impl.bridge.worker.Bridge.prototype.receive = function(id, method, args) { throw new Error('caplin.streamlink.impl.bridge.worker.Bridge.receive :  This is an abstract method'); };

caplin.streamlink.impl.bridge.worker.Bridge.prototype.removeReceiver = function(receiver) { throw new Error('caplin.streamlink.impl.bridge.worker.Bridge.removeReceiver :  This is an abstract method'); };

caplin.streamlink.impl.bridge.worker.Bridge.prototype.send = function(id, method, args) { throw new Error('caplin.streamlink.impl.bridge.worker.Bridge.send :  This is an abstract method'); };



//## BridgeBase.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker");

// caplin.forceload(caplin.streamlink.impl.bridge.worker.Bridge

caplin.streamlink.impl.bridge.worker.BridgeBase = function(runtime, worker) {
	// Constructor body
	this.runtime = runtime;
	this.worker = worker;
	this.nextId = 1;
	this.prefix = (worker) ? "W" : "C";
	this.logPrefix = (worker) ? "WorkerBridge" : "ClientBridge";
	this.receiverMap = {} ;
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.BridgeBase, caplin.streamlink.impl.bridge.worker.Bridge);

caplin.streamlink.impl.bridge.worker.BridgeBase.prototype.getRuntime = function() {
	return this.runtime;
};

caplin.streamlink.impl.bridge.worker.BridgeBase.prototype.findReceiver = function(findObject) {
	for (var objId in this.receiverMap) {
		var receiver = this.receiverMap[objId];
		if (receiver.getReceiverKey() == findObject) {
			return receiver;
		}
	}
	return null;
};

caplin.streamlink.impl.bridge.worker.BridgeBase.prototype.getReceiver = function(id) {
	this.log(this.logPrefix + " getReceiver(" + id + ")");
	return this.receiverMap[id];
};

caplin.streamlink.impl.bridge.worker.BridgeBase.prototype.getNextId = function() {
	var id = this.prefix + this.nextId;
	(this.nextId)++;
	return id;
};

caplin.streamlink.impl.bridge.worker.BridgeBase.prototype.addReceiver = function(receiver) {
	this.log(this.logPrefix + " addReceiver(" + receiver.getReceiverId() + ")");
	this.receiverMap[receiver.getReceiverId()] = receiver;
};

caplin.streamlink.impl.bridge.worker.BridgeBase.prototype.log = function(message) { };

caplin.streamlink.impl.bridge.worker.BridgeBase.prototype.receive = function(id, method, args) {
	this.log(this.logPrefix + " receive(" + id + " " + method + " ...)");
	var receiver = this.receiverMap[id];
	if (receiver == null) {
		this.log(this.logPrefix + " receiver not found");
	}
	else {
		receiver.receive(method, args);
	}
};

caplin.streamlink.impl.bridge.worker.BridgeBase.prototype.removeReceiver = function(receiver) {
	delete this.receiverMap[receiver.getReceiverId()];
};



//## Utils.js ##//
caplin.namespace("caplin.streamlink.impl.utils");

caplin.streamlink.impl.utils.Utils = function() {};

caplin.streamlink.impl.utils.Utils.startsWith = function(text, beginning) {
	return (beginning == (text.substring(0, beginning.length)));
};

caplin.streamlink.impl.utils.Utils.replaceParameters = function(msgTemplate, args) {
	var result = msgTemplate;
	if (args != null) {
		for (var i = 0; i < args.length; (i)++) {
			var replacement = (args[i] == null) ? "null" : args[i].toString();
			result = result.replace(new RegExp("\\{" + i + "\\}", 'g'), replacement);
		}
	}
	return result;
};



//## SubjectType.js ##//
caplin.namespace("caplin.streamlink");

/**
 * @class  <p>Enumeration that represents the available subject types.</p>
 *  <p>The <code>SubjectType</code> will be provided by calling <code>getType()</code> on a 
 *  {@link caplin.streamlink.ContainerElement} or {@link caplin.streamlink.DirectoryChangeElement}.</p>
 *  <p>It is also a parameter to <code>createSubject</code> in {@link caplin.streamlink.StreamLink}.</p>
 * 
 * @name caplin.streamlink.SubjectType
 */
caplin.streamlink.SubjectType = {
	/**
	 *  <p>RTTP chat object type.</p>
	 */
	CHAT: "CHAT",
	/**
	 *  <p>RTTP container object type.</p>
	 */
	CONTAINER: "CONTAINER",
	/**
	 *  <p>RTTP directory object type.</p>
	 */
	DIRECTORY: "DIRECTORY",
	/**
	 *  <p>RTTP news headline object type.</p>
	 */
	NEWS_HEADLINE: "NEWS_HEADLINE",
	/**
	 *  <p>RTTP news story object type.</p>
	 */
	NEWS_STORY: "NEWS_STORY",
	/**
	 *  <p>RTTP page object type.</p>
	 */
	PAGE: "PAGE",
	/**
	 *  <p>RTTP permission object type.</p>
	 */
	PERMISSION: "PERMISSION",
	/**
	 *  <p>RTTP record object type.</p>
	 */
	RECORD: "RECORD",
	/**
	 *  <p>Type used if the RTTP object type is unknown.</p>
	 */
	UNKNOWN: "UNKNOWN",
	values: function() {return [caplin.streamlink.SubjectType.CHAT, caplin.streamlink.SubjectType.CONTAINER, caplin.streamlink.SubjectType.DIRECTORY, caplin.streamlink.SubjectType.NEWS_HEADLINE, caplin.streamlink.SubjectType.NEWS_STORY, caplin.streamlink.SubjectType.PAGE, caplin.streamlink.SubjectType.PERMISSION, caplin.streamlink.SubjectType.RECORD, caplin.streamlink.SubjectType.UNKNOWN];},
	valueOf: function(name) {return name;}
};

//## StatisticsEvent.js ##//
caplin.namespace("caplin.streamlink");

/**
 * @class  <p>Provides access to the statistics relating to an instance of StreamLink.</p>
 *  <p>The <code>StatisticsEvent</code> will be provided on a <code>onStatisticsChange</code> 
 *  callback to a {@link caplin.streamlink.ConnectionListener}.</p>
 * 
 * @name caplin.streamlink.StatisticsEvent
 */
caplin.streamlink.StatisticsEvent = function() {};

/**
 *  <p>Gets the average (one-way) heartbeat message latency between Liberator and StreamLink.</p>
 *  @returns {long} The average heartbeat latency in milliseconds, or -1 if the latency cannot be calculated.
 * @function
 */
caplin.streamlink.StatisticsEvent.prototype.getAverageLatency = function() { throw new Error('caplin.streamlink.StatisticsEvent.getAverageLatency :  This is an abstract method'); };

/**
 *  <p>Gets the clock offset between Liberator and StreamLink.</p>
 *  @returns {long} The clock offset in milliseconds, or -1 if the clock offset cannot be calculated.
 * @function
 */
caplin.streamlink.StatisticsEvent.prototype.getClockOffset = function() { throw new Error('caplin.streamlink.StatisticsEvent.getClockOffset :  This is an abstract method'); };

/**
 *  <p>Gets the latest (one-way) heartbeat message latency between Liberator and StreamLink.</p>
 *  @returns {long} The latest heartbeat latency in milliseconds, or -1 if the latency cannot be calculated.
 * @function
 */
caplin.streamlink.StatisticsEvent.prototype.getLatency = function() { throw new Error('caplin.streamlink.StatisticsEvent.getLatency :  This is an abstract method'); };



//## StatisticsEventImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.StatisticsEvent

caplin.streamlink.impl.event.StatisticsEventImpl = function(lastLatency, averageLatency, clockOffset) {
	// Constructor body
	this._lastLatency = lastLatency;
	this._clockOffset = clockOffset;
	this._averageLatency = averageLatency;
};

// caplin.implement(caplin.streamlink.impl.event.StatisticsEventImpl, caplin.streamlink.StatisticsEvent);

caplin.streamlink.impl.event.StatisticsEventImpl.createFromDictionary = function(dictionary) {
	var lastLatency = dictionary["Latency"];
	var averageLatency = dictionary["AverageLatency"];
	var clockOffset = dictionary["ClockOffset"];
	return new caplin.streamlink.impl.event.StatisticsEventImpl(lastLatency, averageLatency, clockOffset);
};

caplin.streamlink.impl.event.StatisticsEventImpl.prototype._toDictionary = function() {
	var root = {} ;
	root["Latency"] = this._lastLatency;
	root["AverageLatency"] = this._averageLatency;
	root["ClockOffset"] = this._clockOffset;
	return root;
};

caplin.streamlink.impl.event.StatisticsEventImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("StatisticsEventImpl [latency=");
	builder.push(this._lastLatency);
	builder.push(", averageLatency=");
	builder.push(this._averageLatency);
	builder.push(", clockOffset=");
	builder.push(this._clockOffset);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.StatisticsEventImpl.prototype.getAverageLatency = function() {
	return this._averageLatency;
};

caplin.streamlink.impl.event.StatisticsEventImpl.prototype.getClockOffset = function() {
	return this._clockOffset;
};

caplin.streamlink.impl.event.StatisticsEventImpl.prototype.getLatency = function() {
	return this._lastLatency;
};



//## OkEvent.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.impl.event.RttpEventBase

caplin.streamlink.impl.event.OkEvent = function(subject, code) {
	// Super
	caplin.streamlink.impl.event.RttpEventBase.call(this, subject, code, null, null);};

caplin.extend(caplin.streamlink.impl.event.OkEvent, caplin.streamlink.impl.event.RttpEventBase);

caplin.streamlink.impl.event.OkEvent.createFromRttp = function(rttpCode, tokens, fieldMap, encoder) {
	return new caplin.streamlink.impl.event.OkEvent(encoder.rttpDecode(tokens[1]), rttpCode);
};

caplin.streamlink.impl.event.OkEvent.prototype.toString = function() {
	var builder = [] ;
	builder.push("OkEvent [subject=");
	builder.push(this._subject);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.OkEvent.prototype._publishCommandResponse = function(listener) {
	(listener.onCommandOk && listener.onCommandOk(this._subject));
};

caplin.streamlink.impl.event.OkEvent.prototype._publishSubscriptionResponse = function(subscription, subscriptionListener) { };



//## PacketThrottle.js ##//
caplin.namespace("caplin.streamlink.impl.packet");

// caplin.forceload(caplin.streamlink.impl.packet.Packet

caplin.streamlink.impl.packet.PacketThrottle = function(subject, command) {
	// Constructor body
	this.subject = subject;
	this.command = command;
};

// caplin.implement(caplin.streamlink.impl.packet.PacketThrottle, caplin.streamlink.impl.packet.Packet);

caplin.streamlink.impl.packet.PacketThrottle.prototype.getType = function() {
	return caplin.streamlink.impl.packet.PacketType.THROTTLE;
};

caplin.streamlink.impl.packet.PacketThrottle.prototype.getSubject = function() {
	return this.subject;
};

caplin.streamlink.impl.packet.PacketThrottle.prototype.toRttpString = function(encoder) {
	return caplin.streamlink.impl.packet.PacketType.THROTTLE.getCode() + " " + encoder.rttpEncode(this.subject) + ";" + this.command.toString().toLowerCase();
};

caplin.streamlink.impl.packet.PacketThrottle.prototype.toString = function() {
	var builder = [] ;
	builder.push("Throttle [subject=");
	builder.push(this.subject);
	builder.push(", command=");
	builder.push(this.command);
	builder.push("]");
	return builder.join("");
};



//## SubscriptionStatusEvent.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  <p>Provides information about a change in the state of a subscription.</p>
 *  <p>The <code>SubscriptionStatusEvent</code> will be provided on a <code>onSubscriptionStatus</code> callback to a {@link caplin.streamlink.SubscriptionListener}.</p>
 *  
 * 
 * @name caplin.streamlink.SubscriptionStatusEvent
 */
caplin.streamlink.SubscriptionStatusEvent = function() {};

/**
 *  <p>Returns the new status of the subject of the subscription.</p>
 *  @returns {caplin.streamlink.SubscriptionStatusType} The new status.
 * @function
 */
caplin.streamlink.SubscriptionStatusEvent.prototype.getStatus = function() { throw new Error('caplin.streamlink.SubscriptionStatusEvent.getStatus :  This is an abstract method'); };

/**
 *  <p>Returns the subject of the subscription to which the event relates.</p>
 *  @returns {String} The subject.
 * @function
 */
caplin.streamlink.SubscriptionStatusEvent.prototype.getSubject = function() { throw new Error('caplin.streamlink.SubscriptionStatusEvent.getSubject :  This is an abstract method'); };



//## SubjectStatusEventImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.impl.event.RttpEventBase
// caplin.forceload(caplin.streamlink.SubscriptionStatusEvent

caplin.streamlink.impl.event.SubjectStatusEventImpl = function(subject, status, rttpCode, objectNumber, sequenceNumber) {
	// Super
	caplin.streamlink.impl.event.RttpEventBase.call(this, subject, rttpCode, objectNumber, sequenceNumber);
	// Constructor body
	this._status = status;
};

caplin.extend(caplin.streamlink.impl.event.SubjectStatusEventImpl, caplin.streamlink.impl.event.RttpEventBase);

// caplin.implement(caplin.streamlink.impl.event.SubjectStatusEventImpl, caplin.streamlink.SubscriptionStatusEvent);

caplin.streamlink.impl.event.SubjectStatusEventImpl.prototype.getStatus = function() {
	return this._status;
};

caplin.streamlink.impl.event.SubjectStatusEventImpl.createFromDictionary = function(root) {
	var subject = root[caplin.streamlink.impl.event.RttpEventBase.SUBJECT];
	var status = caplin.streamlink.SubscriptionStatusType.valueOf(root["Status"]);
	var rttpCode = caplin.streamlink.impl.utils.RttpUtil.getRttpCode(root[caplin.streamlink.impl.event.RttpEventBase.CODE]);
	var objectNumber = root[caplin.streamlink.impl.event.RttpEventBase.OBJECT_NUMBER];
	var sequenceNumber = root[caplin.streamlink.impl.event.RttpEventBase.SEQUENCE_NUMBER];
	return new caplin.streamlink.impl.event.SubjectStatusEventImpl(subject, status, rttpCode, objectNumber, sequenceNumber);
};

caplin.streamlink.impl.event.SubjectStatusEventImpl.createFromRttp = function(rttpCode, tokens, fieldMap, encoder) {
	var sequenceNumber = tokens[0].substring(2, 4);
	var objectNumber = tokens[0].substring(4, 8);
	switch (rttpCode) {
	case caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_OK:
	return new caplin.streamlink.impl.event.SubjectStatusEventImpl(null, caplin.streamlink.SubscriptionStatusType.STATUS_OK, rttpCode, objectNumber, sequenceNumber);
	case caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_STALE:
	return new caplin.streamlink.impl.event.SubjectStatusEventImpl(null, caplin.streamlink.SubscriptionStatusType.STATUS_STALE, rttpCode, objectNumber, sequenceNumber);
	case caplin.streamlink.impl.protocolhandler.RttpCode.STATUS_LIMITED:
	return new caplin.streamlink.impl.event.SubjectStatusEventImpl(null, caplin.streamlink.SubscriptionStatusType.STATUS_LIMITED, rttpCode, objectNumber, sequenceNumber);
	}
	return null;
};

caplin.streamlink.impl.event.SubjectStatusEventImpl.prototype._toDictionary = function() {
	var root = caplin.streamlink.impl.event.RttpEventBase.prototype._toDictionary.call(this);
	root["Status"] = this._status.toString();
	return root;
};

caplin.streamlink.impl.event.SubjectStatusEventImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("SubjectStatusEvent [subject=");
	builder.push(this._subject);
	builder.push(", status=");
	builder.push(this._status);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.SubjectStatusEventImpl.prototype._publishCommandResponse = function(listener) { };

caplin.streamlink.impl.event.SubjectStatusEventImpl.prototype._publishSubscriptionResponse = function(subscription, subscriptionListener) {
	subscriptionListener.onSubscriptionStatus(subscription, this);
};



//## KeymasterResponseListenerForward.js ##//
caplin.namespace("caplin.streamlink.impl.credentials");

// caplin.forceload(caplin.streamlink.KeymasterResponseListener

caplin.streamlink.impl.credentials.KeymasterResponseListenerForward = function(keymasterResponseListener, keymasterCredentialsProvider, streamLinkContext) {
	// Constructor body
	this.keymasterResponseListener = keymasterResponseListener;
	this.keymasterCredentialsProvider = keymasterCredentialsProvider;
	this.context = streamLinkContext;
};

// caplin.implement(caplin.streamlink.impl.credentials.KeymasterResponseListenerForward, caplin.streamlink.KeymasterResponseListener);

caplin.streamlink.impl.credentials.KeymasterResponseListenerForward.prototype.onKeymasterError = function(reason) {
	if (this.keymasterResponseListener != null) {
		var self = this;
		this.context.runtime.createTimeout(function() {
			self.keymasterResponseListener.onKeymasterError(reason);
		}, 0);
	}
	this.keymasterCredentialsProvider.cancelPolling();
};

caplin.streamlink.impl.credentials.KeymasterResponseListenerForward.prototype.onKeymasterResponse = function(response) {
	if (this.keymasterResponseListener != null) {
		var self = this;
		this.context.runtime.createTimeout(function() {
			(self.keymasterResponseListener.onKeymasterResponse && self.keymasterResponseListener.onKeymasterResponse(response));
		}, 0);
	}
	this.keymasterCredentialsProvider.startKeymasterPollingInterval();
};



//## ConnectionFactoryImpl.js ##//
//## ConnectionFactoryImpl.js ##//
caplin.namespace("caplin.streamlink.js.connection");

caplin.include("caplin.streamlink.impl.core.StreamLinkContext");
caplin.include("caplin.streamlink.ConnectionFactory", true)
// caplin.forceload(caplin.streamlink.ConnectionFactory);
caplin.include("caplin.streamlink.impl.core.connection.Connection");
caplin.include("caplin.streamlink.js.impl.connection.Type2Connection");
caplin.include("caplin.streamlink.js.impl.connection.PollingConnection");
caplin.include("caplin.streamlink.js.impl.connection.ForeverFrame");


caplin.streamlink.js.connection.ConnectionFactoryImpl = function()
{
	var global = new Function("return this;")();
	this._supportsWebSocket = !!(global.WebSocket || global.MozWebSocket);
};

caplin.implement(caplin.streamlink.js.connection.ConnectionFactoryImpl, caplin.streamlink.ConnectionFactory);

caplin.streamlink.js.connection.ConnectionFactoryImpl.prototype.setContext = function(/*caplin.streamlink.impl.StreamLinkContext*/ context)
{
	this._context = context;
};

caplin.streamlink.js.connection.ConnectionFactoryImpl.prototype.getConnection = function(/*String*/ url) 
{
	switch (this._getProtocol(url))
	{
		case "ws":
			return new caplin.streamlink.js.impl.connection.WebSocketConnection(this._context, url, false);
		case "wss":
			return new caplin.streamlink.js.impl.connection.WebSocketConnection(this._context, url, true);
		case "http":
			return new caplin.streamlink.js.impl.connection.ForeverFrame(this._context, url, false);
		case "https":
			return new caplin.streamlink.js.impl.connection.ForeverFrame(this._context, url, true);
		case "xhr":
			return new caplin.streamlink.js.impl.connection.Type2Connection(this._context, url, false);
		case "xhrs":
			return new caplin.streamlink.js.impl.connection.Type2Connection(this._context, url, true);
		case "poll":
			return new caplin.streamlink.js.impl.connection.PollingConnection(this._context, url, false);
		case "polls":
			return new caplin.streamlink.js.impl.connection.PollingConnection(this._context, url, true);
		case "dummy":
			return this._context.getTestConnection();
		default:
			throw new Error("not implemented");
	}
};

caplin.streamlink.js.connection.ConnectionFactoryImpl.prototype._getProtocol = function(/*String*/ url)
{
	return url.substring(0, url.indexOf(":"));
};

caplin.streamlink.js.connection.ConnectionFactoryImpl.prototype.isSupported = function(/*String*/ url) 
{
	var protocol = this._getProtocol(url);
	return (this._supportsWebSocket && (protocol === "ws" || protocol === "wss")) 
			|| (protocol === "xhr" || protocol === "xhrs"  || protocol === "http" || protocol === "https" || protocol === "poll" || protocol === "polls");
};		

caplin.streamlink.js.connection.ConnectionFactoryImpl.prototype.getKeymasterConnection = function(url, keymasterResponseListener, keymasterTimeout, keymasterRetries)
{
	return new caplin.streamlink.js.impl.credentials.KeymasterConnectionImpl(url, keymasterResponseListener, keymasterTimeout, keymasterRetries, this._context.logger);
};


//## ActionFailEvent.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.impl.event.RttpEventBase

caplin.streamlink.impl.event.ActionFailEvent = function(code, objectNumber, sequenceNumber) {
	// Super
	caplin.streamlink.impl.event.RttpEventBase.call(this, null, code, objectNumber, sequenceNumber);};

caplin.extend(caplin.streamlink.impl.event.ActionFailEvent, caplin.streamlink.impl.event.RttpEventBase);

caplin.streamlink.impl.event.ActionFailEvent.createFromRttp = function(rttpCode, tokens, fieldMap, encoder) {
	return new caplin.streamlink.impl.event.ActionFailEvent(rttpCode, tokens[0].substring(4, 8), tokens[0].substring(2, 4));
};

caplin.streamlink.impl.event.ActionFailEvent.prototype.toString = function() {
	var builder = [] ;
	builder.push("ActionFailEvent [subject=");
	builder.push(this._subject);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.ActionFailEvent.prototype._publishCommandResponse = function(listener) {
	listener.onCommandError(this._subject, caplin.streamlink.impl.event.CommandErrorEventImpl._createFromRttpCode(this._rttpCode));
};

caplin.streamlink.impl.event.ActionFailEvent.prototype._publishSubscriptionResponse = function(subscription, subscriptionListener) {
	subscriptionListener.onSubscriptionError(subscription, caplin.streamlink.impl.event.SubjectErrorEventImpl.createFromRttpEvent(this));
};



//## ContainerSubscriptionParameters.js ##//
caplin.namespace("caplin.streamlink");

// caplin.forceload(caplin.streamlink.RecordSubscriptionParameters

/**
 * @class   
 * @private
 *   
 *  <p>The container subscription parameters allows the container "window" to be defined.</p>
 *  <p>The <code>ContainerSubscriptionParameters</code> is passed as a parameter to the <code>subscribe</code>
 *  method in the {@link caplin.streamlink.StreamLink}.</p>
 *  
 *  @extends caplin.streamlink.RecordSubscriptionParameters
 * 
 * @name caplin.streamlink.ContainerSubscriptionParameters
 */
caplin.streamlink.ContainerSubscriptionParameters = function() {};

// caplin.implement(caplin.streamlink.ContainerSubscriptionParameters, caplin.streamlink.RecordSubscriptionParameters);

/**
 *  <p>Sets the container window size.</p> 
 * @param {int} start The index of the first element. Where the first element has an index value of 0 (Zero).
 * @param {int} size The number of elements within the window.
 *  @returns {caplin.streamlink.ContainerSubscriptionParameters} The current ContainerParameters instance to permit a fluent programming style.
 * @function
 */
caplin.streamlink.ContainerSubscriptionParameters.prototype.setWindow = function(start, size) { throw new Error('caplin.streamlink.ContainerSubscriptionParameters.setWindow :  This is an abstract method'); };



//## ContainerModelInternal.js ##//
caplin.namespace("caplin.streamlink.impl.subscription.container");


caplin.streamlink.impl.subscription.container.ContainerModelInternal = function() {
	// Field Definitions
	this.size = 0;
	this.windowStart = 0;
	this.windowEnd = -(1);

	// Constructor body
	this.elements = [];
	this.subjectToElementMap = {} ;
};

caplin.streamlink.impl.subscription.container.ContainerModelInternal.prototype.hasMoves = function(previousElements, additionsAndDeletions) {
	for (var i = 0; i < additionsAndDeletions.length; ++(i)) {
		var item = additionsAndDeletions[i];
		var element = previousElements[item.getObjectNumber()];
		if (element != null) {
			return true;
		}
	}
	return false;
};

caplin.streamlink.impl.subscription.container.ContainerModelInternal.prototype.getElements = function() {
	return this.elements;
};

caplin.streamlink.impl.subscription.container.ContainerModelInternal.prototype.processImage = function(elements, containerEventInternal) {
	var actions = [];
	this.windowStart = containerEventInternal.getWindowStart();
	this.windowEnd = containerEventInternal.getWindowEnd();
	var previousElements = {} ;
	for (var i = 0; i < elements.length; (i)++) {
		var element = elements[i];
		previousElements[element._getObjectNumber()] = element;
	}
	var additionsAndDeletions = containerEventInternal.getItems();
	var anyMoves = this.hasMoves(previousElements, additionsAndDeletions);
	if (!(anyMoves)) {
		this.doClear(actions, elements);
	}
	for (var i = 0; i < additionsAndDeletions.length; ++(i)) {
		var item = additionsAndDeletions[i];
		var element = previousElements[item.getObjectNumber()];
		if (element != null) {
			var pos = this.findObjectNumber(elements, item.getObjectNumber());
			this.doMove(actions, elements, pos, i);
			delete previousElements[item.getObjectNumber()];
		}
		else {
			element = new caplin.streamlink.impl.event.ContainerElementImpl(item.getObjectNumber(), item.getSubject(), item.getType(), 0);
			this.doInsert(actions, elements, i, element);
		}
	}
	if (anyMoves) {
		for (var objectNumber in previousElements) {
			var pos = this.findObjectNumber(elements, objectNumber);
			this.doDelete(actions, elements, pos);
		}
	}
	return actions;
};

caplin.streamlink.impl.subscription.container.ContainerModelInternal.prototype.processUpdate = function(elements, containerEventInternal) {
	var actions = [];
	var additions = {} ;
	var additionsAndDeletions = containerEventInternal.getItems();
	for (var i = 0; i < additionsAndDeletions.length; ++(i)) {
		var item = additionsAndDeletions[i];
		if (item.isAdded()) {
			additions[item.getObjectNumber()] = item;
		}
		else {
			var index = this.findObjectNumber(elements, item.getObjectNumber());
			this.doDelete(actions, elements, index);
		}
	}
	var order = containerEventInternal.getOrder();
	for (var i = 0; i < order.length; (i)++) {
		var orderItem = order[i];
		var newPosition = orderItem.getPos() - this.windowStart;
		var objNum = orderItem.getObjectNumber();
		var item = additions[orderItem.getObjectNumber()];
		if (item != null) {
			var element = new caplin.streamlink.impl.event.ContainerElementImpl(objNum, item.getSubject(), item.getType(), 0);
			this.doInsert(actions, elements, newPosition, element);
		}
		else {
			var oldPosition = this.findObjectNumber(elements, objNum);
			this.doMove(actions, elements, oldPosition, newPosition);
		}
	}
	return actions;
};

caplin.streamlink.impl.subscription.container.ContainerModelInternal.prototype.update = function(containerEvent) {
	return this.updateInternalModel(this.elements, containerEvent);
};

caplin.streamlink.impl.subscription.container.ContainerModelInternal.prototype.updateInternalModel = function(elements, containerEventInternal) {
	this.size = containerEventInternal.getSize();
	var actions = null;
	if (containerEventInternal.isImage()) {
		actions = this.processImage(elements, containerEventInternal);
	}
	else {
		actions = this.processUpdate(elements, containerEventInternal);
	}
	this.reorderElements(elements);
	return actions;
};

caplin.streamlink.impl.subscription.container.ContainerModelInternal.prototype.findObjectNumber = function(elements, objNum) {
	for (var i = 0; i < elements.length; (i)++) {
		var element = elements[i];
		if ((element._getObjectNumber() == (objNum))) {
			return i;
		}
	}
	return -(1);
};

caplin.streamlink.impl.subscription.container.ContainerModelInternal.prototype.findSubject = function(subject) {
	var pos = -(1);
	var element = this.subjectToElementMap[subject];
	if (element != null) {
		pos = element._getPosition();
	}
	return pos;
};

caplin.streamlink.impl.subscription.container.ContainerModelInternal.prototype.getSize = function() {
	return this.size;
};

caplin.streamlink.impl.subscription.container.ContainerModelInternal.prototype.getWindowEnd = function() {
	return this.windowEnd;
};

caplin.streamlink.impl.subscription.container.ContainerModelInternal.prototype.getWindowStart = function() {
	return this.windowStart;
};

caplin.streamlink.impl.subscription.container.ContainerModelInternal.prototype.doClear = function(actions, elements) {
	(elements.length = 0);
	this.subjectToElementMap = {} ;
	actions.splice(0, 0, new caplin.streamlink.impl.subscription.container.ClearAction());
};

caplin.streamlink.impl.subscription.container.ContainerModelInternal.prototype.doDelete = function(actions, elements, pos) {
	var element = elements.splice(pos, 1)[0];
	delete this.subjectToElementMap[element.getSubject()];
	var action = new caplin.streamlink.impl.subscription.container.DeleteAction(element, pos);
	actions.push(action);
};

caplin.streamlink.impl.subscription.container.ContainerModelInternal.prototype.doInsert = function(actions, elements, pos, element) {
	elements.splice(pos, 0, element);
	this.subjectToElementMap[element.getSubject()] = element;
	var action = new caplin.streamlink.impl.subscription.container.InsertAction(element, pos);
	actions.push(action);
};

caplin.streamlink.impl.subscription.container.ContainerModelInternal.prototype.doMove = function(actions, elements, oldPos, newPos) {
	if (oldPos != newPos) {
		var element = elements.splice(oldPos, 1)[0];
		elements.splice(newPos, 0, element);
		var action = new caplin.streamlink.impl.subscription.container.MoveAction(element, oldPos, newPos);
		actions.push(action);
	}
};

caplin.streamlink.impl.subscription.container.ContainerModelInternal.prototype.reorderElements = function(elements) {
	for (var i = 0; i < elements.length; (i)++) {
		var element = elements[i];
		element._setPosition(i + this.windowStart);
	}
};



//## KeymasterCredentialsProviderImpl.js ##//
caplin.namespace("caplin.streamlink.impl.credentials");

// caplin.forceload(caplin.streamlink.KeymasterResponseListener
// caplin.forceload(caplin.streamlink.ConnectionListener
// caplin.forceload(caplin.streamlink.impl.credentials.KeymasterCredentialsProvider

caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl = function(connectionFactory, streamLinkContext) {
	// Field Definitions
	this.cancelled = false;

	// Constructor body
	var keymasterKeepAliveListener = streamLinkContext.configuration._getKeymasterKeepAliveListener();
	this.keymasterResponseListenerForward = new caplin.streamlink.impl.credentials.KeymasterResponseListenerForward(keymasterKeepAliveListener, this, streamLinkContext);
	this.connectionFactory = connectionFactory;
	this.streamLinkContext = streamLinkContext;
	this.keymasterUrl = streamLinkContext.configuration._getKeymasterUrl();
	this.keymasterKeepAliveUrl = streamLinkContext.configuration._getKeymasterKeepAliveUrl();
	this.keepAliveInterval = streamLinkContext.configuration._getKeymasterKeepAliveInterval();
	this.keymasterTimeout = streamLinkContext.configuration._getHttpTimeout();
	this.keymasterAttempts = streamLinkContext.configuration._getKeymasterAttempts();
};

// caplin.implement(caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl, caplin.streamlink.ConnectionListener);
// caplin.implement(caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl, caplin.streamlink.KeymasterResponseListener);
// caplin.implement(caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl, caplin.streamlink.impl.credentials.KeymasterCredentialsProvider);

caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl.prototype.parseKeymasterResponse = function(response) {
	var lines = response.replace(new RegExp("\r", 'g'), "").split("\n");
	var valid = false;
	var username = null;
	var password = null;
	for (var i = 0; i < lines.length; (i)++) {
		var line = lines[i];
		var index = line.indexOf("=");
		var name = line.substring(0, index);
		var value = line.substring(index + 1);
		if ((name == ("username"))) {
			username = ((value == (""))) ? null : value;
		}
		else if ((name == ("token"))) {
			password = ((value == (""))) ? null : value;
		}
		else if ((name == ("credentials"))) {
			if ((value == ("ok"))) {
				valid = true;
			}
		}
	}
	if (valid == true && username != null && password != null) {
		return new caplin.streamlink.impl.credentials.CredentialsImpl(username, password);
	}
	return null;
};

caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl.prototype.cancel = function() {
	this.streamLinkContext.logger.log(caplin.streamlink.LogLevel.FINEST, "Cancelling request for keymaster credentials.");
	this.cancelled = true;
	if (this.keymasterPollConnection != null) {
		this.keymasterPollConnection.cancel();
	}
};

caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl.prototype.cancelPolling = function() {
	this.streamLinkContext.logger.log(caplin.streamlink.LogLevel.FINEST, "Stopping keymaster session keep alive.");
	if (this.id != null) {
		this.id.cancel();
		this.id = null;
	}
	if (this.keymasterPollConnection != null) {
		this.keymasterPollConnection.cancel();
	}
};

caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl.prototype.onConnectionFail = function(reason) { };

caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl.prototype.onConnectionStatusChange = function(statusEvent) {
	var state = statusEvent.getConnectionState();
	if (state == caplin.streamlink.ConnectionState.LOGGEDIN || state == caplin.streamlink.ConnectionState.RECONNECTED) {
		this.cancelPolling();
		this.keymasterPollConnection = this.connectionFactory.getKeymasterConnection(this.keymasterKeepAliveUrl, this.keymasterResponseListenerForward, this.keymasterTimeout, this.keymasterAttempts);
		this.startKeymasterPollingInterval();
	}
	else if (state == caplin.streamlink.ConnectionState.DISCONNECTED) {
		this.cancelPolling();
	}
};

caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl.prototype.onCredentialsRequired = function(listener) {
	this.credentialsReceiver = listener;
	this.connectionFactory.getKeymasterConnection(this.keymasterUrl, this, this.keymasterTimeout, this.keymasterAttempts).start();
};

caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl.prototype.onKeymasterError = function(reason) {
	this.credentialsReceiver.credentialsUnavailable(reason);
};

caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl.prototype.onKeymasterResponse = function(response) {
	if (this.cancelled == false) {
		this.streamLinkContext.logger.log(caplin.streamlink.LogLevel.FINEST, "Credentials received from KeyMaster: {0}", response);
		var cred = this.parseKeymasterResponse(response);
		if (cred != null) {
			this.credentialsReceiver.provideCredentials(cred);
		}
		else {
			this.credentialsReceiver.credentialsUnavailable("cannot parse keymaster response");
		}
	}
	else {
		this.streamLinkContext.logger.log(caplin.streamlink.LogLevel.NOTIFY, "Credentials returned but ignored because this credentials provider has been cancelled: {0}", response);
	}
};

caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl.prototype.onServiceStatusChange = function(svcStatusEvent) { };

caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl.prototype.onSourceStatusChange = function(srcStatusEvent) { };

caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl.prototype.onStatisticsChange = function(statChangeEvent) { };

caplin.streamlink.impl.credentials.KeymasterCredentialsProviderImpl.prototype.startKeymasterPollingInterval = function() {
	this.streamLinkContext.logger.log(caplin.streamlink.LogLevel.FINEST, "Start keymaster session keep alive.");
	var self = this;
	this.id = this.streamLinkContext.runtime.createTimeout(function() {
		self.keymasterPollConnection.start();
	}, this.keepAliveInterval);
};



//## Channel.js ##//
caplin.namespace("caplin.streamlink.js.impl.connection");

caplin.streamlink.js.impl.connection.Channel = (function()
{

	var global = new Function("return this;")();
	var supportsXDR = global.XDomainRequest && (global.XMLHttpRequest && new XMLHttpRequest().responseType === undefined);

	function getChannel(url, timeout, useCacheBuster)
	{
		var thisPageUrl = global.location.protocol + "//" + global.location.host + "/";

		if (supportsXDR == true && url.substring(0, thisPageUrl.length) != thisPageUrl)
		{
			return new XDRChannel(url, timeout, useCacheBuster);
		}
		return new XHRChannel(url, timeout, useCacheBuster);
	}

	function stopTimeout(obj)
	{
		if (obj.id != null)
		{
			clearTimeout(obj.id);
			obj.id = null;
		}
	}

	// Makes sure each request is not cached by the browser
	function getCacheBusterQuery(url, useCacheBuster)
	{
		var cacheBusterString = "";
		if (useCacheBuster)
		{
			cacheBusterQuery = "cachebuster=" + new Date().getTime();
			if (url.indexOf("?") > 0)
			{
				cacheBusterString = "&" + cacheBusterQuery;
			}
			else
			{
				cacheBusterString = "?" + cacheBusterQuery;
			}
		}
		return cacheBusterString;
	}

	// Checks for Firefox and IE to make sure the response is valid
	function isValidResponse(xhr)
	{
		// If there is a non-empty response then this is a valid response otherwise we check response headers
		return xhr.responseText != "" || isResponseHeadersAndResponseTextValid(xhr);
	}
	
	// Check for IE to make sure the request was not 'Aborted'
	function isResponseHeadersAndResponseTextValid(xhr)
	{
		var allResponseHeaders = xhr.getAllResponseHeaders();
		return allResponseHeaders != null && allResponseHeaders.length > 0 && xhr.responseText != null;
	}

	/**
	 * XHR Connection
	 */
	function XHRChannel(url, timeout, useCacheBuster)
	{
		var self = this;
		this.className = "XHRChannel";
		this.xhr = new XMLHttpRequest();
		this.url = url;
		this.timeout = timeout;
		this.useCacheBuster = (useCacheBuster === true);

		this.dataChunkStart = 0;

		var xhr = this.xhr;

		xhr.onreadystatechange = function()
		{
			var readyState = xhr.readyState;
			if (readyState >= 3 && xhr.status == 200 && self.onprogress)
			{
				stopTimeout(self);
				self.onprogress(xhr.responseText.substring(self.dataChunkStart));
				self.dataChunkStart = self.xhr.responseText.length;
			}
			if (readyState == 4)
			{
				stopTimeout(self);
				if (xhr.status == 200 && isValidResponse(xhr))
				{
					if (self.onload) self.onload(xhr.responseText);
				}
				else
				{
					if (self.onerror) self.onerror(xhr.status, xhr.responseText);
				}
			}
		};
	}

	XHRChannel.prototype.send = function(data, contentType)
	{
		this.dataChunkStart = 0;

		var xhr = this.xhr;

		var cacheBuster = getCacheBusterQuery(this.url, this.useCacheBuster);

		if (data != undefined)
		{
			xhr.open("POST", this.url + cacheBuster, true);
		}
		else
		{
			xhr.open("GET", this.url + cacheBuster, true);
		}
		if (contentType != undefined)
		{
			xhr.setRequestHeader("Content-Type", contentType);
		}

		var self = this;
		stopTimeout(self);
		if (this.timeout > 0)
		{
			this.id = setTimeout(function()
			{
				if (self.onerror) self.onerror(-1, "Http Response Timed Out after " + self.timeout + "ms");
				self.abort();
			}, self.timeout);
		}

		xhr.send(data);
	};

	XHRChannel.prototype.abort = function()
	{
		this.xhr.abort();
	};

	/**
	 * XDR Connection
	 */
	function XDRChannel(url, timeout, useCacheBuster)
	{
		this.className = "XDRChannel";
		this.xdr = new XDomainRequest();
		this.url = url;
		this.timeout = timeout;
		this.useCacheBuster = (useCacheBuster === true);

		this.dataChunkStart = 0;

		var self = this;
		var xdr = this.xdr;

		xdr.onload = function()
		{
			if (self.onload) setTimeout(function()
			{
				stopTimeout(self);
				self.onload(xdr.responseText);
			}, 0);
		};
		xdr.onprogress = function()
		{
			if (self.onprogress)
			{
				stopTimeout(self);
				self.onprogress(xdr.responseText.substring(self.dataChunkStart));
				self.dataChunkStart = self.xdr.responseText.length;
			}
		};
		xdr.onerror = function()
		{
			if (self.onerror)
			{
				stopTimeout(self);
				self.onerror(-1, xdr.responseText);
			}
		};
	}

	XDRChannel.prototype.send = function(data, contentType)
	{
		var xdr = this.xdr;
		this.dataChunkStart = 0;

		try
		{
			var cacheBuster = getCacheBusterQuery(this.url, this.useCacheBuster);
			// ie-8 doesn't support this :-(
			// if (contentType != null) xdr.contentType = contentType;
			if (data != undefined)
			{
				xdr.open("POST", this.url + cacheBuster);
			}
			else
			{
				xdr.open("GET", this.url + cacheBuster);
			}

			var self = this;
			this.id = setTimeout(function()
			{
				if (self.onerror) self.onerror(-1, "Http Response Timed Out after " + self.timeout + "ms");
				self.abort();
			}, self.timeout);

			xdr.send(data);
		}
		catch (e)
		{
			stopTimeout(this);
			xdr.onerror();
		}
		;
	};

	XDRChannel.prototype.abort = function()
	{
		this.xdr.abort();
	};

	return getChannel;
})();


//## StreamLinkFactoryInternal.js ##//
caplin.namespace("caplin.streamlink.impl");


caplin.streamlink.impl.StreamLinkFactoryInternal = function() {};

caplin.streamlink.impl.StreamLinkFactoryInternal.prototype.create = function(logger, configuration, keymasterResponseListener) { throw new Error('caplin.streamlink.impl.StreamLinkFactoryInternal.create :  This is an abstract method'); };



//## DirectoryElementImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.impl.Serializable
// caplin.forceload(caplin.streamlink.DirectoryChangeElement

caplin.streamlink.impl.event.DirectoryElementImpl = function(subject, type, added) {
	// Constructor body
	this._subject = subject;
	this._type = type;
	this._added = added;
};

// caplin.implement(caplin.streamlink.impl.event.DirectoryElementImpl, caplin.streamlink.DirectoryChangeElement);
// caplin.implement(caplin.streamlink.impl.event.DirectoryElementImpl, caplin.streamlink.impl.Serializable);

caplin.streamlink.impl.event.DirectoryElementImpl.prototype.isAdded = function() {
	return this._added;
};

caplin.streamlink.impl.event.DirectoryElementImpl.prototype.getType = function() {
	return this._type;
};

caplin.streamlink.impl.event.DirectoryElementImpl.createFromDictionary = function(root) {
	return new caplin.streamlink.impl.event.DirectoryElementImpl(root["Subject"], caplin.streamlink.SubjectType.valueOf(root["Type"]), root["Added"]);
};

caplin.streamlink.impl.event.DirectoryElementImpl.prototype._toDictionary = function() {
	var root = {} ;
	root["Subject"] = this._subject;
	root["Type"] = this._type.toString();
	root["Added"] = this._added;
	return root;
};

caplin.streamlink.impl.event.DirectoryElementImpl.prototype.getSubject = function() {
	return this._subject;
};

caplin.streamlink.impl.event.DirectoryElementImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("DirectoryChangeElement [subject=");
	builder.push(this._subject);
	builder.push(", type=");
	builder.push(this._type);
	builder.push(", added=");
	builder.push(this._added);
	builder.push("]");
	return builder.join("");
};



//## LazyLogInfo.js ##//
caplin.namespace("caplin.streamlink.impl.logger");

// caplin.forceload(caplin.streamlink.LogInfo

caplin.streamlink.impl.logger.LazyLogInfo = function(level, msgTemplate, args) {
	// Field Definitions
	this._message = null;
	this._representation = null;

	// Constructor body
	this._level = level;
	this._messageTemplate = msgTemplate;
	this._args = args;
	this._date = new Date();
};

// caplin.implement(caplin.streamlink.impl.logger.LazyLogInfo, caplin.streamlink.LogInfo);

caplin.streamlink.impl.logger.LazyLogInfo.prototype.getLevel = function() {
	return this._level;
};

caplin.streamlink.impl.logger.LazyLogInfo.prototype.getMessage = function() {
	if (this._message == null) {
		this._message = caplin.streamlink.impl.utils.Utils.replaceParameters(this._messageTemplate, this._args);
	}
	return this._message;
};

caplin.streamlink.impl.logger.LazyLogInfo.prototype.toString = function() {
	if (this._representation == null) {
		this._representation = caplin.utils.CommonUtils.formatDateStamp(this._date) + " - " + this._level.getPaddedName() + " : " + this.getMessage();
	}
	return this._representation;
};

caplin.streamlink.impl.logger.LazyLogInfo.prototype.getDate = function() {
	return this._date;
};



//## ClearAction.js ##//
caplin.namespace("caplin.streamlink.impl.subscription.container");

// caplin.forceload(caplin.streamlink.impl.subscription.container.Action

caplin.streamlink.impl.subscription.container.ClearAction = function() {};

// caplin.implement(caplin.streamlink.impl.subscription.container.ClearAction, caplin.streamlink.impl.subscription.container.Action);

caplin.streamlink.impl.subscription.container.ClearAction.createFromDictionary = function(root) {
	return new caplin.streamlink.impl.subscription.container.ClearAction();
};

caplin.streamlink.impl.subscription.container.ClearAction.prototype._toDictionary = function() {
	var root = {} ;
	root["actionType"] = "Clear";
	return root;
};

caplin.streamlink.impl.subscription.container.ClearAction.prototype.toString = function() {
	return "Clear";
};

caplin.streamlink.impl.subscription.container.ClearAction.prototype.applyToList = function(list) {
	(list.length = 0);
};

caplin.streamlink.impl.subscription.container.ClearAction.prototype.applyToModel = function(model) {
	model.clear();
};



//## PacketDiscard.js ##//
caplin.namespace("caplin.streamlink.impl.packet");

// caplin.forceload(caplin.streamlink.impl.packet.Packet

caplin.streamlink.impl.packet.PacketDiscard = function(subject) {
	// Constructor body
	this.subject = subject;
};

// caplin.implement(caplin.streamlink.impl.packet.PacketDiscard, caplin.streamlink.impl.packet.Packet);

caplin.streamlink.impl.packet.PacketDiscard.prototype.getType = function() {
	return caplin.streamlink.impl.packet.PacketType.DISCARD;
};

caplin.streamlink.impl.packet.PacketDiscard.prototype.getSubject = function() {
	return this.subject;
};

caplin.streamlink.impl.packet.PacketDiscard.prototype.toRttpString = function(encoder) {
	return caplin.streamlink.impl.packet.PacketType.DISCARD.getCode() + " " + encoder.rttpEncode(this.subject);
};

caplin.streamlink.impl.packet.PacketDiscard.prototype.toString = function() {
	var builder = [] ;
	builder.push("Discard [subject=");
	builder.push(this.subject);
	builder.push("]");
	return builder.join("");
};



//## CommandSubscriptionOnWorker.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.server");

// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver

caplin.streamlink.impl.bridge.worker.server.CommandSubscriptionOnWorker = function(id, commandSubscription) {
	// Constructor body
	this.id = id;
	this.commandSubscription = commandSubscription;
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.server.CommandSubscriptionOnWorker, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.server.CommandSubscriptionOnWorker.prototype.getReceiverKey = function() {
	return null;
};

caplin.streamlink.impl.bridge.worker.server.CommandSubscriptionOnWorker.prototype.getReceiverId = function() {
	return this.id;
};

caplin.streamlink.impl.bridge.worker.server.CommandSubscriptionOnWorker.prototype.receive = function(method, args) {
	if ((method == ("unpersist"))) {
		this.commandSubscription.unPersist();
	}
};



//## ContainerOrderItem.js ##//
caplin.namespace("caplin.streamlink.impl.event");

caplin.streamlink.impl.event.ContainerOrderItem = function(objectNumber, pos) {
	// Constructor body
	this.objectNumber = objectNumber;
	this.pos = pos;
};

caplin.streamlink.impl.event.ContainerOrderItem.prototype.getPos = function() {
	return this.pos;
};

caplin.streamlink.impl.event.ContainerOrderItem.prototype.getObjectNumber = function() {
	return this.objectNumber;
};

caplin.streamlink.impl.event.ContainerOrderItem.prototype.toString = function() {
	var builder = [] ;
	builder.push("[objectNumber=");
	builder.push(this.objectNumber);
	builder.push(", pos=");
	builder.push(this.pos);
	builder.push("]");
	return builder.join("");
};



//## PacketType.js ##//
caplin.namespace("caplin.streamlink.impl.packet");

caplin.streamlink.impl.packet.PacketType = function(id, name, responseRequired, system, batchable) {
	// Constructor body
	this.id = id;
	this.code = name;
	this.responseRequired = responseRequired;
	this.system = system;
	this.batchable = batchable;
};

caplin.streamlink.impl.packet.PacketType.REQUEST = new caplin.streamlink.impl.packet.PacketType(0, "REQUEST", true, false, true);
caplin.streamlink.impl.packet.PacketType.DISCARD = new caplin.streamlink.impl.packet.PacketType(1, "DISCARD", true, false, true);
caplin.streamlink.impl.packet.PacketType.CONTRIB = new caplin.streamlink.impl.packet.PacketType(2, "CONTRIB", true, false, true);
caplin.streamlink.impl.packet.PacketType.NOOP = new caplin.streamlink.impl.packet.PacketType(3, "NOOP", true, true, true);
caplin.streamlink.impl.packet.PacketType.SYNC = new caplin.streamlink.impl.packet.PacketType(4, "SYNC", true, true, false);
caplin.streamlink.impl.packet.PacketType.LOGIN = new caplin.streamlink.impl.packet.PacketType(5, "LOGIN", true, true, false);
caplin.streamlink.impl.packet.PacketType.CREATE = new caplin.streamlink.impl.packet.PacketType(6, "CREATE", true, false, true);
caplin.streamlink.impl.packet.PacketType.DELETE = new caplin.streamlink.impl.packet.PacketType(7, "DELETE", true, false, true);
caplin.streamlink.impl.packet.PacketType.GLOBAL_THROTTLE = new caplin.streamlink.impl.packet.PacketType(8, "GLOBAL_THROTTLE", true, false, true);
caplin.streamlink.impl.packet.PacketType.THROTTLE = new caplin.streamlink.impl.packet.PacketType(9, "THROTTLE", true, false, true);
caplin.streamlink.impl.packet.PacketType.AUTO = new caplin.streamlink.impl.packet.PacketType(99, "AUTO", false, false, false);
caplin.streamlink.impl.packet.PacketType.LOGOUT = new caplin.streamlink.impl.packet.PacketType(10, "LOGOUT", true, true, false);
caplin.streamlink.impl.packet.PacketType.values = function() { return [caplin.streamlink.impl.packet.PacketType.REQUEST, caplin.streamlink.impl.packet.PacketType.DISCARD, caplin.streamlink.impl.packet.PacketType.CONTRIB, caplin.streamlink.impl.packet.PacketType.NOOP, caplin.streamlink.impl.packet.PacketType.SYNC, caplin.streamlink.impl.packet.PacketType.LOGIN, caplin.streamlink.impl.packet.PacketType.CREATE, caplin.streamlink.impl.packet.PacketType.DELETE, caplin.streamlink.impl.packet.PacketType.GLOBAL_THROTTLE, caplin.streamlink.impl.packet.PacketType.THROTTLE, caplin.streamlink.impl.packet.PacketType.AUTO, caplin.streamlink.impl.packet.PacketType.LOGOUT];};
caplin.streamlink.impl.packet.PacketType.valueOf = function(name) { return this[name]; };
caplin.streamlink.impl.packet.PacketType.prototype.isBatchable = function() {
	return this.batchable;
};

caplin.streamlink.impl.packet.PacketType.prototype.isResponseRequired = function() {
	return this.responseRequired;
};

caplin.streamlink.impl.packet.PacketType.prototype.isSystem = function() {
	return this.system;
};

caplin.streamlink.impl.packet.PacketType.prototype.getId = function() {
	return this.id;
};

caplin.streamlink.impl.packet.PacketType.prototype.getCode = function() {
	return this.code;
};

caplin.streamlink.impl.packet.PacketType.prototype.toString = function() {
	return "id: " + this.id + ", code: " + this.code + ", isReponseRequired: " + this.responseRequired + ", isSystem: " + this.system + ", isBatchable: " + this.batchable;
};



//## EncoderImpl.js ##//
caplin.namespace("caplin.streamlink.js.impl.protocolhandler");

caplin.include("caplin.streamlink.impl.protocolhandler.Encoder", true);

caplin.streamlink.js.impl.protocolhandler.EncoderImpl = function() 
{};

caplin.implement(caplin.streamlink.js.impl.protocolhandler.EncoderImpl, "caplin.streamlink.impl.protocolhandler.Encoder");

(function () {

var fReplaceDecode = function (sMatched) { return sMatched === "+" ? " " : "%25"; };

caplin.streamlink.js.impl.protocolhandler.EncoderImpl.prototype.rttpDecode = function(sRttp) 
{
	return decodeURIComponent(sRttp.replace(/%(?![0-9a-fA-F][0-9a-fA-F])|\+/g, fReplaceDecode));
};

})();

(function () {

var mEncodingSubtitutions = {
		"%20": "+",
		"*": "%2A",
		"!": "%21",
		"(": "%28",
		")": "%29",
		"'": "%27"
};
var fReplaceEncode = function (sMatched) { return mEncodingSubtitutions[sMatched]; };

caplin.streamlink.js.impl.protocolhandler.EncoderImpl.prototype.rttpEncode = function(sRttp) 
{
	return encodeURIComponent(sRttp).replace(/[*!'()]|%20/g, fReplaceEncode);
};
	
})();



//## Runtime.js ##//
caplin.namespace("caplin.streamlink.impl");


caplin.streamlink.impl.Runtime = function() {};

caplin.streamlink.impl.Runtime.prototype.createInterval = function(runnable, milliseconds) { throw new Error('caplin.streamlink.impl.Runtime.createInterval :  This is an abstract method'); };

caplin.streamlink.impl.Runtime.prototype.createTimeout = function(runnable, milliseconds) { throw new Error('caplin.streamlink.impl.Runtime.createTimeout :  This is an abstract method'); };

caplin.streamlink.impl.Runtime.prototype.parseInt = function(number) { throw new Error('caplin.streamlink.impl.Runtime.parseInt :  This is an abstract method'); };

caplin.streamlink.impl.Runtime.prototype.currentTimeMillis = function() { throw new Error('caplin.streamlink.impl.Runtime.currentTimeMillis :  This is an abstract method'); };

caplin.streamlink.impl.Runtime.prototype.parseLong = function(number) { throw new Error('caplin.streamlink.impl.Runtime.parseLong :  This is an abstract method'); };



//## ConnectionMonitor.js ##//
caplin.namespace("caplin.streamlink.impl.protocolhandler");

// caplin.forceload(caplin.streamlink.ConnectionListener

caplin.streamlink.impl.protocolhandler.ConnectionMonitor = function(protocolHandler, context, clockOffsetInterval, clockOffsetsInWindow, latencysInWindow) {
	// Field Definitions
	this.timeSentFromClient = 0;
	this.inflight = false;

	// Constructor body
	this.protocolHandler = protocolHandler;
	this.runtime = context.runtime;
	this.logger = context.logger;
	this.clockOffsetInterval = clockOffsetInterval;
	this.statistics = new caplin.streamlink.impl.protocolhandler.StatisticsCalculator(clockOffsetsInWindow, latencysInWindow);
};

// caplin.implement(caplin.streamlink.impl.protocolhandler.ConnectionMonitor, caplin.streamlink.ConnectionListener);

caplin.streamlink.impl.protocolhandler.ConnectionMonitor.prototype.getTimeStamp = function(tokens) {
	for (var i = 0; i < tokens.length; (i)++) {
		if (!((tokens[i].indexOf("=")>=0))) {
			continue;
		}
		var pair = tokens[i].split("=");
		if (pair.length == 2 && ("TimeStamp" == ("TimeStamp"))) {
			return pair[1];
		}
	}
	return null;
};

caplin.streamlink.impl.protocolhandler.ConnectionMonitor.prototype.getClockOffset = function() {
	return this.statistics.getClockOffset();
};

caplin.streamlink.impl.protocolhandler.ConnectionMonitor.prototype.updateTimeSentFromClient = function() {
	this.timeSentFromClient = this.runtime.currentTimeMillis();
	return this.timeSentFromClient;
};

caplin.streamlink.impl.protocolhandler.ConnectionMonitor.prototype.onConnectionFail = function(reason) { };

caplin.streamlink.impl.protocolhandler.ConnectionMonitor.prototype.onConnectionStatusChange = function(connectionStatusEvent) {
	var state = connectionStatusEvent.getConnectionState();
	if (caplin.streamlink.ConnectionState.LOGGEDIN == state || caplin.streamlink.ConnectionState.RECONNECTED == state) {
		this.startSyncing();
	}
	else {
		this.inflight = false;
		this.statistics.clear();
		if (this.syncTimer != null) {
			this.syncTimer.cancel();
		}
	}
};

caplin.streamlink.impl.protocolhandler.ConnectionMonitor.prototype.onServiceStatusChange = function(serviceStatusEvent) { };

caplin.streamlink.impl.protocolhandler.ConnectionMonitor.prototype.onSourceStatusChange = function(sourceStatusEvent) { };

caplin.streamlink.impl.protocolhandler.ConnectionMonitor.prototype.onStatisticsChange = function(statisticsEvent) { };

caplin.streamlink.impl.protocolhandler.ConnectionMonitor.prototype.onSyncOk = function(tokens) {
	this.inflight = false;
	if (tokens.length > 3) {
		var timeReceivedOnClient = this.runtime.currentTimeMillis();
		var timeSentFromServer = this.runtime.parseLong(tokens[2]);
		var timeBetweenClientToServer = this.runtime.parseLong(tokens[3]);
		this.statistics.updateSyncStatistics(this.timeSentFromClient, timeSentFromServer, timeBetweenClientToServer, timeReceivedOnClient);
		this.protocolHandler.fireOnStatisticsChange(this.statistics.newStatisticsEvent());
	}
};

caplin.streamlink.impl.protocolhandler.ConnectionMonitor.prototype.onTimedHeartbeat = function(tokens) {
	var rawTimestamp = this.getTimeStamp(tokens);
	if (rawTimestamp == null) {
		return;
	}
	var timeSentFromServer = this.runtime.parseLong(rawTimestamp);
	var timeReceivedOnClient = this.runtime.currentTimeMillis();
	this.statistics.updateLatencyStatistics(timeSentFromServer, timeReceivedOnClient);
	this.protocolHandler.fireOnStatisticsChange(this.statistics.newStatisticsEvent());
};

caplin.streamlink.impl.protocolhandler.ConnectionMonitor.prototype.startSyncing = function() {
	var self = this;
	this.syncTimer = this.runtime.createTimeout(function() {
		self.sync();
		self.startSyncing();
	}, this.clockOffsetInterval);
};

caplin.streamlink.impl.protocolhandler.ConnectionMonitor.prototype.sync = function() {
	if (this.inflight) {
		return;
	}
	try {
		this.inflight = true;
		this.protocolHandler.sendPacket(new caplin.streamlink.impl.packet.PacketSync(this, this.statistics.getAverageLatency(), this.statistics.getClockOffset()));
	}
	catch (e) {
		this.inflight = false;
		this.logger.log(caplin.streamlink.LogLevel.WARN, "Exception while attempting sync {0}", e);
	}
};



//## ContainerModel.js ##//
caplin.namespace("caplin.streamlink");


/**
 * @class  <p>This interface defines a model that represents a container. You need to implement it to process
 *  the <code>updateModel</code> callback in {@link caplin.streamlink.ContainerEvent}s.</p>
 * 
 *  <p>The methods on this interface will be called in the correct order to ensure that your
 *  representation is correct.</p> 
 *  
 * 
 * @name caplin.streamlink.ContainerModel
 */
caplin.streamlink.ContainerModel = function() {};

/**
 *  <p>Called to indicate that you should clear your model of the container.</p>
 * @function
 */
caplin.streamlink.ContainerModel.prototype.clear = function() { throw new Error('caplin.streamlink.ContainerModel.clear :  This is an abstract method'); };

/**
 *  <p>Called to indicate that you should insert the ContainerElement at the specified position.</p>
 * @param {int} index The index at which to insert the ContainerElement.
 * @param {caplin.streamlink.ContainerElement} element The ContainerElement to insert.
 * @function
 */
caplin.streamlink.ContainerModel.prototype.insert = function(index, element) { throw new Error('caplin.streamlink.ContainerModel.insert :  This is an abstract method'); };

/**
 *  <p>Called to indicate that you should move the specified ContainerElement.</p>
 * @param {int} from The index at which the ContainerElement is currently located.
 * @param {int} to The index to which the ContainerElement should be moved to.
 * @param {caplin.streamlink.ContainerElement} element The ContainerElement to move.
 * @function
 */
caplin.streamlink.ContainerModel.prototype.move = function(from, to, element) { throw new Error('caplin.streamlink.ContainerModel.move :  This is an abstract method'); };

/**
 *  <p>Called to indicate that you should remove the ContainerElement from the specified position.</p>
 * @param {int} index The index at which to remove the ContainerElement.
 * @param {caplin.streamlink.ContainerElement} element The ContainerElement to remove.
 * @function
 */
caplin.streamlink.ContainerModel.prototype.remove = function(index, element) { throw new Error('caplin.streamlink.ContainerModel.remove :  This is an abstract method'); };



//## CurrentSubscriptions.js ##//
caplin.namespace("caplin.streamlink.impl.subscription");


caplin.streamlink.impl.subscription.CurrentSubscriptions = function() {
	// Field Definitions
	this.subscriptions = {} ;
};

caplin.streamlink.impl.subscription.CurrentSubscriptions.prototype.containsMessages = function(objectNumber) {
	return this.subscriptions[objectNumber] != null;
};

caplin.streamlink.impl.subscription.CurrentSubscriptions.prototype.getAllMessages = function() {
	var result = [];
	for (var objectNumber in this.subscriptions) {
		var messages = this.subscriptions[objectNumber];
		for (var i = 0; i < messages.length; (i)++) {
			result.push(messages[i]);
		}
	}
	return result;
};

caplin.streamlink.impl.subscription.CurrentSubscriptions.prototype.getAllNonAutoMessages = function() {
	var result = [];
	for (var objectNumber in this.subscriptions) {
		var messages = this.subscriptions[objectNumber];
		for (var i = 0; i < messages.length; (i)++) {
			var message = messages[i];
			if (message.getPacket().getType() != caplin.streamlink.impl.packet.PacketType.AUTO) {
				result.push(message);
			}
		}
	}
	return result;
};

caplin.streamlink.impl.subscription.CurrentSubscriptions.prototype.getArray = function(objectNumber) {
	var messages = this.subscriptions[objectNumber];
	if (messages == null) {
		messages = [];
		this.subscriptions[objectNumber] = messages;
	}
	return messages;
};

caplin.streamlink.impl.subscription.CurrentSubscriptions.prototype.getMessages = function(objectNumber) {
	return this.subscriptions[objectNumber];
};

caplin.streamlink.impl.subscription.CurrentSubscriptions.prototype.clear = function() {
	this.subscriptions = {} ;
};

caplin.streamlink.impl.subscription.CurrentSubscriptions.prototype.remove = function(objectNumber, eventHandler) {
	var messages = this.subscriptions[objectNumber];
	this.removeEventHandlerFromArray(messages, eventHandler);
	if ((messages.length==0)) {
		delete this.subscriptions[objectNumber];
	}
};

caplin.streamlink.impl.subscription.CurrentSubscriptions.prototype.removeAll = function(objectNumber) {
	delete this.subscriptions[objectNumber];
};

caplin.streamlink.impl.subscription.CurrentSubscriptions.prototype.removeEventHandlerFromArray = function(messages, eventHandler) {
	for (var i = 0; i < messages.length; (i)++) {
		var message = messages[i];
		if (message.getHandler() == eventHandler) {
			messages.splice(i,1)[0];
		}
	}
};

caplin.streamlink.impl.subscription.CurrentSubscriptions.prototype.set = function(objectNumber, message) {
	var messages = this.getArray(objectNumber);
	this.removeEventHandlerFromArray(messages, message.getHandler());
	messages.push(message);
};



//## StreamLinkCoreImpl.js ##//
caplin.namespace("caplin.streamlink.impl");

// caplin.forceload(caplin.streamlink.impl.StreamLinkCore

caplin.streamlink.impl.StreamLinkCoreImpl = function(context, protocolHandler, subscriptionManager) {
	// Field Definitions
	this._nextCtrId = 1;

	// Constructor body
	this._context = context;
	this._protocolHandler = protocolHandler;
	this._subscriptionManager = subscriptionManager;
	this._testConnection = new caplin.streamlink.impl.connection.TestConnection(context, "dummy://testconnection", false);
	this._context.setTestConnection(this._testConnection);
	this._configLogged = false;
};

// caplin.implement(caplin.streamlink.impl.StreamLinkCoreImpl, caplin.streamlink.impl.StreamLinkCore);

caplin.streamlink.impl.StreamLinkCoreImpl.prototype.createCommandParameters = function() {
	return new caplin.streamlink.impl.command.CommandParametersImpl();
};

caplin.streamlink.impl.StreamLinkCoreImpl.prototype.createSubject = function(subject, type, listener, parameters) {
	var commandResponse = new caplin.streamlink.impl.command.CommandResponse(listener, this._context);
	var commandParameters = parameters;
	if (commandParameters._isPersistent()) {
		this._subscriptionManager.sendPersistBeforeCommand(commandResponse, new caplin.streamlink.impl.packet.PacketCreate(subject, type));
	}
	else {
		this._subscriptionManager.send(commandResponse, new caplin.streamlink.impl.packet.PacketCreate(subject, type));
	}
	return new caplin.streamlink.impl.command.CommandSubscriptionImpl(this._subscriptionManager, commandResponse, subject, listener);
};

caplin.streamlink.impl.StreamLinkCoreImpl.prototype.publishToSubject = function(subject, fieldData, listener, parameters) {
	var commandResponse = new caplin.streamlink.impl.command.CommandResponse(listener, this._context);
	var commandParameters = parameters;
	if (commandParameters._isPersistent()) {
		this._subscriptionManager.sendPersistBeforeCommand(commandResponse, new caplin.streamlink.impl.packet.PacketContrib(subject, fieldData));
	}
	else {
		this._subscriptionManager.send(commandResponse, new caplin.streamlink.impl.packet.PacketContrib(subject, fieldData));
	}
	return new caplin.streamlink.impl.command.CommandSubscriptionImpl(this._subscriptionManager, commandResponse, subject, listener);
};

caplin.streamlink.impl.StreamLinkCoreImpl.prototype.throttleEverything = function(command, listener, parameters) {
	var commandResponse = new caplin.streamlink.impl.command.CommandResponse(listener, this._context);
	var commandParameters = parameters;
	if (commandParameters._isPersistent()) {
		this._subscriptionManager.sendPersistAfterCommand(commandResponse, new caplin.streamlink.impl.packet.PacketGlobalThrottle(command));
	}
	else {
		this._subscriptionManager.send(commandResponse, new caplin.streamlink.impl.packet.PacketGlobalThrottle(command));
	}
	return new caplin.streamlink.impl.command.CommandSubscriptionImpl(this._subscriptionManager, commandResponse, "ALL", listener);
};

caplin.streamlink.impl.StreamLinkCoreImpl.prototype.throttleSubject = function(subject, command, listener, parameters) {
	var commandResponse = new caplin.streamlink.impl.command.CommandResponse(listener, this._context);
	var commandParameters = parameters;
	if (commandParameters._isPersistent()) {
		this._subscriptionManager.sendPersistAfterCommand(commandResponse, new caplin.streamlink.impl.packet.PacketThrottle(subject, command));
	}
	else {
		this._subscriptionManager.send(commandResponse, new caplin.streamlink.impl.packet.PacketThrottle(subject, command));
	}
	return new caplin.streamlink.impl.command.CommandSubscriptionImpl(this._subscriptionManager, commandResponse, subject, listener);
};

caplin.streamlink.impl.StreamLinkCoreImpl.prototype.createContainerSubscriptionParameters = function() {
	return new caplin.streamlink.impl.subscription.ContainerSubscriptionParametersImpl((this._nextCtrId)++);
};

caplin.streamlink.impl.StreamLinkCoreImpl.prototype.createFreeFormSubscriptionParameters = function() {
	return new caplin.streamlink.impl.subscription.FreeFormSubscriptionParametersImpl();
};

caplin.streamlink.impl.StreamLinkCoreImpl.prototype.getLogger = function() {
	return this._context.logger;
};

caplin.streamlink.impl.StreamLinkCoreImpl.prototype.createRecordSubscriptionParameters = function() {
	return new caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl();
};

caplin.streamlink.impl.StreamLinkCoreImpl.prototype.subscribe = function(subject, listener, parameters) {
	var subscription = new caplin.streamlink.impl.subscription.SubscriptionImpl(this._subscriptionManager, this._context, subject, listener, parameters);
	this._subscriptionManager.send(subscription, new caplin.streamlink.impl.packet.PacketRequest(subject, parameters));
	return subscription;
};

caplin.streamlink.impl.StreamLinkCoreImpl.prototype._getTestConnection = function() {
	return this._testConnection;
};

caplin.streamlink.impl.StreamLinkCoreImpl.prototype.getVersion = function() {
	return caplin.streamlink.impl.Version.getVersion();
};

caplin.streamlink.impl.StreamLinkCoreImpl.prototype.addConnectionListener = function(listener) {
	this._protocolHandler.addConnectionListener(listener);
};

caplin.streamlink.impl.StreamLinkCoreImpl.prototype.connect = function() {
	if (this._configLogged == false) {
		this._context.logger.log(caplin.streamlink.LogLevel.CONFIG, "StreamLink Version {0}", this.getVersion());
		this._context.configuration._log(this._context.logger);
		this._configLogged = true;
	}
	this._protocolHandler.connect();
};

caplin.streamlink.impl.StreamLinkCoreImpl.prototype.deleteSubject = function(subject, listener) {
	var commandResponse = new caplin.streamlink.impl.command.CommandResponse(listener, this._context);
	this._subscriptionManager.send(commandResponse, new caplin.streamlink.impl.packet.PacketDelete(subject));
};

caplin.streamlink.impl.StreamLinkCoreImpl.prototype.disconnect = function() {
	this._protocolHandler.disconnect();
};

caplin.streamlink.impl.StreamLinkCoreImpl.prototype.removeConnectionListener = function(listener) {
	this._protocolHandler.removeConnectionListener(listener);
};



//## ClientBridge.js ##//
caplin.streamlink.js.impl.ClientBridge = function(configuration) 
{
	var runtime = new caplin.streamlink.js.impl.RuntimeImpl();
	caplin.streamlink.impl.bridge.worker.BridgeBase.call(this, runtime, false);
	var self = this;

	this.webWorker = new Worker(configuration._getWorkerUrl()); 
	
	this.webWorker.onmessage = function(event) 
	{
		var batch = event.data;
		for ( var i = 0; i < batch.length; i++)
		{
			var message = batch[i];
			if (message.id !== "BRIDGE")
			{
				self.receive(message.id, message.method, message.args);
			}
		}		
	};
};

caplin.extend(caplin.streamlink.js.impl.ClientBridge, caplin.streamlink.impl.bridge.worker.BridgeBase);

caplin.streamlink.js.impl.ClientBridge.prototype.log = function(message) 
{
};

caplin.streamlink.js.impl.ClientBridge.prototype.send = function(id, method, args) 
{
	this.webWorker.postMessage({'id': id, 'method': method, 'args': args});
};

//## DeleteAction.js ##//
caplin.namespace("caplin.streamlink.impl.subscription.container");

// caplin.forceload(caplin.streamlink.impl.subscription.container.Action

caplin.streamlink.impl.subscription.container.DeleteAction = function(obj, at) {
	// Constructor body
	this._obj = obj;
	this._at = at;
};

// caplin.implement(caplin.streamlink.impl.subscription.container.DeleteAction, caplin.streamlink.impl.subscription.container.Action);

caplin.streamlink.impl.subscription.container.DeleteAction.createFromDictionary = function(root) {
	var at = root["At"];
	var dict = root["Obj"];
	var obj = caplin.streamlink.impl.event.ContainerElementImpl.createFromDictionary(dict);
	return new caplin.streamlink.impl.subscription.container.DeleteAction(obj, at);
};

caplin.streamlink.impl.subscription.container.DeleteAction.prototype._toDictionary = function() {
	var root = {} ;
	root["actionType"] = "Delete";
	root["At"] = this._at;
	root["Obj"] = this._obj._toDictionary();
	return root;
};

caplin.streamlink.impl.subscription.container.DeleteAction.prototype.toString = function() {
	return "Delete " + this._at;
};

caplin.streamlink.impl.subscription.container.DeleteAction.prototype.applyToList = function(list) {
	var deleted = list.splice(this._at,1)[0];
	if ((deleted == (this._obj)) == false) {
		throw new Error(deleted + " was not " + this._obj);
	}
};

caplin.streamlink.impl.subscription.container.DeleteAction.prototype.applyToModel = function(model) {
	model.remove(this._at, this._obj);
};



//## PacketDelete.js ##//
caplin.namespace("caplin.streamlink.impl.packet");

// caplin.forceload(caplin.streamlink.impl.packet.Packet

caplin.streamlink.impl.packet.PacketDelete = function(subject) {
	// Constructor body
	this.subject = subject;
};

// caplin.implement(caplin.streamlink.impl.packet.PacketDelete, caplin.streamlink.impl.packet.Packet);

caplin.streamlink.impl.packet.PacketDelete.prototype.getType = function() {
	return caplin.streamlink.impl.packet.PacketType.DELETE;
};

caplin.streamlink.impl.packet.PacketDelete.prototype.getSubject = function() {
	return this.subject;
};

caplin.streamlink.impl.packet.PacketDelete.prototype.toRttpString = function(encoder) {
	var result = [] ;
	result.push(caplin.streamlink.impl.packet.PacketType.DELETE.getCode());
	result.push(" ");
	result.push(encoder.rttpEncode(this.subject));
	return result.join("");
};

caplin.streamlink.impl.packet.PacketDelete.prototype.toString = function() {
	var builder = [] ;
	builder.push("Delete [subject=");
	builder.push(this.subject);
	builder.push("]");
	return builder.join("");
};



//## ThrottleCommand.js ##//
caplin.namespace("caplin.streamlink");

/**
 * @class  <p>Enumeration that defines the available throttle levels and throttle commands.</p> 
 *  <p>A ThrottleCommand is passed as a parameter to the <code>throttleSubject()</code> 
 *     and <code>throttleEverything()</code> methods of {@link caplin.streamlink.StreamLink}.</p>
 *  <p>For an explanation of throttling, refer the Throttling section of the document <b>StreamLink Overview.</b></p>
 * 
 * @name caplin.streamlink.ThrottleCommand
 */
caplin.streamlink.ThrottleCommand = {
	/**
	 *  <p>Increase the throttle level.</p>
	 */
	UP: "UP",
	/**
	 *  <p>Decrease the throttle level.</p>
	 */
	DOWN: "DOWN",
	/**
	 *  <p>Set the throttle level to the minimum amount of throttling.</p>
	 */
	MIN: "MIN",
	/**
	 *  <p>Set the throttle level to the maximum amount of throttling.</p>
	 */
	MAX: "MAX",
	/**
	 *  <p>Stop sending updates to the client.</p>
	 */
	STOP: "STOP",
	/**
	 *  <p>Start sending updates to the client again.</p>
	 */
	START: "START",
	/**
	 *  <p>Set the throttle level to the default level.</p>
	 */
	DEF: "DEF",
	values: function() {return [caplin.streamlink.ThrottleCommand.UP, caplin.streamlink.ThrottleCommand.DOWN, caplin.streamlink.ThrottleCommand.MIN, caplin.streamlink.ThrottleCommand.MAX, caplin.streamlink.ThrottleCommand.STOP, caplin.streamlink.ThrottleCommand.START, caplin.streamlink.ThrottleCommand.DEF];},
	valueOf: function(name) {return name;}
};

//## SubscriptionOnClient.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.client");

// caplin.forceload(caplin.streamlink.Subscription
// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver

caplin.streamlink.impl.bridge.worker.client.SubscriptionOnClient = function(bridge, id, subject, listener) {
	// Constructor body
	this._bridge = bridge;
	this._id = id;
	this._subject = subject;
	this._listener = listener;
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.client.SubscriptionOnClient, caplin.streamlink.Subscription);
// caplin.implement(caplin.streamlink.impl.bridge.worker.client.SubscriptionOnClient, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.client.SubscriptionOnClient.prototype.getSubscriptionListener = function() {
	return this._listener;
};

caplin.streamlink.impl.bridge.worker.client.SubscriptionOnClient.prototype.getReceiverKey = function() {
	return null;
};

caplin.streamlink.impl.bridge.worker.client.SubscriptionOnClient.prototype.getReceiverId = function() {
	return this._id;
};

caplin.streamlink.impl.bridge.worker.client.SubscriptionOnClient.prototype.getSubject = function() {
	return this._subject;
};

caplin.streamlink.impl.bridge.worker.client.SubscriptionOnClient.prototype.receive = function(method, args) { };

caplin.streamlink.impl.bridge.worker.client.SubscriptionOnClient.prototype.setWindow = function(start, size) {
	var args = [start, size];
	this._bridge.send(this._id, "setWindow", args);
};

caplin.streamlink.impl.bridge.worker.client.SubscriptionOnClient.prototype.unsubscribe = function() {
	this._bridge.send(this._id, "unsubscribe", []);
};



//## StreamLinkFactory.js ##//
//## StreamLinkFactory.js ##//
caplin.namespace("caplin.streamlink");

caplin.include("caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl");
caplin.include("caplin.streamlink.impl.StreamLinkImpl");
caplin.include("caplin.streamlink.js.impl.protocolhandler.EncoderImpl");
caplin.include("caplin.streamlink.impl.StreamLinkContext");
caplin.include("caplin.streamlink.impl.connection.ConnectionProviderImpl");
caplin.include("caplin.streamlink.impl.subscription.SubscriptionManagerImpl");
caplin.include("caplin.streamlink.js.connection.ConnectionFactoryImpl");
caplin.include("caplin.streamlink.StreamLink");
caplin.include("caplin.streamlink.ConnectionFactory");
caplin.include("caplin.streamlink.Logger");
caplin.include("caplin.streamlink.impl.logger.LoggerImpl");
caplin.include("caplin.streamlink.impl.protocolhandler.ProtocolHandler");
caplin.include("caplin.streamlink.impl.subscription.SubscriptionManager");
caplin.include("caplin.streamlink.impl.connection.ConnectionProvider");
caplin.include("caplin.streamlink.impl.protocolhandler.Encoder");
caplin.include("caplin.streamlink.CredentialsProvider");
caplin.include("caplin.streamlink.impl.credentials.PasswordCredentialsProvider");

/**
 * @class  <p>Factory that creates {@link caplin.streamlink.StreamLink} instances.</p>
 * 
 * @name caplin.streamlink.StreamLinkFactory
 */
caplin.streamlink.StreamLinkFactory = function() 
{
};

caplin.streamlink.StreamLinkFactory._nextStreamLinkId = 0;
caplin.streamlink.StreamLinkFactory._streamLinkMap = {};

caplin.streamlink.StreamLinkFactory._getStreamLink = function(id) 
{
	return caplin.streamlink.StreamLinkFactory._streamLinkMap[id];
};

/**
 *  <p>Creates a StreamLink object that has the supplied StreamLink configuration settings.</p>
 *  @param {Map<String>} configuration StreamLink configuration as JSON attributes:
 *  
 *  <table>
 *  <tr>
 *  	<th>Attribute name</th>
 *  	<th>Type</th>
 *  	<th>Default</th>
 *  	<th>Optional</th>
 *      <th>Description</th>
 *  </tr>
 *  <tr>
 *  	<td>liberator_urls</td>
 *  	<td>string, [string], [[string]]</td>
 *  	<td>None</td>
 *  	<td>No</td>
 *  	<td>The Liberator URL groups to connect to (see below).</td>
 *  </tr>
 *  <tr>
 *  	<td>keymaster_url</td>
 *  	<td>string</td>
 *  	<td>None (KeyMaster not used)</td>
 *  	<td>Yes</td>
 *  	<td>URL of KeyMaster for secure authentication of Liberator login requests.</td>
 *  </tr>
 *  <tr>
 *  	<td>keymaster_keep_alive_url</td>
 *  	<td>string</td>
 *  	<td>None (KeyMaster not used)</td>
 *  	<td>Yes</td>
 *  	<td>URL that StreamLink uses to check the connection to KeyMaster every 
 *          <code>keymaster_keep_alive_interval</code> milliseconds.</td>
 *  </tr>
 *  <tr>
 *  	<td>keymaster_keep_alive_interval</td>
 *  	<td>number</td>
 *  	<td>600000 milliseconds <br />(10 minutes)</td>
 *  	<td>Yes</td>
 *  	<td>Interval in milliseconds at which StreamLink will poll KeyMaster 
 *          (on <code>keymaster_keep_alive_url</code>) to keep the KeyMaster session open.</td>
 *  </tr>
 *  <tr>
 *  	<td>keymaster_attempts</td>
 *  	<td>number</td>
 *  	<td>3 retries</td>
 *  	<td>Yes</td>
 *  	<td><p>The number of times that StreamLink tries to get the user-credentials 
 *             from KeyMaster before notifying an error.</p>
 *          <p>Also the number of times that StreamLink allows KeyMaster to successively time out 
 *             in response to a keep-alive poll (see <code>http-timeout</code>) before StreamLink notifies an error.</p></td>
 *  </tr>
 *  <tr>
 *  	<td>keymaster_keep_alive_listener</td>
 *  	<td>KeymasterResponseListener</td>
 *  	<td>None</td>
 *  	<td>Yes</td>
 *  	<td>The implementation of the {@link caplin.streamlink.KeymasterResponseListener} interface. 
 *          This receives information about responses to keep-alive requests made to KeyMaster.</td>
 *  </tr>
 *  <tr>
 *  	<td>username</td>
 *  	<td>string</td>
 *  	<td>None</td>
 *  	<td>Yes</td>
 *  	<td>Liberator login user-name to use if logins are not being managed by KeyMaster.</td>
 *  </tr>
 *  <tr>
 *  	<td>password</td>
 *  	<td>string</td>
 *  	<td>None</td>
 *  	<td>Yes</td>
 *  	<td>Liberator login password to use if logins are not being managed by KeyMaster.</td>
 *  </tr>
 *  <tr>
 *  	<td>poll_interval</td>
 *  	<td>number</td>
 *  	<td>1000 milliseconds <br />(1 second)</td>
 *  	<td>Yes</td>
 *  	<td>Time interval in  milliseconds between polls of the Liberator. 
 *          This only applies to HTTP polling type connections 
 *			(see 'RTTP connection types' in the document <b>StreamLink Overview</b>).</td>
 *  </tr>
 *  <tr>
 *  	<td>forever_frame_reconnect_count</td>
 *  	<td>number</td>
 *  	<td>10000</td>
 *  	<td>Yes</td>
 *  	<td>After receiving this number of messages, a forever frame connection will perform a fast reconnect 
 *          to prevent memory leaks.</td>
 *  </tr>
 *  <tr>
 *  	<td>xhr_streaming_reconnect_count</td>
 *  	<td>number</td>
 *  	<td>10000</td>
 *  	<td>Yes</td>
 *  	<td>After receiving this number of messages, an xhr streaming connection will perform a fast reconnect 
 *          to prevent memory leaks.</td>
 *  </tr>
 *  <tr>
 *  	<td>request_timeout</td>
 *  	<td>number</td>
 *  	<td>10000 milliseconds <br />(10 seconds)</td>
 *  	<td>Yes</td>
 *  	<td>Time in milliseconds allowed for the response to each request before StreamLink reconnects to the Liberator.</td>
 *  </tr>
 *  <tr>
 *  	<td>http_timeout</td>
 *  	<td>number</td>
 *  	<td>30000 milliseconds <br />(30 seconds)</td>
 *  	<td>Yes</td>
 *  	<td>Time in milliseconds allowed for an HTTP Request to finish successfully before StreamLink times out.</td>
 *  </tr>
 *  <tr>
 *  	<td>clock_offset_interval</td>
 *  	<td>number</td>
 *  	<td>10000 milliseconds <br />(10 seconds)</td>
 *  	<td>Yes</td>
 *  	<td>StreamLink calculates a new clock offset value every clock_offset_interval milliseconds.</td>
 *  </tr>
 *  <tr>
 *  	<td>clock_offset_window</td>
 *  	<td>number</td>
 *  	<td>120000 milliseconds <br />(2 minutes)</td>
 *  	<td>Yes</td>
 *  	<td>Time in milliseconds a calculated clock offset is valid for use in producing statistics.</td>
 *  </tr>
 *  <tr>
 *  	<td>latency_interval</td>
 *  	<td>number</td>
 *  	<td>10000 milliseconds <br />(10 seconds)</td>
 *  	<td>Yes</td>
 *  	<td>Time in milliseconds between calculating a new latency value.</td>
 *  </tr>
 *  <tr>
 *  	<td>latency_window</td>
 *  	<td>number</td>
 *  	<td>120000 milliseconds <br />(2 minutes)</td>
 *  	<td>Yes</td>
 *  	<td>Time in milliseconds a calculated latency is valid for use in producing statistics.</td>
 *  </tr>
 *  <tr>
 *  	<td>log_window_log_level</td>
 *  	<td>caplin.streamlink.LogLevel</td>
 *  	<td>caplin.streamlink.LogLevel.FINE</td>
 *  	<td>Yes</td>
 *  	<td>When the debug log window is shown, (see log_window_visible), only messages 
 *  		at or above the log level defined by log_window_log_level appear in the window.<br/>
 *  </td>
 *  </tr>
 *  <tr>
 *  	<td>log_window_visible</td>
 *  	<td>boolean</td>
 *  	<td>none</td>
 *  	<td>Yes</td>
 *  	<td>When true, the debug log window is shown when StreamLink first connects.<br/>
 *          When false, the debug log window cannot be shown.<br/>
 *          When left undefined the debug window can be shown if the page url contains a fragment of the form
 *          <code>?debug=[LogLevel name]</code>.
 *  	</tr>
 *  </table>
 *  
 *  <p><b>Failover configuration using liberator_urls.</b></p>
 *  <p>A URL group is selected at random. StreamLink tries to connect to them sequentially.</p>
 *  <p>If StreamLink did not successfully connect the next group will be tried.</p>
 *  <p>URL's must include the protocol (http, https, ws, wss, poll, polls).</p>
 *  <p>The protocol rttp means use the best non-ssl supported protocols.</p>
 *  <p>The protocol rttps means use the best ssl supported protocols.</p>
 *  
 *  <p><b>Examples:</b></p>
 *  
 *  
 *  <pre>
 *  caplin.streamlink.StreamLinkFactory.create({
 *    liberator_urls: "rttp://liberator1:8080",
 *    username: "test@example.com",
 *    password: "my password"
 *  });
 *  </pre>
 *  
 *  <pre>
 *  caplin.streamlink.StreamLinkFactory.create({
 *    liberator_urls: ["rttp://liberator1:8080", "rttp://liberator2:8080"],
 *    keymaster_url: "keymaster",
 *    keymaster_keep_alive_url: "keymasterkeepaliveurl"
 *  });
 *  </pre>
 *  
 *  <pre>
 *  caplin.streamlink.StreamLinkFactory.create({
 *    liberator_urls: [
 *      ["rttp://liberator1:8080", "rttp://liberator2:8080"],
 *      ["rttp://liberator3:8080", "rttp://liberator4:8080"]
 *    ],
 *    keymaster_url: "keymaster",
 *    keymaster_keep_alive_url: "keymasterkeepaliveurl",
 *    keymaster_keep_alive_interval: 300000,
 *    http_timeout: 30000,
 *    keymaster_attempts: 3
 *  });
 *  </pre>
 *  
 *  @returns {caplin.streamlink.StreamLink} A StreamLink object.
 *  @function
 */
caplin.streamlink.StreamLinkFactory.create = function(configuration) 
{
	if (!configuration)
	{
		throw new Error('JSON configuration must be provided.');
	}
	var self = caplin.streamlink.StreamLinkFactory;
	var streamLinkConfiguration = self._toStreamLinkConfiguration(configuration);
	var streamLink = self._newStreamLink(streamLinkConfiguration);
	self._streamLinkMap[self._nextStreamLinkId] = streamLink;
	return streamLink;
};

caplin.streamlink.StreamLinkFactory._toStreamLinkConfiguration = function(configuration) 
{
	var utils = caplin.streamlink.js.impl.Utils;
	var typeSchema = {
		username: utils.assertString,
		password: utils.assertString,
		keymaster_url: utils.assertString,
		keymaster_keep_alive_url : utils.assertString,
		keymaster_keep_alive_interval : utils.assertInteger,
		keymaster_attempts : utils.assertInteger,
		keymaster_keep_alive_listener : utils.assertKeymasterResponseListener,
		http_timeout: utils.assertInteger,
		forever_frame_reconnect_count : utils.assertInteger,
		xhr_streaming_reconnect_count : utils.assertInteger,
		poll_interval: utils.assertInteger,
		request_timeout: utils.assertInteger,
		clock_offset_interval: utils.assertInteger,
		clock_offset_window: utils.assertInteger,
		latency_interval: utils.assertInteger,
		latency_window: utils.assertInteger,
		application_id: utils.assertString,
		log_window_log_level: utils.assertLogLevel,
		log_window_visible: utils.assertBoolean,
		liberator_urls: function () {}
	};
	utils.validateTypeSchema(typeSchema, configuration);
	
	var result = caplin.streamlink.impl.StreamLinkConfigurationImpl.create();
	caplin.streamlink.StreamLinkFactory._addLiberatorUrls(result, configuration.liberator_urls);
	
	for (var attribute in configuration)
	{
		var value = configuration[attribute];
		var camelCase = attribute.replace(/_([a-z])/g, function(all, group) { return group.toUpperCase(); });
		var setter = "set" + camelCase.slice(0, 1).toUpperCase() + camelCase.slice(1);
		if (typeof result[setter] === 'function')
		{
			result[setter](value);
		}
	}
	result._validate();
	return result;
};

caplin.streamlink.StreamLinkFactory._addLiberatorUrls = function(streamLinkConfiguration, liberatorUrls) 
{
	var utils = caplin.streamlink.js.impl.Utils;
	if (!liberatorUrls)
	{
		throw new Error('liberator_urls is required on the configuration.')
	}
	
	if (utils.isString(liberatorUrls))
	{
		streamLinkConfiguration.addLiberatorUrlGroup(liberatorUrls);
	}
	else if (utils.isStringArray(liberatorUrls))
	{
		streamLinkConfiguration.addLiberatorUrlGroup.apply(streamLinkConfiguration, liberatorUrls);
	}
	else if (utils.isArray(liberatorUrls))
	{
		for ( var i = 0; i !== liberatorUrls.length; ++i)
		{
			if (!utils.isStringArray(liberatorUrls[i]))
			{
				throw new Error('liberator_urls must be a String, [String] or [[String]].')
			}
			streamLinkConfiguration.addLiberatorUrlGroup.apply(streamLinkConfiguration, liberatorUrls[i]);
		}
	}
	else
	{
		throw new Error('liberator_urls must be a String, [String] or [[String]].')
	}
};

caplin.streamlink.StreamLinkFactory._newStreamLink = function(config) 
{
	var self = caplin.streamlink.StreamLinkFactory;
	var streamLinkId = self._nextStreamLinkId++;
	var streamLink = config._isRunOnWorker() ? self._newStreamLinkOnWorker(config) : self._newStreamLinkNoWorker(config);
	self._streamLinkMap[self._nextStreamLinkId] = streamLink;
	return streamLink;
};

caplin.streamlink.StreamLinkFactory._newStreamLinkOnWorker = function(configuration) 
{
	var sl = caplin.streamlink;
	var bridge = new sl.js.impl.ClientBridge(configuration);
	var clientFactory = new sl.impl.bridge.worker.client.FactoryOnClient(bridge);
	var streamLinkCore = clientFactory.createStreamLink(configuration);
	var streamLinkId = sl.StreamLinkFactory._nextStreamLinkId;
	return new sl.StreamLink(streamLinkCore, streamLinkId, streamLinkCore.getLogger(), configuration);
};

caplin.streamlink.StreamLinkFactory._newStreamLinkNoWorker = function(configuration) 
{
	var sl = caplin.streamlink;
	var connectionFactory = new sl.js.connection.ConnectionFactoryImpl();
	var runtime = new sl.js.impl.RuntimeImpl();
	var encoder = new sl.js.impl.protocolhandler.EncoderImpl();
	var streamLinkFactoryImpl = new sl.impl.StreamLinkFactoryImpl(connectionFactory, "SLJS", runtime, encoder);
	var logger = new sl.impl.logger.LoggerImpl(runtime);
	var streamLinkCore = streamLinkFactoryImpl.create(logger, configuration, null);
	var streamLinkId = sl.StreamLinkFactory._nextStreamLinkId;
	return new sl.StreamLink(streamLinkCore, streamLinkId, logger, configuration);
};



//## CommandSubscriptionOnClient.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.client");

// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver
// caplin.forceload(caplin.streamlink.CommandSubscription

caplin.streamlink.impl.bridge.worker.client.CommandSubscriptionOnClient = function(bridge, id, subject, listener) {
	// Constructor body
	this._bridge = bridge;
	this._id = id;
	this._subject = subject;
	this._listener = listener;
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.client.CommandSubscriptionOnClient, caplin.streamlink.CommandSubscription);
// caplin.implement(caplin.streamlink.impl.bridge.worker.client.CommandSubscriptionOnClient, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.client.CommandSubscriptionOnClient.prototype.getCommandListener = function() {
	return this._listener;
};

caplin.streamlink.impl.bridge.worker.client.CommandSubscriptionOnClient.prototype.getReceiverKey = function() {
	return null;
};

caplin.streamlink.impl.bridge.worker.client.CommandSubscriptionOnClient.prototype.getReceiverId = function() {
	return this._id;
};

caplin.streamlink.impl.bridge.worker.client.CommandSubscriptionOnClient.prototype.getSubject = function() {
	return this._subject;
};

caplin.streamlink.impl.bridge.worker.client.CommandSubscriptionOnClient.prototype.receive = function(method, args) { };

caplin.streamlink.impl.bridge.worker.client.CommandSubscriptionOnClient.prototype.unPersist = function() {
	this._bridge.send(this._id, "unpersist", []);
};



//## ResponseQueue.js ##//
caplin.namespace("caplin.streamlink.impl.protocolhandler");


caplin.streamlink.impl.protocolhandler.ResponseQueue = function(protocolHandler, runtime, logger, requestTimeout) {
	// Field Definitions
	this.responseTimeoutQueue = [];
	this.responseQueue = [];

	// Constructor body
	this.protocolHandler = protocolHandler;
	this.runtime = runtime;
	this.logger = logger;
	this.requestTimeout = requestTimeout;
};

caplin.streamlink.impl.protocolhandler.ResponseQueue.prototype.isEmpty = function() {
	return (this.responseQueue.length==0);
};

caplin.streamlink.impl.protocolhandler.ResponseQueue.prototype.getTimer = function(timeout, duration) {
	var self = this;
	return this.runtime.createTimeout(function() {
		if (self.timeLastUpdateReceived > timeout.timeResponseSent) {
			timeout.timeResponseSent = self.runtime.currentTimeMillis();
			timeout.timer = self.getTimer(timeout, self.timeLastUpdateReceived - timeout.timeResponseSent + self.requestTimeout);
		}
		else {
			self.logger.log(caplin.streamlink.LogLevel.FINE, "Response not received within timeout period " + self.requestTimeout + " ms");
			self.cancelAllTimers();
			self.protocolHandler.reconnect();
		}
	}, duration);
};

caplin.streamlink.impl.protocolhandler.ResponseQueue.prototype.shift = function() {
	var timeout = this.responseTimeoutQueue.shift();
	if (timeout != null) {
		timeout.timer.cancel();
	}
	return this.responseQueue.shift();
};

caplin.streamlink.impl.protocolhandler.ResponseQueue.prototype.getTimeout = function() {
	var timeout = new caplin.streamlink.impl.protocolhandler.Timeout();
	timeout.timeResponseSent = this.runtime.currentTimeMillis();
	timeout.timer = this.getTimer(timeout, this.requestTimeout);
	return timeout;
};

caplin.streamlink.impl.protocolhandler.ResponseQueue.prototype.add = function(packet) {
	this.responseQueue.push(packet);
	this.responseTimeoutQueue.push(this.getTimeout());
};

caplin.streamlink.impl.protocolhandler.ResponseQueue.prototype.cancelAllTimers = function() {
	for (var i = 0; i != this.responseTimeoutQueue.length; ++(i)) {
		this.responseTimeoutQueue[i].timer.cancel();
	}
};

caplin.streamlink.impl.protocolhandler.ResponseQueue.prototype.checkResponseReceived = function(rttpEvent) {
	var packet = this.shift();
	if (packet == null) {
		return;
	}
	var packetSubject = packet.getSubject();
	var eventSubject = rttpEvent.getSubject();
	if (eventSubject != null && packetSubject != null && packetSubject.length > 0 && !((eventSubject == (packetSubject)))) {
		throw new Error("Request/Response mismatch - request: \'" + packetSubject + "\' response: \'" + eventSubject + "\'");
	}
};

caplin.streamlink.impl.protocolhandler.ResponseQueue.prototype.clear = function() {
	this.cancelAllTimers();
	(this.responseQueue.length = 0);
	(this.responseTimeoutQueue.length = 0);
};

caplin.streamlink.impl.protocolhandler.ResponseQueue.prototype.clearSystemMessages = function() {
	this.cancelAllTimers();
	var newResponseQueue = [];
	var newResponseTimeoutQueue = [];
	for (var i = 0; i < this.responseQueue.length; (i)++) {
		var packet = this.responseQueue[i];
		if (!(packet.getType().isSystem())) {
			newResponseQueue.push(packet);
			newResponseTimeoutQueue.push(this.responseTimeoutQueue[i]);
		}
	}
	this.responseQueue = newResponseQueue;
	this.responseTimeoutQueue = newResponseTimeoutQueue;
};

caplin.streamlink.impl.protocolhandler.ResponseQueue.prototype.responseReceived = function() {
	this.shift();
};

caplin.streamlink.impl.protocolhandler.ResponseQueue.prototype.updateReceived = function() {
	this.timeLastUpdateReceived = this.runtime.currentTimeMillis();
};



//## CommandSubscriptionImpl.js ##//
caplin.namespace("caplin.streamlink.impl.command");

// caplin.forceload(caplin.streamlink.CommandSubscription

caplin.streamlink.impl.command.CommandSubscriptionImpl = function(subscriptionManager, commandResponse, subject, listener) {
	// Constructor body
	this._subscriptionManager = subscriptionManager;
	this._commandResponse = commandResponse;
	this._subject = subject;
	this._listener = listener;
};

// caplin.implement(caplin.streamlink.impl.command.CommandSubscriptionImpl, caplin.streamlink.CommandSubscription);

/**
 *  <p>Returns the {@link caplin.streamlink.CommandListener} of this Subscription.</p>
 *  @returns {caplin.streamlink.CommandListener} {@link caplin.streamlink.CommandListener} commandListener The Command Listener
 * @function
 */
caplin.streamlink.impl.command.CommandSubscriptionImpl.prototype.getCommandListener = function() {
	return this._listener;
};

caplin.streamlink.impl.command.CommandSubscriptionImpl.prototype.getSubject = function() {
	return this._subject;
};

/**
 *  <p>Stop this Command from being Persistant. After this method has been invoked, the StreamLink will no longer send the Command to the Liberator
 *  on Reconnect</p>
 * @function
 */
caplin.streamlink.impl.command.CommandSubscriptionImpl.prototype.unPersist = function() {
	this._subscriptionManager.unPersist(this._commandResponse);
	this._commandResponse.unPersist();
};



//## SubscriptionOnWorker.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.server");

// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver

caplin.streamlink.impl.bridge.worker.server.SubscriptionOnWorker = function(id, sub) {
	// Constructor body
	this.id = id;
	this.subscription = sub;
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.server.SubscriptionOnWorker, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.server.SubscriptionOnWorker.prototype.getReceiverKey = function() {
	return null;
};

caplin.streamlink.impl.bridge.worker.server.SubscriptionOnWorker.prototype.getReceiverId = function() {
	return this.id;
};

caplin.streamlink.impl.bridge.worker.server.SubscriptionOnWorker.prototype.receive = function(method, args) {
	if ((method == ("unsubscribe"))) {
		this.subscription.unsubscribe();
	}
	else if ((method == ("setWindow"))) {
		this.subscription.setWindow(args[0], args[1]);
	}
};



//## FactoryOnWorker.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.server");

// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver

caplin.streamlink.impl.bridge.worker.server.FactoryOnWorker = function(bridge, streamLinkFactory) {
	// Constructor body
	this.bridge = bridge;
	this.id = "Factory";
	this.streamLinkFactory = streamLinkFactory;
	bridge.addReceiver(this);
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.server.FactoryOnWorker, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.server.FactoryOnWorker.prototype.getReceiverKey = function() {
	return null;
};

caplin.streamlink.impl.bridge.worker.server.FactoryOnWorker.prototype.getReceiverId = function() {
	return this.id;
};

caplin.streamlink.impl.bridge.worker.server.FactoryOnWorker.prototype.createStreamLink = function(streamLinkId, loggerId, configurationDict, keymasterResponseListenerId) {
	var configuration = caplin.streamlink.impl.StreamLinkConfigurationImpl.createFromDictionary(configurationDict);
	var logger = new caplin.streamlink.impl.bridge.worker.server.LoggerOnWorker(this.bridge, loggerId);
	this.bridge.addReceiver(logger);
	var keymasterResponseListener = null;
	if (keymasterResponseListenerId != null) {
		keymasterResponseListener = new caplin.streamlink.impl.bridge.worker.server.KeymasterResponseListenerOnWorker(this.bridge, keymasterResponseListenerId);
		this.bridge.addReceiver(keymasterResponseListener);
	}
	var streamLink = this.streamLinkFactory.create(logger, configuration, keymasterResponseListener);
	this.bridge.addReceiver(new caplin.streamlink.impl.bridge.worker.server.StreamLinkOnWorker(this.bridge, streamLinkId, streamLink));
};

caplin.streamlink.impl.bridge.worker.server.FactoryOnWorker.prototype.receive = function(method, args) {
	if ((method == ("createStreamLink"))) {
		this.createStreamLink(args[0], args[1], args[2], args[3]);
	}
};



//## ConnectionListenerOnClient.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.client");

// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver

caplin.streamlink.impl.bridge.worker.client.ConnectionListenerOnClient = function(id, listener) {
	// Constructor body
	this._id = id;
	this._listener = listener;
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.client.ConnectionListenerOnClient, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.client.ConnectionListenerOnClient.prototype.getReceiverKey = function() {
	return this._listener;
};

caplin.streamlink.impl.bridge.worker.client.ConnectionListenerOnClient.prototype.getReceiverId = function() {
	return this._id;
};

caplin.streamlink.impl.bridge.worker.client.ConnectionListenerOnClient.prototype.receive = function(method, args) {
	if ((method == ("onConnectionStatusChange"))) {
		(this._listener.onConnectionStatusChange && this._listener.onConnectionStatusChange(caplin.streamlink.impl.event.ConnectionStatusEventImpl.createFromDictionary(args[0])));
	}
	else if ((method == ("onSourceStatusChange"))) {
		(this._listener.onSourceStatusChange && this._listener.onSourceStatusChange(caplin.streamlink.impl.event.SourceStatusEventImpl.createFromDictionary(args[0])));
	}
	else if ((method == ("onServiceStatusChange"))) {
		(this._listener.onServiceStatusChange && this._listener.onServiceStatusChange(caplin.streamlink.impl.event.ServiceStatusEventImpl.createFromDictionary(args[0])));
	}
	else if ((method == ("onStatisticsChange"))) {
		(this._listener.onStatisticsChange && this._listener.onStatisticsChange(caplin.streamlink.impl.event.StatisticsEventImpl.createFromDictionary(args[0])));
	}
};



//## MoveAction.js ##//
caplin.namespace("caplin.streamlink.impl.subscription.container");

// caplin.forceload(caplin.streamlink.impl.subscription.container.Action

caplin.streamlink.impl.subscription.container.MoveAction = function(obj, from, to) {
	// Constructor body
	this.obj = obj;
	this.from = from;
	this.to = to;
};

// caplin.implement(caplin.streamlink.impl.subscription.container.MoveAction, caplin.streamlink.impl.subscription.container.Action);

caplin.streamlink.impl.subscription.container.MoveAction.createFromDictionary = function(root) {
	var from = root["From"];
	var to = root["To"];
	var dict = root["Obj"];
	var obj = caplin.streamlink.impl.event.ContainerElementImpl.createFromDictionary(dict);
	return new caplin.streamlink.impl.subscription.container.MoveAction(obj, from, to);
};

caplin.streamlink.impl.subscription.container.MoveAction.prototype._toDictionary = function() {
	var root = {} ;
	root["actionType"] = "Move";
	root["From"] = this.from;
	root["To"] = this.to;
	root["Obj"] = this.obj._toDictionary();
	return root;
};

caplin.streamlink.impl.subscription.container.MoveAction.prototype.toString = function() {
	return "Move " + this.obj.getSubject() + " from " + this.from + " to " + this.to;
};

caplin.streamlink.impl.subscription.container.MoveAction.prototype.applyToList = function(list) {
	var removedObj = list.splice(this.from,1)[0];
	list.splice(this.to, 0, removedObj);
};

caplin.streamlink.impl.subscription.container.MoveAction.prototype.applyToModel = function(model) {
	model.move(this.from, this.to, this.obj);
};



//## ContainerItem.js ##//
caplin.namespace("caplin.streamlink.impl.event");


caplin.streamlink.impl.event.ContainerItem = function(subject, type, objectNumber, add) {
	// Constructor body
	this.subject = subject;
	this.type = type;
	this.objectNumber = objectNumber;
	this.add = add;
};

caplin.streamlink.impl.event.ContainerItem.prototype.isAdded = function() {
	return this.add;
};

caplin.streamlink.impl.event.ContainerItem.prototype.getType = function() {
	return this.type;
};

caplin.streamlink.impl.event.ContainerItem.prototype.getObjectNumber = function() {
	return this.objectNumber;
};

caplin.streamlink.impl.event.ContainerItem.prototype.getSubject = function() {
	return this.subject;
};

caplin.streamlink.impl.event.ContainerItem.prototype.toString = function() {
	var builder = [] ;
	builder.push("[subject=");
	builder.push(this.subject);
	builder.push(", type=");
	builder.push(this.type);
	builder.push(", added=");
	builder.push(this.add);
	builder.push("]");
	return builder.join("");
};



//## DirectoryEventImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.impl.event.RttpEventBase
// caplin.forceload(caplin.streamlink.DirectoryEvent

caplin.streamlink.impl.event.DirectoryEventImpl = function(subject, image, elements, rttpCode, objectNumber, sequenceNumber) {
	// Super
	caplin.streamlink.impl.event.RttpEventBase.call(this, subject, rttpCode, objectNumber, sequenceNumber);
	// Constructor body
	this._image = image;
	this._elements = elements;
};

caplin.extend(caplin.streamlink.impl.event.DirectoryEventImpl, caplin.streamlink.impl.event.RttpEventBase);

// caplin.implement(caplin.streamlink.impl.event.DirectoryEventImpl, caplin.streamlink.DirectoryEvent);

caplin.streamlink.impl.event.DirectoryEventImpl.createFromDictionary = function(root) {
	var elements = [];
	var jsonElements = root["Elements"];
	for (var i = 0; i < jsonElements.length; (i)++) {
		var elm = caplin.streamlink.impl.event.DirectoryElementImpl.createFromDictionary(jsonElements[i]);
		elements.push(elm);
	}
	return new caplin.streamlink.impl.event.DirectoryEventImpl(root[caplin.streamlink.impl.event.RttpEventBase.SUBJECT], root["Image"], elements, caplin.streamlink.impl.utils.RttpUtil.getRttpCode(root[caplin.streamlink.impl.event.RttpEventBase.CODE]), root[caplin.streamlink.impl.event.RttpEventBase.OBJECT_NUMBER], root[caplin.streamlink.impl.event.RttpEventBase.SEQUENCE_NUMBER]);
};

caplin.streamlink.impl.event.DirectoryEventImpl.createFromRttp = function(rttpCode, tokens, fieldMap, encoder) {
	var objectNumber = null;
	var sequenceNumber = null;
	var subject = null;
	var image = false;
	var dirListingStartToken = 1;
	var elements = [];
	if (rttpCode == caplin.streamlink.impl.protocolhandler.RttpCode.DIRECTORY_RESP) {
		objectNumber = tokens[0].substring(2, 6);
		dirListingStartToken = 2;
		subject = encoder.rttpDecode(tokens[1]);
		image = true;
	}
	else if (rttpCode == caplin.streamlink.impl.protocolhandler.RttpCode.DIR_UPD) {
		sequenceNumber = tokens[0].substring(2, 4);
		objectNumber = tokens[0].substring(4, 8);
		dirListingStartToken = 1;
	}
	for (var i = dirListingStartToken; i < tokens.length; (i)++) {
		var subjectType = tokens[i].split(";");
		var type = subjectType[1];
		var childSubject = encoder.rttpDecode(subjectType[0]);
		var childType = caplin.streamlink.impl.event.ParseUtils.getSubjectType(type);
		var added = !((type == ("0")));
		elements.push(new caplin.streamlink.impl.event.DirectoryElementImpl(childSubject, childType, added));
	}
	return new caplin.streamlink.impl.event.DirectoryEventImpl(subject, image, elements, rttpCode, objectNumber, sequenceNumber);
};

caplin.streamlink.impl.event.DirectoryEventImpl.prototype._toDictionary = function() {
	var root = caplin.streamlink.impl.event.RttpEventBase.prototype._toDictionary.call(this);
	var jsonElements = [];
	for (var i = 0; i < this._elements.length; (i)++) {
		var elm = this._elements[i];
		jsonElements.push(elm._toDictionary());
	}
	root["Elements"] = jsonElements;
	root["Image"] = this._image;
	return root;
};

caplin.streamlink.impl.event.DirectoryEventImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("DirectoryChangeEventImpl [subject=");
	builder.push(this._subject);
	builder.push(", elements=");
	builder.push(this._elements);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.DirectoryEventImpl.prototype.getChangedElements = function() {
	return this._elements;
};

caplin.streamlink.impl.event.DirectoryEventImpl.prototype._publishCommandResponse = function(listener) { };

caplin.streamlink.impl.event.DirectoryEventImpl.prototype._publishSubscriptionResponse = function(subscription, subscriptionListener) {
	(subscriptionListener.onDirectoryUpdate && subscriptionListener.onDirectoryUpdate(subscription, this));
};



//## KeymasterResponseListenerOnClient.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.client");

// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver

caplin.streamlink.impl.bridge.worker.client.KeymasterResponseListenerOnClient = function(id, keymasterResponseListener) {
	// Constructor body
	this.id = id;
	this.keymasterResponseListener = keymasterResponseListener;
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.client.KeymasterResponseListenerOnClient, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.client.KeymasterResponseListenerOnClient.prototype.getReceiverKey = function() {
	return this.keymasterResponseListener;
};

caplin.streamlink.impl.bridge.worker.client.KeymasterResponseListenerOnClient.prototype.getReceiverId = function() {
	return this.id;
};

caplin.streamlink.impl.bridge.worker.client.KeymasterResponseListenerOnClient.prototype.receive = function(method, args) {
	if ((method == ("onKeymasterResponse"))) {
		(this.keymasterResponseListener.onKeymasterResponse && this.keymasterResponseListener.onKeymasterResponse(args[0]));
	}
	else if ((method == ("onKeymasterError"))) {
		this.keymasterResponseListener.onKeymasterError(args[0]);
	}
};



//## ContainerEventImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.impl.Serializable
// caplin.forceload(caplin.streamlink.ContainerEvent

caplin.streamlink.impl.event.ContainerEventImpl = function(subject, size, windowStart, windowEnd, changes) {
	// Constructor body
	this._subject = subject;
	this._size = size;
	this._windowStart = windowStart;
	this._windowEnd = windowEnd;
	this._changes = changes;
	this._image = false;
};

// caplin.implement(caplin.streamlink.impl.event.ContainerEventImpl, caplin.streamlink.ContainerEvent);
// caplin.implement(caplin.streamlink.impl.event.ContainerEventImpl, caplin.streamlink.impl.Serializable);

caplin.streamlink.impl.event.ContainerEventImpl.prototype.isImage = function() {
	return this._image;
};

caplin.streamlink.impl.event.ContainerEventImpl.createFromDictionary = function(root) {
	var subject = root["subject"];
	var size = root["size"];
	var windowStart = root["windowStart"];
	var windowEnd = root["windowEnd"];
	var chgs = root["changes"];
	var changes = [];
	for (var i = 0; i < chgs.length; (i)++) {
		var dict = chgs[i];
		var actionType = dict["actionType"];
		var action = null;
		if ((actionType == ("Clear"))) {
			action = caplin.streamlink.impl.subscription.container.ClearAction.createFromDictionary(dict);
		}
		else if ((actionType == ("Insert"))) {
			action = caplin.streamlink.impl.subscription.container.InsertAction.createFromDictionary(dict);
		}
		else if ((actionType == ("Delete"))) {
			action = caplin.streamlink.impl.subscription.container.DeleteAction.createFromDictionary(dict);
		}
		else if ((actionType == ("Move"))) {
			action = caplin.streamlink.impl.subscription.container.MoveAction.createFromDictionary(dict);
		}
		changes.push(action);
	}
	return new caplin.streamlink.impl.event.ContainerEventImpl(subject, size, windowStart, windowEnd, changes);
};

caplin.streamlink.impl.event.ContainerEventImpl.prototype._getChanges = function() {
	return this._changes;
};

caplin.streamlink.impl.event.ContainerEventImpl.prototype._toDictionary = function() {
	var root = {} ;
	root["subject"] = this._subject;
	root["size"] = this._size;
	root["windowStart"] = this._windowStart;
	root["windowEnd"] = this._windowEnd;
	var chgs = [];
	for (var i = 0; i < this._changes.length; (i)++) {
		var action = this._changes[i];
		var dict = action._toDictionary();
		chgs.push(dict);
	}
	root["changes"] = chgs;
	return root;
};

caplin.streamlink.impl.event.ContainerEventImpl.prototype.getSize = function() {
	return this._size;
};

caplin.streamlink.impl.event.ContainerEventImpl.prototype.getWindowEnd = function() {
	return this._windowEnd;
};

caplin.streamlink.impl.event.ContainerEventImpl.prototype.getWindowStart = function() {
	return this._windowStart;
};

caplin.streamlink.impl.event.ContainerEventImpl.prototype.getSubject = function() {
	return this._subject;
};

caplin.streamlink.impl.event.ContainerEventImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("ContainerEvent [subject=");
	builder.push(this._subject);
	builder.push(", size=");
	builder.push(this._size);
	builder.push(", windowStart=");
	builder.push(this._windowStart);
	builder.push(", windowEnd=");
	builder.push(this._windowEnd);
	builder.push(", changes=[");
	builder.push(this._changes.toString());
	builder.push("]");
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.ContainerEventImpl.prototype.updateModel = function(containerModel) {
	for (var i = 0; i < this._changes.length; (i)++) {
		var action = this._changes[i];
		action.applyToModel(containerModel);
	}
};



//## RecordType3Event.js ##//
caplin.namespace("caplin.streamlink");

// caplin.forceload(caplin.streamlink.DataEvent

/**
 * @class  <p>Provides information about a change to type 3 data in a record.</p>
 *  <p>The <code>RecordType3Event</code> will be provided on a <code>onRecordType3Update</code> 
 *  callback to a {@link caplin.streamlink.SubscriptionListener}.</p>
 *  
 *  <p>Type 3 data stores the history of updates to the record. A common use for records
 *  of this type is holding and viewing daily trade activity for an instrument.</p>
 *  
 *  @extends caplin.streamlink.DataEvent 
 * 
 * @name caplin.streamlink.RecordType3Event
 */
caplin.streamlink.RecordType3Event = function() {};

// caplin.implement(caplin.streamlink.RecordType3Event, caplin.streamlink.DataEvent);

/**
 *  <p>Returns whether all type 3 entries should be deleted.</p>
 *  @returns {boolean} true if all entries should be deleted, otherwise false.
 * @function
 */
caplin.streamlink.RecordType3Event.prototype.deleteAllEntries = function() { throw new Error('caplin.streamlink.RecordType3Event.deleteAllEntries :  This is an abstract method'); };

/**
 *  <p>Returns the record fields relating to the event, as a collection of key-value pairs.</p>
 * 
 *  <p>When a call to isImage() returns true, the subscription data event contains an image of the data
 *  for the subscribed item. All the fields comprising the record are present in the collection. The application
 *  must discard any values previously received and replace them with the values contained within the
 *  collection.</p>
 *  
 *  <p>When a call to isImage() returns false, the subscription data event contains new data for the subscribed
 *  record. The values in the collection returned are an update to any values previously received and any
 *  application caches should be updated appropriately.</p>
 *  
 *  @returns {Object} the fields
 * @function
 */
caplin.streamlink.RecordType3Event.prototype.getFields = function() { throw new Error('caplin.streamlink.RecordType3Event.getFields :  This is an abstract method'); };



//## RecordType3EventImpl.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.RecordType3Event
// caplin.forceload(caplin.streamlink.impl.Serializable
// caplin.forceload(caplin.streamlink.impl.event.RttpEventBase
// caplin.forceload(caplin.streamlink.DataEvent

caplin.streamlink.impl.event.RecordType3EventImpl = function(subject, image, fields, deleteAll, rttpCode, objectNumber, sequenceNumber) {
	// Super
	caplin.streamlink.impl.event.RttpEventBase.call(this, subject, rttpCode, objectNumber, sequenceNumber);
	// Constructor body
	this._image = image;
	this._fields = fields;
	this._deleteAllEntries = deleteAll;
};

caplin.extend(caplin.streamlink.impl.event.RecordType3EventImpl, caplin.streamlink.impl.event.RttpEventBase);

// caplin.implement(caplin.streamlink.impl.event.RecordType3EventImpl, caplin.streamlink.DataEvent);
// caplin.implement(caplin.streamlink.impl.event.RecordType3EventImpl, caplin.streamlink.RecordType3Event);
// caplin.implement(caplin.streamlink.impl.event.RecordType3EventImpl, caplin.streamlink.impl.Serializable);

caplin.streamlink.impl.event.RecordType3EventImpl.prototype.deleteAllEntries = function() {
	return this._deleteAllEntries;
};

caplin.streamlink.impl.event.RecordType3EventImpl.prototype.isImage = function() {
	return this._image;
};

caplin.streamlink.impl.event.RecordType3EventImpl.createFromDictionary = function(root) {
	return new caplin.streamlink.impl.event.RecordType3EventImpl(root[caplin.streamlink.impl.event.RttpEventBase.SUBJECT], root["Image"], root["Fields"], root["DeleteAll"], caplin.streamlink.impl.utils.RttpUtil.getRttpCode(root[caplin.streamlink.impl.event.RttpEventBase.CODE]), root[caplin.streamlink.impl.event.RttpEventBase.OBJECT_NUMBER], root[caplin.streamlink.impl.event.RttpEventBase.SEQUENCE_NUMBER]);
};

caplin.streamlink.impl.event.RecordType3EventImpl.createFromRttp = function(rttpCode, tokens, fieldMap, encoder) {
	var objectNumber = tokens[0].substring(4, 8);
	var sequenceNumber = tokens[0].substring(2, 4);
	if (rttpCode == caplin.streamlink.impl.protocolhandler.RttpCode.REC3_UPD) {
		var fields = caplin.streamlink.impl.event.ParseUtils.getFields(tokens, fieldMap, encoder, 1);
		return new caplin.streamlink.impl.event.RecordType3EventImpl(null, false, fields, false, rttpCode, objectNumber, sequenceNumber);
	}
	else if (rttpCode == caplin.streamlink.impl.protocolhandler.RttpCode.REC3_IMG || rttpCode == caplin.streamlink.impl.protocolhandler.RttpCode.REC3_CLR) {
		var fields = caplin.streamlink.impl.event.ParseUtils.getFields(tokens, fieldMap, encoder, 1);
		return new caplin.streamlink.impl.event.RecordType3EventImpl(null, true, fields, true, rttpCode, objectNumber, sequenceNumber);
	}
	return null;
};

caplin.streamlink.impl.event.RecordType3EventImpl.prototype._toDictionary = function() {
	var root = caplin.streamlink.impl.event.RttpEventBase.prototype._toDictionary.call(this);
	root["DeleteAll"] = this._deleteAllEntries;
	root["Image"] = this._image;
	root["Fields"] = this._fields;
	return root;
};

caplin.streamlink.impl.event.RecordType3EventImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("RecordType3DataEvent [subject=");
	builder.push(this._subject);
	builder.push(", fields=[");
	builder.push(caplin.xlang.Utils.dictionaryToString(this._fields));
	builder.push("]");
	builder.push(", image=");
	builder.push(this._image);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.RecordType3EventImpl.prototype.getFields = function() {
	return this._fields;
};

caplin.streamlink.impl.event.RecordType3EventImpl.prototype._publishCommandResponse = function(listener) { };

caplin.streamlink.impl.event.RecordType3EventImpl.prototype._publishSubscriptionResponse = function(subscription, subscriptionListener) {
	(subscriptionListener.onRecordType3Update && subscriptionListener.onRecordType3Update(subscription, this));
};



//## RecordSubscriptionParametersImpl.js ##//
caplin.namespace("caplin.streamlink.impl.subscription");

// caplin.forceload(caplin.streamlink.RecordSubscriptionParameters
// caplin.forceload(caplin.streamlink.impl.SubscriptionParametersInternal
// caplin.forceload(caplin.streamlink.impl.Serializable

caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl = function() {};

// caplin.implement(caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl, caplin.streamlink.RecordSubscriptionParameters);
// caplin.implement(caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl, caplin.streamlink.impl.Serializable);
// caplin.implement(caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl, caplin.streamlink.impl.SubscriptionParametersInternal);

caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl.prototype.setFields = function(fields) {
	this._fields = fields;
	return this;
};

caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl.prototype.setFilter = function(filter, isImage) {
	this._imageFilter = isImage;
	this._filter = filter;
	return this;
};

caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl.prototype._toDictionary = function() {
	var root = {} ;
	root["type"] = "SubscriptionParameters";
	root["filter"] = this._filter;
	root["imageFilter"] = this._imageFilter;
	root["fields"] = this._fields;
	return root;
};

caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl.prototype.toRttpString = function(encoder) {
	if (this._fields == null && this._filter == null) {
		return "";
	}
	var sb = [] ;
	sb.push(";");
	if (this._filter != null) {
		if (this._imageFilter) {
			sb.push("imagefilter=");
		}
		else {
			sb.push("filter=");
		}
		sb.push(this._filter);
	}
	if (this._fields != null && this._fields.length > 0) {
		if (this._filter != null) {
			sb.push(",");
		}
		for (var i = 0; i < this._fields.length; (i)++) {
			if (i != 0) {
				sb.push(",");
			}
			var encodedField = encoder.rttpEncode(this._fields[i]);
			sb.push(encodedField);
		}
	}
	return sb.join("");
};

caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("SubscriptionParameters [fields=");
	builder.push(this._fields);
	builder.push(", filter=");
	builder.push(this._filter);
	builder.push(", imageFilter=");
	builder.push(this._imageFilter);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl.prototype._fromDictionary = function(root) {
	this._filter = root["filter"];
	this._imageFilter = root["imageFilter"];
	this._fields = root["fields"];
};



//## ContainerSubscriptionParametersImpl.js ##//
caplin.namespace("caplin.streamlink.impl.subscription");

// caplin.forceload(caplin.streamlink.ContainerSubscriptionParameters
// caplin.forceload(caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl

caplin.streamlink.impl.subscription.ContainerSubscriptionParametersImpl = function(ctrid) {
	// Super
	caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl.call(this);
	// Field Definitions
	this._windowStart = -(1);
	this._windowSize = -(1);

	// Constructor body
	this._ctrid = ctrid;
};

caplin.extend(caplin.streamlink.impl.subscription.ContainerSubscriptionParametersImpl, caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl);

// caplin.implement(caplin.streamlink.impl.subscription.ContainerSubscriptionParametersImpl, caplin.streamlink.ContainerSubscriptionParameters);

caplin.streamlink.impl.subscription.ContainerSubscriptionParametersImpl.prototype._inWindow = function(position) {
	return this._windowStart != -(1) && this._windowSize != -(1) && position >= this._windowStart && position < this._windowStart + this._windowSize;
};

caplin.streamlink.impl.subscription.ContainerSubscriptionParametersImpl.prototype.setWindow = function(start, size) {
	this._windowStart = start;
	this._windowSize = size;
	return this;
};

caplin.streamlink.impl.subscription.ContainerSubscriptionParametersImpl.prototype._toDictionary = function() {
	var root = caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl.prototype._toDictionary.call(this);
	root["type"] = "ContainerSubscriptionParameters";
	root["windowStart"] = this._windowStart;
	root["windowSize"] = this._windowSize;
	root["ctrid"] = this._ctrid;
	return root;
};

caplin.streamlink.impl.subscription.ContainerSubscriptionParametersImpl.prototype._getCtrid = function() {
	return this._ctrid;
};

caplin.streamlink.impl.subscription.ContainerSubscriptionParametersImpl.prototype.toRttpString = function(encoder) {
	var result = [] ;
	result.push(";ctrid=");
	result.push(this._ctrid);
	if (this._windowStart != -(1)) {
		result.push(",ctrstart=");
		result.push(this._windowStart);
		result.push(",ctrend=");
		result.push(this._windowStart + this._windowSize - 1);
	}
	var recordParams = caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl.prototype.toRttpString.call(this, encoder);
	if (recordParams.length > 0) {
		result.push(",");
		result.push(recordParams.substring(1));
	}
	return result.join("");
};

caplin.streamlink.impl.subscription.ContainerSubscriptionParametersImpl.prototype.toString = function() {
	var builder = [] ;
	builder.push("SubscriptionParameters [fields=");
	builder.push(this._fields);
	builder.push(", filter=");
	builder.push(this._filter);
	builder.push(", imageFilter=");
	builder.push(this._imageFilter);
	builder.push(", windowStart=");
	builder.push(this._windowStart);
	builder.push(", windowSize=");
	builder.push(this._windowSize);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.subscription.ContainerSubscriptionParametersImpl.prototype._copy = function(other) {
	this._windowSize = other._windowSize;
	this._windowStart = other._windowStart;
	this._ctrid = other._ctrid;
	this._fields = other._fields;
	this._filter = other._filter;
	this._imageFilter = other._imageFilter;
};

caplin.streamlink.impl.subscription.ContainerSubscriptionParametersImpl.prototype._fromDictionary = function(root) {
	caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl.prototype._fromDictionary.call(this, root);
	this._windowStart = root["windowStart"];
	this._windowSize = root["windowSize"];
	this._ctrid = root["ctrid"];
};



//## PacketLogout.js ##//
caplin.namespace("caplin.streamlink.impl.packet");

// caplin.forceload(caplin.streamlink.impl.packet.Packet

caplin.streamlink.impl.packet.PacketLogout = function() {};

// caplin.implement(caplin.streamlink.impl.packet.PacketLogout, caplin.streamlink.impl.packet.Packet);

caplin.streamlink.impl.packet.PacketLogout.prototype.getType = function() {
	return caplin.streamlink.impl.packet.PacketType.LOGOUT;
};

caplin.streamlink.impl.packet.PacketLogout.prototype.getSubject = function() {
	return null;
};

caplin.streamlink.impl.packet.PacketLogout.prototype.toRttpString = function(encoder) {
	return "LOGOUT";
};

caplin.streamlink.impl.packet.PacketLogout.prototype.toString = function() {
	return "Logout";
};



//## StreamLinkOnClient.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.client");

// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver
// caplin.forceload(caplin.streamlink.impl.StreamLinkCore

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient = function(bridge, id, logger) {
	// Field Definitions
	this._nextCtrId = 1;

	// Constructor body
	this._bridge = bridge;
	this._id = id;
	this._logger = logger;
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient, caplin.streamlink.impl.StreamLinkCore);
// caplin.implement(caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.getReceiverId = function() 
{
	if (arguments.length == 0) return caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.getReceiverId_.apply(this, arguments);
	if (arguments.length == 1 && (arguments[0] == null || (arguments[0] instanceof java.lang.Object))) return caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.getReceiverId__javaXlangXObject.apply(this, arguments);
	throw new Error('No function found for these arguments.');
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.createCommandParameters = function() {
	return new caplin.streamlink.impl.command.CommandParametersImpl();
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.createSubject = function(subject, type, listener, parameters) {
	var jsonParameters = parameters._toDictionary();
	this.send("createSubject", subject, type, this.getReceiverId(listener), jsonParameters);
	return null;
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.publishToSubject = function(subject, fieldData, listener, parameters) {
	var commandListener = this._bridge.findReceiver(listener);
	if (commandListener == null) {
		commandListener = new caplin.streamlink.impl.bridge.worker.client.CommandListenerOnClient(this._bridge.getNextId(), listener);
		this._bridge.addReceiver(commandListener);
	}
	var commandSubscription = new caplin.streamlink.impl.bridge.worker.client.CommandSubscriptionOnClient(this._bridge, this._bridge.getNextId(), subject, listener);
	this._bridge.addReceiver(commandSubscription);
	var jsonParameters = parameters._toDictionary();
	this.send("publishToSubject", commandSubscription.getReceiverId(), subject, fieldData, commandListener.getReceiverId(), jsonParameters);
	return commandSubscription;
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.throttleEverything = function(command, listener, parameters) {
	var jsonParameters = parameters._toDictionary();
	this.send("throttleEverything", command, this.getReceiverId(listener), jsonParameters);
	return null;
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.throttleSubject = function(subject, command, listener, parameters) {
	var jsonParameters = parameters._toDictionary();
	this.send("throttleSubject", subject, command, this.getReceiverId(listener), jsonParameters);
	return null;
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.createContainerSubscriptionParameters = function() {
	return new caplin.streamlink.impl.subscription.ContainerSubscriptionParametersImpl((this._nextCtrId)++);
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.createFreeFormSubscriptionParameters = function() {
	return new caplin.streamlink.impl.subscription.FreeFormSubscriptionParametersImpl();
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.getLogger = function() {
	return this._logger;
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.createRecordSubscriptionParameters = function() {
	return new caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl();
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.subscribe = function(subject, listener, parameters) {
	var receiver = this._bridge.findReceiver(listener);
	if (receiver == null) {
		receiver = new caplin.streamlink.impl.bridge.worker.client.SubscriptionListenerOnClient(this._bridge, this._bridge.getNextId(), listener);
		this._bridge.addReceiver(receiver);
	}
	var subscription = new caplin.streamlink.impl.bridge.worker.client.SubscriptionOnClient(this._bridge, this._bridge.getNextId(), subject, listener);
	this._bridge.addReceiver(subscription);
	var params = (parameters != null) ? parameters._toDictionary() : null;
	this.send("subscribe", subscription.getReceiverId(), subject, receiver.getReceiverId(), params);
	return subscription;
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.getReceiverKey = function() {
	return null;
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.getReceiverId_ = function() {
	return this._id;
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.getReceiverId__javaXlangXObject = function(object) {
	return this._bridge.findReceiver(object).getReceiverId();
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.getVersion = function() {
	return caplin.streamlink.impl.Version.getVersion();
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.addConnectionListener = function(listener) {
	var receiver = this._bridge.findReceiver(listener);
	if (receiver == null) {
		receiver = new caplin.streamlink.impl.bridge.worker.client.ConnectionListenerOnClient(this._bridge.getNextId(), listener);
		this._bridge.addReceiver(receiver);
	}
	this.send("addConnectionListener", receiver.getReceiverId());
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.connect = function() {
	this.send("connect");
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.deleteSubject = function(subject, listener) {
	this.send("deleteSubject", subject, this.getReceiverId(listener));
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.disconnect = function() {
	this.send("disconnect");
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.receive = function(method, args) { };

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.removeConnectionListener = function(listener) {
	var receiver = this._bridge.findReceiver(listener);
	if (receiver != null) {
		this.send("removeConnectionListener", receiver.getReceiverId());
		this._bridge.removeReceiver(receiver);
	}
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.send = function(method) {
	var args = Array.prototype.slice.call(arguments, 1);	this._bridge.send(this._id, method, args);
};

caplin.streamlink.impl.bridge.worker.client.StreamLinkOnClient.prototype.setReachability = function(isReachable) {
	this.send("setReachability", isReachable);
};



//## StreamLinkOnWorker.js ##//
caplin.namespace("caplin.streamlink.impl.bridge.worker.server");

// caplin.forceload(caplin.streamlink.impl.bridge.worker.Receiver

caplin.streamlink.impl.bridge.worker.server.StreamLinkOnWorker = function(bridge, id, streamLink) {
	// Constructor body
	this.bridge = bridge;
	this.id = id;
	this.streamlink = streamLink;
};

// caplin.implement(caplin.streamlink.impl.bridge.worker.server.StreamLinkOnWorker, caplin.streamlink.impl.bridge.worker.Receiver);

caplin.streamlink.impl.bridge.worker.server.StreamLinkOnWorker.prototype.createSubscriptionParameters = function(params) {
	if (params == null) {
		return null;
	}
	else if ((params["type"] == ("SubscriptionParameters"))) {
		var subscriptionParameters = new caplin.streamlink.impl.subscription.RecordSubscriptionParametersImpl();
		subscriptionParameters._fromDictionary(params);
		return subscriptionParameters;
	}
	else {
		var subscriptionParameters = this.streamlink.createContainerSubscriptionParameters();
		subscriptionParameters._fromDictionary(params);
		return subscriptionParameters;
	}
};

caplin.streamlink.impl.bridge.worker.server.StreamLinkOnWorker.prototype.getReceiverKey = function() {
	return null;
};

caplin.streamlink.impl.bridge.worker.server.StreamLinkOnWorker.prototype.getReceiverId = function() {
	return this.id;
};

caplin.streamlink.impl.bridge.worker.server.StreamLinkOnWorker.prototype.addConnectionListener = function(connectionListenerId) {
	var listener = this.bridge.getReceiver(connectionListenerId);
	if (listener == null) {
		listener = new caplin.streamlink.impl.bridge.worker.server.ConnectionListenerOnWorker(this.bridge, connectionListenerId);
		this.bridge.addReceiver(listener);
	}
	this.streamlink.addConnectionListener(listener);
};

caplin.streamlink.impl.bridge.worker.server.StreamLinkOnWorker.prototype.publishToSubject = function(subject, commandSubscriptionId, fieldData, commandListenerId, commandParametersData) {
	var commandParameters = caplin.streamlink.impl.command.CommandParametersImpl._createFromDictionary(commandParametersData);
	var listener = this.bridge.getReceiver(commandListenerId);
	if (listener == null) {
		listener = new caplin.streamlink.impl.bridge.worker.server.CommandListenerOnWorker(this.bridge, commandListenerId);
		this.bridge.addReceiver(listener);
	}
	var commandSubscription = this.streamlink.publishToSubject(subject, fieldData, listener, commandParameters);
	this.bridge.addReceiver(new caplin.streamlink.impl.bridge.worker.server.CommandSubscriptionOnWorker(commandSubscriptionId, commandSubscription));
};

caplin.streamlink.impl.bridge.worker.server.StreamLinkOnWorker.prototype.receive = function(method, args) {
	this.bridge.log("call method " + method);
	if ((method == ("connect"))) {
		this.streamlink.connect();
	}
	else if ((method == ("disconnect"))) {
		this.streamlink.disconnect();
	}
	else if ((method == ("addConnectionListener"))) {
		this.addConnectionListener(args[0]);
	}
	else if ((method == ("removeConnectionListener"))) {
		this.removeConnectionListener(args[0]);
	}
	else if ((method == ("subscribe"))) {
		var subscriberId = args[0];
		var subject = args[1];
		var subscriptionListenerId = args[2];
		var params = args[3];
		this.subscribe(subscriberId, subject, subscriptionListenerId, params);
	}
	else if ((method == ("publishToSubject"))) {
		var commandSubscriptionId = args[0];
		var subject = args[1];
		var fieldData = args[2];
		var commandListenerId = args[3];
		var commandParametersData = args[4];
		this.publishToSubject(subject, commandSubscriptionId, fieldData, commandListenerId, commandParametersData);
	}
};

caplin.streamlink.impl.bridge.worker.server.StreamLinkOnWorker.prototype.removeConnectionListener = function(connectionListenerId) {
	var listener = this.bridge.getReceiver(connectionListenerId);
	if (listener != null) {
		this.streamlink.removeConnectionListener(listener);
	}
};

caplin.streamlink.impl.bridge.worker.server.StreamLinkOnWorker.prototype.subscribe = function(subscriberId, subject, subscriptionListenerId, params) {
	var subscriptionParameters = this.createSubscriptionParameters(params);
	var listener = this.bridge.getReceiver(subscriptionListenerId);
	if (listener == null) {
		listener = new caplin.streamlink.impl.bridge.worker.server.SubscriptionListenerOnWorker(this.bridge, subscriberId, subscriptionListenerId);
		this.bridge.addReceiver(listener);
	}
	var subscription = this.streamlink.subscribe(subject, listener, subscriptionParameters);
	this.bridge.addReceiver(new caplin.streamlink.impl.bridge.worker.server.SubscriptionOnWorker(subscriberId, subscription));
};



//## PacketContrib.js ##//
caplin.namespace("caplin.streamlink.impl.packet");

// caplin.forceload(caplin.streamlink.impl.packet.Packet

caplin.streamlink.impl.packet.PacketContrib = function(subject, fieldData) {
	// Constructor body
	this.subject = subject;
	this.fieldData = fieldData;
};

// caplin.implement(caplin.streamlink.impl.packet.PacketContrib, caplin.streamlink.impl.packet.Packet);

caplin.streamlink.impl.packet.PacketContrib.prototype.getType = function() {
	return caplin.streamlink.impl.packet.PacketType.CONTRIB;
};

caplin.streamlink.impl.packet.PacketContrib.prototype.getSubject = function() {
	return this.subject;
};

caplin.streamlink.impl.packet.PacketContrib.prototype.toRttpString = function(encoder) {
	var result = [] ;
	result.push(caplin.streamlink.impl.packet.PacketType.CONTRIB.getCode());
	result.push(" ");
	result.push(encoder.rttpEncode(this.subject));
	for (var key in this.fieldData) {
		var encodedKey = encoder.rttpEncode(key);
		var encodedValue = encoder.rttpEncode(this.fieldData[key]);
		result.push(" ");
		result.push(encodedKey);
		result.push("=");
		result.push(encodedValue);
	}
	return result.join("");
};

caplin.streamlink.impl.packet.PacketContrib.prototype.toString = function() {
	var builder = [] ;
	builder.push("Contrib [subject=");
	builder.push(this.subject);
	builder.push(", fields=");
	if (this.fieldData == null) {
		builder.push("null");
	}
	else {
		builder.push("[");
		builder.push(caplin.xlang.Utils.dictionaryToString(this.fieldData));
		builder.push("]");
	}
	builder.push("]");
	return builder.join("");
};



//## StreamLinkFactoryImpl.js ##//
caplin.namespace("caplin.streamlink.impl");

// caplin.forceload(caplin.streamlink.impl.StreamLinkFactoryInternal

caplin.streamlink.impl.StreamLinkFactoryImpl = function(connectionFactory, applicationIdBase, runtime, encoder) {
	// Constructor body
	this.connectionFactory = connectionFactory;
	this.runtime = runtime;
	this.encoder = encoder;
	this.applicationIdBase = applicationIdBase;
};

// caplin.implement(caplin.streamlink.impl.StreamLinkFactoryImpl, caplin.streamlink.impl.StreamLinkFactoryInternal);

caplin.streamlink.impl.StreamLinkFactoryImpl.prototype.getConnectionFactory = function() {
	return this.connectionFactory;
};

caplin.streamlink.impl.StreamLinkFactoryImpl.prototype.getRuntime = function() {
	return this.runtime;
};

caplin.streamlink.impl.StreamLinkFactoryImpl.prototype.create = function(logger, configuration, keymasterResponseListener) {
	var connectionProvider = new caplin.streamlink.impl.connection.ConnectionProviderImpl(logger, configuration._getLiberatorUrls());
	var context = new caplin.streamlink.impl.StreamLinkContext(this.runtime, logger, this.applicationIdBase, configuration, this.encoder);
	if (keymasterResponseListener != null) {
		configuration.setKeymasterKeepAliveListener(keymasterResponseListener);
	}
	var credentialsProvider = configuration._getCredentialsProvider(context, this.connectionFactory);
	this.connectionFactory.setContext(context);
	connectionProvider.setConnectionFactory(this.connectionFactory);
	var protocolHandler = new caplin.streamlink.impl.protocolhandler.ProtocolHandlerImpl(context, connectionProvider, credentialsProvider);
	var subscriptionManager = new caplin.streamlink.impl.subscription.SubscriptionManager(protocolHandler, new caplin.streamlink.impl.subscription.CurrentSubscriptions(), context.logger);
	protocolHandler.setProtocolListener(subscriptionManager);
	var clockOffsetInterval = configuration._getClockOffsetInterval();
	var clockOffsetsInWindow = configuration._getClockOffsetsInWindow();
	var latencysInWindow = configuration._getLatencysInWindow();
	var connectionMonitor = new caplin.streamlink.impl.protocolhandler.ConnectionMonitor(protocolHandler, context, clockOffsetInterval, clockOffsetsInWindow, latencysInWindow);
	protocolHandler.setConnectionMonitor(connectionMonitor);
	protocolHandler.addConnectionListener(connectionMonitor);
	return new caplin.streamlink.impl.StreamLinkCoreImpl(context, protocolHandler, subscriptionManager);
};

caplin.streamlink.impl.StreamLinkFactoryImpl.prototype.getEncoder = function() {
	return this.encoder;
};



//## ContribEvent.js ##//
caplin.namespace("caplin.streamlink.impl.event");

// caplin.forceload(caplin.streamlink.impl.event.RttpEventBase

caplin.streamlink.impl.event.ContribEvent = function(subject, code, objectNumber, sequenceNumber) {
	// Super
	caplin.streamlink.impl.event.RttpEventBase.call(this, subject, code, objectNumber, sequenceNumber);};

caplin.extend(caplin.streamlink.impl.event.ContribEvent, caplin.streamlink.impl.event.RttpEventBase);

caplin.streamlink.impl.event.ContribEvent.createFromRttp = function(rttpCode, tokens, fieldMap, encoder) {
	if (rttpCode == caplin.streamlink.impl.protocolhandler.RttpCode.CONTRIB_WAIT) {
		return new caplin.streamlink.impl.event.ContribEvent(encoder.rttpDecode(tokens[1]), rttpCode, tokens[0].substring(2, 6), null);
	}
	else if (rttpCode == caplin.streamlink.impl.protocolhandler.RttpCode.CONTRIB_OK_DELAY) {
		return new caplin.streamlink.impl.event.ContribEvent(encoder.rttpDecode(tokens[1]), rttpCode, tokens[0].substring(4, 8), tokens[0].substring(2, 4));
	}
	return null;
};

caplin.streamlink.impl.event.ContribEvent.prototype.toString = function() {
	var builder = [] ;
	builder.push("ContribEvent [subject=");
	builder.push(this._subject);
	builder.push("]");
	return builder.join("");
};

caplin.streamlink.impl.event.ContribEvent.prototype._publishCommandResponse = function(listener) {
	if (this._rttpCode == caplin.streamlink.impl.protocolhandler.RttpCode.CONTRIB_OK_DELAY) {
		(listener.onCommandOk && listener.onCommandOk(this._subject));
	}
};

caplin.streamlink.impl.event.ContribEvent.prototype._publishSubscriptionResponse = function(subscription, subscriptionListener) { };


caplin.streamlink.js.WorkerBridge = function(runtime)
{
	caplin.streamlink.impl.bridge.worker.BridgeBase.call(this, runtime, true);
	this.queue = [];
	this.timeoutId = null;
};

caplin.extend(caplin.streamlink.js.WorkerBridge, caplin.streamlink.impl.bridge.worker.BridgeBase);

caplin.streamlink.js.WorkerBridge.prototype.send = function(id, method, args)
{
	 this.queue.push({'id': id, 'method': method, 'args': args});
	 if (!this.timeoutId)
	 {
		 var self = this;
		 this.timeoutId = setTimeout(function () 
		 {
			 postMessage(self.queue);
			 self.queue = [];
			 self.timeoutId = null;
		 }, 5);
	 }
};

caplin.streamlink.js.WorkerBridge.setup = function()
{
	var sl = caplin.streamlink;
	var runtime = new sl.js.impl.RuntimeImpl();
	var bridge = new sl.js.WorkerBridge(runtime);
	try
	{
		if (typeof window === "undefined") // test if we are inside a webworker
		{
			onmessage = function(event)
			{
				var data = event.data;
				bridge.receive(data.id, data.method, data.args);
			};

			var connectionFactory = new sl.js.connection.ConnectionFactoryImpl();
			var encoder = new sl.js.impl.protocolhandler.EncoderImpl();
			var streamlinkFactory = new sl.impl.StreamLinkFactoryImpl(connectionFactory, "SLJS", runtime, encoder);
			new sl.impl.bridge.worker.server.FactoryOnWorker(bridge, streamlinkFactory);
		}
	}
	catch (error)
	{
		bridge.log(error.toString());
	}
}

caplin.streamlink.js.WorkerBridge.setup();
